#include "code/framework/Properties.h"
#include "code/framework/utils.h"
#include <fstream>
#include <iostream>
#include <sstream>
#include <string>
#include "code/framework/check/loaders.h"

using namespace utils;
using namespace std;

Properties::Properties()
{
	properties=new map<string, string>();
	m_fatalErrorEnabled=true;
}

Properties::Properties(string filename)
{
	properties=new map<string, string>();
	loadFromFile(filename);
	m_fatalErrorEnabled=true;
}

Properties::Properties(istream *stream)
{
	properties=new map<string, string>();
	loadFromStream(stream);
	m_fatalErrorEnabled=true;
}

Properties::~Properties()
{
	delete properties;
}

bool Properties::containsKey(string key)
{
	if(properties->find(key)==properties->end())
		return false;
	return true;
}

void Properties::loadFromStream(istream *stream)
{
	string line;
	while(getline(*stream,line))
	{
		if(line.find("[")!=0 && line.find("#")!=0 && line.find("//")!=0) //ne pas prendre les commentaires en compte
		{
			if(line.size()>2) //ne pas prendre en compte les lignes vides, au minimum "a=b"
			{
				int indice=line.find("=");
				string key=line.substr(0, indice); //longueur = indice
				string value=line.substr(indice+1, line.size()-1-indice);
				
				value=deleteAll(value, "\r");
				setValue(key, value);
			}
		}
	}
}

void Properties::loadFromString(string str)
{
	istringstream iss(str);
	loadFromStream(&iss);
}

void Properties::loadFromFile(string filename)
{
	ifstream preffile;
	
	fwkOpenTextFile(filename, &preffile);
		
	loadFromStream(&preffile);
	
	preffile.close();
}

void Properties::saveToFile(string filename)
{
//	ofstream outfile (filename.c_str(), ios_base::trunc | ios_base::out);
//	if(!outfile.good())
//	{
//		fatalError("Properties::saveToFile : cannot open file "+filename);
//	}

	ofstream outfile;
	fwkOpenTextFile(filename, &outfile, ios_base::trunc | ios_base::out);
	
	map<string,string>::iterator it;
	for(it=properties->begin() ; it!=properties->end(); it++)
	{
    	string key=(*it).first;
    	string value=(*it).second;
    	outfile << key << "=" << value << endl;
	}
	
	outfile.close();
}

string Properties::toString()
{
	ostringstream oss;
	map<string,string>::iterator it;
	for(it=properties->begin() ; it!=properties->end(); it++)
	{
    	string key=(*it).first;
    	string value=(*it).second;
    	oss << key << "=" << value << endl;
	}
	return oss.str();
}

string Properties::getStrValue(string key)
{
	map<string,string>::iterator it=properties->find(key);
	if(it!=properties->end())
	{
		return string((*it).second);
	}
	if(m_fatalErrorEnabled)
	{
		fatalError("Properties::getStrValue : key '"+key+"' does not exist.");
	}
	return string("");
}

OSL_COLOR Properties::getColorValue(string key)
{
	map<string,string>::iterator it=properties->find(key);
	if(it!=properties->end())
	{
		string s=string((*it).second);
		
		vector<string> vect=split(s, ",");
		if(vect.size()<3)
			fatalError("Properties::getColorValue : color property malformed: "+s);
			
		int r, g, b, a=255;
		r=toInt(vect[0]);
		g=toInt(vect[1]);
		b=toInt(vect[2]);
		if(vect.size()>3)
			a=toInt(vect[3]);
		
		return RGBA(r, g, b, a);
		
	}
	if(m_fatalErrorEnabled)
	{
		fatalError("Properties::getColorValue : key '"+key+"' does not exist.");
	}
	return 0;
}

int Properties::getIntValue(string key)
{
	map<string,string>::iterator it=properties->find(key);
	if(it!=properties->end())
	{
		string res=it->second;
		return toInt(res);
	}
	if(m_fatalErrorEnabled)
	{
		fatalError("Properties::getIntValue : key '"+key+"' does not exist.");
	}
	return 0;
}

float Properties::getFloatValue(string key)
{
	map<string,string>::iterator it=properties->find(key);
	if(it!=properties->end())
	{
		string res=it->second;
		return toFloat(res);
	}
	if(m_fatalErrorEnabled)
	{
		fatalError(string("Properties::getFloatValue : key '"+key+"' does not exist.").c_str());
	}
	return 0.0f;
}

bool Properties::getBoolValue(string key)
{
	map<string,string>::iterator it=properties->find(key);
	if(it!=properties->end())
	{
		string res=it->second;
		if(res=="true" || res=="yes")
			return true;
		else if(res=="false" || res=="no")
			return false;
		else
			fatalError(string("Properties::getBoolValue : key '"+key+"' is not a boolean.").c_str());
			
	}
	if(m_fatalErrorEnabled)
	{
		fatalError(string("Properties::getBoolValue : key '"+key+"' does not exist.").c_str());
	}
	return 0.0f;
}

string Properties::getStrValue(string key, string def)
{
	if(!containsKey(key))
		return def;
	return getStrValue(key);
}

OSL_COLOR Properties::getColorValue(string key, OSL_COLOR def)
{
	if(!containsKey(key))
		return def;
	return getColorValue(key);
}

int Properties::getIntValue(string key, int def)
{
	if(!containsKey(key))
		return def;
	return getIntValue(key);
}

float Properties::getFloatValue(string key, float def)
{
	if(!containsKey(key))
		return def;
	return getFloatValue(key);
}

bool Properties::getBoolValue(string key, bool def)
{
	if(!containsKey(key))
		return def;
	return getBoolValue(key);
}

void Properties::setValue(string key, string value)
{
	(*properties)[key]=value;
}

void Properties::setValue(string key, int value)
{
	(*properties)[key]=utils::toString(value);
}

void Properties::setFatalError(bool enable)
{
	m_fatalErrorEnabled=enable;
}




