#ifndef ANIM_H_
#define ANIM_H_

#include <list>
#include <vector>
#include "code/framework/anim/ImageAndAttr.h"
#include "OSLib/oslib.h"
#include "code/framework/interfaces/Positionable.h"
#include "code/framework/State.h"


//TODO: const pour les fonctions get
//TODO: playable, drawable, resizable, positionable

//class d'animations de sprites, basés sur plusieurs images.
//pas de spritesheets pour le moment.

//pour friend:
class AbstractResRepo;

class Anim : public Positionable
{
	private:	   	
	   	//variables de classe
	   	static list<ImageAndAttr *> anim_loaded_images;
	   	
	   	//membres
	   	vector<ImageAndAttr*> m_frames; //les images utilisées
	   	
	   	float m_delay_total; //le temps total
	   	float m_delay_frames; //le temps entre chaque frame
	   	float m_delay_left; //le temps qu'on a de trop, reste entre le temps passé et le temps mis pour arriver a la frame actuelle
	   	
	   	int m_currentFrame; //l'image courante
	   	int m_nbImages; //le nombre d'images que contient l'animation
	   	int m_loop; //le numero de loop en cours.
	   	bool m_running; //est ce que l'animation est en marche?
	   	float m_nbTicks; //le "timer" depuis le precedent changement, en ticks. utilisé pour le delay
	   	int m_maxLoops; //combien de boucles elle doit faire
		//position
	   	int m_x;
	   	int m_y;
	   
	protected:
		
   		
   		
	   	
   	public:
   		//methodes de classe (static)
   		static int getNbBufferedImages();
   		static ImageAndAttr *getImageWithName(string name);
   		static ImageAndAttr *bufferImage(ImageAndAttr *iaa);
	   	static void unbufferImage(ImageAndAttr *iaa);
	   	static void logFilesInBuffer();
   		
   		//TODO: remettre en protected
   		//construction d'une anim a 1 image, on n'est pas sensé en rajouter (anim bloquée)
   		Anim(OSL_IMAGE *image, string imageID);
   		
   		//construction d'une anim a 1 image, on peut ajouter des images
   		Anim(OSL_IMAGE *image, string imageId, float delay_total, int maxLoops=-1);
   		
   		void addImage(OSL_IMAGE *image, string imageId);
   		//end todo
   		
   		
   		
   		//const/dest
   		Anim(unsigned int nbImages, string header, string where, float delay_total, short transparency=OSL_PF_8888, int maxLoops=-1); //-1=infinite
   		
   		//construction d'une anim a 1 image, on n'est pas sencé en rajouter (anim bloquée)
   		Anim(string filename, OSL_PIXELFORMATS transparency=OSL_PF_8888);
   		
   		//construction d'une anim a 1 image, on peut ajouter des images
   		Anim(string filename, float delay_total, short transparency=OSL_PF_8888, int maxLoops=-1);
   		
   		~Anim();
   		Anim(const Anim &an);//copie
   		Anim& operator =(const Anim& Other);
   		
   		//methodes
   		void start();
   		void restart();
   		void reset();
   		void stop();
   		void update();
   		void drawXY(int x, int y);
   		void draw();
   		void setX(int x);
		void setY(int y);
		int getX();
		int getY();
		State *savestate(State * state);
		void restorestate(State * state);
   		//TODO: createcopy //normalement operator = ou constructeur par copie OK.
   		//Anim * createCopy();
   		
   		//TODO: draw avec des effets.
   		
   		//getters et setters
   		int getCurrentFrameNum() const;
   		bool isRunning() const;
   		OSL_IMAGE *getImage() const;
   		string getImageName() const;
   		void setReferencePoint(short x, short y);
   		void setReferencePointCenter();
   		bool finished() const;
   		
   		void addImage(string filename, short transparency=OSL_PF_8888);
   		
   		float getDelayTotal();
   		void setDelayTotal(float d);
   		float getDelayBetweenFrames();
   		int getNbImages();
   		void setNbLoops(int loops);
   		
   		void sync(Anim *anim);
   		
   		
   		
   		friend class AbstractResRepo;
   		//friend void resrepo::addImageToAnim(AbstractResRepo *res, Anim *anim, string filename, int transparency);
};

#endif /*ANIM_H_*/














