#ifndef BMPFONT_H_
#define BMPFONT_H_

#include "code/framework/bmpFont/CharRowCol.h"
#include "code/framework/tree.hh"
#include <string>
#include <oslib/oslib.h>
#include "code/framework/Pos.h"
#include <list>
#include "code/framework/bmpFont/BmpText.h"
#include <iostream>
#include <sstream>
#include "code/framework/fx/EffectsLib.h"

#define BMPFONT_INIT_DEFAULT_FOLDER "res/fonts/bmpFonts/"

//classe permettant l'utilisation d'un police sous forme de bitmap.
//elle permet d'avoir la totalité des 256 charactères ascii.
class BmpText; //pour friend

class BmpFont
{
	private:
		//variables statiques
		static bool isInit; //lazy init.
		static CharRowCol *chars; //256 entrées
		static tree<short> *bytesTree;
		
		//fonctions statiques
		static unsigned short * loadSizes(string fileradix); //renvoi un tableau
		static unsigned short * loadSizes(istream *fichier_sizes); //renvoi un tableau
		
		
		//données membres
		unsigned short * m_sizes; //les tailles de chaque caractere (numérotés)
		unsigned short * m_dsizes; //les tailles de dessin de chaque caractere (numérotés)
		Pos * m_positions; //tableau de Pos, positions de chaque caractère dans l'image
		OSL_IMAGE * m_img; //l'image de la police
		unsigned short m_width; //width of one cell 
		unsigned short m_height; //height of one cell
		
		bool m_cursive; //si la police est trop penchée, elle risque d'être tronquée.
		//si tel est le cas, en notifier la class avec cursive=true
		
	public:
		//fonctions statiques
		static void initBmpFont(string folder=BMPFONT_INIT_DEFAULT_FOLDER);
		static void initBmpFont(istream *stream_chars, istream *stream_bytes);
		static list<short> * getIndicesFromStr(string str);
		static void freeRes(); //libere les ressources utilisées pour créer des fonts
		
		//construction, destruction, operations
		BmpFont(OSL_IMAGE *img, istream *fichier_sizes, bool cursive=false); //a utiliser avec les resRepo
		BmpFont(string where, string filename, short transparency, bool cursive=false);
		BmpFont(const BmpFont &);
		~BmpFont();
		BmpFont& operator =(const BmpFont& iaa);
		
		//fonctions membres
		unsigned short getTextSize(string) const;
		unsigned short getTextSize(list<short> *indices) const;
		unsigned short getTextDrawSize(string) const;
		unsigned short getTextDrawSize(list<short> *indices) const;
		OSL_IMAGE * getImage();
		unsigned short getHeight() const;
		unsigned short getSize(int indice);
		unsigned short getDSize(int indice);
		void drawChar(short indice, short x, short y, AbstractLocalEffect *fx1=NULL, AbstractLocalEffect *fx2=NULL, AbstractLocalEffect *fx3=NULL, AbstractLocalEffect *fx4=NULL, AbstractLocalEffect *fx5=NULL);
		bool isCursive();
		
		//test
		static CharRowCol *getCRC(int indice);
		
		friend class BmpText;
};

#endif /*BMPFONT_H_*/










