#include "code/framework/bmpFont/BmpText.h"
#include "code/framework/utils.h"
//#include "code/framework/semaphores.h"
#include <pspsdk.h>

using namespace utils;

BmpText::BmpText(string s, BmpFont *bmpf)
{

	m_text=s;
	m_font=bmpf;
	m_img=NULL;
	m_width=0;
	m_dwidth=0;
	m_height=0;

	m_indices=BmpFont::getIndicesFromStr(s.c_str());

				
	m_angle=0;
	
	m_x=0;
	m_y=0;
}

string BmpText::getText()
{
	return m_text;
}

void BmpText::setText(string text)
{
	m_text=text;
	if(m_indices!=NULL)
		delete m_indices;
	m_indices=BmpFont::getIndicesFromStr(text.c_str());
	if(m_img!=NULL)
	{
		oslDeleteImage(m_img);
		m_img=NULL;
		compile();
		m_img->angle=-m_angle;
	}
	m_width=0;
	m_dwidth=0;
}

BmpText::BmpText(const BmpText &other)
{
	message("BmpText::construction par copie");
	m_text=other.m_text;
	m_font=other.m_font;
	
	m_x=other.m_x;
	m_y=other.m_y;
	
	if(other.m_img==NULL)
		m_img=NULL;
	else
		m_img=oslCreateImageCopy(other.m_img, other.m_img->location);	

	m_width=other.m_width;
	m_dwidth=other.m_dwidth;
	m_height=other.m_height;
	
	m_indices=NULL;
	if(other.m_indices!=NULL)
	{
		m_indices=new list<short>();
		list<short>::iterator it;
	  	for (it=other.m_indices->begin() ; it != other.m_indices->end(); it++ )
	    {
	    	m_indices->push_back(*it);
	    }
	}
	
	m_angle=other.m_angle;
}

BmpText::~BmpText()
{
	if(m_img!=NULL)
		oslDeleteImage(m_img);
	if(m_indices!=NULL)
		delete m_indices;
}

BmpText& BmpText::operator =(const BmpText& other)
{
	message("BmpText::operator =");
	
	m_text=other.m_text;
	m_font=other.m_font;
	m_x=other.m_x;
	m_y=other.m_y;
	
	if(m_img!=NULL)
	{
		oslDeleteImage(m_img);
	}
	
	if(other.m_img==NULL)
	{	
		m_img=NULL;
	}
	else
	{
		m_img=oslCreateImageCopy(other.m_img, other.m_img->location);
	}	

	m_width=other.m_width;
	m_dwidth=other.m_dwidth;
	m_height=other.m_height;
	
	if(m_indices!=NULL)
		delete m_indices;
	m_indices=NULL;
	if(other.m_indices!=NULL)
	{
		m_indices=new list<short>();
		list<short>::iterator it;
	  	for (it=other.m_indices->begin() ; it != other.m_indices->end(); it++ )
	    {
	    	m_indices->push_back(*it);
	    }
	}
	
	m_angle=other.m_angle;
	
	return *this;
}

/*int BmpText::getAngle()
{
	return m_angle;
}*/

/*void BmpText::setAngle(int angle)
{
	m_angle=angle;
}*/



OSL_IMAGE * BmpText::compile() 
{

	if(m_img==NULL)
	{
		
		m_dwidth=m_font->getTextDrawSize(m_indices);
		m_width=m_font->getTextSize(m_indices);
		m_height=m_font->getHeight();
		
		//creation de l'image
		OSL_IMAGE * img=oslCreateImage(m_font->isCursive()?m_dwidth:m_width, m_height, OSL_IN_RAM, /*m_font->getImage()->pixelFormat*/ OSL_PF_8888);
		if(img==NULL)
			fatalError("BmpText::compile : impossible de creer l'image");
				
		oslClearImage(img, RGBA(0, 0, 0, 0));
		
		short x=0, y=0; //position actuelle
		list<short>::iterator it;
	  	for (it=m_indices->begin() ; it != m_indices->end(); it++ )
	    {
	    	short indice=*it;
	    	
	    	//m_font->drawChar(indice, curX, y);
	    	//m_img,m_positions[indice].x,m_positions[indice].y,m_dsizes[indice],m_height
	    	//OSL_IMAGE *m_fontImg=m_font->m_img;
	    	drawImageOnImage(m_font->m_img, m_font->m_positions[indice].x, m_font->m_positions[indice].y, 
	    					m_font->m_dsizes[indice], m_font->m_height,	img, x, y);
	    	
	    	x+=m_font->getSize(indice);
	    }
		
		m_img=img;
		
		delete m_indices; //plus besoin une fois compilé
		m_indices=NULL;	
		
		m_img->angle=-m_angle;
		
		//oslSwizzleImage(m_img);
	}
	
	return m_img;
}

OSL_IMAGE * BmpText::getImage()
{
	if(m_img==NULL)
		compile();
	return m_img;
}


/*void BmpText::draw(int x, int y)//dessine sur le buffer courant en utilisant la font si l'image est nulle
{
	if(m_img==NULL)
	{
		float floatAngle=float(m_angle);
		m_font->getImage()->angle=-m_angle;
		float curX=float(x);
		float curY=float(y);
		
		list<short>::iterator it;
	  	for (it=m_indices->begin() ; it != m_indices->end(); it++ )
	    {
	    	//short indice=*it;
	    	m_font->drawChar(*it, utils::round(curX), utils::round(curY));
	    	//m_font->drawChar(*it, int(curX), int(curY));
	    	
	    	float charWidth=float(m_font->getSize(*it));
	    	
	    	curX+=oslCos(floatAngle, charWidth);
	    	curY-=oslSin(floatAngle, charWidth);
	    	
	    	//curX+=charWidth*vfpu_cosf(m_angle*0.0174532925f);
	    }
	    m_font->getImage()->angle=0;
	}
	else
	{
		oslDrawImageXY(m_img, x, y);
	}
}*/

unsigned short BmpText::getWidth()
{
	if(m_width==0) //n'a pas été compilé, ou n'a pas été appelé encore
		m_width=m_font->getTextSize(m_indices);
	return m_width;
}

unsigned short BmpText::getDWidth()
{
	if(m_dwidth==0) //n'a pas été compilé, ou n'a pas été appelé encore
		m_dwidth=m_font->getTextDrawSize(m_indices);
	return m_dwidth;
}

unsigned short BmpText::getHeight()
{
	if(m_height==0)
		m_height=m_font->getHeight();
	return m_height;
}

OSL_IMAGE * BmpText::createSwizzledImageCopy()
{
	if(m_img==NULL)
		compile();
	OSL_IMAGE *ret_img=oslCreateSwizzledImage(m_img, OSL_IN_RAM);
	ret_img->angle=m_angle;
	return ret_img;
}

//Drawable:
void BmpText::draw(AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3, AbstractLocalEffect *fx4, AbstractLocalEffect *fx5)
{
	drawXY(m_x, m_y, fx1, fx2, fx3, fx4, fx5);
}

void BmpText::drawXY(int x, int y, AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3, AbstractLocalEffect *fx4, AbstractLocalEffect *fx5)
{
	if(m_img==NULL)
	{
		float floatAngle=float(m_angle);
		m_font->getImage()->angle=-m_angle;
		float curX=float(x);
		float curY=float(y);
		
		list<short>::iterator it;
	  	for (it=m_indices->begin() ; it != m_indices->end(); it++ )
	    {
	    	//short indice=*it;
	    	m_font->drawChar(*it, utils::round(curX), utils::round(curY), fx1, fx2, fx3, fx4, fx5);
	    	
	    	float charWidth=float(m_font->getSize(*it));
	    	
	    	curX+=oslCos(floatAngle, charWidth);
	    	curY-=oslSin(floatAngle, charWidth);
	    }
	    m_font->getImage()->angle=0;
	}
	else
	{
		fx::drawXY(m_img, x, y, fx1, fx2, fx3, fx4, fx5);
		//oslDrawImageXY(m_img, x, y);
	}
}

//Positionable:
void BmpText::setX(int x)
{
	m_x=x;
}

void BmpText::setY(int y)
{
	m_y=y;
}

int BmpText::getX()
{
	return m_x;
}

int BmpText::getY()
{
	return m_y;
}


//Orientable:
int BmpText::getOrientation()
{
	return m_angle;
}

void BmpText::setOrientation(int angle)
{
	m_angle=angle;
}


//StateSavable:
State *BmpText::savestate(State * state)
{
	state->m_angle=m_angle;
	state->m_x=m_x;
	state->m_y=m_y;
	
	return state;
}

void BmpText::restorestate(State * state)
{
	m_angle=state->m_angle;
	m_x=state->m_x;
	m_y=state->m_y;
}




















