#ifndef BMPTEXT_H_
#define BMPTEXT_H_

#include <string>
#include "code/framework/bmpFont/BmpFont.h"
using namespace std;

#include "code/framework/interfaces/Drawable.h"
#include "code/framework/interfaces/Positionable.h"
#include "code/framework/interfaces/Orientable.h"
#include "code/framework/fx/EffectsLib.h"

//classe utilisant la classe BmpFont pour stocker des textes.
//par défaut elle ne stocke que les indices des caractères au départ
//(1er traitement couteux) mais peut le compiler en image bitmap
//(2ème traitement couteux) pour gagner en rapidité et permettre
//de les dessiner avec des effets.
class BmpFont; //pour friend

class BmpText : public Positionable, public Drawable, public Orientable
{
	private:
		//members
		string m_text;
		list<short> * m_indices; //TODO: utiliser ca partout constr, destr, copie
		BmpFont *m_font;
		OSL_IMAGE * m_img;
		//bool m_compiled; //utilise m_img, si c'est NULL, c'est pas compilé
		unsigned short m_width;
		unsigned short m_dwidth;
		unsigned short m_height; 
		
		int m_angle;
		int m_x, m_y;
	
	public:
		//construction/destruction
		BmpText(string s, BmpFont *bmpf);
		BmpText(const BmpText &other);
		~BmpText();
		BmpText& operator =(const BmpText& other);
		

		OSL_IMAGE * compile();
		OSL_IMAGE * getImage();
		OSL_IMAGE * createSwizzledImageCopy(); //crée une image détachée du texte,
		//pour pouvoir le deleter après et garder l'image uniquement
		
		
		//void draw(int x, int y);//dessine sur le buffer courant en utilisant la font
		unsigned short getWidth();
		unsigned short getDWidth();
		unsigned short getHeight();
		
		void setText(string text);
		string getText();
		
		//Drawable:
		void draw(AbstractLocalEffect *fx1=NULL, AbstractLocalEffect *fx2=NULL, AbstractLocalEffect *fx3=NULL, AbstractLocalEffect *fx4=NULL, AbstractLocalEffect *fx5=NULL);
		void drawXY(int x, int y, AbstractLocalEffect *fx1=NULL, AbstractLocalEffect *fx2=NULL, AbstractLocalEffect *fx3=NULL, AbstractLocalEffect *fx4=NULL, AbstractLocalEffect *fx5=NULL);
		
		//Positionable:
		void setX(int x);
		void setY(int y);
		int getX();
		int getY();
		
		//Orientable:
		int getOrientation();
		void setOrientation(int angle);
		
		//StateSavable:
		State *savestate(State * state);
		void restorestate(State * state);
		
};

#endif /*BMPTEXT_H_*/
