#include "code/framework/fx/impl/local/RotateBackAndForthEffect.h"
#include "code/framework/utils.h"
using namespace utils;

RotateBackAndForthEffect::RotateBackAndForthEffect(int time, int from, int to, int nbLoops) //-1 = infinite
{
	
	m_time=time;
	m_curLoop=0;
	m_curTime=0;
	m_nbLoops=nbLoops;
	m_running=false;
	m_from=from;
	m_to=to;
	m_direction=true;
	
	
	if(m_time<=0)
		oslFatalError("RotateBackAndForthEffect: time must be > 0");
}
 
	
void RotateBackAndForthEffect::start()
{
	m_running=true;
}
 
void RotateBackAndForthEffect::restart()
{
	m_curLoop=0;
	m_curTime=0;
	m_running=true;
	m_direction=true;
}
 
void RotateBackAndForthEffect::reset()
{
	m_curLoop=0;
	m_curTime=0;
	m_direction=true;
}
 
void RotateBackAndForthEffect::stop()
{
	m_running=false;
}
 
void RotateBackAndForthEffect::update()
{
	if(m_running)
	{
		m_curTime++;
		if(m_curTime==m_time)
		{
			m_curTime=0;
			m_curLoop++;
			m_direction=!m_direction;	
		}
		if(finished())
		{
			m_running=false;
		}
	}
	
}
 
bool RotateBackAndForthEffect::finished()
{
	if(m_nbLoops==-1) //infini
	{
		
		return false; //ca fini jamais
	}
	
	if(m_nbLoops==m_curLoop)
	{
		return true;
	}
	return false;
}
 
bool RotateBackAndForthEffect::isRunning()
{
	return m_running;
}
 
OSL_IMAGE * RotateBackAndForthEffect::applyOn(OSL_IMAGE *img)
{
	if(img==NULL)
		return NULL;
	
	
	int dist=m_to-m_from;
	int rot=int((m_curTime*dist)/m_time);
	if(m_direction) //de from a to
	{
		img->angle+=(m_from+rot);
	}
	else //de to a from
	{
		img->angle+=(m_to-rot);
	}

	return img;
}

Orientable *RotateBackAndForthEffect::applyOnOrientable(Orientable *obj)
{
	if(obj==NULL)
		return NULL;
	
	
	int dist=m_to-m_from;
	int rot=int((m_curTime*dist)/m_time);
	if(m_direction) //de from a to
	{
		//img->angle+=(m_from+rot);
		obj->setOrientation(obj->getOrientation()+m_from+rot);
	}
	else //de to a from
	{
		//img->angle+=(m_to-rot);
		obj->setOrientation(obj->getOrientation()+m_to-rot);
	}

	return obj;
}

RotateBackAndForthEffect* RotateBackAndForthEffect::clone() const
{
	return new RotateBackAndForthEffect(m_time, m_from, m_to, m_nbLoops);
}

void RotateBackAndForthEffect::setTime(int time)
{
	if(time<=0)
		oslFatalError("RotateBackAndForthEffect: time must be > 0");
	m_time=time;
}

int RotateBackAndForthEffect::getTime()
{
	return m_time;
}

int RotateBackAndForthEffect::getOrientation()
{
	int dist=m_to-m_from;
	int rot=int((m_curTime*dist)/m_time);
	if(m_direction) //de from a to
	{
		return m_from+rot;
	}
	//else //de to a from
	return m_to-rot;
}















