#include "code/framework/fx/impl/local/TranslationEffect.h"
#include "code/framework/utils.h"

#include <math.h>

using namespace utils;

TranslationEffect::TranslationEffect(int x0, int y0, int x1, int y1, int time01, int time10, bool start0, int nbLoops) //-1 = infinite
{
	m_curLoop=0;
	m_curTime=0;
	m_direction=start0;
	m_nbLoops=nbLoops;
	m_running=false;
	m_start0=start0;
	m_time01=time01;
	m_time10=time10;
	m_x0=float(x0);
	m_x1=float(x1);
	m_y0=float(y0);
	m_y1=float(y1);
	
	//distance entre chaque frame
	m_dist01FramesX=(m_x1-m_x0)/float(time01);
	m_dist01FramesY=(m_y1-m_y0)/float(time01);
	if(time10!=0)
	{
		m_dist10FramesX=(m_x0-m_x1)/float(time10);
		m_dist10FramesY=(m_y0-m_y1)/float(time10);
	}
	else
	{
		m_dist10FramesX=(m_x0-m_x1);
		m_dist10FramesY=(m_y0-m_y1);
	}
	
	//position actuelle
	if(start0)
	{
		m_curX=m_x0;
		m_curY=m_y0;
	}
	else
	{
		m_curX=m_x1;
		m_curY=m_y1;
	}
	
	//distance à parcourir
	m_distanceTotalX=abs(x1-x0);
	m_distanceTotalY=abs(y1-y0);
	
	//distance parcourue
	m_distanceCurrentX=0.0f;
	m_distanceCurrentY=0.0f;
}
 
	
void TranslationEffect::start()
{
	m_running=true;
}
 
void TranslationEffect::restart()
{
	reset();
	start();
}
 
void TranslationEffect::reset()
{
	m_curLoop=0;
	m_curTime=0;
	m_direction=m_start0;
	if(m_start0)
	{
		m_curX=m_x0;
		m_curY=m_y0;
	}
	else
	{
		m_curX=m_x1;
		m_curY=m_y1;
	}
	
	m_distanceCurrentX=0.0f;
	m_distanceCurrentY=0.0f;
}
 
void TranslationEffect::stop()
{
	m_running=false;
}
 
void TranslationEffect::update()
{
	int time=m_time01;
	float distFramesX=m_dist01FramesX;
	float distFramesY=m_dist01FramesY;
	if(!m_direction)
	{
		time=m_time10;
		distFramesX=m_dist10FramesX;
		distFramesY=m_dist10FramesY;
	}
	
	
	if(m_running)
	{
		m_curTime++;
		
		m_curX+=distFramesX;
		m_curY+=distFramesY;
				
		//distance parcourue
		m_distanceCurrentX+=fabs(distFramesX);
		m_distanceCurrentY+=fabs(distFramesY);
			
		//if(m_curTime>=time)
		if( m_distanceCurrentX>=m_distanceTotalX && m_distanceCurrentY>=m_distanceTotalY )
		{
			m_curTime=0;
			m_distanceCurrentX=0.0f;
			m_distanceCurrentY=0.0f;
			m_curLoop++;
			
			m_direction=!m_direction;
			
			if(m_direction)
			{
				if(m_start0)
				{
					m_curX=m_x0;
					m_curY=m_y0;
				}
				else
				{
					m_curX=m_x1;
					m_curY=m_y1;
				}
			}
			else
			{
				if(!m_start0)
				{
					m_curX=m_x0;
					m_curY=m_y0;
				}
				else
				{
					m_curX=m_x1;
					m_curY=m_y1;
				}
			}
			
			
			
		}
		if(finished())
		{
			m_running=false;
		}
	}
	
}
 
bool TranslationEffect::finished()
{
	
	if(m_nbLoops==-1) //infini
	{
		
		return false; //ca fini jamais
	}
	
	if(m_nbLoops==m_curLoop)
	{
		return true;
	}
	return false;
}
 
bool TranslationEffect::isRunning()
{
	return m_running;
}
 
OSL_IMAGE * TranslationEffect::applyOn(OSL_IMAGE *img)
{
	if(img==NULL)
		return NULL;
	
	img->x+=int(m_curX);
	img->y+=int(m_curY);

	return img;
}

Positionable *TranslationEffect::applyOnPositionable(Positionable *obj)
{
	if(obj==NULL)
		return NULL;
		
	obj->setX(obj->getX()+int(m_curX));
	obj->setY(obj->getY()+int(m_curY));

	return obj;
}
 
TranslationEffect* TranslationEffect::clone() const
{
	return new TranslationEffect(int(m_x0), int(m_y0), int(m_x1), int(m_y1), m_time01, m_time10, m_start0, m_nbLoops);
}


int TranslationEffect::getTimeForward()
{
	return m_time01;
}

void TranslationEffect::setTimeForward(int time)
{
	m_time01=time;
	
	//distance entre chaque frame
	m_dist01FramesX=(m_x1-m_x0)/float(time);
	m_dist01FramesY=(m_y1-m_y0)/float(time);
}

int TranslationEffect::getTimeBackWard()
{
	return m_time10;
}

void TranslationEffect::setTimeBackward(int time)
{
	m_time10=time;
	
	//distance entre chaque frame
	m_dist10FramesX=(m_x0-m_x1)/float(time);
	m_dist10FramesY=(m_y0-m_y1)/float(time);
}

Pos TranslationEffect::getStartPos()
{
	return Pos(int(m_x0), int(m_y0));
}

Pos TranslationEffect::getEndPos()
{
	return Pos(int(m_x1), int(m_y1));
}
		
Pos TranslationEffect::getCurrentPos()
{
	return Pos(int(m_curX), int(m_curY));
}

int TranslationEffect::getPosX()
{
	return int(m_curX);
}

int TranslationEffect::getPosY()
{
	return int(m_curY);
}











