#include "code/framework/fx/impl/local/multi/SerialLocalEffect.h"
#include "code/framework/check/PrettyFileLogger.h"
#include "code/framework/utils.h"

using namespace std;
using namespace utils;

SerialLocalEffect::SerialLocalEffect(int nbLoops)
{
	m_effects=new vector<AbstractLocalEffect *>();
	m_currentEffect=0;
	m_nbLoops=nbLoops;
	m_currentLoop=0;
}

SerialLocalEffect::~SerialLocalEffect()
{
	vector<AbstractLocalEffect *>::iterator it;
	for (it=m_effects->begin(); it<m_effects->end();it++)
    {
    	delete *it;
    }
    delete m_effects;
}


void SerialLocalEffect::start()
{
	if(m_currentEffect<m_effects->size())
	{
		m_effects->at(m_currentEffect)->start();	
	}
}

void SerialLocalEffect::restart()
{
	vector<AbstractLocalEffect *>::iterator it;
	for (it=m_effects->begin(); it<m_effects->end();it++)
    {
    	AbstractLocalEffect *alfx = *it;
    	alfx->stop();
    	alfx->reset();
    }
	
	/*if(m_currentEffect<m_effects->size())
	{
		m_effects->at(m_currentEffect)->stop();
		m_effects->at(m_currentEffect)->reset();
	}*/
	m_currentEffect=0;
	m_currentLoop=0;
	start();
}

void SerialLocalEffect::reset()
{
	if(m_currentEffect<m_effects->size())
	{
		m_effects->at(m_currentEffect)->reset();
	}
	m_currentEffect=0;
	m_currentLoop=0;
}

void SerialLocalEffect::stop()
{
	if(m_currentEffect<m_effects->size())
	{
		m_effects->at(m_currentEffect)->stop();
	}
}

void SerialLocalEffect::update()
{
	if(m_currentEffect<m_effects->size() && m_effects->at(m_currentEffect)->isRunning())
	{
		m_effects->at(m_currentEffect)->update();
		if(m_effects->at(m_currentEffect)->finished())
		{
			//m_effects[m_currentEffect]->stop();
			//m_effects->at(m_currentEffect)->reset();
			m_effects->at(m_currentEffect)->stop();
			m_currentEffect++;
			if(m_currentEffect<m_effects->size())
			{
				m_effects->at(m_currentEffect)->reset();
				m_effects->at(m_currentEffect)->start(); //on a vu que le précedent etait en marche, c'est une condition pour etre la
			}
			else
			{
				m_currentLoop++;
				if(m_nbLoops==-1 || m_currentLoop<m_nbLoops)
				{
					m_currentEffect=0;
					m_effects->at(m_currentEffect)->reset();
					m_effects->at(m_currentEffect)->start();	
				}	
			}
		}
	}
}

bool SerialLocalEffect::finished()
{
	//return !isRunning();
	if(m_nbLoops!=-1 && m_currentLoop>=m_nbLoops)
		return true;
	return false;
}

bool SerialLocalEffect::isRunning()
{
	if(m_effects->size()==0)
		return false;
		
	if(m_currentEffect<m_effects->size())
	{
		return m_effects->at(m_currentEffect)->isRunning();
	}
	return false;
}

OSL_IMAGE * SerialLocalEffect::applyOn(OSL_IMAGE *img)
{
	if(m_effects->size()==0)
		return img;
		
	if(m_currentEffect<m_effects->size())
	{
		return m_effects->at(m_currentEffect)->applyOn(img);
	}
	//return img;
	return m_effects->at(m_effects->size()-1)->applyOn(img);
}

Positionable *SerialLocalEffect::applyOnPositionable(Positionable *obj)
{
	if(m_effects->size()==0)
		return obj;
		
	if(m_currentEffect<m_effects->size())
	{
		return m_effects->at(m_currentEffect)->applyOnPositionable(obj);
	}
	//return img;
	return m_effects->at(m_effects->size()-1)->applyOnPositionable(obj);
}

Orientable *SerialLocalEffect::applyOnOrientable(Orientable *obj)
{
	if(m_effects->size()==0)
		return obj;
		
	if(m_currentEffect<m_effects->size())
	{
		return m_effects->at(m_currentEffect)->applyOnOrientable(obj);
	}
	//return img;
	return m_effects->at(m_effects->size()-1)->applyOnOrientable(obj);
}

Resizable *SerialLocalEffect::applyOnResizable(Resizable *obj)
{
	if(m_effects->size()==0)
		return obj;
		
	if(m_currentEffect<m_effects->size())
	{
		return m_effects->at(m_currentEffect)->applyOnResizable(obj);
	}
	//return img;
	return m_effects->at(m_effects->size()-1)->applyOnResizable(obj);
}


void SerialLocalEffect::addEffect(AbstractLocalEffect * fx)
{
	m_effects->push_back(fx);
}

SerialLocalEffect* SerialLocalEffect::clone() const
{
	SerialLocalEffect *sle = new SerialLocalEffect(m_nbLoops);
	vector<AbstractLocalEffect *>::iterator it;
	for (it=m_effects->begin(); it!=m_effects->end();it++)
    {
    	sle->addEffect((*it)->clone());
    }
	return sle;
}

int SerialLocalEffect::getPosX()
{
	if(m_effects->size()==0)
		return AbstractLocalEffect::getPosX();
		
	if(m_currentEffect<m_effects->size())
	{
		return m_effects->at(m_currentEffect)->getPosX();
	}
	//return img;
	return m_effects->at(m_effects->size()-1)->getPosX();
}

int SerialLocalEffect::getPosY()
{
	if(m_effects->size()==0)
		return AbstractLocalEffect::getPosY();
		
	int y=-1;	
	//PFL::logMessage("SerialLocalEffect::getPosY start");
	if(m_currentEffect<m_effects->size())
	{
		y = m_effects->at(m_currentEffect)->getPosY();
		//PFL::logMessage("SerialLocalEffect::getPosY end current effect : "+toString(y));
		return y;
	}
	
	y=m_effects->at(m_effects->size()-1)->getPosY();
	//PFL::logMessage("SerialLocalEffect::getPosY end current effect -1 : "+toString(y));
	return y;
}























