#include "code/framework/fx/impl/managers/LocalEffectManager.h"
#include <oslib/oslib.h>
#include <vector>
#include "code/framework/fx/abstract/AbstractLocalEffect.h"
using namespace std;

//ImgState *LocalEffectManager::m_state=NULL;

LocalEffectManager::LocalEffectManager()
{
	m_state=new ImgState();
	m_effects=new vector<AbstractLocalEffect *>();
}

LocalEffectManager::~LocalEffectManager()
{
	delete m_state;
	delete m_effects;
}

void LocalEffectManager::deleteAll()
{
	while(!m_effects->empty())
	{
		AbstractLocalEffect * back=m_effects->back();
		m_effects->pop_back();
		delete back;
	}
}



void LocalEffectManager::start()
{
	for(vector<AbstractLocalEffect *>::iterator it=m_effects->begin() ; it!=m_effects->end();it++)	
	{
		(*it)->start();	
	}
}
 
void LocalEffectManager::restart()
{
	for(vector<AbstractLocalEffect *>::iterator it=m_effects->begin() ; it!=m_effects->end();it++)	
	{
		(*it)->restart();	
	}
}

void LocalEffectManager::reset()
{
	for(vector<AbstractLocalEffect *>::iterator it=m_effects->begin() ; it!=m_effects->end();it++)	
	{
		(*it)->reset();	
	}
}

void LocalEffectManager::stop()
{
	for(vector<AbstractLocalEffect *>::iterator it=m_effects->begin() ; it!=m_effects->end();it++)	
	{
		(*it)->stop();	
	}
}

void LocalEffectManager::update()
{
	for(vector<AbstractLocalEffect *>::iterator it=m_effects->begin() ; it!=m_effects->end();it++)	
	{
		(*it)->update();	
	}
}

bool LocalEffectManager::finished()
{
	for(vector<AbstractLocalEffect *>::iterator it=m_effects->begin() ; it!=m_effects->end();it++)	
	{
		if(!(*it)->finished())
		{
			return false;	
		}
	}
	return true;
}

bool LocalEffectManager::isRunning()
{
	for(vector<AbstractLocalEffect *>::iterator it=m_effects->begin();it!=m_effects->end();it++)	
	{
		if((*it)->isRunning())
		{
			return true;	
		}
	}
	return false;
}


AbstractLocalEffect * LocalEffectManager::addEffect(AbstractLocalEffect * fx)
{
	m_effects->push_back(fx);
	return fx;
}



OSL_IMAGE * LocalEffectManager::applyOn(OSL_IMAGE *img)
{
	OSL_IMAGE *res=img;
	for(vector<AbstractLocalEffect *>::iterator it=m_effects->begin();it!=m_effects->end();it++)	
	{
		res=(*it)->applyOn(res);
		if(res==NULL)
			return NULL;
	}
	return res;
}

void LocalEffectManager::draw(OSL_IMAGE *img)
{
	//1) sauvegarder son etat car on va la triturer
	m_state->saveFrom(img);
	
	//2) appliquer tous les effets
	img=applyOn(img);
	
	//3) et si le resultat est != null, dessiner l'image
	if (img!=NULL)
	{
		oslDrawImage(img);
	}
	
	//4) et enfin restaurer l'image comme elle etait
	m_state->restoreTo(img);
}

void LocalEffectManager::drawXY(OSL_IMAGE *img, int x, int y)
{
	m_state->saveFrom(img);
	img=applyOn(img);
	if (img!=NULL)
		oslDrawImageXY(img, x, y);
	m_state->restoreTo(img);
}

/*void LocalEffectManager::freeRes()
{
	delete m_state;
}*/

AbstractLocalEffect *LocalEffectManager::getAt(unsigned int num)
{
	if(num>=m_effects->size())
		return NULL;
	return m_effects->at(num);	
}

int LocalEffectManager::getNbEffects()
{
	return m_effects->size();
}



