#include "code/framework/imgFilters/imgFilters.h"
#include "code/framework/utils.h"
#include <oslib/oslib.h>

using namespace utils;



void rgbToHsl(int rgb[], float hsl[])
{
	int r=0, g=1, b=2;
	int h=0, s=1, l=2;
	
	float lightness=0.0f, saturation=0.0f, hue=0.0f;
	
	unsigned char minval = oslMin(rgb[r], oslMin(rgb[g], rgb[b]));
	unsigned char maxval = oslMax(rgb[r], oslMax(rgb[g], rgb[b]));
	float mdiff  = float(maxval) - float(minval);
	float msum   = float(maxval) + float(minval);
	
	lightness = msum / 510.0f;
	
	if (maxval == minval) 
	{
		saturation = 0.0f;
		hue = 0.0f; 
	}   
	else 
	{ 
		float rnorm = (maxval - rgb[r]) / mdiff;      
		float gnorm = (maxval - rgb[g]) / mdiff;
		float bnorm = (maxval - rgb[b]) / mdiff;   
		
		saturation = (lightness <= 0.5f) ? (mdiff / msum) : (mdiff / (510.0f - msum));
		
		if(rgb[r] == maxval) 
			hue = 60.0f * (6.0f + bnorm - gnorm);
		else if(rgb[g] == maxval) 
			hue = 60.0f * (2.0f + rnorm - bnorm);
		else //if (rgb[b] == maxval) 
			hue = 60.0f * (4.0f + gnorm - rnorm);
			
		if (hue > 360.0f) 
			hue = hue - 360.0f;
	}
	
	hsl[h]=int(hue*255.0f/360.0f);
	hsl[s]=int(saturation*255.0f);
	hsl[l]=int(lightness*255.0f);
}

void hslToRgb(float hsl[], int rgb[])
{
	int r=0, g=1, b=2;
	int h=0, s=1, l=2;
	
	float hue=hsl[h]/255.0f*360.0f;
	float saturation=hsl[s]/255.0f;
	float lightness=hsl[l]/255.0f;
	
	if (saturation == 0.0) //gris
	{
		rgb[r] = rgb[g] = rgb[b] = int(hsl[l]);//lightness * 255.0f;
	}
	else
	{
		float rm1, rm2;
		
		if (lightness <= 0.5f) 
			rm2 = lightness + lightness * saturation;  
		else                     
			rm2 = lightness + saturation - lightness * saturation;
		rm1 = 2.0f * lightness - rm2;   
		rgb[r] = toRGB1(rm1, rm2, hue + 120.0f);   
		rgb[g] = toRGB1(rm1, rm2, hue);
		rgb[b] = toRGB1(rm1, rm2, hue - 120.0f);
	}
}

int toRGB1(float rm1, float rm2, float rh)
{
	if(rh > 360.0f) 
		rh -= 360.0f;
	else if (rh <   0.0f) 
		rh += 360.0f;
	
	if(rh <  60.0f) 
		rm1 = rm1 + (rm2 - rm1) * rh / 60.0f;   
	else if(rh < 180.0f) 
		rm1 = rm2;
	else if(rh < 240.0f) 
		rm1 = rm1 + (rm2 - rm1) * (240.0f - rh) / 60.0f;      
	
	return int(rm1 * 255.0f);
}


OSL_IMAGE *getHSLRecoloredBufferCopy(OSL_COLOR refcolor, int lum_min, int lum_max, int alpha)
{
	OSL_IMAGE *pixelcopy=oslCreateImage(480, 272, OSL_IN_RAM, OSL_PF_8888);
	if(pixelcopy==NULL)
	{
		//retry..
		pixelcopy=oslCreateImage(480, 272, OSL_IN_RAM, OSL_PF_8888);
		if(pixelcopy==NULL)
			fatalError("getHSLRecoloredBufferCopy : impossible de creer une image");
		else
			fatalError("getHSLRecoloredBufferCopy : l'image a eu besoin d'un 2e essai pour etre creee");
	}
	
	//on a besoin de:
	float hsl_ref[3];
	int rgb_ref[3];
	int a;  //stockage temp
	oslRgbaGet8888(refcolor, rgb_ref[0], rgb_ref[1], rgb_ref[2], a);
	rgbToHsl(rgb_ref, hsl_ref); //hsl_ref ok :)
	
	
		
	oslSyncDrawing();
	oslLockImage(OSL_SECONDARY_BUFFER);
	oslLockImage(pixelcopy);
	
	for(int i=0;i<480;i++)
	{
		for(int j=0;j<272;j++)
		{
			//int pixel=oslConvertColorEx(OSL_DEFAULT_BUFFER ->palette, OSL_PF_8888, OSL_DEFAULT_BUFFER ->pixelFormat, oslGetImagePixel(OSL_DEFAULT_BUFFER , i, j));
			int pixelsrc=oslGetImagePixel(OSL_SECONDARY_BUFFER, i, j);
			int pixel8888src=oslConvertColorEx(OSL_SECONDARY_BUFFER->palette, OSL_PF_8888, OSL_SECONDARY_BUFFER->pixelFormat, pixelsrc);
			int rgb_src[3];
			oslRgbaGet8888(pixel8888src, rgb_src[0], rgb_src[1], rgb_src[2], a);
			float hsl_src[3];
			rgbToHsl(rgb_src, hsl_src);
			
			float hsl_res[3];
			int rgb_res[3];
			
			//teinte et saturation on touche pas
			hsl_res[0]=hsl_ref[0];
			
			
			hsl_res[1]=hsl_ref[1];
			//test:
			//hsl_res[1]=hsl_src[1];
			//hsl_res[1]=(hsl_ref[1]+hsl_src[1])/2.0f;
			
			
			//calcul luminance resultat:
			float lightness_res=hsl_src[2];
			int lum_size=lum_max-lum_min;
			lightness_res=lightness_res*float(lum_size)/255.0f;
			lightness_res+=float(lum_min);
			hsl_res[2]=lightness_res;
			
			hslToRgb(hsl_res, rgb_res);			
			
			int pixel8888result=RGBA(rgb_res[0],rgb_res[1],rgb_res[2],alpha);
			
			oslSetImagePixel(pixelcopy, i, j, pixel8888result);
		}
	}
	oslUnlockImage(pixelcopy);
	oslUnlockImage(OSL_SECONDARY_BUFFER);
	
	oslSwizzleImage(pixelcopy);
	
	return pixelcopy;
}

OSL_IMAGE *getBlueRecoloredBufferCopy(float multblue, float multothers)
{
	OSL_IMAGE *pixelcopy=oslCreateImage(480, 272, OSL_IN_RAM, OSL_PF_8888);
	if(pixelcopy==NULL)
	{
		//retry..
		pixelcopy=oslCreateImage(480, 272, OSL_IN_RAM, OSL_PF_8888);
		if(pixelcopy==NULL)
			fatalError("getBlueRecoloredBufferCopy : impossible de creer une image");
		else
			fatalError("getBlueRecoloredBufferCopy : l'image a eu besoin d'un 2e essai pour etre creee");
	}
	oslSyncDrawing();
	oslLockImage(OSL_SECONDARY_BUFFER);
	oslLockImage(pixelcopy);
	
	/*float tp_r=oslConvertColorEx(OSL_DEFAULT_BUFFER->palette, OSL_PF_8888, OSL_DEFAULT_BUFFER->pixelFormat, t_r) / 255.0f;
	float tp_g=oslConvertColorEx(OSL_DEFAULT_BUFFER->palette, OSL_PF_8888, OSL_DEFAULT_BUFFER->pixelFormat, t_g) / 255.0f;
	float tp_b=oslConvertColorEx(OSL_DEFAULT_BUFFER->palette, OSL_PF_8888, OSL_DEFAULT_BUFFER->pixelFormat, t_b) / 255.0f;
	*/
	
	for(int i=0;i<480;i++)
	{
		for(int j=0;j<272;j++)
		{
			//int pixel=oslConvertColorEx(OSL_DEFAULT_BUFFER ->palette, OSL_PF_8888, OSL_DEFAULT_BUFFER ->pixelFormat, oslGetImagePixel(OSL_DEFAULT_BUFFER , i, j));
			int pixelsrc=oslGetImagePixel(OSL_SECONDARY_BUFFER, i, j);
			int pixel8888src=oslConvertColorEx(OSL_SECONDARY_BUFFER->palette, OSL_PF_8888, OSL_SECONDARY_BUFFER->pixelFormat, pixelsrc);
			int r, g, b, a;
			oslRgbaGet8888(pixel8888src, r, g, b, a);
			
			//max
			float max;
			max=r>g?r:g;
			max=max>b?max:b;
			max=max*multblue;
			
			float others=multothers*float(r+g+b-max)/2.0f;
			
			//int total=r+b+g;
			
			/*int r_r, r_g, r_b;
			r_r=total*tp_r;
			r_g=total*tp_g;
			r_b=total*tp_b;
			*/
			int pixel8888result=RGBA(int(others),int(others),int(max),255);
			
			oslSetImagePixel(pixelcopy, i, j, pixel8888result);
		}
	}
	oslUnlockImage(pixelcopy);
	oslUnlockImage(OSL_SECONDARY_BUFFER);
	
	oslSwizzleImage(pixelcopy);
	
	return pixelcopy;
}

OSL_IMAGE *getOrangeRecoloredBufferCopy(float mult_r, float mult_g, float mult_b, int alpha)
{
	OSL_IMAGE *pixelcopy=oslCreateImage(480, 272, OSL_IN_RAM, OSL_PF_8888);
	if(pixelcopy==NULL)
	{
		//retry..
		pixelcopy=oslCreateImage(480, 272, OSL_IN_RAM, OSL_PF_8888);
		if(pixelcopy==NULL)
			fatalError("getOrangeRecoloredBufferCopy : impossible de creer une image");
		else
			fatalError("getOrangeRecoloredBufferCopy : l'image a eu besoin d'un 2e essai pour etre creee");
	}
	oslSyncDrawing();
	oslLockImage(OSL_SECONDARY_BUFFER);
	oslLockImage(pixelcopy);
	
	for(int i=0;i<480;i++)
	{
		for(int j=0;j<272;j++)
		{
			int pixelsrc=oslGetImagePixel(OSL_SECONDARY_BUFFER, i, j);
			int pixel8888src=oslConvertColorEx(OSL_SECONDARY_BUFFER->palette, OSL_PF_8888, OSL_SECONDARY_BUFFER->pixelFormat, pixelsrc);
			int r, g, b, a;
			oslRgbaGet8888(pixel8888src, r, g, b, a);
			
			int tmp;
			for(int k=0;k<2;k++)
			{
				if(g>r)
				{
					tmp=r;
					r=g;
					g=tmp;
				}
				if(b>g)
				{
					tmp=b;
					b=g;
					g=tmp;
				}
			}
			
			int pixel8888result=RGBA(int(mult_r*r),int(mult_g*g),int(mult_b*b),alpha);
			
			oslSetImagePixel(pixelcopy, i, j, pixel8888result);
		}
	}
	oslUnlockImage(pixelcopy);
	oslUnlockImage(OSL_SECONDARY_BUFFER);
	
	oslSwizzleImage(pixelcopy);
	
	return pixelcopy;
}

OSL_IMAGE *getOrangeRecoloredBufferCopy2(float mult_r, float mult_g, float mult_b, int alpha)
{
	OSL_IMAGE *pixelcopy=oslCreateImage(480, 272, OSL_IN_RAM, OSL_PF_8888);
	if(pixelcopy==NULL)
	{
		//retry..
		pixelcopy=oslCreateImage(480, 272, OSL_IN_RAM, OSL_PF_8888);
		if(pixelcopy==NULL)
			fatalError("getOrangeRecoloredBufferCopy2 : impossible de creer une image");
		else
			fatalError("getOrangeRecoloredBufferCopy2 : l'image a eu besoin d'un 2e essai pour etre creee");
	}
	oslSyncDrawing();
	oslLockImage(OSL_SECONDARY_BUFFER);
	oslLockImage(pixelcopy);
	
	for(int i=0;i<480;i++)
	{
		for(int j=0;j<272;j++)
		{
			int pixelsrc=oslGetImagePixel(OSL_SECONDARY_BUFFER, i, j);
			int pixel8888src=oslConvertColorEx(OSL_SECONDARY_BUFFER->palette, OSL_PF_8888, OSL_SECONDARY_BUFFER->pixelFormat, pixelsrc);
			int r, g, b, a;
			oslRgbaGet8888(pixel8888src, r, g, b, a);
			
			int tmp;
			for(int k=0;k<2;k++)
			{
				if(g>r)
				{
					tmp=r;
					r=g;
					g=tmp;
				}
				if(b>g)
				{
					tmp=b;
					b=g;
					g=tmp;
				}
			}
			
			float r_proportion=0.66f;
			float g_proportion=1.0f-r_proportion;
			
			int sum_rg=r+g;
			
			float new_r=r_proportion*sum_rg;
			float new_g=g_proportion*sum_rg;
			
			if(new_r>255.0f)
			{
				new_g=new_g+(new_r-255.0f);
				new_r=255.0f;
			}
			
			int pixel8888result=RGBA(int(mult_r*new_r),int(mult_g*new_g),int(mult_b*b),alpha);
			
			oslSetImagePixel(pixelcopy, i, j, pixel8888result);
		}
	}
	oslUnlockImage(pixelcopy);
	oslUnlockImage(OSL_SECONDARY_BUFFER);
	
	oslSwizzleImage(pixelcopy);
	
	return pixelcopy;
}

//reorganisation des couleurs dans l'ordre r,g,b
//r=r, g=r/2, b=moy(g,b)
OSL_IMAGE *getOrangeRecoloredBufferCopy3(float mult_r, float mult_g, float mult_b, int alpha)
{
	OSL_IMAGE *pixelcopy=oslCreateImage(480, 272, OSL_IN_RAM, OSL_PF_8888);
	if(pixelcopy==NULL)
	{
		//retry..
		pixelcopy=oslCreateImage(480, 272, OSL_IN_RAM, OSL_PF_8888);
		if(pixelcopy==NULL)
			fatalError("getOrangeRecoloredBufferCopy3 : impossible de creer une image");
		else
			fatalError("getOrangeRecoloredBufferCopy3 : l'image a eu besoin d'un 2e essai pour etre creee");
	}
	oslSyncDrawing();
	oslLockImage(OSL_SECONDARY_BUFFER);
	oslLockImage(pixelcopy);
	
	for(int i=0;i<480;i++)
	{
		for(int j=0;j<272;j++)
		{
			int pixelsrc=oslGetImagePixel(OSL_SECONDARY_BUFFER, i, j);
			int pixel8888src=oslConvertColorEx(OSL_SECONDARY_BUFFER->palette, OSL_PF_8888, OSL_SECONDARY_BUFFER->pixelFormat, pixelsrc);
			int r, g, b, a;
			oslRgbaGet8888(pixel8888src, r, g, b, a);
			
			int tmp;
			for(int k=0;k<2;k++)
			{
				if(g>r)
				{
					tmp=r;
					r=g;
					g=tmp;
				}
				if(b>g)
				{
					tmp=b;
					b=g;
					g=tmp;
				}
			}
			
			float new_r=float(r);
			float new_g=float(r)/2.0f;
			float new_b=(float(g)+float(b))/2.0f;
			
			int pixel8888result=RGBA(int(mult_r*new_r),int(mult_g*new_g),int(mult_b*new_b),alpha);
			
			oslSetImagePixel(pixelcopy, i, j, pixel8888result);
		}
	}
	oslUnlockImage(pixelcopy);
	oslUnlockImage(OSL_SECONDARY_BUFFER);
	
	oslSwizzleImage(pixelcopy);
	
	return pixelcopy;
}


























