#include "code/framework/particles/complete/Particle.h"

Particle::Particle(int x, int y, Anim *anim, int lifespan, AbstractLocalEffect *lfx1, AbstractLocalEffect *lfx2, AbstractLocalEffect *lfx3, AbstractLocalEffect *lfx4, AbstractLocalEffect *lfx5, AbstractGlobalEffect *gfx1, AbstractGlobalEffect *gfx2)
//: Trackable("Particle")
{
	m_anim=anim;
	m_x=x;
	m_y=y;
	
	//jusqu'a 5 effets locaux appliqués
	m_lfx1=lfx1;
	m_lfx2=lfx2;
	m_lfx3=lfx3;
	m_lfx4=lfx4;
	m_lfx5=lfx5;
	
	//jusqu'a 2 effets globaux appliqués
	m_gfx1=gfx1;		
	m_gfx2=gfx2;
	
	m_lifespan=lifespan;
	m_lifeSpent=0;
}

Particle::Particle(int x, int y, Anim *anim, int lifespan, AbstractLocalEffect *lfx1, AbstractGlobalEffect *gfx1, AbstractGlobalEffect *gfx2)
//: Trackable("Particle")
{
	//Particle(x, y, anim, lifespan, lfx1, NULL, NULL, NULL, NULL, gfx1, gfx2);
	m_anim=anim;
	m_x=x;
	m_y=y;
	
	//jusqu'a 5 effets locaux appliqués
	m_lfx1=lfx1;
	m_lfx2=NULL;
	m_lfx3=NULL;
	m_lfx4=NULL;
	m_lfx5=NULL;
	
	//jusqu'a 2 effets globaux appliqués
	m_gfx1=gfx1;		
	m_gfx2=gfx2;
	
	m_lifespan=lifespan;
	m_lifeSpent=0;
}

Particle::Particle(int x, int y, Anim *anim, int lifespan, AbstractGlobalEffect *gfx1, AbstractGlobalEffect *gfx2, AbstractLocalEffect *lfx1, AbstractLocalEffect *lfx2, AbstractLocalEffect *lfx3, AbstractLocalEffect *lfx4, AbstractLocalEffect *lfx5)
//: Trackable("Particle")
{
	m_anim=anim;
	m_x=x;
	m_y=y;
	
	//jusqu'a 5 effets locaux appliqués
	m_lfx1=lfx1;
	m_lfx2=lfx2;
	m_lfx3=lfx3;
	m_lfx4=lfx4;
	m_lfx5=lfx5;
	
	//jusqu'a 2 effets globaux appliqués
	m_gfx1=gfx1;		
	m_gfx2=gfx2;
	
	m_lifespan=lifespan;
	m_lifeSpent=0;
}
		

Particle::~Particle()
{
	//old: on detruit pas l'image, elle est partagée parmis les particules
	//new: on detruit l'anim, ce sont des copies qui sont données aux particules
	delete m_anim;
	if(m_gfx1!=NULL)
		delete m_gfx1;
	if(m_gfx2!=NULL)
		delete m_gfx2;
	if(m_lfx1!=NULL)
		delete m_lfx1;
	if(m_lfx2!=NULL)
		delete m_lfx2;
	if(m_lfx3!=NULL)
		delete m_lfx3;
	if(m_lfx4!=NULL)
		delete m_lfx4;
	if(m_lfx5!=NULL)
		delete m_lfx5;
}

void Particle::drawXY(int x, int y, AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3, AbstractLocalEffect *fx4, AbstractLocalEffect *fx5)
{
	//on dessine que si l'un des effets est en marche
	if(isRunning())
	{

		fx::drawXY(m_anim->getImage(), x, y, m_lfx1, m_lfx2, m_lfx3, fx1, fx2);
;
	}
}

//inline void Particle::draw() dans .h



//update appelé uniquement si le generateur n'est pas en pause
void Particle::update()
{
	//if(isRunning())
	//{
		m_anim->update();
		if(m_gfx1!=NULL)
			m_gfx1->update();
		if(m_gfx2!=NULL)
			m_gfx2->update();
		if(m_lfx1!=NULL)
			m_lfx1->update();
		if(m_lfx2!=NULL)
			m_lfx2->update();
		if(m_lfx3!=NULL)
			m_lfx3->update();
		if(m_lfx4!=NULL)
			m_lfx4->update();
		if(m_lfx5!=NULL)
			m_lfx5->update();
	
	
		m_lifeSpent++;
	//}
}

void Particle::start()
{
	m_anim->start();
	if(m_gfx1!=NULL)
		m_gfx1->start();
	if(m_gfx2!=NULL)
		m_gfx2->start();
	if(m_lfx1!=NULL)
		m_lfx1->start();
	if(m_lfx2!=NULL)
		m_lfx2->start();
	if(m_lfx3!=NULL)
		m_lfx3->start();
	if(m_lfx4!=NULL)
		m_lfx4->start();
	if(m_lfx5!=NULL)
		m_lfx5->start();	
}

bool Particle::isRunning()
{
	if(m_lfx1!=NULL && m_lfx1->isRunning())
		return true;
	if(m_lfx2!=NULL && m_lfx2->isRunning())
		return true;
	if(m_lfx3!=NULL && m_lfx3->isRunning())
		return true;
	if(m_lfx4!=NULL && m_lfx4->isRunning())
		return true;
	if(m_lfx5!=NULL && m_lfx5->isRunning())
		return true;
	if(m_anim->isRunning())
		return true;
	if(m_gfx1!=NULL && m_gfx1->isRunning())
		return true;
	if(m_gfx2!=NULL && m_gfx2->isRunning())
		return true;
		
	return false;
}

void Particle::reset()
{
	m_anim->reset();
	if(m_gfx1!=NULL)
		m_gfx1->reset();
	if(m_gfx2!=NULL)
		m_gfx2->reset();
	if(m_lfx1!=NULL)
		m_lfx1->reset();
	if(m_lfx2!=NULL)
		m_lfx2->reset();
	if(m_lfx3!=NULL)
		m_lfx3->reset();
	if(m_lfx4!=NULL)
		m_lfx4->reset();
	if(m_lfx5!=NULL)
		m_lfx5->reset();
}

void Particle::restart()
{
	m_anim->restart();
	if(m_gfx1!=NULL)
		m_gfx1->restart();
	if(m_gfx2!=NULL)
		m_gfx2->restart();
	if(m_lfx1!=NULL)
		m_lfx1->restart();
	if(m_lfx2!=NULL)
		m_lfx2->restart();
	if(m_lfx3!=NULL)
		m_lfx3->restart();
	if(m_lfx4!=NULL)
		m_lfx4->restart();
	if(m_lfx5!=NULL)
		m_lfx5->restart();
}

void Particle::stop()
{
	m_anim->stop();
	if(m_gfx1!=NULL)
		m_gfx1->stop();
	if(m_gfx2!=NULL)
		m_gfx2->stop();
	if(m_lfx1!=NULL)
		m_lfx1->stop();
	if(m_lfx2!=NULL)
		m_lfx2->stop();
	if(m_lfx3!=NULL)
		m_lfx3->stop();
	if(m_lfx4!=NULL)
		m_lfx4->stop();
	if(m_lfx5!=NULL)
		m_lfx5->stop();
}

bool Particle::isDead()
{
	if(m_lifespan==-1)
		return false;
	if(m_lifeSpent<=m_lifespan)
		return false;
	return true;
}

int Particle::getX()
{
	return m_x;
}

int Particle::getY()
{
	return m_y;
}

void Particle::setX(int x)
{
	m_x=x;	
}

void Particle::setY(int y)
{
	m_y=y;
}

int Particle::getXwithFX()
{
	int x=m_x;
	
	if(m_lfx1!=NULL)
		x += m_lfx1->getPosX();
	if(m_lfx2!=NULL)
		x += m_lfx2->getPosX();
	if(m_lfx3!=NULL)
		x += m_lfx3->getPosX();
	if(m_lfx4!=NULL)
		x += m_lfx4->getPosX();
	if(m_lfx5!=NULL)
		x += m_lfx5->getPosX();
	
	return x;
}

int Particle::getYwithFX()
{
	int y=m_y;
	
	if(m_lfx1!=NULL)
		y += m_lfx1->getPosY();
	if(m_lfx2!=NULL)
		y += m_lfx2->getPosY();
	if(m_lfx3!=NULL)
		y += m_lfx3->getPosY();
	if(m_lfx4!=NULL)
		y += m_lfx4->getPosY();
	if(m_lfx5!=NULL)
		y += m_lfx5->getPosY();
	
	return y;
}

State *Particle::savestate(State * state)
{
	state->m_x=m_x;
	state->m_y=m_y;
	
	return state;
}

void Particle::restorestate(State * state)
{
	m_x=state->m_x;
	m_y=state->m_y;
}

AbstractGlobalEffect *Particle::getGlobalEffect1()
{
	return m_gfx1;
}

AbstractGlobalEffect *Particle::getGlobalEffect2()
{
	return m_gfx2;
}

int Particle::getLifeSpent()
{
	return m_lifeSpent;
}








