#include "code/framework/particles/complete/fx/ParticlesProgressiveTintEffect.h"
#include <oslib/oslib.h>
#include "code/framework/utils.h"
using namespace utils;


ParticlesProgressiveTintEffect::ParticlesProgressiveTintEffect(int timeAB, int timeBA, u32 colorFrom, u32 colorTo, int nbLoops)
{
	m_running=false;
	m_curTick=0;
	m_nbLoops=nbLoops;
	m_currentLoop=0;
	m_aToB=true;
	m_timeAB=timeAB;
	m_timeBA=timeBA;
	m_colorFrom=new u8[4];
	m_colorTo=new u8[4];
	oslRgbaGet8888(colorFrom, m_colorFrom[0], m_colorFrom[1], m_colorFrom[2], m_colorFrom[3]);
	oslRgbaGet8888(colorTo, m_colorTo[0], m_colorTo[1], m_colorTo[2], m_colorTo[3]);
}
	
void ParticlesProgressiveTintEffect::start()
{
	m_running=true;
}

void ParticlesProgressiveTintEffect::restart()
{
	m_curTick=0;
	m_running=true;
	m_currentLoop=0;
	m_aToB=true;
}

void ParticlesProgressiveTintEffect::reset()
{
	m_curTick=0;
	m_currentLoop=0;
	m_aToB=true;
}

void ParticlesProgressiveTintEffect::stop()
{
	m_running=false;
}

void ParticlesProgressiveTintEffect::update()
{
	if(m_running)
	{
		m_curTick++;
		if	(
				(m_aToB && m_curTick==m_timeAB) ||
				(!m_aToB && m_curTick==m_timeBA)
			)
		{
			m_curTick=0;
			m_aToB=!m_aToB;
			m_currentLoop++;
			if(finished())
			{
				m_running=false;
				return;	
			}	
		}
	}
}

bool ParticlesProgressiveTintEffect::finished()
{
	if(m_nbLoops==-1)//infini
		return false;
		
	if(m_currentLoop==m_nbLoops)
		return true;
	return false;
}

bool ParticlesProgressiveTintEffect::isRunning()
{
	return m_running;
}

void ParticlesProgressiveTintEffect::activate()
{
	int setres[4];
	for(int i=0;i<4;i++)
	{
		if(m_colorFrom[i]!=m_colorTo[i])
		{
			int compFrom=m_aToB?m_colorFrom[i]:m_colorTo[i];
			int compTo=m_aToB?m_colorTo[i]:m_colorFrom[i];
		
			if(m_curTick==0)
			{
				setres[i]=compFrom;
			}	
			else
			{
				int time=m_aToB?m_timeAB:m_timeBA;
				
//				if(time==0)
//					oslFatalError("ProgressiveTintEffect::activate() time = 0");
				
				//double dist=double(compTo-compFrom);
				int curDist=int((float(compTo-compFrom)*m_curTick)/float(time));
				setres[i]=int(curDist+compFrom);
			}
		}	
		else
		{
			setres[i]=m_colorFrom[i];
		}
	}
	
	//oslSetAlpha(OSL_FX_TINT, RGBA(setres[0], setres[1], setres[2], setres[3]));


	OSL_ALPHA_PARAMS *alpha;
	alpha = new OSL_ALPHA_PARAMS;
	oslGetAlphaEx(alpha);
	
	u8 * curColor=new u8[4];
	u8 * newColor=new u8[4];
	oslRgbaGet8888(alpha->coeff1, curColor[0], curColor[1], curColor[2], curColor[3]);
	
	alpha->effect=OSL_FX_ALPHA | OSL_FX_COLOR;
	for(int i=0;i<4;i++)
	{	
		if(curColor[i]!=setres[i])
			newColor[i]=u8(float(curColor[i]*setres[i])/255.0f);
		else
			newColor[i]=curColor[i];
		//newColor[i]=u8(0.5f*curColor[i]+0.5f*setres[i]);
	}
	//newColor[3]=u8((curColor[3]*setres[3])/255.0f);
	alpha->coeff1=RGBA(newColor[0], newColor[1], newColor[2], newColor[3]);
	
	oslSetAlphaEx(alpha);
	
	delete alpha;
	delete[] curColor;
	delete[] newColor;
	
}


ParticlesProgressiveTintEffect::~ParticlesProgressiveTintEffect()
{
	delete[] m_colorFrom;
	delete[] m_colorTo;
}

ParticlesProgressiveTintEffect* ParticlesProgressiveTintEffect::clone() const
{
	return new ParticlesProgressiveTintEffect(m_timeAB, m_timeBA, RGBA(m_colorFrom[0], m_colorFrom[1], m_colorFrom[2], m_colorFrom[3]), RGBA(m_colorTo[0], m_colorTo[1], m_colorTo[2], m_colorTo[3]), m_nbLoops);
}
		
void ParticlesProgressiveTintEffect::setTimeForward(int time)
{
	m_timeAB=time;
}

int ParticlesProgressiveTintEffect::getTimeForward()
{ 
	return m_timeAB;
} 

void ParticlesProgressiveTintEffect::setTimeBackward(int time)
{
	m_timeBA=time;
}

int ParticlesProgressiveTintEffect::getTimeBackward()
{
	return m_timeBA;
}	
		
		
		
		
		
		






