#include "code/framework/resRepo/FolderRepo.h"
#include <stdio.h>
#include <oslib/oslib.h>
#include <iostream>
#include <sstream>
#include <fstream>
#include <string>
#include "code/framework/utils.h"
#include <list>
#include "code/framework/check/loaders.h"

using namespace utils;

FolderRepo::FolderRepo(string folder)
{
	if(folder.substr( folder.size()-1, 1)!="/")
	{
		folder=folder+"/";
	}
	m_folderName=uniformizePath(folder);
	m_files=NULL; //lazy init
	m_nbFiles=-1; //lazy init
	
	if(!directoryExists(m_folderName))
	{
		/*
		oslFatalError(string("FolderRepo: folder "+m_folderName+" does not exist").c_str());
		*/

	}
}

FolderRepo::~FolderRepo()
{
	if(m_files!=NULL)
		delete m_files;
}


long FolderRepo::getNbFiles()
{
	if(m_nbFiles==-1)
	{
		init();
	}
	return m_nbFiles;
}

vector<string> FolderRepo::getFilesNames()
{
	if(m_nbFiles==-1)
	{
		init();
	}
	
	vector<string> res;
	
	map<string, ResFileDetails>::iterator it;
	for (it=m_files->begin(); it!=m_files->end(); it++)
	{
		res.push_back((*it).first);
	}
	return res;
}

vector<ResFileDetails> FolderRepo::getFilesDetails()
{
	if(m_nbFiles==-1)
	{
		init();
	}
	
	vector<ResFileDetails> res;
	
	map<string, ResFileDetails>::iterator it;
	for (it=m_files->begin(); it!=m_files->end(); it++)
	{
		res.push_back((*it).second);
	}
	return res;
}

ResFileDetails FolderRepo::getFileDetails(string filename)
{

	ResFileDetails rfd;
	rfd.m_name=uniformizePath(filename);
	filename=uniformizePath(m_folderName+filename);
	rfd.m_offset=0;
	
	ifstream is;

  	is.open (filename.c_str(), ios::binary );
  	if(is.fail())
  	{
  		rfd.error=true;
  		return rfd;
  	}
	// get length of file:
	is.seekg (0, ios::end);
	rfd.m_size = long(is.tellg());
	
	is.close();
	
	return rfd;
}

//ouvre le fichier et place le pointeur au bon endroit
FILE *FolderRepo::open(string filename)
{
	filename=uniformizePath(m_folderName+filename);
	
	FILE *fd=fopen (filename.c_str(), "rb");
	if(fd==NULL)
		return NULL;
	
	return fd;
}

string FolderRepo::getName()
{
	return m_folderName;
}


string FolderRepo::readTextFileString(string filename)
{
	ResFileDetails rfd=getFileDetails(filename);
	
	filename=uniformizePath(m_folderName+filename);
	
	if(rfd.error)
		return "";
	
	
	FILE *fd=fopen (filename.c_str(), "r");
	if(fd==NULL)
		return "";
		
	char *data=(char*)malloc(rfd.m_size+1);
	fread(data,1,rfd.m_size,fd);
	data[rfd.m_size]='\0';
	
	fclose(fd);
	
	string res(data);
	
	free(data);
	
	return res;
}

istream *FolderRepo::readTextFileStream(string filename)
{
	istringstream *iss=new istringstream(readTextFileString(filename), istringstream::in);
	return iss;
}

OSL_IMAGE *FolderRepo::readPngFile(string filename, int location, int pixelFormat)
{
	filename=uniformizePath(m_folderName+filename);
	OSL_IMAGE *img=fwkLoadImageFilePNG(filename, location, pixelFormat);
	return img;
}

OSL_SOUND *FolderRepo::readBgmFile(string filename)
{
	filename=uniformizePath(m_folderName+filename);
	OSL_SOUND *res=fwkLoadSoundFileBGM(filename, OSL_FMT_NONE);
	return res;
}


bool FolderRepo::extractFileTo(string archivefile, string memorystickfile) //renvoie true si succès
{

	//archivefile=uniformizePath(m_folderName+archivefile);
	
	ResFileDetails resfd=getFileDetails(archivefile);
	if(resfd.error)
	{
		debug("resfd error");
		return false;
	}
	
	archivefile=uniformizePath(m_folderName+archivefile);
	
	FILE *fd=fopen (archivefile.c_str(), "rb");
	if(fd==NULL)
	{
		debug("fd==NULL!");
		return false;
	}
	//on va lire/ecrire des morceaux de fichiers par blocs de 16
	
	//création du répertoire cible s'il n'existe pas
	string dir=filepath_getpath(memorystickfile);
	if(!createDirectory(dir))
	{
		debug("impossible to create directory \""+dir+"\"");
		fclose(fd);
		return false;
	}
	
	//fichier ou on va ecrire
	ofstream outfile(memorystickfile.c_str(), ofstream::out|ofstream::binary|ofstream::trunc);
	
	long int done=0; //quantité déja écrite
	long int total=resfd.m_size; //quantité à écrire
	
	char * buffer = new char [16]; //le buffer temporaire
	
	while(done<total)
	{
		//lecture
		//taille à lire (16 a chaque fois, sauf le dernier morceau s'il est + petit);
		int sizetoread=16;
		if(total-done < 16)
			sizetoread = total - done;
			
		//lecture
		fread(buffer,sizetoread,1,fd);
		
		//ecriture
		outfile.write(buffer, sizetoread);
		if(outfile.bad())
		{
			debug("write failure into file \""+memorystickfile+"\"");
			fclose(fd);
			delete[] buffer;
			outfile.close();
			return false;
		}
		
		done+=sizetoread;	
	}	
	
	fclose(fd);
	delete[] buffer;
	outfile.close();
	

	return true;
}

//TODO: untested!!
bool FolderRepo::extractAllToFolder(string dir)
{

	if(m_nbFiles==-1)
	{
		init();
	}
	
	if(dir.at(dir.length()-1)!='/') //faut mettre le dernier '/'
	{
		dir=dir+"/";
	}
	
	map<string, ResFileDetails>::iterator it;
	for (it=m_files->begin(); it!=m_files->end(); it++)
	{
		ResFileDetails resfd=(*it).second;
		bool ok=extractFileTo(resfd.m_name, dir+resfd.m_name);
		if(!ok)
			return false;
	}
	
	return true;
}

string FolderRepo::uniformizePath(string path)
{
	path=replaceAll(path, "//", "/");
	path=replaceAll(path, "./", "");
	return path;
}

void FolderRepo::init() //l'initialisation est lazy, forcer avec cette fonction
{
	m_nbFiles=0;
	m_files=new map<string, ResFileDetails>();
	recurseIntoFolders("");
}


void FolderRepo::recurseIntoFolders(string startdir)
{
	list<string> *files=directoryListFiles(m_folderName+startdir);
	list<string> *dirs=directoryListFolders(m_folderName+startdir);
	
	m_nbFiles+=files->size();
	
	//on va chercher les details
	list<string>::iterator itf;
  	for(itf=files->begin() ; itf != files->end(); itf++ )
    {
    	string filename=uniformizePath(startdir+(*itf));
    	ResFileDetails rfd=getFileDetails(filename);
    	m_files->insert(pair<string,ResFileDetails>(rfd.m_name,rfd));
    }
    
    //on récursivite :)
    list<string>::iterator itd;
  	for(itd=dirs->begin(); itd != dirs->end(); itd++ )
    {
    	recurseIntoFolders(startdir+(*itd)+"/");
    }
	
	delete files;
	delete dirs;
}



















