#ifndef UTILS_H_
#define UTILS_H_

#include <sstream>
#include <string>
#include <iostream>
#include <oslib/oslib.h>
#include <vector>
#include <stdlib.h>
#include <time.h>
#include <list>
#include <math.h>

using namespace std;

namespace utils
{
	void enableDebug(bool enable=true);
	
	
	template<class T> inline string toString(T number)
	{
		ostringstream ost;
		ost << number;
		return ost.str();
	}
	
	int toInt(string s);
	float toFloat(string s);
	
	string replaceAll(string in, string find, string replacement);
	string deleteAll(string in, string find);
	void debug(string message);
	vector<string> split(string str, string delim);
	bool endsWith(string str, string with);
	
	void oslImageCenterScreen(OSL_IMAGE *); //dans l'ecran, et utilise son centre pour les placements
	void oslImageSetPosition(OSL_IMAGE *img, int x, int y);
	void oslImageSetCenter(OSL_IMAGE *img, int x, int y);
	
	//ces 2 fonctions random vont toujours générer la même suite (pas de srand)
	//float vfpu_rand(float min, float max);
	//int fastrandom(int min, int max);
	void initrandom();
	//va generer des nombres differents suivant ce qu'on passe a srand(), a appeler avant les appels a random
	int random(int min, int max);
	float randomf(float min, float max);
	
	//TODO: ecrire des fonctions de lecture/ecriture dans des fichiers .properties
	
	//tests
	string testReplace();
	string testReadLine();
	
	void screenshot();
	
	long int wavDuration(string wavfile); //en ms
	long int bgmDuration(string bgmfile); //en ms
	
	void drawImageOnImage(OSL_IMAGE *src, int srcX, int srcY, int width, int height, OSL_IMAGE *dest, int destX, int destY);
	
	//filesystem
	list<string> *directoryListFiles(string folder);
	list<string> *directoryListFolders(string folder);
	bool fileExists(string file);
	bool directoryExists(string dir);
	string getCurrentDirectory();
	bool createDirectory(string dir); //créé un répertoire sur une profondeur quelconque
	bool deleteDirectory(string dir); //efface un répertoire et tous ses fichiers/dossiers
	
	
	//ex: avec "dans/mon/dossier/unfichier.ext" :
	string filepath_getpath(string filepath); //renvoie dans/mon/dossier/"
	string filepath_getfile(string filepath); //renvoie "unfichier.ext"
	
	int oslAnalogGetAngle(int x, int y);
	
	OSL_IMAGE *getBufferCopy();
	
	//string trim(string str);
	
	int round(float x);
	
	void fatalError(string message);
	void message(string mess);
	
	string getTimestamp();
	
	void oslSetSoundVolume(OSL_SOUND *sound, int volume);
}



/************************
 *  fonctions inline
 ***********************/


//fonctions inline toujours dans le header!
inline int utils::toInt(string s)
{
	istringstream istr(s);
    int i;
    istr >> i;
    return i;
}

inline float utils::toFloat(string s)
{
	istringstream istr(s);
    float f;
    istr >> f;
    return f;
}

inline void utils::oslImageCenterScreen(OSL_IMAGE * img)
{
	oslSetImageRotCenter(img);
	//img->centerX=img->sizeX >> 1; // /2
	//img->centerY=img->sizeY >> 1; // /2
	img->x=240; //480/2
	img->y=136; //272/2
}

inline void utils::oslImageSetPosition(OSL_IMAGE *img, int x, int y)
{
	img->x=x;
	img->y=y;
}

inline void utils::oslImageSetCenter(OSL_IMAGE *img, int x, int y)
{
	img->centerX=x;
	img->centerY=y;
}

//ces 2 fonctions random vont toujours générer la même suite (pas de srand)
/*inline float utils::vfpu_rand(float min, float max) 
{
    float result;
    __asm__ volatile (
        "mtv      %1, S000\n"
        "mtv      %2, S001\n"
        "vsub.s   S001, S001, S000\n"
        "vone.s   S002\n"
        "vrndf1.s S003\n"
        "vsub.s   S003, S003, S002\n"
        "vmul.s   S001, S003, S001\n"
        "vadd.s   S000, S000, S001\n"
        "mfv      %0, S000\n"
        : "=r"(result) : "r"(min), "r"(max));
    return result;
}*/
/*inline int utils::fastrandom(int min, int max)
{
	return int(vfpu_rand(float(min), float(max+1)));
}*/

//enfin trouvé srand pour vfpu

/*inline void utils::vfpu_srand(unsigned int x) 
{
	__asm__ volatile ( "mtv %0, S000\n vrnds.s S000" : "=r"(x));
}*/

//va generer des nombres differents suivant ce qu'on passe a srand(), a appeler avant les appels a random
inline int utils::random(int min, int max)
{
	return int(randomf(float(min), float(max+1)));
}

inline float utils::randomf(float min, float max)
{
	if(min==max)
		return min;
	return vfpu_randf(min, max);
//	float result;
//    __asm__ volatile (
//		"mtv      %1, S000\n"
//        "mtv      %2, S001\n"
//        "vsub.s   S001, S001, S000\n"
//        "vrndf1.s S002\n"
//        "vone.s	  S003\n"
//        "vsub.s   S002, S002, S003\n"
//        "vmul.s   S001, S002, S001\n"
//        "vadd.s   S000, S000, S001\n"
//        "mfv      %0, S000\n"
//        : "=r"(result) : "r"(min), "r"(max));
//    return result;
}

inline void utils::initrandom()
{
	vfpu_srand(time(NULL));
}

inline void utils::screenshot()
{
	time_t seconds=time(NULL);
	
	ostringstream ost;
	
	ost << "screens/";	
	ost << seconds;
	ost << "-";
	
	clock_t ticks = clock ();
	
	ost << ticks;
	ost << ".png";
	
	oslWriteImageFilePNG(OSL_SECONDARY_BUFFER, ost.str().c_str(), 0);
}

inline int utils::round(float x)
{
	return int(x+0.5f);
}

inline string utils::getTimestamp()
{
	clock_t ticks = clock ();
	ostringstream ost;
	ost << ticks;
	return ost.str();
}

inline void utils::oslSetSoundVolume(OSL_SOUND *sound, int volume)
{
	sound->volumeLeft=volume;
	sound->volumeRight=volume;
}

#endif /*UTILS_H_*/














