#include "code/game/EventsScenario.h"
#include <list>
#include <string>
#include <map>
#include <fstream>
#include "code/framework/utils.h"
#include "code/framework/time/Timer.h"
using namespace std;
using namespace utils;

#define EVENT_TYPE_INSTANT 0
#define EVENT_TYPE_REPEAT 1

EventsScenario::EventsScenario(string configFile)
{
	m_running=false;
	m_currentEvents = new list<int>();
	m_events = new multimap<long int, int>();
	m_timer=new Timer();
	m_lastUpdate=0;
	
	
	ifstream fichier_config(configFile.c_str());
	
	if(fichier_config.fail())
		fatalError("cannot open file "+configFile);
	//TODO: ceci est un test, a enlever!
	/*if(fichier_config.fail())
	{
		fichier_config.clear();
		//oslFatalError(("EventsScenario : cannot open scenario: "+configFile).c_str());
		enableDebug();
		debug(("EventsScenario : cannot open scenario: "+configFile).c_str());
		
		bool ok=false;
		int i=0;
		while(!ok)
		{
			fichier_config.clear();
			fichier_config.open(configFile.c_str());
			if(!fichier_config.fail())
			{
				ok=true;
				debug("finalement reussi!");
			}
			i++;
			if(i==1000)
			{
				debug("peine perdue");
				oslQuit();
			}
		}
		
		enableDebug(false);
		
	}*/
	string line;
	while(getline(fichier_config,line))
	{
		if(line.find("[") != 0 && line.find("#") != 0) //ne pas prendre les commentaires en compte
		{
			if(line.size()>2) //ne pas prendre en compte les lignes vides, au minimum "a=b"
			{
				//debug("line = "+line);
				
				int indice=line.find("=");
				long int key=toInt(line.substr(0, indice)); //longueur = indice
				string value=line.substr(indice+1, line.size()-1-indice);
				
				value=deleteAll(value, "\r");
				
				//debug("key = "+toString(key)+" - value = "+value);
				
				//recherche du type d'évènement décrit
				int type; //nous dira si la valeur est de type "instant" ou "repeat"
				//recherche ':' dans value
				int colon=value.find(":");
				if(colon!=-1) //trouvé
				{
					string strtype=value.substr(0,colon);
					value=value.substr(colon+1,value.size()-1-colon);
					if(strtype.compare("instant") == 0) //==
						type=EVENT_TYPE_INSTANT;
					else //if(strtype.compare("repeat") == 0)
						type=EVENT_TYPE_REPEAT;
				}
				else //non trouvé
				{
					type=EVENT_TYPE_INSTANT;
				}
				
				
				//on extrait les options
				string option="";
				int arobase=value.find("@");
				if(arobase!=-1) //trouvé, il y a des options
				{
					option=value.substr(arobase+1,value.size()-1-arobase);
					value=value.substr(0,arobase);
				}
				
				//séparation des valeurs
				vector<string> values=split(value, "|");
				
				//séparation des options
				vector<string> options=split(option, "|");
				
				//insertion
				if(type==EVENT_TYPE_INSTANT)
				{
					//debug("type is instant");
					vector<int>::iterator it;
					for(vector<string>::iterator it=values.begin(); it<values.end(); it++)
					{
						//debug("inserting ("+toString(key)+", "+*it+")");
						m_events->insert(pair<long int, int>(key, toInt(*it)));
					}
				}
				else //if(type==EVENT_TYPE_REPEAT)
				{
					//debug("type is repeat");
					if(options.size()!=2)
					{
						fichier_config.close();
						fatalError("EventsScenario: a repeat event must have 2 options, the number of occurences and the interval");
					}
					int times=toInt(options.at(0));
					int interval=toInt(options.at(1));
					
					for(vector<string>::iterator it=values.begin(); it<values.end(); it++)
					{
						for(int i=0;i<times;i++)
						{
							//debug("inserting ("+toString(key+(i*interval))+", "+*it+")");
							m_events->insert(pair<long int, int>(key+(i*interval), toInt(*it)));
						}
					}
				}
			}
		}
	}
	fichier_config.close();
	//enableDebug(false);
}

EventsScenario::~EventsScenario()
{
	delete m_currentEvents;
	delete m_events;
	delete m_timer;
}

list<int> *EventsScenario::getCurrentEvents()
{
	return m_currentEvents;
}

bool EventsScenario::hasEvent(int eventNum)
{
	for(list<int>::iterator it=m_currentEvents->begin(); it!=m_currentEvents->end(); it++)
    {
    	int i=*it;
    	if(i==eventNum)
    		return true;
    }
    return false;
}

void EventsScenario::update()
{
	if(m_running)
	{
		//m_curTime++; //ne sert plus
		long int newUpdate=long(m_timer->getTime()); //now
		
		//on vide les evenements courants (qui ne sont plus a jour)
		m_currentEvents->clear();
		
		//et on met les nouveaux
		for(long int i=m_lastUpdate+1;i<=newUpdate;i++)
		{
			pair<multimap<long int,int>::iterator,multimap<long int,int>::iterator> ret;
			ret=m_events->equal_range(/*m_curTime*/i);
			for(multimap<long int,int>::iterator it=ret.first; it!=ret.second; ++it)
      		{
      			m_currentEvents->push_back((*it).second);
      		}
		}
		m_lastUpdate=newUpdate;
	}
}

void EventsScenario::start()
{
	m_running=true;
	m_timer->start();
}

void EventsScenario::pause()
{
	m_running=false;
	m_timer->stop();
}

void EventsScenario::reset()
{
	m_currentEvents->clear();
	m_timer->reset();
	m_lastUpdate=0;
}

void EventsScenario::restart()
{
	reset();
	start();
	//m_timer->start(); //certainement en trop
}

bool EventsScenario::isRunning()
{
	return m_running;	
}

double EventsScenario::getTime()
{
	return m_timer->getTime();	
}


