#ifndef PAIR_H_
#define PAIR_H_

#include "code/game/pieces/Abstract/AbstractPiece.h"
#include "code/framework/Matrix.h"
#include "code/game/grid/MatrixContentObject.h"
#include "code/framework/fx/EffectsLib.h"
#include "code/game/grid/GameGrid.h"
#include "code/framework/Pos.h"
//#include "code/framework/check/Trackable.h"

//orientations
#define ORIENTATION_NORTH 0
#define ORIENTATION_SOUTH 1
#define ORIENTATION_EAST 2
#define ORIENTATION_WEST 3

#define DIRECTION_UP 0
#define DIRECTION_DOWN 1
#define DIRECTION_RIGHT 2
#define DIRECTION_LEFT 3

#define APPLY_FX_ON_NONE 0
#define APPLY_FX_ON_1 1
#define APPLY_FX_ON_2 2
#define APPLY_FX_ON_BOTH 3
#define APPLY_FX_ON_1_OR_2 4
#define APPLY_FX_ON_1_2_OR_BOTH 5


//Assemblage de 2 pièces
class Pair //: public Trackable
{
	private:
		AbstractPiece **m_piece; //tableau de 2 pieces (pointeurs)
		int m_orientation;
		int m_gridPosX;
		int m_gridPosY;
				
	public:
		Pair(GameGrid *grid); //random
		Pair(GameGrid *grid, int shape1, int color1, int shape2, int color2);
		Pair(GameGrid *grid, AbstractPiece *piece0, AbstractPiece *piece1);
		Pair(AbstractPiece *piece0, AbstractPiece *piece1);
		~Pair();
		AbstractPiece *getPiece(int num);
		int getOrientation();
		void setOrientation(int orientation);
		int rotate(bool clockwise=true); //ne fait aucune vérif
		void draw(GameGrid *grid, SerialLocalEffect *fx_transl, int applyFxOn=APPLY_FX_ON_BOTH, AbstractLocalEffect *fx1=NULL, AbstractLocalEffect *fx2=NULL, AbstractLocalEffect *fx3=NULL, AbstractLocalEffect *fx4=NULL);
		void drawXY(int x, int y, GameGrid *grid, SerialLocalEffect *fx_transl, int applyFxOn=APPLY_FX_ON_BOTH, AbstractLocalEffect *fx1=NULL, AbstractLocalEffect *fx2=NULL, AbstractLocalEffect *fx3=NULL, AbstractLocalEffect *fx4=NULL);
		void drawXY(int x, int y, int x2, int y2, SerialLocalEffect *fx_transl, int applyFxOn, AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3, AbstractLocalEffect *fx4);
		//void setGridPos(int x, int y);
		void move(int direction);  //ne fait aucune vérif
		AbstractPiece *detach(int num); //renvoie la piece "num" et met null à la place dans la paire
		
		//void setGrid(Matrix<MatrixContentObject> *grid);
		void update();
		Pos getGridPos(int pieceNum); 
		void switchPieces(); //permute les 2 pieces
		void goTo(int col, int line); //positionne la piece a une autre place
		
		void setPiece0(AbstractPiece *piece0);
		void setPiece1(AbstractPiece *piece1);
};

#endif /*PAIR_H_*/



