#include "code/game/grid/MatrixContentObject.h"
#include "code/game/gamedef.h"
#include "code/framework/fx/EffectsLib.h"

MatrixContentObject::MatrixContentObject(AbstractPiece *p)
//:Trackable("mco")
{
	//debug("new MatrixContentObject");
	m_piece=p;
	m_colorExploding=false;
	m_shapeExploding=false;
	m_specialExploding=false;
	m_state=MCO_STATE_FIXED;
	//debug("new TranslationEffect");
	m_fx_fall=new TranslationEffect(0, 0, 0, GAMEDEF_PIECE_HEIGHT, GAMEDEF_COLLAPSE_SPEED, 0, true, 1);
	//debug("new MatrixContentObject ok");
	
	m_mcoContent=NULL;
}
MatrixContentObject::~MatrixContentObject()
{
	if(m_piece!=NULL)
		delete m_piece;
	delete m_fx_fall;
	
	if(m_mcoContent!=NULL)
		delete m_mcoContent;
}
		
MatrixContentObject::MatrixContentObject(const MatrixContentObject &mco) //copie
//:Trackable("mco")
{
	if(mco.m_piece==NULL)
		m_piece=NULL;
	else
		m_piece=mco.m_piece->clone();
	m_colorExploding=false;
	m_shapeExploding=false;
	m_specialExploding=false;
	m_state=MCO_STATE_FIXED;
	m_fx_fall=mco.m_fx_fall->clone();
	m_mcoContent=mco.m_mcoContent;
}

MatrixContentObject& MatrixContentObject::operator =(const MatrixContentObject& mco)
{
	//this->Trackable::operator=(mco);
	
	if(mco.m_piece==NULL)
		m_piece=NULL;
	else
		m_piece=mco.m_piece->clone();
	m_colorExploding=false;
	m_shapeExploding=false;
	m_specialExploding=false;
	m_state=MCO_STATE_FIXED;
	m_fx_fall=mco.m_fx_fall->clone();
	m_mcoContent=mco.m_mcoContent;
	return *this;
}

AbstractPiece *MatrixContentObject::getPiece()
{
	return m_piece;	
}

bool MatrixContentObject::isColorExploding()
{
	return m_colorExploding;
}

bool MatrixContentObject::isShapeExploding()
{
	return m_shapeExploding;
}

bool MatrixContentObject::isSpecialExploding()
{
	return m_specialExploding;
}

void MatrixContentObject::setColorExploding(bool b)
{
	m_colorExploding=b;
}

void MatrixContentObject::setShapeExploding(bool b)
{
	m_shapeExploding=b;
}

void MatrixContentObject::setSpecialExploding(bool b)
{
	m_specialExploding=b;
}


bool MatrixContentObject::isEmpty()
{
	return m_piece==NULL;
}

bool MatrixContentObject::isExploding(int mode)
{
	if(mode==PIECE_MODE_COLOR)
		return m_colorExploding;
	else if(mode==PIECE_MODE_SHAPE)
		return m_shapeExploding;
	else if(mode==PIECE_MODE_SPECIAL)
		return m_specialExploding;
	
	//else
	return m_colorExploding || m_shapeExploding || m_specialExploding;
}

void MatrixContentObject::startExplosion(int mode)
{
	if(m_piece==NULL)
		fatalError("MatrixContentObject::startExplosion : la piece est NULL");
	m_piece->startDestruct();
	if(mode==PIECE_MODE_COLOR)
		setColorExploding();
	else if(mode==PIECE_MODE_SHAPE)
		setShapeExploding();
	else
		setSpecialExploding();
}

void MatrixContentObject::setState(int state)
{
	m_state=state;
}

int MatrixContentObject::getState()
{
	return m_state;
}

void MatrixContentObject::update()
{
	if(m_piece!=NULL)
	{
		m_piece->update();
		m_fx_fall->update();
	}
}

bool MatrixContentObject::tryCleaning(bool force)
{
	if(m_piece!=NULL && (m_piece->destructed() || force)) //il y a une piece et elle a fini de se detruire. ou alors on force la destruction
	{
		delete m_piece;
		m_piece=NULL;
		
		m_colorExploding=false;
		m_shapeExploding=false;
		m_specialExploding=false;
		m_fx_fall->stop();
		m_fx_fall->reset();
		m_state=MCO_STATE_FIXED;
		return true;
	}
	
	return false;
}

void MatrixContentObject::startFalling()
{
	m_state=MCO_STATE_FALLING;
	m_fx_fall->start();
}

void MatrixContentObject::restartFalling()
{
	m_state=MCO_STATE_FALLING;
	m_fx_fall->restart();
}

AbstractPiece *MatrixContentObject::detachPiece()
{
	AbstractPiece *p=m_piece;
	m_piece=NULL;
	return p;
}

void MatrixContentObject::setPiece(AbstractPiece *p)
{
	if(m_piece != NULL)
	{
		delete m_piece;
	}
	m_piece=p;
}

bool MatrixContentObject::isFinishedFalling()
{
	if(/*m_state==MCO_STATE_FALLING && */m_piece!=NULL && m_fx_fall->finished())
	{
		return true;
	}
	return false;
}

void MatrixContentObject::dontFall()
{
	m_state=MCO_STATE_FIXED;
	m_fx_fall->stop();
	m_fx_fall->reset();
}

bool MatrixContentObject::isFalling()
{
	return m_state==MCO_STATE_FALLING;
}

void MatrixContentObject::setMCOContent(AbstractMCOContent *cont)
{
	if(m_mcoContent!=NULL)
		delete m_mcoContent;
	m_mcoContent=cont;
}

AbstractMCOContent *MatrixContentObject::getMCOContent()
{
	return m_mcoContent;
}

void MatrixContentObject::deleteMCOContent()
{
	if(m_mcoContent==NULL)
		fatalError("MatrixContentObject::deleteMCOContent : mcoContent is NULL");
	delete m_mcoContent;
	m_mcoContent=NULL;
}













