#include "code/game/levels/LinePattern.h"
#include <fstream>
#include "code/framework/utils.h"
#include "code/game/pieces/abstract/AbstractPiece.h"
#include "code/game/gamedef.h"
#include "code/game/pieces/impl/PieceCombo.h"
#include "code/game/pieces/impl/PieceSimple.h"
#include "code/game/pieces/impl/PieceShielded.h"
#include "code/game/pieces/impl/PieceLevel.h"
#include <oslib/oslib.h>

using namespace std;
using namespace utils;

LinePattern::LinePattern(string file, int nbLoops)
{
	//enableDebug();
	//debug("construction linepattern");
	
	m_currentLineNum=0;
	m_pieces=new vector< list<AbstractPiece *> *>();
	
	//chargement du fichier
	//debug("chargement du fichier : "+file);
	ifstream patternfile(file.c_str());
	if(patternfile.fail())
		fatalError("LinePattern : impossible de charger le fichier");
	
	string line;
	int linenum=1;
	while(getline(patternfile,line))
	{
		//debug("ligne: "+line);
		if(line.find(">") == 0) //la ligne doit commencer par ">"
		{
			//debug("trouve une ligne qui correspond : " + line);
			line=deleteAll(line, " ");
			line=line.substr(1, string::npos); //on enlève le ">"
			line=deleteAll(line, "\r"); //le caractère de fin de ligne aussi nous embete
			
			//debug("apres trans: "+line);
			
			vector<string> piecesStr = split(line, "|"); //on a un vecteur de string ou chaque string est le code d'une pièce
			
			//AbstractPiece ** pline=new AbstractPiece *[GAMEDEF_NB_COLS]; //allocation d'une ligne
			list<AbstractPiece *> *pline=new list<AbstractPiece *>();
			
			//int i=0;
			//parcours de piecesStr
			vector<string>::iterator it;
			int piecenum=1;
			for (it=piecesStr.begin(); it<piecesStr.end();it++)
		    {
		    	AbstractPiece *p=getPieceFromString(*it, linenum, piecenum);
		    	pline->push_back(p);
		    	piecenum++;
		    }
			
			m_pieces->push_back(pline);
			linenum++;
		}
	}
	
	patternfile.close();
	
	m_nbLoops=nbLoops;
	m_currentLoop=0;
}

LinePattern::~LinePattern()
{
	vector< list<AbstractPiece *> *>::iterator it;
	for (it=m_pieces->begin(); it<m_pieces->end();it++)
    {
    	list<AbstractPiece *> *pline=*it;
    	
    	list<AbstractPiece *>::iterator it2;
    	for (it2=pline->begin(); it2!=pline->end(); it2++)
    	{
    		AbstractPiece *p=*it2;
    		if(p!=NULL)
    			delete p;
    	}
    	delete pline;
//    	for(int i=0;i<GAMEDEF_NB_COLS;i++)
//    	{
//    		if(pline[i]!=NULL)
//	    		delete pline[i];
//    	}
//    	delete[] pline;

    }
    delete m_pieces;
}

void LinePattern::reset()
{
	m_currentLineNum=0;
	m_currentLoop=0;
}

AbstractPiece *LinePattern::getPieceFromString(string s, int linenum, int piecenum)
{
	//debug("getPieceFromString : "+s);
	int type=0; //defines de AbstractPiece.h
	int color=0; //defines de AbstractPiece.h
	int shape=0; //defines de AbstractPiece.h
	
	string colorstr=""; //la couleur sous forme de lettre
	
	if(s.find(".")==0) //pas de pièce
		return NULL;
	
	//pieces combo
	if(s.find("(")==0)
	{
		//debug("piece combo");
		string number=s.substr(1, s.size()-2);
		int num=toInt(number);
		
		//debug(toString(num) + " combos, return");
		return new PieceCombo(num);		    		
	}
	
	//piece simple mais aleatoire
	if(s.find("!!")==0) 
	{
		//debug("piece aleatoire, return");
		return new PieceSimple();
	}
	
	//pieces spéciales
	if(s.find("#")==0)
	{
		//enableDebug();
		//debug("trouvé une piece spéciale");
		//debug("s="+s);
		s=deleteAll(s, "#");
		int id;
		if(s.size()==0)
			id=0;
		else
			id=toInt(s);
		//debug("id="+toString(id));
		
		//TODO: ces pieces devraient gérer les couleurs
		PieceLevel *piece=new PieceLevel(PIECE_SHAPE_NONE, PIECE_COLOR_NONE);
		piece->setId(id);
		//enableDebug(false);
		return piece;		
	}
	
	//pieces blindées
	if(s.find("[")==0)
	{
		//debug("piece blindee");
		type=PIECE_TYPE_SHIELDED;
		s=s.substr(1, s.size()-2);
		//debug("la piece normale serait : "+s);
		if(s.find("!!")==0)
		{
			//debug("piece blindee aleatoire");
			return new PieceShielded();
		}
	}
	//pieces normales
	else
	{
		//debug("piece normale");
		type=PIECE_TYPE_NORMAL;
	}
	
	//pieces normales et blindées: recherche couleur et forme
	if(s.find("*")==0)
	{
		shape=PIECE_SHAPE_STAR;
		colorstr=s.substr(string("*").size(), string::npos);
		//debug("etoile, color="+colorstr);
	}
	else if(s.find("O")==0)
	{
		shape=PIECE_SHAPE_CIRCLE;
		colorstr=s.substr(string("O").size(), string::npos);
		//debug("rond, color="+colorstr);
	}
	else if(s.find("□")==0)
	{
		shape=PIECE_SHAPE_SQUARE;
		colorstr=s.substr(string("□").size(), string::npos);
		//debug("carre, color="+colorstr);
	}
	else if(s.find("X")==0)
	{
		shape=PIECE_SHAPE_CROSS;
		colorstr=s.substr(string("X").size(), string::npos);
		//debug("croix, color="+colorstr);
	}
	else if(s.find("▲")==0)
	{
		shape=PIECE_SHAPE_TRIANGLE;
		colorstr=s.substr(string("▲").size(), string::npos);
		//debug("triangle, color="+colorstr);
	}
	else if(s.find("◊")==0)
	{
		shape=PIECE_SHAPE_DIAMOND;
		colorstr=s.substr(string("◊").size(), string::npos);
		//debug("lozange, color="+colorstr);
	}
	else
		fatalError("LinePattern: ne peut connaitre la forme desiree, s='"+s+"'\n at position ("+toString(piecenum)+","+toString(linenum)+")");
	
	//recherche couleur 
	//debug("recherche couleur avec str : "+colorstr);
	if(type==PIECE_TYPE_NORMAL || type==PIECE_TYPE_SHIELDED)
	{
		if(colorstr.find("B")==0)
		{
			//debug("BLEU");
			color=PIECE_COLOR_BLUE;
		}
		else if(colorstr.find("R")==0)
		{
			//debug("ROUGE");
			color=PIECE_COLOR_RED;
		}
		else if(colorstr.find("Y")==0)
		{
			//debug("JAUNE");
			color=PIECE_COLOR_YELLOW;
		}
		else if(colorstr.find("G")==0)
		{
			//debug("VERT");
			color=PIECE_COLOR_GREEN;
		}
		else if(colorstr.find("O")==0)
		{
			//debug("ORANGE");
			color=PIECE_COLOR_ORANGE;
		}
		else if(colorstr.find("P")==0)
		{
			//debug("VIOLET");
			color=PIECE_COLOR_PURPLE;
		}
		else
			fatalError("LinePattern: ne peut connaitre la couleur desiree: '"+colorstr+"', s='"+s+"'\n at position ("+toString(piecenum)+","+toString(linenum)+")");
	}
	
	
	//construction	
	//debug("construction");	    	
	if(type==PIECE_TYPE_NORMAL) //pieces normales
	{
		//debug("return piecesimple");
		return new PieceSimple(shape, color);
	}
	else if(type==PIECE_TYPE_SHIELDED)
	{
		//debug("return pieceshielded");
		return new PieceShielded(shape, color);
	}
	
	//debug("return NULL!!");
	fatalError("LinePattern : ne sait pas quelle piece c'est : "+s+"\n at position ("+toString(piecenum)+","+toString(linenum)+")");
	return NULL;
}



list<AbstractPiece *> *LinePattern::getNextLine()
{
	if(m_currentLineNum>=m_pieces->size())
	{
		if(m_nbLoops==-1) //infinite
			m_currentLineNum=0;
		else
		{
			m_currentLoop++;
			if(m_currentLoop>=m_nbLoops)
				return NULL;
			//else
			m_currentLineNum=0;	
		}
	}
	list<AbstractPiece *> *pline=m_pieces->at(m_currentLineNum);
	m_currentLineNum++;
	return pline;
}

int LinePattern::getNbLinesAvailable()
{
	return m_pieces->size();
}




















