#ifndef LEVEL8BITS_H_
#define LEVEL8BITS_H_

#include <oslib/oslib.h>
#include "code/game/levels/abstract/AbstractLevel.h"
#include "code/game/Pair.h"
#include "code/game/levels/LinePattern.h"
#include "code/game/SongBanner.h"
#include "code/game/combos/ComboCounter.h"
#include "code/game/combos/ComboNumberDrawer.h"
#include "code/framework/anim/Anim.h"
#include "code/game/EventsScenario.h"
#include "code/framework/particles/complete/animFactories/ControlledAnimationFactory.h"
#include "code/framework/particles/complete/impl/ParticleGeneratorSegment.h"
#include "code/framework/particles/complete/fxFactories/PFXFRandomGravity.h"
#include "code/framework/Properties.h"

//inner struct
typedef struct
{
	int particles;
	int gentime;
	int movetime;
	int dist;
	int lifespan;
	float gravitymin;
	float gravitymax;
	
}gen8bitsstruct;

class Level8Bits : public AbstractLevel
{
	private:
		void start_specific(); //démarre la musique, le timer etc
		void update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans);
		void pause_specific(); 
		void resume_specific();
		void drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer);
		void drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer);
		
		Anim *m_animBlue,*m_animRed,*m_animYellow,*m_animPurple,*m_animGreen,*m_animGrey;
		Anim *m_animBlueShield,*m_animRedShield,*m_animYellowShield,*m_animPurpleShield,*m_animGreenShield,*m_animGreyShield;
		void initAnims();
		AbstractPiece *randomPiece();
		void drawPairXY(Pair *pair, int x, int y, GameGrid *grid, AbstractLocalEffect *fx_transl=NULL, AbstractLocalEffect *fx1=NULL);
		void drawPieceXY(AbstractPiece *piece, int x, int y, AbstractLocalEffect *fx_transl=NULL);
		Anim *animFromColor(int color, bool shielded=false);
		void initRandomPart(Properties *props);
		void setGeneratorsSpeedSlow();
		void setGeneratorsSpeedNormal();
		void setGeneratorsSpeedFast();
		SongBanner *m_songbanner1;
		SongBanner *m_songbanner2;
		OSL_SOUND *m_music1;
		OSL_SOUND *m_music2;
		int m_part; //voir defines
		int m_difficulty;
		EventsScenario *m_scenario1;
		EventsScenario *m_scenario2;
		ComboNumberDrawer * m_comboDrawer;
		int m_upspeed, m_dropspeed, m_upspeed2, m_dropspeed2;
		float m_duration;
		LinePattern *m_linepattern;
		
		//generateurs de particules:
		//les 2 cotés
		ControlledAnimationFactory *m_gen_animfact_left;
		ControlledAnimationFactory *m_gen_animfact_right;
		Anim *m_gen_animBlue, *m_gen_animRed, *m_gen_animYellow, *m_gen_animPurple, *m_gen_animGreen, *m_gen_animGrey;
		ParticleGeneratorSegment *m_gen_left;
		ParticleGeneratorSegment *m_gen_right;
		PFXFRandomGravity *m_gen_right_pfxfrg;
		PFXFRandomGravity *m_gen_left_pfxfrg;
		
		void initParticleGenerators(Properties *props);
		void deleteParticleGenerators();
		Anim *particleFromColor(int color);
		
		int m_addline;
		
		gen8bitsstruct *m_gen_props_speed1;
		gen8bitsstruct *m_gen_props_speed2;
		gen8bitsstruct *m_gen_props_speed3;
		
		bool hasEvent(int event);
		
		OSL_IMAGE *m_img_overlay;
		
		//generateurs pour brutus
		ParticleGeneratorSegment *m_gen_brutus1;
		ParticleGeneratorSegment *m_gen_brutus2;
		ShakeEffect *m_fx_gen_brutus_shake;
		void initGeneratorsBrutus(Properties *props);
		void deleteGeneratorsBrutus();
		
		//effet pour eurhawt
		ProgressiveAlphaEffect *m_fx_eurhawt_alpha;
		
		//effet pour hedgehog
		TranslationEffect *m_fx_hedgehog_transl;
		
		RotationEffect *m_fx_particlesRot;
		
		
		//pour les boums finaux
		ShakeEffect *m_fx_boum_shake;
		
		OSL_IMAGE *m_img_songbannerbg;
		SerialLocalEffect *m_fx_songbanner;
		WaitLocalEffect *m_fx_songbanner_wait_ptr; //pointeur uniquement!
		
		OSL_IMAGE *m_img_pause;
		
		int m_color_pause;
		
		bool m_test;
		
	public:
		Level8Bits(int difficulty=GAMEDEF_DIFFICULTY_NORMAL, int nbPlayers=1);
		~Level8Bits();
		
		//hérité:
		int getDropSpeed(); //la vitesse a un instant t (elle peut etre modifiée par le scénario)
		int getUpSpeed(); //la vitesse ou les lignes montent a un instant t
		void draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer);
		string getCode();
		
		void buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid);
		
		//LinePattern *getLevelPieces(); 
		bool getAddLineEvent(); //dit quand une ligne doit monter, dû au scénario
		
		bool finished();
		
		list<AbstractPiece *> *getNextLine();
		
		//surchargé pour avoir les bonnes pieces
		Pair *getNewPair();
		
		float getDuration();
		
		void reset();
};

#endif /*LEVEL8BITS_H_*/
