#include "code/game/levels/impl/bioteckprojekt/LevelBioteck.h"
#include "code/framework/Pos.h"
#include "code/game/gamedef.h"
#include "code/framework/utils.h"
#include <oslib/oslib.h>
#include "code/framework/fx/EffectsLib.h"
#include "code/game/pieces/abstract/AbstractPiece.h"
#include "code/game/pieces/impl/PieceSimple.h"
#include "code/game/pieces/impl/PieceJoker.h"
#include "code/framework/Properties.h"
#include "code/framework/fx/utils/AlphaParamsState.h"
#include "code/framework/check/loaders.h"

using namespace utils;

#define LEVEL_BIOTECK_DURATION 247000.0f

string LevelBioteck::getCode()
{
	return "bioteck";
}

LevelBioteck::LevelBioteck(int difficulty, int nbPlayers):AbstractLevel(0, -1, difficulty, nbPlayers)
{		
	Properties *props=NULL;
		
	if(difficulty==GAMEDEF_DIFFICULTY_EASY)
	{
		props=new Properties("res/levels/bioteckprojekt/bioteckprojekt_easy.properties");
		m_linepattern=new LinePattern("res/levels/bioteckprojekt/bioteckprojekt_easy.pattern");
		m_gridImage=fwkLoadImageFilePNG("res/levels/bioteckprojekt/grid.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_4444);
	}
	else if(difficulty==GAMEDEF_DIFFICULTY_NORMAL)
	{
		props=new Properties("res/levels/bioteckprojekt/bioteckprojekt_normal.properties");
		m_linepattern=new LinePattern("res/levels/bioteckprojekt/bioteckprojekt_normal.pattern");
		m_gridImage=fwkLoadImageFilePNG("res/levels/bioteckprojekt/grid.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_4444);
	}
	else //hard
	{
		props=new Properties("res/levels/bioteckprojekt/bioteckprojekt_hard.properties");
		m_nbCols=16;
		m_linepattern=new LinePattern("res/levels/bioteckprojekt/bioteckprojekt_hard.pattern");
		m_gridImage=fwkLoadImageFilePNG("res/levels/bioteckprojekt/grid_hard.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_4444);
	}
	
	m_dropspeed=props->getIntValue("dropspeed.start");
	m_dropspeed_normal=props->getIntValue("dropspeed"); 
	m_upspeed_normal=props->getIntValue("upspeed");
	m_jokerRate=props->getIntValue("joker.rate"); //toutes les X pieces, un joker
	m_difficulty=difficulty;
	
	m_songbanner=new SongBanner(GAMEDEF_SONGBANNER_Y, "res/fonts/bmpFonts/russelsquare/14/", "normal", "italic", props->getStrValue("songbanner.text1"), props->getStrValue("songbanner.text2", ""), props->getStrValue("songbanner.text3", ""), props->getStrValue("songbanner.text4", ""));
	m_comboDrawer=new ComboNumberDrawer();
	m_scenario=new EventsScenario("res/levels/bioteckprojekt/bioteckprojekt.scenario");
	m_music=oslLoadSoundFileBGM("res/levels/bioteckprojekt/bioteckprojekt.bgm", OSL_FMT_STREAM);
	
	
	m_upspeed=-1;
	m_pieceCount=0;
		
	m_addline=0;
	
	m_fx1=new SerialLocalEffect(1);
	m_fx2=new SerialLocalEffect(1);
	
	int growth=props->getIntValue("fx.growth");
	
	//tu iii	
	m_fx1->addEffect(new SizeWowEffect(0, growth, 0, growth, props->getIntValue("fx.part1.time"), props->getIntValue("fx.part1.time"), true, 1));
	m_fx2->addEffect(new WaitLocalEffect(props->getIntValue("fx.part1.time")));
	
	//pause
	WaitLocalEffect *wait=new WaitLocalEffect(props->getIntValue("fx.part1.pause"));
	wait->setSizePercent(100+growth, 100+growth);
	m_fx1->addEffect(wait);
	
	m_fx2->addEffect(new WaitLocalEffect(props->getIntValue("fx.part1.pause")));
	
	//ti ouh
	m_fx1->addEffect(new SizeWowEffect(0, growth, 0, growth, props->getIntValue("fx.part2.time"), props->getIntValue("fx.part2.time"), false, 1));
	m_fx2->addEffect(new WaitLocalEffect(props->getIntValue("fx.part2.time")));
	
	//pause
	m_fx1->addEffect(new WaitLocalEffect(props->getIntValue("fx.part2.pause")));
	m_fx2->addEffect(new WaitLocalEffect(props->getIntValue("fx.part2.pause")));
	
	//touh ouh ouh iii
	m_fx1->addEffect(new SizeWowEffect(0, growth, 0, growth, props->getIntValue("fx.part3.time"), props->getIntValue("fx.part3.time"), true, 1));
	m_fx2->addEffect(new ShakeEffect(0, props->getIntValue("fx.part3.shake.height"), props->getIntValue("fx.part3.shake.time"), true, props->getIntValue("fx.part3.shake.loops")));
	
	//pause
	wait=new WaitLocalEffect(props->getIntValue("fx.part3.pause"));
	wait->setSizePercent(100+growth, 100+growth);
	m_fx1->addEffect(wait);
	m_fx2->addEffect(new WaitLocalEffect(props->getIntValue("fx.part3.pause")));
	
	//ouhiiouhiiouh
	m_fx1->addEffect(new SizeWowEffect(0, growth, 0, growth, props->getIntValue("fx.part4.time"), props->getIntValue("fx.part4.time"), false, 1));
	m_fx2->addEffect(new RotationEffect(props->getIntValue("fx.part4.rotation.time"), true, props->getIntValue("fx.part4.rotation.loops")));
	
	//pause
	m_fx1->addEffect(new WaitLocalEffect(props->getIntValue("fx.part4.pause"), true));
	m_fx2->addEffect(new WaitLocalEffect(props->getIntValue("fx.part4.pause")));
	
	//ouh iii iii iiii
	SerialLocalEffect *slfx=new SerialLocalEffect(4);
	slfx->addEffect(new SizeWowEffect(0, growth, 0, growth, props->getIntValue("fx.part5.growth.time"), props->getIntValue("fx.part5.growth.time"), true, 1));
	m_fx1->addEffect(slfx);
	
	m_fx_fadeout=new ProgressiveAlphaEffect(props->getIntValue("fx.fadeout.time"), props->getIntValue("fx.fadeout.time"), props->getIntValue("fx.fadeout.from"), props->getIntValue("fx.fadeout.to"), 1);
	m_fx_fadein=new ProgressiveAlphaEffect(props->getIntValue("fx.fadein.time"), props->getIntValue("fx.fadein.time"), props->getIntValue("fx.fadein.from"), props->getIntValue("fx.fadein.to"), 1);
	
	delete props;
	
}

LevelBioteck::~LevelBioteck()
{
	oslDeleteImage(m_gridImage);
	
	delete m_linepattern;
	delete m_songbanner;
	delete m_comboDrawer;
	delete m_scenario;
	
	delete m_fx1;
	delete m_fx2;
	
	delete m_fx_fadeout;
	delete m_fx_fadein;
	
	oslDeleteSound(m_music);
	
}


//dessin avec une paire
void LevelBioteck::draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	if(!isPaused())
	{
		currentPair->draw(grid, fx_trans, APPLY_FX_ON_NONE);
	}
	drawAfter(grid, nextPair, cc);
}

//dessin avec 2 pieces
void LevelBioteck::draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	Pos pixPosConnected=grid->getContentPosition(pieceConnectedPos.x, pieceConnectedPos.y);
	Pos pixPosUnconnected=grid->getContentPosition(pieceUnconnectedPos.x, pieceUnconnectedPos.y);
	
	pieceConnected->drawXY(pixPosConnected.x, pixPosConnected.y);
	pieceUnconnected->drawXY(pixPosUnconnected.x, pixPosUnconnected.y, fx_trans);
	
	drawAfter(grid, nextPair, cc);
}

//dessin avec rien
void LevelBioteck::draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	drawAfter(grid, nextPair, cc);
}

void LevelBioteck::drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	oslDrawImageXY(m_gridImage, grid->getPosX(), grid->getPosY());
	
	if(isPaused())
	{
		//TODO: vrai menu pause
		oslDrawString(220, 130, "pause");
	}
	else
	{
		//dessin du contenu de la grille
		
		if(m_fx_fadeout->isRunning())
		{
			AlphaParamsState *alphap=new AlphaParamsState();
			alphap->saveState();
			m_fx_fadeout->activate();
			for(int i=0;i<grid->getNbCols();i++)
			{
				for(int j=0;j<grid->getNbLines();j++)
				{
					MatrixContentObject *mco=grid->getAt(i,j);
					if(mco->getPiece()!=NULL)
					{
						Pos pos=grid->getContentPosition(i, j);
						AbstractPiece *p=mco->getPiece();
											
						p->drawXY(pos.x,pos.y);					
					}
				}	
			}
			alphap->restoreState();
			delete alphap;
		}
		
		if(!m_fx_fadeout->isRunning() && !m_fx_fadein->isRunning())
		{		
			for(int i=0;i<grid->getNbCols();i++)
			{
				for(int j=0;j<grid->getNbLines();j++)
				{
					MatrixContentObject *mco=grid->getAt(i,j);
					if(mco->getPiece()!=NULL)
					{
						Pos pos=grid->getContentPosition(i, j);
						AbstractPiece *p=mco->getPiece();
											
						p->drawXY(pos.x,pos.y,m_fx1->isRunning()?m_fx1:NULL,m_fx1->isRunning()?m_fx2:NULL);					
					}
				}	
			}
		}
		
		if(m_fx_fadein->isRunning())
		{
			AlphaParamsState *alphap=new AlphaParamsState();
			alphap->saveState();
			m_fx_fadein->activate();
			for(int i=0;i<grid->getNbCols();i++)
			{
				for(int j=0;j<grid->getNbLines();j++)
				{
					MatrixContentObject *mco=grid->getAt(i,j);
					if(mco->getPiece()!=NULL)
					{
						Pos pos=grid->getContentPosition(i, j);
						AbstractPiece *p=mco->getPiece();
											
						p->drawXY(pos.x,pos.y,m_fx1->isRunning()?m_fx1:NULL,m_fx1->isRunning()?m_fx2:NULL);					
					}
				}	
			}
			alphap->restoreState();
			delete alphap;
		}
	}
	
	
}

void LevelBioteck::drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	nextPair->drawXY(60, 30, grid, APPLY_FX_ON_NONE);
	oslDrawFillRect(400, 270-int((cc->getTime()*260)/cc->getTimeTotalAvailable()), 415, 270, RGB(255,0,0));
	
	m_songbanner->draw();
	m_comboDrawer->drawXY(200, 180); //TODO: calculer le placement
	oslDrawString(10, 190, (toString(m_scenario->getTime())).c_str());
}

void LevelBioteck::start_specific()
{
	oslPlaySound(m_music, 0);
	m_scenario->restart();
}

//appelé uniquement si le level n'est pas en pause.
void LevelBioteck::update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
{
	m_songbanner->update();
	m_comboDrawer->sync(cc);
	m_comboDrawer->update();
	m_scenario->update();
	m_fx1->update();
	m_fx2->update();
	m_fx_fadeout->update();
	m_fx_fadein->update();
		
	//evenements du scenario
	if(m_scenario->hasEvent(1)) //debut basses
	{
		if(m_difficulty==GAMEDEF_DIFFICULTY_EASY)
			m_addline++;
		else
			m_addline+=3;
		m_dropspeed=m_dropspeed_normal;
		m_upspeed=m_upspeed_normal;
	}
	
	if(m_scenario->hasEvent(4)) //tou iiii
	{
		m_fx1->restart();
		m_fx2->restart();
	}
	
	if(m_scenario->hasEvent(9)) //fade out
	{
		m_fx_fadeout->restart();
	}
	
	if(m_scenario->hasEvent(11)) //fade in
	{
		m_fx_fadein->restart();
	}
	
}

void LevelBioteck::pause_specific() 
{
	oslPauseSound(m_music, 1);
	m_scenario->pause();
}

void LevelBioteck::resume_specific()
{
	oslPauseSound(m_music, 0);
	m_scenario->start();
}
		

list<AbstractPiece *> *LevelBioteck::getNextLine()
{
	return m_linepattern->getNextLine();
}

bool LevelBioteck::getAddLineEvent() //dit quand une ligne doit monter, dû au scénario
{
	if(m_addline>0)
	{
		m_addline--;
		return true;
	}
	return false;
}

int LevelBioteck::getDropSpeed() //la vitesse a un instant t (elle peut etre modifiée par le scénario)
{
	return m_dropspeed;
}

int LevelBioteck::getUpSpeed() //la vitesse ou les lignes montent a un instant t
{
	return m_upspeed;
}

void LevelBioteck::buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid)
{
	if(key==OSL_KEY_TRIANGLE)
	{
				
	}
	else if(key==OSL_KEY_SQUARE)
	{
		m_fx1->restart();
		m_fx2->restart();
	}
}

bool LevelBioteck::finished()
{
	if(m_scenario->getTime()>=LEVEL_BIOTECK_DURATION)
	{
		return true;
	}
	return false;
}

float LevelBioteck::getDuration()
{
	return LEVEL_BIOTECK_DURATION;
}

Pair *LevelBioteck::getNewPair()
{
	AbstractPiece *p1=NULL, *p2=new PieceSimple();
	if(m_jokerRate!=-1 && m_pieceCount>=m_jokerRate)
	{
		p1=new PieceJoker();
		m_pieceCount=0;
	}
	else 
	{
		if(m_jokerRate!=-1)
		{
			m_pieceCount++;
		}
		p1=new PieceSimple();
	}
	
	if(p1==NULL)
		oslFatalError("LevelBioteck::getNewPair : p1 is NULL");
	
	Pair *p=new Pair(p1,p2);
	
	int posX=int(m_nbCols/2)-1;
	int posY=-1; //au dessus de tout en haut
	p->goTo(posX, posY);
	
	return p;
}




