#ifndef LEVELBIOTECK_H_
#define LEVELBIOTECK_H_

#include <oslib/oslib.h>
#include "code/game/levels/abstract/AbstractLevel.h"
#include "code/game/grid/GameGrid.h"
#include "code/game/Pair.h"
#include "code/game/levels/LinePattern.h"
#include "code/game/pieces/abstract/AbstractPiece.h"
#include <list>
#include "code/game/SongBanner.h"
#include "code/game/combos/ComboCounter.h"
#include "code/game/combos/ComboNumberDrawer.h"
#include "code/game/EventsScenario.h"
#include "code/framework/fx/EffectsLib.h"


using namespace std;

class LevelBioteck : public AbstractLevel
{
	private:
			
		OSL_IMAGE *m_gridImage;
		LinePattern *m_linepattern;
			
		void start_specific(); //démarre la musique, le timer etc
		void update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans);
		void pause_specific(); 
		void resume_specific();
				
		SongBanner *m_songbanner;
		
		ComboNumberDrawer * m_comboDrawer;
		
		EventsScenario *m_scenario;
		OSL_SOUND* m_music;
		
		int m_dropspeed;
		int m_upspeed;
		
		int m_dropspeed_normal;
		int m_upspeed_normal;
		
		int m_addline; //ajoute 1 qd on veut qu'une ligne monte
		
		void drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc);
		void drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc);
		
		int m_jokerRate;
		int m_pieceCount; //compteur pour savoir qd le joker arrive
		
		//effets (2 en simultané)
		SerialLocalEffect *m_fx1;
		SerialLocalEffect *m_fx2;
		
		ProgressiveAlphaEffect *m_fx_fadeout;
		ProgressiveAlphaEffect *m_fx_fadein;
		
		int m_difficulty;
		
	public:
		LevelBioteck(int difficulty=GAMEDEF_DIFFICULTY_NORMAL, int nbPlayers=1);
		~LevelBioteck();
		
		//hérité:
		int getDropSpeed(); //la vitesse a un instant t (elle peut etre modifiée par le scénario)
		int getUpSpeed(); //la vitesse ou les lignes montent a un instant t
		void draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer);
		string getCode();
		
		void buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid);
		
		//LinePattern *getLevelPieces(); 
		bool getAddLineEvent(); //dit quand une ligne doit monter, dû au scénario
		
		bool finished();
		
		list<AbstractPiece *> *getNextLine();
		
		//surchargé pour avoir des jokers de temps en temps
		Pair *getNewPair();
		
		float getDuration();
};

#endif /*LEVELBIOTECK_H_*/
