#include "code/game/levels/impl/cathedral/LevelCathedral.h"
#include "code/framework/Properties.h"
#include <string>
#include "code/game/gamedef.h"
#include "code/game/levels/utils/DJMCOContentSimple.h"
#include "code/game/grid/AbstractMCOContent.h"
#include "code/game/Pair.h"
#include "code/framework/utils.h"
#include "code/game/pieces/impl/PieceNull.h"
#include "code/game/pieces/impl/PieceLevel.h"
#include "code/game/levels/utils/DJMCOContentLayered.h"
#include "code/framework/particles/complete/impl/ParticleGeneratorPoint.h"
#include "code/framework/check/loaders.h"

using namespace utils;
using namespace std;

#define LEVEL_CATHEDRAL_PART_SHELTER 0
#define LEVEL_CATHEDRAL_PART_TECHNO 1
#define LEVEL_CATHEDRAL_PART_BOSS 2

#define LEVEL_CATHEDRAL_DURATION_SHELTER 72070
#define LEVEL_CATHEDRAL_DURATION_TECHNO 93960
#define LEVEL_CATHEDRAL_DURATION_BOSS 95390

string LevelCathedral::getCode()
{
	return "cathedral";
}

LevelCathedral::LevelCathedral(int difficulty, int nbPlayers):AbstractLevel(0, -1, difficulty, nbPlayers)
{
	Properties *props=new Properties("res/levels/cathedral/cathedral.properties");
	
	//m_difficulty=difficulty;
	m_gridImage=fwkLoadImageFilePNG("res/levels/cathedral/grid.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_gridImage, props->getIntValue("image.grid.pos.x"), props->getIntValue("image.grid.pos.y"));
	
	m_pos_nextPair.x=props->getIntValue("nextpair.pos.x");
	m_pos_nextPair.y=props->getIntValue("nextpair.pos.y");
	
	if(m_difficulty==GAMEDEF_DIFFICULTY_EASY)
	{
		m_linepattern=new LinePattern("res/levels/cathedral/cathedral_easy.pattern");
		
		m_dropspeed_shelter_normal=props->getIntValue("easy.dropspeed.shelter.normal");
		m_dropspeed_shelter_fast=props->getIntValue("easy.dropspeed.shelter.fast");
		m_dropspeed_techno_normal=props->getIntValue("easy.dropspeed.techno.normal");
		m_dropspeed_techno_fast=props->getIntValue("easy.dropspeed.techno.fast");
		m_dropspeed_boss_normal=props->getIntValue("easy.dropspeed.boss.normal");
		m_dropspeed_boss_fast=props->getIntValue("easy.dropspeed.boss.fast");
		
		m_upspeed_shelter_normal=props->getIntValue("easy.upspeed.shelter.normal");
		m_upspeed_shelter_fast=props->getIntValue("easy.upspeed.shelter.fast");
		m_upspeed_techno_normal=props->getIntValue("easy.upspeed.techno.normal");
		m_upspeed_techno_fast=props->getIntValue("easy.upspeed.techno.fast");
		m_upspeed_boss_normal=props->getIntValue("easy.upspeed.boss.normal");
		m_upspeed_boss_fast=props->getIntValue("easy.upspeed.boss.fast");
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_NORMAL)
	{
		m_linepattern=new LinePattern("res/levels/cathedral/cathedral_normal.pattern");
		
		m_dropspeed_shelter_normal=props->getIntValue("normal.dropspeed.shelter.normal");
		m_dropspeed_shelter_fast=props->getIntValue("normal.dropspeed.shelter.fast");
		m_dropspeed_techno_normal=props->getIntValue("normal.dropspeed.techno.normal");
		m_dropspeed_techno_fast=props->getIntValue("normal.dropspeed.techno.fast");
		m_dropspeed_boss_normal=props->getIntValue("normal.dropspeed.boss.normal");
		m_dropspeed_boss_fast=props->getIntValue("normal.dropspeed.boss.fast");
		
		m_upspeed_shelter_normal=props->getIntValue("normal.upspeed.shelter.normal");
		m_upspeed_shelter_fast=props->getIntValue("normal.upspeed.shelter.fast");
		m_upspeed_techno_normal=props->getIntValue("normal.upspeed.techno.normal");
		m_upspeed_techno_fast=props->getIntValue("normal.upspeed.techno.fast");
		m_upspeed_boss_normal=props->getIntValue("normal.upspeed.boss.normal");
		m_upspeed_boss_fast=props->getIntValue("normal.upspeed.boss.fast");
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_HARD)
	{
		m_linepattern=new LinePattern("res/levels/cathedral/cathedral_hard.pattern");
		
		m_dropspeed_shelter_normal=props->getIntValue("hard.dropspeed.shelter.normal");
		m_dropspeed_shelter_fast=props->getIntValue("hard.dropspeed.shelter.fast");
		m_dropspeed_techno_normal=props->getIntValue("hard.dropspeed.techno.normal");
		m_dropspeed_techno_fast=props->getIntValue("hard.dropspeed.techno.fast");
		m_dropspeed_boss_normal=props->getIntValue("hard.dropspeed.boss.normal");
		m_dropspeed_boss_fast=props->getIntValue("hard.dropspeed.boss.fast");
		
		m_upspeed_shelter_normal=props->getIntValue("hard.upspeed.shelter.normal");
		m_upspeed_shelter_fast=props->getIntValue("hard.upspeed.shelter.fast");
		m_upspeed_techno_normal=props->getIntValue("hard.upspeed.techno.normal");
		m_upspeed_techno_fast=props->getIntValue("hard.upspeed.techno.fast");
		m_upspeed_boss_normal=props->getIntValue("hard.upspeed.boss.normal");
		m_upspeed_boss_fast=props->getIntValue("hard.upspeed.boss.fast");
	}
	else
	{
		oslFatalError("no such difficulty in level cathedral");
	}
	
	m_upspeed=m_upspeed_shelter_normal;
	m_dropspeed=m_dropspeed_shelter_normal;
	
	m_songbanner_shelter=new SongBanner(GAMEDEF_SONGBANNER_Y, "res/fonts/bmpFonts/russelsquare/14/", "normal", "italic", 
								props->getStrValue("songbanner.shelter.text1"), props->getStrValue("songbanner.shelter.text2", ""), 
								props->getStrValue("songbanner.shelter.text3", ""), props->getStrValue("songbanner.shelter.text4", ""));
	m_songbanner_techno=new SongBanner(GAMEDEF_SONGBANNER_Y, "res/fonts/bmpFonts/russelsquare/14/", "normal", "italic", 
								props->getStrValue("songbanner.techno.text1"), props->getStrValue("songbanner.techno.text2", ""), 
								props->getStrValue("songbanner.techno.text3", ""), props->getStrValue("songbanner.techno.text4", ""));
	m_songbanner_boss=new SongBanner(GAMEDEF_SONGBANNER_Y, "res/fonts/bmpFonts/russelsquare/14/", "normal", "italic", 
								props->getStrValue("songbanner.boss.text1"), props->getStrValue("songbanner.boss.text2", ""), 
								props->getStrValue("songbanner.boss.text3", ""), props->getStrValue("songbanner.boss.text4", ""));
	m_comboDrawer=new ComboNumberDrawer();
	m_scenario_shelter=new EventsScenario("res/levels/cathedral/shelter.scenario");
	m_scenario_techno=new EventsScenario("res/levels/cathedral/techno.scenario");
	m_scenario_boss=new EventsScenario("res/levels/cathedral/boss.scenario");
	
	m_music_techno=fwkLoadSoundFileMP3("res/levels/cathedral/techno_cathedral.mp3", OSL_FMT_STREAM);
	m_music_boss=fwkLoadSoundFileMP3("res/levels/cathedral/boss_at_cathedral.mp3", OSL_FMT_STREAM);
	m_music_shelter=fwkLoadSoundFileMP3("res/levels/cathedral/ancient_shelter.mp3", OSL_FMT_STREAM);
	oslSetSoundVolume(m_music_techno, props->getIntValue("music.techno.volume"));
	oslSetSoundVolume(m_music_boss, props->getIntValue("music.boss.volume"));
	oslSetSoundVolume(m_music_shelter, props->getIntValue("music.shelter.volume"));
	
	m_part=LEVEL_CATHEDRAL_PART_SHELTER;
	
	
	m_anim_bell=new Anim(props->getIntValue("piece.bell.nbframes"), "bell", "res/levels/cathedral/pieces/bell/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_anim_bible=new Anim(props->getIntValue("piece.bible.nbframes"), "bible", "res/levels/cathedral/pieces/bible/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_anim_candle=new Anim(props->getIntValue("piece.candle.nbframes"), "candle", "res/levels/cathedral/pieces/candle/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_anim_chalice=new Anim(props->getIntValue("piece.chalice.nbframes"), "chalice", "res/levels/cathedral/pieces/chalice/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_anim_cross=new Anim(props->getIntValue("piece.cross.nbframes"), "cross", "res/levels/cathedral/pieces/cross/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_anim_ictus=new Anim(props->getIntValue("piece.ictus.nbframes"), "ictus", "res/levels/cathedral/pieces/ictus/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_anim_bible_paper=new Anim(props->getIntValue("piece.bible.nbframes"), "paper", "res/levels/cathedral/pieces/bible/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_anim_candle_flame=new Anim(props->getIntValue("piece.candle.flame.nbframes"), "fire", "res/levels/cathedral/pieces/candle/", props->getIntValue("piece.candle.flame.duration"), OSL_PF_8888);
	m_anim_candle_flame_explode=new Anim(props->getIntValue("piece.candle.flame.explosion.nbframes"), "fire_expl", "res/levels/cathedral/pieces/candle/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	//m_anim_bible_sheet=new Anim(props->getIntValue("piece.bible.nbframes"), "sheet", "res/levels/cathedral/pieces/bible/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_anim_bell->setReferencePointCenter();
	m_anim_bible->setReferencePointCenter();
	m_anim_candle->setReferencePointCenter();
	m_anim_chalice->setReferencePointCenter();
	m_anim_cross->setReferencePointCenter();
	m_anim_ictus->setReferencePointCenter();
	m_anim_bible_paper->setReferencePointCenter();
	m_anim_candle_flame->setReferencePointCenter();
	m_anim_candle_flame_explode->setReferencePointCenter();
	//m_anim_bible_sheet->setReferencePointCenter();
	
	m_color_blue=RGB(props->getIntValue("color.blue.r"), props->getIntValue("color.blue.g"), props->getIntValue("color.blue.b"));
	m_color_red=RGB(props->getIntValue("color.red.r"), props->getIntValue("color.red.g"), props->getIntValue("color.red.b"));
	m_color_green=RGB(props->getIntValue("color.green.r"), props->getIntValue("color.green.g"), props->getIntValue("color.green.b"));
	m_color_yellow=RGB(props->getIntValue("color.yellow.r"), props->getIntValue("color.yellow.g"), props->getIntValue("color.yellow.b"));
	m_color_orange=RGB(props->getIntValue("color.orange.r"), props->getIntValue("color.orange.g"), props->getIntValue("color.orange.b"));
	m_color_purple=RGB(props->getIntValue("color.purple.r"), props->getIntValue("color.purple.g"), props->getIntValue("color.purple.b"));
	
	
	m_mcoc_bell=newMCOContentBell();
	m_mcoc_bible=newMCOContentBible();
	m_mcoc_candle=newMCOContentCandle();
	m_mcoc_chalice=newMCOContentChalice();
	m_mcoc_cross=newMCOContentCross();
	m_mcoc_ictus=newMCOContentIctus();
	
		
	//rayons
	m_img_rays=new OSL_IMAGE *[5];
	for(int i=1;i<=5;i++)
	{
		m_img_rays[i-1]=fwkLoadImageFilePNG("res/levels/cathedral/ray"+toString(i)+".png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
		oslImageSetPosition(m_img_rays[i-1], props->getIntValue("ray"+toString(i)+".pos.x"), props->getIntValue("ray"+toString(i)+".pos.y"));
	}
	
	m_meteor=constructMeteor(props);
	m_meteor_event=false;
	m_meteor_fxtrans=false;
	
	m_overlap_sheltertechno=props->getIntValue("music.overlap.shelter-techno");
	m_overlap_technoboss=props->getIntValue("music.overlap.techno-boss");
	
	m_addline=0;
	
	initWavesViolin(props);
	
	m_fx_shake_bells=new ShakeEffect(
		props->getIntValue("fx.techno.bell.shake.width"), 
		props->getIntValue("fx.techno.bell.shake.height"), 
		props->getIntValue("fx.techno.bell.shake.time"), 
		true, 
		props->getIntValue("fx.techno.bell.shake.loops"));
	
	//orgue
	m_fx_size_organ=new SizeWowEffect *[getNbCols()]; //tableau
	int growpercent=props->getIntValue("fx.techno.organ.resize.grow.percent");
	int growtime=props->getIntValue("fx.techno.organ.resize.grow.time");
	int reducetime=props->getIntValue("fx.techno.organ.resize.reduce.time");
	int sizeloops=props->getIntValue("fx.techno.organ.resize.loops");
	for(int i=0;i<getNbCols();i++)
	{
		m_fx_size_organ[i]=new SizeWowEffect(0, growpercent, 0, growpercent, growtime, reducetime, true, sizeloops);
	}
	m_repeater_organ=new Repeater(props->getIntValue("fx.techno.organ.resize.repeat.time"));
		
	m_nb_specialexpl=0;
	
	delete props;
	
	
	//enableDebug();
	debug("construction ok");
}

Meteor *LevelCathedral::constructMeteor(Properties *props)
{	
	DJMCOContentLayered *mco=new DJMCOContentLayered();
	
	Anim *anim_normal=new Anim("res/levels/cathedral/pieces/meteor/meteor1.png");
	anim_normal->setReferencePointCenter();
	
	Anim *anim_expl=new Anim(props->getIntValue("piece.meteor.nbframes"), "meteor", "res/levels/cathedral/pieces/meteor/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	anim_expl->setReferencePointCenter();
	
	mco->addLayer(anim_normal, false);
	mco->addLayerExplosion(anim_expl, false);
	
	int lifespan=props->getIntValue("piece.meteor.gen.lifespan");
	
	ParticleGeneratorPoint *partgen=new ParticleGeneratorPoint(
								-50, -50, 270, props->getIntValue("piece.meteor.gen.scale"), 
								props->getIntValue("piece.meteor.gen.throughput.particles"), 
								props->getIntValue("piece.meteor.gen.throughput.frames"), 
								lifespan, 
								props->getIntValue("piece.meteor.gen.distance"),
								lifespan,
								"res/levels/cathedral/pieces/meteor/particle.png", OSL_PF_8888);
	partgen->setLocalEffect1SimpleTranslation();
	partgen->setLocalEffect2(
		new GravityEffect(0, 0, 0, props->getFloatValue("piece.meteor.gen.fx.gravity"))
	);
	
	int part1_duration=props->getIntValue("piece.meteor.gen.fx.tint.part1.duration");
	SerialGlobalEffect *fx_tint=new SerialGlobalEffect(1);
	fx_tint->addEffect(
		new ProgressiveTintEffect(
			part1_duration, part1_duration, 
			RGBA(props->getIntValue("piece.meteor.gen.fx.tint.part1.from.r"), props->getIntValue("piece.meteor.gen.fx.tint.part1.from.g"), props->getIntValue("piece.meteor.gen.fx.tint.part1.from.b"), props->getIntValue("piece.meteor.gen.fx.tint.part1.from.a")), 
			RGBA(props->getIntValue("piece.meteor.gen.fx.tint.part1.to.r"), props->getIntValue("piece.meteor.gen.fx.tint.part1.to.g"), props->getIntValue("piece.meteor.gen.fx.tint.part1.to.b"), props->getIntValue("piece.meteor.gen.fx.tint.part1.to.a")), 
			1)
	);
	fx_tint->addEffect(
		new ProgressiveTintEffect(
			lifespan-part1_duration, lifespan-part1_duration, 
			RGBA(props->getIntValue("piece.meteor.gen.fx.tint.part2.from.r"), props->getIntValue("piece.meteor.gen.fx.tint.part2.from.g"), props->getIntValue("piece.meteor.gen.fx.tint.part2.from.b"), props->getIntValue("piece.meteor.gen.fx.tint.part2.from.a")), 
			RGBA(props->getIntValue("piece.meteor.gen.fx.tint.part2.to.r"), props->getIntValue("piece.meteor.gen.fx.tint.part2.to.g"), props->getIntValue("piece.meteor.gen.fx.tint.part2.to.b"), props->getIntValue("piece.meteor.gen.fx.tint.part2.to.a")), 
			1)
	);
	partgen->setGlobalEffect1(fx_tint);
	
	Meteor *meteor=new Meteor(mco, partgen, props->getIntValue("piece.meteor.gen.pos.y.shift"));
	
	meteor->setX(-50);
	meteor->setY(-50);
	
	return meteor;
}

DJMCOContentSimple *LevelCathedral::newMCOContentBell(OSL_COLOR color)
{
	return new DJMCOContentSimple(new Anim(*m_anim_bell), color);
}

DJMCOContentLayered *LevelCathedral::newMCOContentBible(OSL_COLOR color)
{
	DJMCOContentLayered *mcoc=new DJMCOContentLayered(color);
	
	//partie normale
	mcoc->addLayer(new Anim(*m_anim_bible_paper), false);
	mcoc->addLayer(new Anim(*m_anim_bible), true);
	
	//partie explosion
	mcoc->addLayerExplosion(new Anim(*m_anim_bible_paper), false);
	//mcoc->addLayerExplosion(new Anim(*m_anim_bible_sheet), false);
	mcoc->addLayerExplosion(new Anim(*m_anim_bible), true);
	
	return mcoc;
}

DJMCOContentLayered *LevelCathedral::newMCOContentCandle(OSL_COLOR color)
{
	DJMCOContentLayered *mcoc=new DJMCOContentLayered(color);
	
	//partie normale
	mcoc->addLayer(new Anim(*m_anim_candle), true);
	Anim *anim_flame=new Anim(*m_anim_candle_flame);
	anim_flame->start();
	mcoc->addLayer(anim_flame, false);
	
	//partie explosion
	mcoc->addLayerExplosion(new Anim(*m_anim_candle), true);
	mcoc->addLayerExplosion(new Anim(*m_anim_candle_flame_explode), false);
		
	return mcoc;
}

DJMCOContentSimple *LevelCathedral::newMCOContentChalice(OSL_COLOR color)
{
	return new DJMCOContentSimple(new Anim(*m_anim_chalice), color);
}

DJMCOContentSimple *LevelCathedral::newMCOContentCross(OSL_COLOR color)
{
	return new DJMCOContentSimple(new Anim(*m_anim_cross), color);
}

DJMCOContentSimple *LevelCathedral::newMCOContentIctus(OSL_COLOR color)
{
	return new DJMCOContentSimple(new Anim(*m_anim_ictus), color);
}

void LevelCathedral::initWavesViolin(Properties *props)
{
	m_fxmgr_wave_odd=new LocalEffectManager();
	m_fxmgr_wave_even=new LocalEffectManager();
	
	int size=props->getIntValue("fx.techno.violin.wave.size");
//	int timeraise=props->getIntValue("fx.techno.violin.wave.time.raise");
//	int timewaitup=props->getIntValue("fx.techno.violin.wave.time.waitup");
//	int timefall=props->getIntValue("fx.techno.violin.wave.time.fall");
	int timenextline=props->getIntValue("fx.techno.violin.wave.time.nextline");
	int timeshake=props->getIntValue("fx.techno.violin.wave.time.shake");
	int nbshakes=props->getIntValue("fx.techno.violin.wave.shakes.nb");
	
	int wait_first=0;
	for(int i=0;i<m_nbCols;i++)
	{
		SerialLocalEffect *fx=new SerialLocalEffect(1);
		
		if(wait_first!=0)
		{
			fx->addEffect(new WaitLocalEffect(wait_first));
		}
		wait_first+=timenextline;
		
		fx->addEffect(new ShakeEffect(size, 0, timeshake, true, nbshakes));
		
		m_fxmgr_wave_odd->addEffect(fx);
		m_fxmgr_wave_even->addEffect(fx->clone());
	}
	
	m_repeater_wave=new Repeater(props->getIntValue("fx.techno.violin.waves.interval")/*, int times=-1*/);
	m_waves_is_odd=true;
}

//void LevelCathedral::initWavesViolin(Properties *props)
//{
//	m_fxmgr_wave_odd=new LocalEffectManager();
//	m_fxmgr_wave_even=new LocalEffectManager();
//	
//	int size=props->getIntValue("fx.techno.violin.wave.size");
//	int timeraise=props->getIntValue("fx.techno.violin.wave.time.raise");
//	int timewaitup=props->getIntValue("fx.techno.violin.wave.time.waitup");
//	int timefall=props->getIntValue("fx.techno.violin.wave.time.fall");
//	int timenextcol=props->getIntValue("fx.techno.violin.wave.time.nextcolumn");
//	
//	int wait_first=0;
//	for(int i=0;i<m_nbCols;i++)
//	{
//		SerialLocalEffect *fx=new SerialLocalEffect(1);
//		
//		if(wait_first!=0)
//		{
//			fx->addEffect(new WaitLocalEffect(wait_first));
//		}
//		wait_first+=timenextcol;
//		
//		fx->addEffect(new TranslationEffect(0, 0, 0, -size, timeraise, timeraise, true, 1));
//		if(timewaitup!=0)
//		{
//			WaitLocalEffect *wle=new WaitLocalEffect(timewaitup);
//			wle->setPosition(0, -size);
//			fx->addEffect(wle);
//		}
//		fx->addEffect(new TranslationEffect(0, -size, 0, 0, timefall, timefall, true, 1));
//		
//		m_fxmgr_wave_odd->addEffect(fx);
//		m_fxmgr_wave_even->addEffect(fx->clone());
//	}
//	
//	m_repeater_wave=new Repeater(props->getIntValue("fx.techno.violin.waves.interval")/*, int times=-1*/);
//	m_waves_is_odd=true;
//}

LevelCathedral::~LevelCathedral()
{
	oslDeleteImage(m_gridImage);
	
	delete m_linepattern;
	delete m_songbanner_techno;
	delete m_songbanner_boss;
	delete m_songbanner_shelter;
	delete m_comboDrawer;
	delete m_scenario_techno;
	delete m_scenario_boss;
	delete m_scenario_shelter;
	
	oslDeleteSound(m_music_techno);
	oslDeleteSound(m_music_boss);
	oslDeleteSound(m_music_shelter);
	
	delete m_anim_bell;
	delete m_anim_bible;
	delete m_anim_candle;
	delete m_anim_chalice;
	delete m_anim_cross;
	delete m_anim_ictus;
	delete m_anim_bible_paper;
	delete m_anim_candle_flame;
	delete m_anim_candle_flame_explode;
	//delete m_anim_bible_sheet;
	
	delete m_mcoc_bell;
	delete m_mcoc_bible;
	delete m_mcoc_candle;
	delete m_mcoc_chalice;
	delete m_mcoc_cross;
	delete m_mcoc_ictus;
	
	for(int i=0;i<5;i++)
	{
		oslDeleteImage(m_img_rays[i]);
	}
	delete[] m_img_rays;
	
	delete m_meteor;
	
	m_fxmgr_wave_odd->deleteAll();
	m_fxmgr_wave_even->deleteAll();
	delete m_fxmgr_wave_odd;
	delete m_fxmgr_wave_even;
	delete m_repeater_wave;
	
	delete m_fx_shake_bells;
	
	for(int i=0;i<getNbCols();i++)
	{
		delete m_fx_size_organ[i];
	}
	delete[] m_fx_size_organ;
	delete m_repeater_organ;
}



OSL_COLOR LevelCathedral::getOslColor(int color_num)
{
	switch(color_num)
	{
		case PIECE_COLOR_BLUE:
		{
			return m_color_blue;
			break;
		}
		case PIECE_COLOR_GREEN:
		{
			return m_color_green;
			break;
		}
		case PIECE_COLOR_ORANGE:
		{
			return m_color_orange;
			break;
		}
		case PIECE_COLOR_PURPLE:
		{
			return m_color_purple;
			break;
		}
		case PIECE_COLOR_RED:
		{
			return m_color_red;
			break;
		}
		case PIECE_COLOR_YELLOW:
		{
			return m_color_yellow;
			break;
		}
	}
	
	oslFatalError("LevelCathedral::getOslColor : color undefined");
	return 0;
}

AbstractMCOContent *LevelCathedral::mcoContentFromShapeAndColor(int shape, int color)
{
	switch(shape)
	{
		case PIECE_SHAPE_CIRCLE:
		{
			return newMCOContentCandle(getOslColor(color));
			break;	
		}
		case PIECE_SHAPE_CROSS:
		{
			return newMCOContentCross(getOslColor(color));
			break;	
		}
		case PIECE_SHAPE_DIAMOND:
		{
			return newMCOContentBible(getOslColor(color));
			break;	
		}
		case PIECE_SHAPE_SQUARE:
		{
			return newMCOContentChalice(getOslColor(color));
			break;	
		}
		case PIECE_SHAPE_STAR:
		{
			return newMCOContentBell(getOslColor(color));
			break;	
		}
		case PIECE_SHAPE_TRIANGLE:
		{
			return newMCOContentIctus(getOslColor(color));
			break;	
		}
	}
	
	oslFatalError("LevelCathedral::mcoContentFromShapeAndColor : shape undefined");
	return NULL;
}

void LevelCathedral::drawPieceXY(int x, int y, int shape, int color, AbstractLocalEffect *fx1)
{
	debug("drawpiecexy");
	AbstractMCOContent *mcoc=NULL;
	switch(shape)
	{
		case PIECE_SHAPE_CIRCLE:
		{
			m_mcoc_candle->setColor(getOslColor(color));
			mcoc=m_mcoc_candle;			
			break;	
		}
		case PIECE_SHAPE_CROSS:
		{
			m_mcoc_cross->setColor(getOslColor(color));
			mcoc=m_mcoc_cross;	
			break;	
		}
		case PIECE_SHAPE_DIAMOND:
		{
			m_mcoc_bible->setColor(getOslColor(color));
			mcoc=m_mcoc_bible;	
			break;	
		}
		case PIECE_SHAPE_SQUARE:
		{
			m_mcoc_chalice->setColor(getOslColor(color));
			mcoc=m_mcoc_chalice;	
			break;	
		}
		case PIECE_SHAPE_STAR:
		{
			m_mcoc_bell->setColor(getOslColor(color));
			mcoc=m_mcoc_bell;	
			break;	
		}
		case PIECE_SHAPE_TRIANGLE:
		{
			m_mcoc_ictus->setColor(getOslColor(color));
			mcoc=m_mcoc_ictus;	
			break;	
		}
	}
	
	if(mcoc==NULL)
		oslFatalError("LevelCathedral::drawPieceXY : mcoc est NULL");
		
	mcoc->setX(x);
	mcoc->setY(y);
	
	mcoc->draw(fx1);	
	debug("fin drawpiecexy");
}

void LevelCathedral::drawPairXY(Pair *pair, int x, int y, AbstractLocalEffect *fx1)
{
	debug("drawpairxy");
	int x2=x, y2=y;
	switch(pair->getOrientation())
	{
		case ORIENTATION_EAST:
			x2+=GAMEDEF_PIECE_WIDTH;
			break;	
		
		case ORIENTATION_NORTH:
			y2-=GAMEDEF_PIECE_HEIGHT;
			break;
			
		case ORIENTATION_SOUTH:
			y2+=GAMEDEF_PIECE_HEIGHT;
			break;
			
		case ORIENTATION_WEST:
			x2-=GAMEDEF_PIECE_WIDTH;
			break;
	}
	
	if(pair->getPiece(0)->getType()==PIECE_TYPE_NULL)
	{
		//rien
	}
	else if(pair->getPiece(0)->getType()==PIECE_TYPE_LEVEL)
	{
		//oslDrawFillRect(x-10, y-10, x+10, y+10, RGB(255, 0, 255));
	}
	else
		drawPieceXY(x, y, pair->getPiece(0)->getShape(), pair->getPiece(0)->getColor(), fx1);
		
	if(pair->getPiece(1)->getType()==PIECE_TYPE_NULL)
	{
		//rien
	}
	else if(pair->getPiece(1)->getType()==PIECE_TYPE_LEVEL)
	{
		//oslDrawFillRect(x-10, y-10, x+10, y+10, RGB(0, 255, 255));
	}
	else
		drawPieceXY(x2, y2, pair->getPiece(1)->getShape(), pair->getPiece(1)->getColor(), fx1);

	debug("fin drawpairxy");
}

//void LevelCathedral::drawNextPair(Pair *nextPair, GameGrid *grid, SerialLocalEffect *fx_trans)
//{
//	
//}

//dessin avec une paire
void LevelCathedral::draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	debug("draw avec une paire");
	drawBefore(grid, nextPair, cc);
	
	//currentPair->draw(grid, fx_trans, APPLY_FX_ON_NONE);
	Pos gridp=currentPair->getGridPos(0);
	Pos pixp=grid->getContentPosition(gridp.x, gridp.y);
	drawPairXY(currentPair, pixp.x, pixp.y, fx_trans);
	
	drawAfter(grid, nextPair, cc, fx_trans);
	debug("fin draw avec une paire");
}

//dessin avec 2 pieces
void LevelCathedral::draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	debug("draw avec 2 pieces");
	drawBefore(grid, nextPair, cc);
	Pos pixPosConnected=grid->getContentPosition(pieceConnectedPos.x, pieceConnectedPos.y);
	Pos pixPosUnconnected=grid->getContentPosition(pieceUnconnectedPos.x, pieceUnconnectedPos.y);
	
	//pieceConnected->drawXY(pixPosConnected->x, pixPosConnected->y);
	//pieceUnconnected->drawXY(pixPosUnconnected->x, pixPosUnconnected->y, fx_trans);
	if(pieceConnected->getType()==PIECE_TYPE_LEVEL)
	{
		//oslDrawFillRect(pixPosConnected->x-10, pixPosConnected->y-10, pixPosConnected->x+10, pixPosConnected->y+10, RGB(0, 255, 255));
	}
	else if(pieceConnected->getType()!=PIECE_TYPE_NULL)
	{
		drawPieceXY(pixPosConnected.x, pixPosConnected.y, pieceConnected->getShape(), pieceConnected->getColor());
	}
	if(pieceUnconnected->getType()==PIECE_TYPE_LEVEL)
	{
		//oslDrawFillRect(pixPosUnconnected->x-10, pixPosUnconnected->y-10, pixPosUnconnected->x+10, pixPosUnconnected->y+10, RGB(0, 255, 255));
	}
	else if(pieceUnconnected->getType()!=PIECE_TYPE_NULL)
	{
		drawPieceXY(pixPosUnconnected.x, pixPosUnconnected.y, pieceUnconnected->getShape(), pieceUnconnected->getColor(), fx_trans);
	}
	drawAfter(grid, nextPair, cc, fx_trans);
	debug("fin draw avec 2pieces");
}

//dessin avec rien
void LevelCathedral::draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer)
{
	debug("draw avec rien");
	drawBefore(grid, nextPair, cc);
	drawAfter(grid, nextPair, cc);
	debug("fin draw avec rien");
}

void LevelCathedral::drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc, AbstractLocalEffect *fx_trans)
{
	debug("drawBefore");
	oslDrawImage(m_gridImage);
	
//	for(int i=0;i<grid->getNbCols();i++)
//	{
//		for(int j=0;j<grid->getNbLines();j++)
//		{
//			MatrixContentObject *mco=grid->getAt(i,j);
//			if(mco->getPiece()!=NULL)
//			{
//				Pos *pos=grid->getContentPosition(i, j);
//				mco->getPiece()->drawXY(pos->x,pos->y);
//			}
//		}	
//	}


	if(isPaused())
	{
		//TODO: pause
	}
	else
	{
		debug("dessin contenu grille");
		//dessin du contenu de la grille
		//pieces qui explosent pas
		
		AbstractLocalEffect *fx1=NULL;
		AbstractLocalEffect *fx2=NULL;
		AbstractLocalEffect *fx3=m_fx_shake_bells->isRunning()?m_fx_shake_bells:NULL;
		
		for(int i=0;i<getNbCols();i++)
		{
			AbstractLocalEffect *fx4=m_fx_size_organ[i]->isRunning()?m_fx_size_organ[i]:NULL;
				
			for(int j=0;j<getNbLines();j++)
			{
				MatrixContentObject *mco=grid->getAt(i,j);
				if(mco==NULL)
					debug("drawBefore : mco est null");
				
				if(m_repeater_wave->isRunning()) //on est en phase violons!
				{
					fx1=m_fxmgr_wave_odd->getAt(11-j);
					fx2=m_fxmgr_wave_even->getAt(j);
				}
				
				AbstractPiece *p=mco->getPiece();
				if(p!=NULL && !p->destructing() && !p->destructed())//pieces qui explosent pas
				{
					//Pos *pos=grid->getContentPosition(i, j);
					if(p->getType()==PIECE_TYPE_NORMAL)
					{
						if(mco->getMCOContent()!=NULL)
						{
							mco->getMCOContent()->draw(fx1, fx2, fx3, fx4);
						}
					} //TODO: pieces combo speciales mcocontent
					else if(p->getType()==PIECE_TYPE_COMBO)
					{
						Pos pos=grid->getContentPosition(i, j);
						p->drawXY(pos.x, pos.y, fx1, fx2, fx3, fx4);
					}
					else if(p->getType()==PIECE_TYPE_LEVEL)
					{
						//Pos *pos=grid->getContentPosition(i, j);
						//oslDrawFillRect(pos->x-10, pos->y-10, pos->x+10, pos->y+10, RGB(255, 0, 255));
					}
				}
			}	
		}
		
		//pieces qui explosent
		for(int i=0;i<getNbCols();i++)
		{
			AbstractLocalEffect *fx4=m_fx_size_organ[i]->isRunning()?m_fx_size_organ[i]:NULL;
			
			for(int j=0;j<getNbLines();j++)
			{
				MatrixContentObject *mco=grid->getAt(i,j);
				if(mco==NULL)
					debug("drawBefore : mco est null");
				
				if(m_repeater_wave->isRunning()) //on est en phase violons!
				{
					fx1=m_fxmgr_wave_odd->getAt(11-j);
					fx2=m_fxmgr_wave_even->getAt(j);
				}
				
				AbstractPiece *p=mco->getPiece();
				if(p!=NULL) //deja dessinees avant
				{
					//Pos *pos=grid->getContentPosition(i, j);
					if(p->destructing() || p->destructed())//pieces qui explosent
					{
						if(mco->getMCOContent()!=NULL)
						{
							mco->getMCOContent()->draw(fx1, fx2, fx3, fx4);
						}
						else if(p->getType()==PIECE_TYPE_LEVEL)
						{
							//Pos *pos=grid->getContentPosition(i, j);
							//oslDrawFillRect(pos->x-10, pos->y-10, pos->x+10, pos->y+10, RGB(255, 0, 255));
						}
						else
						{	
							Pos pos=grid->getContentPosition(i, j);
							p->drawXY(pos.x,pos.y, fx1, fx2, fx3, fx4);
						}
					}
				}
			}	
		}
	}
	debug("fin drawBefore");
}

void LevelCathedral::drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc, AbstractLocalEffect *fx_trans)
{
	debug("drawAfter");
	//nextPair->drawXY(m_pos_nextPair.x, m_pos_nextPair.y, grid, APPLY_FX_ON_NONE);
	drawPairXY(nextPair, m_pos_nextPair.x, m_pos_nextPair.y);
	
	//drawMeteor(fx_trans);
	m_meteor->draw(m_meteor_fxtrans?fx_trans:NULL);
	
	//oslDrawFillRect(400, 270-int((cc->getTime()*260)/cc->getTimeTotalAvailable()), 415, 270, RGB(255,0,0));
	
	//jauge
	int one5th=int(float(cc->getTimeTotalAvailable())/5.0f);
	if(cc->getTime()>0)
	{
		oslDrawImage(m_img_rays[4]);
		for(int i=1;i<=4;i++)
		{
			if(cc->getTime()>i*one5th)
				oslDrawImage(m_img_rays[4-i]);
		}
	}
	
	
	if(m_part==LEVEL_CATHEDRAL_PART_SHELTER)
	{
		m_songbanner_shelter->draw();
	}
	if(m_part==LEVEL_CATHEDRAL_PART_TECHNO)
	{
		m_songbanner_techno->draw();
	}
	else
	{
		m_songbanner_boss->draw();
	}
	m_comboDrawer->drawXY(240, 180);
	debug("fin drawAfter");
}

//void LevelCathedral::drawMeteor(AbstractLocalEffect *fx_trans)
//{
//	//m_meteor->draw(m_meteor_fxtrans?fx_trans:NULL);
//	m_meteor->draw();
//}

void LevelCathedral::start_specific()
{
	oslPlaySound(m_music_shelter, 0);
	m_scenario_shelter->restart();
}

void LevelCathedral::explodeMeteor(int gridX, int gridY, GameGrid *grid)
{
	MatrixContentObject *mco;//=grid->getAt(gridX,gridY);
	//mco->startExplosion(PIECE_MODE_SPECIAL);
	
	for(int i=gridX-1;i<=gridX+1;i++)
	{
		for(int j=gridY-1;j<=gridY+1;j++)
		{
			if(i>=0 && i<m_nbCols && j>=0 && j<m_nbLines)
			{
				mco=grid->getAt(i, j);
				AbstractPiece *piece=mco->getPiece();
				if(piece!=NULL)
				{
					mco->startExplosion(PIECE_MODE_SPECIAL);
					m_nb_specialexpl++;
					//mco->update();
				}
			}			
		}
	}
	
	m_meteor->start();
}

void LevelCathedral::startOrganNote()
{
	int r=utils::random(0,getNbCols()-1);
	m_fx_size_organ[r]->restart();
}

int LevelCathedral::getNbSpecialExplosions()
{
	if(m_nb_specialexpl==0)
		return 0;
	
	//else
	
	int tmp=m_nb_specialexpl;
	m_nb_specialexpl=0;
	return tmp;
}

void LevelCathedral::update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
{
	debug("update");
	//update objets fixes
	m_mcoc_bell->update();
	m_mcoc_bible->update();
	m_mcoc_candle->update();
	m_mcoc_chalice->update();
	m_mcoc_cross->update();
	m_mcoc_ictus->update();
	
	m_fx_shake_bells->update();
	
	//on met le meteor à sa place
	//un reset au cas ou il n'a pas a etre affiché
	//m_meteor->setX(-50);
	//m_meteor->setY(-50);
	if(m_meteor->finished())
	{
//		//m_meteor->reset();
		m_meteor->setX(-50);
		m_meteor->setY(-50);
		m_meteor_fxtrans=false;
	}
	
	if(nextPair!=NULL && nextPair->getPiece(0)!=NULL && nextPair->getPiece(0)->getType()==PIECE_TYPE_LEVEL)
	{
		//Pos p=nextPair->getGridPos(0);
		//Pos *gp=grid->getContentPosition(p.x, p.y);
		m_meteor->setX(m_pos_nextPair.x+int(GAMEDEF_PIECE_WIDTH/2.0f));
		m_meteor->setY(m_pos_nextPair.y);
		m_meteor_fxtrans=false;
	}
	else if(currentPair!=NULL && currentPair->getPiece(0)!=NULL && currentPair->getPiece(0)->getType()==PIECE_TYPE_LEVEL)
	{
		Pos p=currentPair->getGridPos(0);
		Pos gp=grid->getContentPosition(p.x, p.y);
		m_meteor->setX(gp.x);
		m_meteor->setY(gp.y);
		m_meteor_fxtrans=true;
	}
	
	
	//vagues du violon
	m_fxmgr_wave_odd->update();
	m_fxmgr_wave_even->update();
	m_repeater_wave->update();
	if(m_repeater_wave->isEvent())
	{
		if(m_waves_is_odd)
			m_fxmgr_wave_odd->restart();
		else
			m_fxmgr_wave_even->restart();
		m_waves_is_odd=!m_waves_is_odd;
	}
	
	//orgue
	m_repeater_organ->update();
	for(int i=0;i<getNbCols();i++)
	{
		m_fx_size_organ[i]->update();
	}
	if(m_repeater_organ->isEvent())
	{
		startOrganNote();
	}
	
	if(m_part==LEVEL_CATHEDRAL_PART_SHELTER)
	{
		m_songbanner_shelter->update();
		m_scenario_shelter->update();
	}
	else if(m_part==LEVEL_CATHEDRAL_PART_TECHNO)
	{
		m_songbanner_techno->update();
		m_scenario_techno->update();
	}
	else
	{
		m_songbanner_boss->update();
		m_scenario_boss->update();
	}
	m_comboDrawer->sync(cc);
	m_comboDrawer->update();
	
	if(m_part==LEVEL_CATHEDRAL_PART_SHELTER && m_scenario_shelter->getTime()>=LEVEL_CATHEDRAL_DURATION_SHELTER-m_overlap_sheltertechno)
	{
		m_part=LEVEL_CATHEDRAL_PART_TECHNO;
		oslPlaySound(m_music_techno, 1);
		m_scenario_techno->restart();
		m_upspeed=m_upspeed_techno_normal;
		m_dropspeed=m_dropspeed_techno_normal;
	}
	else if(m_part==LEVEL_CATHEDRAL_PART_TECHNO && m_scenario_techno->getTime()>=LEVEL_CATHEDRAL_DURATION_TECHNO-m_overlap_sheltertechno-m_overlap_technoboss)
	{
		m_part=LEVEL_CATHEDRAL_PART_BOSS;
		oslPlaySound(m_music_boss, 0);
		m_scenario_boss->restart();
		m_upspeed=m_upspeed_boss_normal;
		m_dropspeed=m_dropspeed_boss_normal;
	}
	
	//evenements du scenario
	if(hasEvent(1)) //debut orgue
	{	
		m_upspeed=m_upspeed_techno_fast;
		startOrganNote();
		m_repeater_organ->restart();
	}
	if(hasEvent(2)) //fin orgue
	{	
		m_upspeed=m_upspeed_techno_normal;
		m_repeater_organ->stop();
		m_repeater_organ->reset();
	}
	if(hasEvent(3)) //Debut Violon = effet de deplacement de lignes avec menu de va et vient si possible comme sur violon
	{	
		m_repeater_wave->restart();
		if(m_waves_is_odd)
			m_fxmgr_wave_odd->restart();
		else
			m_fxmgr_wave_even->restart();
		m_waves_is_odd=!m_waves_is_odd;
	}
	if(hasEvent(4)) //Fin Violon = fin lignes mouvantes
	{	
		m_repeater_wave->stop();
		m_fxmgr_wave_odd->stop();
		m_fxmgr_wave_even->stop();
		m_fxmgr_wave_odd->reset();
		m_fxmgr_wave_even->reset();
	}
	if(hasEvent(12))  //orgue
	{	
		m_addline+=2;
	}
	if(hasEvent(5)) //Techno: Debut Drum acceleration vitesse de chute
	{	
		m_dropspeed=m_dropspeed_techno_fast;
	}
	if(hasEvent(52)) //Techno: Debut Drum acceleration vitesse de chute
	{	
		m_dropspeed=m_dropspeed_techno_fast;
		m_addline+=2;
	}
	if(hasEvent(6)) //Techno: Fin Drum = retour a la vitesse de base
	{	
		m_dropspeed=m_dropspeed_techno_normal;
	}
	if(hasEvent(7)) //Techno: Cloches d'ajout de depart sur 5 s = ajout de 5 lignes environ pr faire apparaitre un pattern avec ailes par ex 
	{	
		m_addline++;
		m_fx_shake_bells->restart();
	}
	
	
	//cas special comme pour la pdrom: on utilise des anims séparées
	for(int i=0;i<grid->getNbCols();i++)
	{
		for(int j=0;j<grid->getNbLines();j++)
		{
			MatrixContentObject *mco=grid->getAt(i,j);
			AbstractPiece *p=mco->getPiece();
			if(p!=NULL)
			{
				if(p->getType()==PIECE_TYPE_NORMAL)
				{
					//on cree si une piece n'en a pas
					if(mco->getMCOContent()==NULL)
					{
						mco->setMCOContent(mcoContentFromShapeAndColor(p->getShape(), p->getColor()));
					}
						
				}
				else if(p->getType()==PIECE_TYPE_LEVEL)
				{
					Pos gp=grid->getContentPosition(i, j);
					m_meteor->setX(gp.x);
					m_meteor->setY(gp.y);
					m_meteor_fxtrans=false;
					
					//p->startDestruct();
					//setSpecialExploding();
					
					//mco->startExplosion(PIECE_MODE_SPECIAL);
					if(!mco->isSpecialExploding())
					{
						explodeMeteor(i, j, grid);
						//mco->update(); //TODO: chercher pourquoi une pieceLevel met + de temps qu'une autre (anim vs repeater)
					}
				}
				/*else if(p->getType()==PIECE_TYPE_COMBO)
				{
					//remplir si c'est une piece combo
					if(mco->getMCOContent()==NULL)
					{
						int nbCombos=p->specialEvent(PIECE_EVENT_COMBO_GET);
						Anim *shine=new Anim(*m_animComboChange);
						Anim *explode=new Anim(*m_animComboExplode);
						mco->setMCOContent(new MCOContentCombo(explode, shine, m_imgCombo, nbCombos));
					}
					
					//on met a jour le nombre de combos
					int nbCombosPiece=p->specialEvent(PIECE_EVENT_COMBO_GET);
					int nbCombosMCOContent=mco->getMCOContent()->specialEvent(MCOCONTENT_EVENT_COMBO_GETNB);
					if(nbCombosPiece!=nbCombosMCOContent)
					{
						mco->getMCOContent()->specialEvent(MCOCONTENT_EVENT_COMBO_DECREMENT);
					}
				}*/
				
				//on démarre si la piece explose
				if(mco->isColorExploding() || mco->isShapeExploding() || mco->isSpecialExploding())
				{
					if(mco->getMCOContent()!=NULL)
						mco->getMCOContent()->start();
				}
			}
			else//mco->getPiece()==NULL
			{
				if(mco->getMCOContent()!=NULL)
				{
					mco->deleteMCOContent();
				}
			}
			
			if(mco->getMCOContent()!=NULL) //peu importe le cas, on update
			{
				Pos pos=grid->getContentPosition(i, j);
				mco->getMCOContent()->setX(pos.x);
				mco->getMCOContent()->setY(pos.y);		
				mco->getMCOContent()->update();
			}
		}
	}
	
	//State *state=new State();
	//m_meteor->savestate(state);
	///fx_trans->applyOn(m_meteor);
	
	m_meteor->update(m_meteor_fxtrans?fx_trans:NULL);
	
	
	//m_meteor->restorestate(state);
	//delete state;
	
	debug("fin update");
}

bool LevelCathedral::hasEvent(int event)
{
	if(m_scenario_shelter->hasEvent(event))
		return true;
	
	if(m_scenario_techno->hasEvent(event))
		return true;
	
	if(m_scenario_boss->hasEvent(event))
		return true;
	
	return false;
}

void LevelCathedral::pause_specific() 
{
//	if(m_part==LEVEL_CATHEDRAL_PART_SHELTER)
//	{
//		oslPauseSound(m_music_shelter, 1);
//		m_scenario_shelter->pause();
//	}
//	else if(m_part==LEVEL_CATHEDRAL_PART_TECHNO)
//	{
//		oslPauseSound(m_music_techno, 1);
//		oslPauseSound(m_music_shelter, 1);
//		m_scenario_techno->pause();
//		m_scenario_shelter->pause();
//	}
//	else
//	{
//		oslPauseSound(m_music_boss, 1);
//		oslPauseSound(m_music_techno, 1);
//		m_scenario_boss->pause();
//		m_scenario_techno->pause();
//	}

	oslPauseSound(m_music_shelter, 1);
	oslPauseSound(m_music_techno, 1);
	oslPauseSound(m_music_boss, 1);
	m_scenario_shelter->pause();
	m_scenario_techno->pause();
	m_scenario_boss->pause();
	
}

void LevelCathedral::resume_specific()
{
//	if(m_part==LEVEL_CATHEDRAL_PART_SHELTER)
//	{
		oslPauseSound(m_music_shelter, 0);
		m_scenario_shelter->start();
//	}
//	else if(m_part==LEVEL_CATHEDRAL_PART_TECHNO)
//	{
		oslPauseSound(m_music_techno, 0);
		m_scenario_techno->start();
//	}
//	else
//	{
		oslPauseSound(m_music_boss, 0);
		m_scenario_boss->start();
//	}
}
		

list<AbstractPiece *> *LevelCathedral::getNextLine()
{
	return m_linepattern->getNextLine();
}

bool LevelCathedral::getAddLineEvent() //dit quand une ligne doit monter, dû au scénario
{
	if(m_addline>0)
	{
		m_addline--;
		return true;
	}
	return false;
}

int LevelCathedral::getDropSpeed() //la vitesse a un instant t (elle peut etre modifiée par le scénario)
{
	return m_dropspeed;
}

int LevelCathedral::getUpSpeed() //la vitesse ou les lignes montent a un instant t
{
	return m_upspeed;
}

void LevelCathedral::buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid)
{
	if(key==OSL_KEY_TRIANGLE)
	{
		m_meteor_event=true;
	}
	if(key==OSL_KEY_SQUARE)
	{
		m_part=LEVEL_CATHEDRAL_PART_TECHNO;
		oslPlaySound(m_music_techno, 1);
		oslStopSound(m_music_shelter);
		oslStopSound(m_music_boss);
		m_scenario_techno->restart();
		m_scenario_shelter->pause();
		m_scenario_boss->pause();
		m_upspeed=m_upspeed_techno_normal;
		m_dropspeed=m_dropspeed_techno_normal;
	}
}

bool LevelCathedral::finished()
{
	if(m_part==LEVEL_CATHEDRAL_PART_BOSS && m_scenario_boss->getTime()>=LEVEL_CATHEDRAL_DURATION_BOSS)
	{
		return true;
	}
	return false;
}

float LevelCathedral::getDuration()
{
	return LEVEL_CATHEDRAL_DURATION_TECHNO+LEVEL_CATHEDRAL_DURATION_BOSS+LEVEL_CATHEDRAL_DURATION_SHELTER;
}

Pair *LevelCathedral::getNewPair()
{
	if(!m_meteor_event)
	{
		return AbstractLevel::getNewPair();
	}
	//else
	
	m_meteor_event=false;
	Pair *p=new Pair(new PieceLevel(), new PieceNull());
	
	int posX=int(m_nbCols/2)-1;
	int posY=-1; //au dessus de tout en haut
	p->goTo(posX, posY);
	
	m_meteor->reset();
	
	return p;
}























