#ifndef LEVELCATHEDRAL_H_
#define LEVELCATHEDRAL_H_

#include <oslib/oslib.h>
#include "code/game/levels/abstract/AbstractLevel.h"
#include "code/game/grid/GameGrid.h"
#include "code/game/Pair.h"
#include "code/game/levels/LinePattern.h"
#include "code/game/pieces/abstract/AbstractPiece.h"
#include <list>
#include "code/game/SongBanner.h"
#include "code/game/combos/ComboCounter.h"
#include "code/game/combos/ComboNumberDrawer.h"
#include "code/game/EventsScenario.h"
#include "code/framework/fx/EffectsLib.h"
#include "code/framework/Pos.h"
#include "code/framework/anim/Anim.h"
#include "code/game/grid/AbstractMCOContent.h"
#include "code/game/levels/utils/DJMCOContentSimple.h"
#include "code/game/levels/utils/DJMCOContentLayered.h"
#include "code/game/levels/impl/cathedral/Meteor.h"
#include "code/framework/Properties.h"
#include "code/framework/time/Repeater.h"

using namespace std;

class LevelCathedral : public AbstractLevel
{
	private:
		OSL_IMAGE *m_gridImage;
		LinePattern *m_linepattern;
		SongBanner *m_songbanner_shelter;
		SongBanner *m_songbanner_techno;
		SongBanner *m_songbanner_boss;
		ComboNumberDrawer * m_comboDrawer;
		EventsScenario *m_scenario_shelter;
		EventsScenario *m_scenario_techno;
		EventsScenario *m_scenario_boss;
		OSL_SOUND *m_music_shelter;
		OSL_SOUND *m_music_techno;
		OSL_SOUND *m_music_boss;
		int m_overlap_sheltertechno;
		int m_overlap_technoboss;
		
		int m_dropspeed;
		int m_dropspeed_shelter_normal;
		int m_dropspeed_shelter_fast;
		int m_dropspeed_techno_normal;
		int m_dropspeed_techno_fast;
		int m_dropspeed_boss_normal;
		int m_dropspeed_boss_fast;
		
		int m_upspeed;
		int m_upspeed_shelter_normal;
		int m_upspeed_shelter_fast;
		int m_upspeed_techno_normal;
		int m_upspeed_techno_fast;
		int m_upspeed_boss_normal;
		int m_upspeed_boss_fast;
		
		//int m_difficulty; //dans le parent
				
		int m_part;
		
		int m_addline;
		
		Pos m_pos_nextPair;
		
		//les animations séparées
		Anim *m_anim_bell;
		Anim *m_anim_bible;
		Anim *m_anim_bible_paper;
		//Anim *m_anim_bible_sheet;
		Anim *m_anim_candle;
		Anim *m_anim_candle_flame;
		Anim *m_anim_candle_flame_explode;
		Anim *m_anim_chalice;
		Anim *m_anim_cross;
		Anim *m_anim_ictus;
		
		
		
		OSL_COLOR m_color_blue;
		OSL_COLOR m_color_red;
		OSL_COLOR m_color_green;
		OSL_COLOR m_color_yellow;
		OSL_COLOR m_color_orange;
		OSL_COLOR m_color_purple;
		
		//pour pouvoir dessiner les pièces pas dans la grille (next pair, current pair, piece connected et unconnected
		//un de chaque forme
		DJMCOContentSimple *m_mcoc_bell;
		DJMCOContentSimple *newMCOContentBell(OSL_COLOR color=0);
		DJMCOContentLayered *m_mcoc_bible;
		DJMCOContentLayered *newMCOContentBible(OSL_COLOR color=0);
		DJMCOContentLayered *m_mcoc_candle;
		DJMCOContentLayered *newMCOContentCandle(OSL_COLOR color=0);
		DJMCOContentSimple *m_mcoc_chalice;
		DJMCOContentSimple *newMCOContentChalice(OSL_COLOR color=0);
		DJMCOContentSimple *m_mcoc_cross;
		DJMCOContentSimple *newMCOContentCross(OSL_COLOR color=0);
		DJMCOContentSimple *m_mcoc_ictus;
		DJMCOContentSimple *newMCOContentIctus(OSL_COLOR color=0);
		
		//vagues violon techno
		LocalEffectManager *m_fxmgr_wave_odd;
		LocalEffectManager *m_fxmgr_wave_even;
		Repeater *m_repeater_wave;
		bool m_waves_is_odd;
		void initWavesViolin(Properties *props);
		
		//shake des cloches
		ShakeEffect *m_fx_shake_bells;
		
		//grossissement pendant l'orgue
		SizeWowEffect **m_fx_size_organ; //tableau
		Repeater *m_repeater_organ;
		void startOrganNote();
		
		//rayons
		OSL_IMAGE **m_img_rays;
		
		void drawPieceXY(int x, int y, int shape, int color, AbstractLocalEffect *fx1=NULL);
		void drawPairXY(Pair *pair, int x, int y, AbstractLocalEffect *fx1=NULL);
		//void drawNextPair(Pair *nextPair, GameGrid *grid, SerialLocalEffect *fx_trans);
		
		AbstractMCOContent *mcoContentFromShapeAndColor(int shape, int color);
		OSL_COLOR getOslColor(int color_num);		
		
		void drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc, AbstractLocalEffect *fx_trans=NULL);
		void drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc, AbstractLocalEffect *fx_trans=NULL);
	
		void start_specific(); //démarre la musique, le timer etc
		void update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans);
		void pause_specific(); 
		void resume_specific();
		
		bool hasEvent(int event);
		
		
		Meteor *m_meteor;
		Meteor *constructMeteor(Properties *props);
		bool m_meteor_fxtrans;
		bool m_meteor_event; //
		void explodeMeteor(int gridX, int gridY, GameGrid *grid);
		//void drawMeteor(AbstractLocalEffect *fx_trans);
		int m_nb_specialexpl;
	
	public:
		LevelCathedral(int difficulty=GAMEDEF_DIFFICULTY_NORMAL, int nbPlayers=1);
		~LevelCathedral();
		
		//hérité:
		int getDropSpeed(); //la vitesse a un instant t (elle peut etre modifiée par le scénario)
		int getUpSpeed(); //la vitesse ou les lignes montent a un instant t
		void draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer);
		string getCode();
		
		void buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid);
		
		bool getAddLineEvent(); //dit quand une ligne doit monter, dû au scénario
		bool finished();
		list<AbstractPiece *> *getNextLine();
		float getDuration();
		
		//surchargé pour avoir des meteores de temps en temps
		Pair *getNewPair();
		
		int getNbSpecialExplosions();
};

#endif /*LEVELCATHEDRAL_H_*/
