#include "code/game/levels/impl/chiptunes/LevelChiptunes.h"
#include "code/framework/check/loaders.h"
#include "code/framework/Properties.h"
#include <math.h>
#include "code/framework/utils.h"
#include "code/framework/particles/complete/animFactories/RandomAnimationFactory.h"
#include "code/framework/particles/complete/fxFactories/PFXFRandomPosition.h"

using namespace utils;

/*
 * easy: machine1
 * normal: machine1 + sunday
 * hard: wolves + machine1
 * 
 * easy: 2 colonnes vides a gauche
 * normal: 2 colonnes vides a droite
 * hard: 2 colonnes vides au milieu
 * 
 */

#define LEVEL_CHIPTUNES_DURATION_WOLVES 167900
#define LEVEL_CHIPTUNES_DURATION_MACHINE 184400
#define LEVEL_CHIPTUNES_DURATION_SUNDAY 201000
#define LEVEL_CHIPTUNES_PART_WOLVES 0
#define LEVEL_CHIPTUNES_PART_MACHINE 1
#define LEVEL_CHIPTUNES_PART_SUNDAY 2


string LevelChiptunes::getCode()
{
	return "chiptunes";
}

LevelChiptunes::LevelChiptunes(int difficulty, int nbPlayers)
:AbstractLevel(0, -1, difficulty, nbPlayers)
{
	Properties *props=new Properties("res/levels/chiptunes/chiptunes.properties");
	
	m_nbCols=14;
	
	
	m_songbanner_wolves=new SongBanner(
		GAMEDEF_SONGBANNER_Y, "res/fonts/bmpFonts/russelsquare/14/", "normal", "italic", 
		props->getStrValue("songbanner.wolves.text1"), props->getStrValue("songbanner.wolves.text2", ""), 
		props->getStrValue("songbanner.wolves.text3", ""), props->getStrValue("songbanner.wolves.text4", "")
	);
	m_songbanner_machine=new SongBanner(
		GAMEDEF_SONGBANNER_Y, "res/fonts/bmpFonts/russelsquare/14/", "normal", "italic", 
		props->getStrValue("songbanner.machine.text1"), props->getStrValue("songbanner.machine.text2", ""), 
		props->getStrValue("songbanner.machine.text3", ""), props->getStrValue("songbanner.machine.text4", "")
	);
	m_songbanner_sunday=new SongBanner(
		GAMEDEF_SONGBANNER_Y, "res/fonts/bmpFonts/russelsquare/14/", "normal", "italic", 
		props->getStrValue("songbanner.sunday.text1"), props->getStrValue("songbanner.sunday.text2", ""), 
		props->getStrValue("songbanner.sunday.text3", ""), props->getStrValue("songbanner.sunday.text4", "")
	);
	
	
	m_fx_tint_ul=new ProgressiveTintEffect(70, 60, RGB(0,0,0), RGB(0,0,0));
	m_fx_tint_dl=new ProgressiveTintEffect(40, 80, RGB(0,0,0), RGB(0,0,0));
	m_fx_tint_ur=new ProgressiveTintEffect(50, 50, RGB(0,0,0), RGB(0,0,0));
	m_fx_tint_dr=new ProgressiveTintEffect(70, 40, RGB(0,0,0), RGB(0,0,0));
	
	m_fx_tint_ul->start();
	m_fx_tint_dl->start();
	m_fx_tint_ur->start();
	m_fx_tint_dr->start();
	
	m_music_wolves=fwkLoadSoundFileMP3("res/levels/chiptunes/wolves.mp3",OSL_FMT_STREAM);
	m_music_machine=fwkLoadSoundFileMP3("res/levels/chiptunes/machine.mp3",OSL_FMT_STREAM);
	m_music_sunday=fwkLoadSoundFileMP3("res/levels/chiptunes/sunday.mp3",OSL_FMT_STREAM);
	
	//sunday est enregistrée -fort
	oslSetSoundVolume(m_music_sunday, props->getIntValue("music.sunday.volume"));
	
	m_scenario_wolves=new EventsScenario("res/levels/chiptunes/wolves.scenario");
	m_scenario_machine=new EventsScenario("res/levels/chiptunes/machine.scenario");
	m_scenario_sunday=new EventsScenario("res/levels/chiptunes/sunday.scenario");
	
	if(m_difficulty==GAMEDEF_DIFFICULTY_EASY)
	{
		m_part=LEVEL_CHIPTUNES_PART_MACHINE;
		m_addline=props->getIntValue("easy.nblines.start");
		m_linepattern=new LinePattern("res/levels/chiptunes/chiptunes_easy.pattern", 1);
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_NORMAL)
	{
		m_part=LEVEL_CHIPTUNES_PART_MACHINE;
		m_addline=props->getIntValue("normal.nblines.start");
		m_linepattern=new LinePattern("res/levels/chiptunes/chiptunes_normal.pattern", 1);
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_HARD)
	{
		m_part=LEVEL_CHIPTUNES_PART_WOLVES;
		m_addline=props->getIntValue("hard.nblines.start");
		m_linepattern=new LinePattern("res/levels/chiptunes/chiptunes_hard.pattern", 1);
	}
	else
	{
		fatalError("no such difficulty in level chiptunes");
	}
	
	m_img_pieces_big[0]=fwkLoadImageFilePNG("res/levels/chiptunes/pieces_big/square.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	m_img_pieces_big[1]=fwkLoadImageFilePNG("res/levels/chiptunes/pieces_big/star.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	m_img_pieces_big[2]=fwkLoadImageFilePNG("res/levels/chiptunes/pieces_big/triangle.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	m_img_pieces_big[3]=fwkLoadImageFilePNG("res/levels/chiptunes/pieces_big/circle.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	m_img_pieces_big[4]=fwkLoadImageFilePNG("res/levels/chiptunes/pieces_big/diamond.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	m_img_pieces_big[5]=fwkLoadImageFilePNG("res/levels/chiptunes/pieces_big/cross.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
		
	for(int i=0;i<6;i++)
	{
		oslImageSetRotCenter(m_img_pieces_big[i]);
	}
	
	m_fx_sizewow_images_big=new SizeWowEffect(
		props->getIntValue("images.big.fx.sizewow.shrinkpercent"), 
		props->getIntValue("images.big.fx.sizewow.growpercent"),
		props->getIntValue("images.big.fx.sizewow.shrinkpercent"), 
		props->getIntValue("images.big.fx.sizewow.growpercent"), 
		props->getIntValue("images.big.fx.sizewow.time"), 
		props->getIntValue("images.big.fx.sizewow.time"), 
		true, 1
	);
	
	int fx_tint_time=props->getIntValue("images.big.fx.tint.time");
	int fx_tint_color=props->getColorValue("images.big.fx.tint.color");
	m_fx_tint_image_big_right=new ProgressiveTintEffect(
		fx_tint_time, fx_tint_time, fx_tint_color, fx_tint_color
	);
	m_fx_tint_image_big_left=new ProgressiveTintEffect(
		fx_tint_time, fx_tint_time, fx_tint_color, fx_tint_color
	);
	m_fx_tint_image_big_right->start();
	m_fx_tint_image_big_left->start();
	
	m_upspeed=500;
	
	m_pos_images_big_right.x=props->getIntValue("images.big.pos.right.x");
	m_pos_images_big_right.y=props->getIntValue("images.big.pos.y");
	m_pos_images_big_left.x=props->getIntValue("images.big.pos.left.x");
	m_pos_images_big_left.y=props->getIntValue("images.big.pos.y");
	
	initParticleGeneratorsTurning(props);
	
	m_fx_gen_transl_right=new TranslationEffect(
		0, 0, 
		0, 0, 
		1, 1, true, 1);
		
	m_fx_gen_transl_left=new TranslationEffect(
		0, 0, 
		0, 0, 
		1, 1, true, 1);
	
		
	m_gen_moving=false;
	
	
	//parametres de translation
	m_translparams_left.x_min=props->getIntValue("images.big.fx.translation.xmin.left");
	m_translparams_left.x_max=props->getIntValue("images.big.fx.translation.xmax.left");
	m_translparams_left.y_min=props->getIntValue("images.big.fx.translation.ymin");
	m_translparams_left.y_max=props->getIntValue("images.big.fx.translation.ymax");
	m_translparams_left.speed_slow=props->getFloatValue("images.big.fx.translation.speed_slow");
	m_translparams_left.speed_fast=props->getFloatValue("images.big.fx.translation.speed_fast");

	m_translparams_right.x_min=props->getIntValue("images.big.fx.translation.xmin.right");
	m_translparams_right.x_max=props->getIntValue("images.big.fx.translation.xmax.right");
	m_translparams_right.y_min=m_translparams_left.y_min;
	m_translparams_right.y_max=m_translparams_left.y_max;
	m_translparams_right.speed_slow=m_translparams_left.speed_slow;
	m_translparams_right.speed_fast=m_translparams_left.speed_fast;
	
	initParticleGeneratorsNoise(props);
		
	m_music_speed_is_fast=false;
	
	m_fx_rotation_raggae=new RotationEffect(props->getIntValue("images.big.fx.turn.time"), false);
	
	//la jauge
	int gaugefxtime=props->getIntValue("gauge.fx.time");
	for(int i=0;i<12;i++)
	{
		m_fx_gauge_tint_left[i]=new ProgressiveTintEffect(gaugefxtime, gaugefxtime, 0, 0, 1);
		m_fx_gauge_tint_right[i]=new ProgressiveTintEffect(gaugefxtime, gaugefxtime, 0, 0, 1);
		m_fx_gauge_alpha[i]=new ProgressiveAlphaEffect(gaugefxtime, gaugefxtime, 255, 0, 1);
	}
	
	//l'effet de depart de wolves
	m_fx_wolves_alpha=new SerialGlobalEffect();
	for(int i=0;i<random(5,20);i++)
	{
		bool forward=random(0,1)==0;
		ProgressiveAlphaEffect *fx=new ProgressiveAlphaEffect(
			random(20,80), random(10,60), forward?random(200,255):random(0,120), forward?random(0,120):random(200,255), random(1,2)
		);
		m_fx_wolves_alpha->addEffect(fx);
	}
	
	m_img_hello=fwkLoadImageFilePNG("res/levels/chiptunes/hello.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	m_img_people=fwkLoadImageFilePNG("res/levels/chiptunes/people.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	m_repeater_txt=new Repeater(props->getIntValue("images.txt.time"), 1);
	
	delete props;
}

LevelChiptunes::~LevelChiptunes()
{
	delete m_songbanner_wolves;
	delete m_songbanner_machine;
	delete m_songbanner_sunday;
	
	delete m_fx_tint_ul;
	delete m_fx_tint_dl;
	delete m_fx_tint_ur;
	delete m_fx_tint_dr;
	
	fwkDeleteSound(m_music_wolves);
	fwkDeleteSound(m_music_machine);
	fwkDeleteSound(m_music_sunday);
	
	delete m_scenario_wolves;
	delete m_scenario_machine;
	delete m_scenario_sunday;
	
	delete m_linepattern;
	
	for(int i=0;i<6;i++)
	{
		oslDeleteImage(m_img_pieces_big[i]);
	}
	
	delete m_fx_sizewow_images_big;
	
	delete m_partgen_turning_right_1;
	delete m_partgen_turning_right_2;
	delete m_partgen_turning_left_1;
	delete m_partgen_turning_left_2;
	
	delete m_anim_particle_square;
	delete m_anim_particle_circle;
	delete m_anim_particle_triangle;
	delete m_anim_particle_diamond;
	delete m_anim_particle_star;
	delete m_anim_particle_cross;
	
	delete m_fx_partgen_turn;
	delete m_fx_partgen_size;
	
	delete m_fx_gen_transl_right;
	delete m_fx_gen_transl_left;
	
	delete m_partgen_noise_right;
	delete m_partgen_noise_left;
	
	delete m_fx_rotation_raggae;
	
	for(int i=0;i<12;i++)
	{
		delete m_fx_gauge_tint_left[i];
		delete m_fx_gauge_tint_right[i];
		delete m_fx_gauge_alpha[i];
	}
	
	delete m_fx_wolves_alpha;
	
	delete m_fx_tint_image_big_right;
	delete m_fx_tint_image_big_left;
	
	oslDeleteImage(m_img_hello);
	oslDeleteImage(m_img_people);
	delete m_repeater_txt;
}

void LevelChiptunes::reset()
{
	Properties *props=new Properties("res/levels/chiptunes/chiptunes.properties");
	
	m_scenario_wolves->pause();
	m_scenario_wolves->reset();
	m_scenario_machine->pause();
	m_scenario_machine->reset();
	m_scenario_sunday->pause();
	m_scenario_sunday->reset();
	
	m_linepattern->reset();
	
	if(m_difficulty==GAMEDEF_DIFFICULTY_EASY)
	{
		m_part=LEVEL_CHIPTUNES_PART_MACHINE;
		m_addline=props->getIntValue("easy.nblines.start");
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_NORMAL)
	{
		m_part=LEVEL_CHIPTUNES_PART_MACHINE;
		m_addline=props->getIntValue("normal.nblines.start");
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_HARD)
	{
		m_part=LEVEL_CHIPTUNES_PART_WOLVES;
		m_addline=props->getIntValue("hard.nblines.start");
	}
	
	//a partir de la rien n'est sur:
	m_partgen_turning_left_2->generationOff();
	m_partgen_turning_left_1->generationOff();
	m_partgen_turning_right_2->generationOff();
	m_partgen_turning_right_1->generationOff();
	
	m_gen_moving=false;
	m_music_speed_is_fast=false;
	
	changePartgenTurnParams(m_partgenturnparams_slow);
	
	m_fx_rotation_raggae->stop();
	
	m_fx_wolves_alpha->stop();
	
	m_songbanner_wolves->reset();
	m_songbanner_machine->reset();
	m_songbanner_sunday->reset();
	
	delete props;
}

void LevelChiptunes::initParticleGeneratorsTurning(Properties *props)
{
	m_partgenturnparams_slow.scale=props->getIntValue("partgen.turning.scale.slow");
	m_partgenturnparams_slow.throughput_particles=props->getIntValue("partgen.turning.throughput.particles.slow");
	m_partgenturnparams_slow.throughput_frames=props->getIntValue("partgen.turning.throughput.frames.slow");
	m_partgenturnparams_slow.transl_time=props->getIntValue("partgen.turning.translation.time.slow");
	m_partgenturnparams_slow.transl_dist=props->getIntValue("partgen.turning.translation.distance.slow");
	m_partgenturnparams_slow.lifespan=props->getIntValue("partgen.turning.lifespan.slow");
	m_partgenturnparams_slow.turn_speed=props->getIntValue("partgen.turning.fx.turn.revolutiontime.slow");
	
	m_partgenturnparams_fast.scale=props->getIntValue("partgen.turning.scale.fast");
	m_partgenturnparams_fast.throughput_particles=props->getIntValue("partgen.turning.throughput.particles.fast");
	m_partgenturnparams_fast.throughput_frames=props->getIntValue("partgen.turning.throughput.frames.fast");
	m_partgenturnparams_fast.transl_time=props->getIntValue("partgen.turning.translation.time.fast");
	m_partgenturnparams_fast.transl_dist=props->getIntValue("partgen.turning.translation.distance.fast");
	m_partgenturnparams_fast.lifespan=props->getIntValue("partgen.turning.lifespan.fast");
	m_partgenturnparams_fast.turn_speed=props->getIntValue("partgen.turning.fx.turn.revolutiontime.fast");
	
	
	
	m_anim_particle_square=new Anim("res/pieces/square/square1.png");
	m_anim_particle_circle=new Anim("res/pieces/circle/circle1.png");
	m_anim_particle_triangle=new Anim("res/pieces/triangle/triangle1.png");
	m_anim_particle_diamond=new Anim("res/pieces/diamond/diamond1.png");
	m_anim_particle_star=new Anim("res/pieces/star/star1.png");
	m_anim_particle_cross=new Anim("res/pieces/cross/cross1.png");
	
	m_pfxf_partgen_turn_right1_color=new PFXFRandomColor(RGB(0,0,0));
	m_pfxf_partgen_turn_right1_color->addColor(RGB(255,255,255));
	m_pfxf_partgen_turn_right2_color=new PFXFRandomColor(RGB(0,0,0));
	m_pfxf_partgen_turn_right2_color->addColor(RGB(255,255,255));
	m_pfxf_partgen_turn_left1_color=new PFXFRandomColor(RGB(0,0,0));
	m_pfxf_partgen_turn_left1_color->addColor(RGB(255,255,255));
	m_pfxf_partgen_turn_left2_color=new PFXFRandomColor(RGB(0,0,0));
	m_pfxf_partgen_turn_left2_color->addColor(RGB(255,255,255));
	
	m_fx_partgen_size=new SizeWowEffect(
		0, props->getIntValue("partgen.turning.particle.size.add.percent"),
		0, props->getIntValue("partgen.turning.particle.size.add.percent"), 
		1, 1, false, 1
	);
	
	//RIGHT
	m_animf_turning_right1=new ControlledAnimationFactory();
	m_animf_turning_right1->setAnim(m_anim_particle_square);
	m_partgen_turning_right_1=new ParticleGeneratorPoint(
		m_pos_images_big_right.x, m_pos_images_big_right.y, 										/* pos */
		0, m_partgenturnparams_slow.scale, 															/* angle, scale */
		m_partgenturnparams_slow.throughput_particles, m_partgenturnparams_slow.throughput_frames, 	/* throughput */
		m_partgenturnparams_slow.transl_time, m_partgenturnparams_slow.transl_dist,  				/* translation time & dist */
		m_partgenturnparams_slow.lifespan, 															/* lifespan */
		m_animf_turning_right1
	);
	m_partgen_turning_right_1->setLocalEffect1SimpleTranslation();
	m_partgen_turning_right_1->setGlobalEffect1(m_pfxf_partgen_turn_right1_color);
	
	
	
	m_animf_turning_right2=new ControlledAnimationFactory();
	m_animf_turning_right2->setAnim(m_anim_particle_square);
	m_partgen_turning_right_2=new ParticleGeneratorPoint(
		m_pos_images_big_right.x, m_pos_images_big_right.y, 										/* pos */
		180, m_partgenturnparams_slow.scale, 														/* angle, scale */
		m_partgenturnparams_slow.throughput_particles, m_partgenturnparams_slow.throughput_frames, 	/* throughput */
		m_partgenturnparams_slow.transl_time, m_partgenturnparams_slow.transl_dist,  				/* translation time & dist */
		m_partgenturnparams_slow.lifespan, 															/* lifespan */
		m_animf_turning_right2
	);
	m_partgen_turning_right_2->setLocalEffect1SimpleTranslation();
	m_partgen_turning_right_2->setGlobalEffect1(m_pfxf_partgen_turn_right2_color);
	
	
	//LEFT
	m_animf_turning_left1=new ControlledAnimationFactory();
	m_animf_turning_left1->setAnim(m_anim_particle_square);
	m_partgen_turning_left_1=new ParticleGeneratorPoint(
		m_pos_images_big_left.x, m_pos_images_big_left.y, 											/* pos */
		0, m_partgenturnparams_slow.scale, 															/* angle, scale */
		m_partgenturnparams_slow.throughput_particles, m_partgenturnparams_slow.throughput_frames, 	/* throughput */
		m_partgenturnparams_slow.transl_time, m_partgenturnparams_slow.transl_dist,  				/* translation time & dist */
		m_partgenturnparams_slow.lifespan, 															/* lifespan */
		m_animf_turning_left1
	);
	m_partgen_turning_left_1->setLocalEffect1SimpleTranslation();
	m_partgen_turning_left_1->setGlobalEffect1(m_pfxf_partgen_turn_left1_color);
	
	
	m_animf_turning_left2=new ControlledAnimationFactory();
	m_animf_turning_left2->setAnim(m_anim_particle_square);
	m_partgen_turning_left_2=new ParticleGeneratorPoint(
		m_pos_images_big_left.x, m_pos_images_big_left.y, 											/* pos */
		180, m_partgenturnparams_slow.scale, 														/* angle, scale */
		m_partgenturnparams_slow.throughput_particles, m_partgenturnparams_slow.throughput_frames, 	/* throughput */
		m_partgenturnparams_slow.transl_time, m_partgenturnparams_slow.transl_dist,  				/* translation time & dist */
		m_partgenturnparams_slow.lifespan, 															/* lifespan */
		m_animf_turning_left2
	);
	m_partgen_turning_left_2->setLocalEffect1SimpleTranslation();
	m_partgen_turning_left_2->setGlobalEffect1(m_pfxf_partgen_turn_left2_color);
	
	
	//l'effet tournant
	//m_fx_partgen_turn_speed_fast=props->getIntValue("partgen.turning.fx.turn.revolutiontime.fast");
	//m_fx_partgen_turn_speed_slow=props->getIntValue("partgen.turning.fx.turn.revolutiontime.slow");
	m_fx_partgen_turn=new RotationEffect(m_partgenturnparams_slow.turn_speed, false);
	
	
	m_partgen_turning_left_2->generationOff();
	m_partgen_turning_left_2->start();
	m_partgen_turning_left_1->generationOff();
	m_partgen_turning_left_1->start();
	m_partgen_turning_right_2->generationOff();
	m_partgen_turning_right_2->start();
	m_partgen_turning_right_1->generationOff();
	m_partgen_turning_right_1->start();
	m_fx_partgen_turn->start();
}

void LevelChiptunes::initParticleGeneratorsNoise(Properties *props)
{
	RandomAnimationFactory *animfRight=new RandomAnimationFactory();
	RandomAnimationFactory *animfLeft=new RandomAnimationFactory();
	
	for(int i=1;i<=props->getIntValue("partgen.noise.nbimages");i++)
	{
		string image="res/levels/chiptunes/noise/noise"+toString(i)+".png";
		Anim *anim=new Anim(image);
		anim->setReferencePointCenter();
		animfRight->addAnimation(anim);
		animfLeft->addAnimation(new Anim(*anim));
	}
	
	int lifespan=props->getIntValue("partgen.noise.lifespan");
	
	m_partgen_noise_right=new ParticleGeneratorSegment(
		360, 480, 90, 240, 
		props->getIntValue("partgen.noise.throughput.particles"), 
		props->getIntValue("partgen.noise.throughput.frames"), 
		1, 
		480, 
		lifespan, 
		animfRight, 
		props->getIntValue("partgen.noise.nbloops")
	);
	m_partgen_noise_left=new ParticleGeneratorSegment(
		120, 480, 90, 240, 
		props->getIntValue("partgen.noise.throughput.particles"), 
		props->getIntValue("partgen.noise.throughput.frames"), 
		1, 
		480, 
		lifespan, 
		animfLeft, 
		props->getIntValue("partgen.noise.nbloops")
	);
	
	m_partgen_noise_right->setLocalEffect1(new PFXFRandomPosition());
	m_partgen_noise_left->setLocalEffect1(new PFXFRandomPosition());
	
	m_partgen_noise_right->setGlobalEffect1(new ProgressiveAlphaEffect(lifespan, lifespan, 255, 0));
	m_partgen_noise_left->setGlobalEffect1(new ProgressiveAlphaEffect(lifespan, lifespan, 255, 0));
	
	
	
	m_partgen_noise_right->generationOn();
	m_partgen_noise_left->generationOn();
}

void LevelChiptunes::start_specific()
{
	if(m_part==LEVEL_CHIPTUNES_PART_WOLVES)
	{
		oslPlaySound(m_music_wolves, 0);
		m_scenario_wolves->restart();
	}
	else if(m_part==LEVEL_CHIPTUNES_PART_MACHINE)
	{
		oslPlaySound(m_music_machine, 1);
		m_scenario_machine->restart();
	}
	else if(m_part==LEVEL_CHIPTUNES_PART_SUNDAY)
	{
		oslPlaySound(m_music_sunday, 2);
		m_scenario_sunday->restart();
	}
}

//dessin avec une paire
void LevelChiptunes::draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	if(m_fx_wolves_alpha->isRunning())
		m_fx_wolves_alpha->activate();
	currentPair->draw(grid, fx_trans, APPLY_FX_ON_NONE);
	oslSetAlpha(OSL_FX_DEFAULT, 0);
	drawAfter(grid, nextPair, cc);
}

//dessin avec 2 pieces
void LevelChiptunes::draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	if(m_fx_wolves_alpha->isRunning())
		m_fx_wolves_alpha->activate();
	Pos pixPosConnected=grid->getContentPosition(pieceConnectedPos.x, pieceConnectedPos.y);
	Pos pixPosUnconnected=grid->getContentPosition(pieceUnconnectedPos.x, pieceUnconnectedPos.y);
	
	pieceConnected->drawXY(pixPosConnected.x, pixPosConnected.y);
	pieceUnconnected->drawXY(pixPosUnconnected.x, pixPosUnconnected.y, fx_trans);
	oslSetAlpha(OSL_FX_DEFAULT, 0);
	
	drawAfter(grid, nextPair, cc);
}

//dessin avec rien
void LevelChiptunes::draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	drawAfter(grid, nextPair, cc);
}

void LevelChiptunes::drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	//le dégradé:
	oslDrawGradientRect(
		0, 0, 480, 272,
		m_fx_tint_ul->getCurrentColor(),
		m_fx_tint_ur->getCurrentColor(),
		m_fx_tint_dl->getCurrentColor(),
		m_fx_tint_dr->getCurrentColor()
	);
	
	m_partgen_noise_right->draw();
	m_partgen_noise_left->draw();
	
	//les grosses images avec les générateurs qui tournent
	
	m_partgen_turning_right_1->draw(m_fx_partgen_size);
	m_partgen_turning_right_2->draw(m_fx_partgen_size);
	m_partgen_turning_left_1->draw(m_fx_partgen_size);
	m_partgen_turning_left_2->draw(m_fx_partgen_size);
	
	m_fx_tint_image_big_left->activate();
	fx::drawXY(
		m_img_piece_left_current, 46, 136, 
		m_fx_sizewow_images_big->isRunning()?m_fx_sizewow_images_big:NULL, 
		m_gen_moving?m_fx_gen_transl_left:NULL, 
		m_fx_rotation_raggae->isRunning()?m_fx_rotation_raggae:NULL
	);
	oslSetAlpha(OSL_FX_DEFAULT, 0);
	m_fx_tint_image_big_right->activate();
	fx::drawXY(
		m_img_piece_right_current, 480-46, 136, 
		m_fx_sizewow_images_big->isRunning()?m_fx_sizewow_images_big:NULL, 
		m_gen_moving?m_fx_gen_transl_right:NULL, 
		m_fx_rotation_raggae->isRunning()?m_fx_rotation_raggae:NULL
	);
	oslSetAlpha(OSL_FX_DEFAULT, 0);
	
	//contenu de la grille
	if(m_fx_wolves_alpha->isRunning())
		m_fx_wolves_alpha->activate();
	for(int i=0;i<grid->getNbCols();i++)
	{
		for(int j=0;j<grid->getNbLines();j++)
		{
			MatrixContentObject *mco=grid->getAt(i,j);
			if(mco->getPiece()!=NULL)
			{
				Pos pos=grid->getContentPosition(i, j);
				mco->getPiece()->drawXY(pos.x,pos.y);
			}
		}	
	}
	oslSetAlpha(OSL_FX_DEFAULT, 0);
}

void LevelChiptunes::drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{	
	//nextPair->drawXY(400, 80, grid, APPLY_FX_ON_NONE);
	//oslDrawFillRect(400, 270-int((cc->getTime()*260)/cc->getTimeTotalAvailable()), 415, 270, RGB(255,0,0));
	drawGauge();
	
	if(m_repeater_txt->isRunning())
	{
		if(m_part==LEVEL_CHIPTUNES_PART_MACHINE)
			oslDrawImage(m_img_hello);
		else if(m_part==LEVEL_CHIPTUNES_PART_SUNDAY)
			oslDrawImage(m_img_people);
	}
	
	if(m_part==LEVEL_CHIPTUNES_PART_WOLVES)
		m_songbanner_wolves->draw();
	else if(m_part==LEVEL_CHIPTUNES_PART_MACHINE)
		m_songbanner_machine->draw();
	else
		m_songbanner_sunday->draw();
}

void LevelChiptunes::drawGauge()
{
	//TODO: vraies positions!
	int origin_x=400;
	int origin_y=10;
	
	for(int i=0;i<12;i++)
	{
		int y=origin_y+i*GAMEDEF_PIECE_HEIGHT;
		
		
		if(m_fx_gauge_alpha[i]->isRunning())
		{
			//gauche
			//m_fx_gauge_tint_left[i]->activate();
			m_fx_gauge_alpha[i]->activate();
			oslDrawFillRect(origin_x, y, origin_x+GAMEDEF_PIECE_WIDTH, y+GAMEDEF_PIECE_HEIGHT, m_fx_gauge_tint_left[i]->getCurrentColor());
			//oslSetAlpha(OSL_FX_NONE, 0);
			
			//droite
			//m_fx_gauge_tint_right[i]->activate();
			//m_fx_gauge_alpha[i]->activate();
			oslDrawFillRect(origin_x+GAMEDEF_PIECE_WIDTH, y, origin_x+2*GAMEDEF_PIECE_WIDTH, y+GAMEDEF_PIECE_HEIGHT, m_fx_gauge_tint_right[i]->getCurrentColor());
			oslSetAlpha(OSL_FX_DEFAULT, 0);
		}
	}
}

void LevelChiptunes::updateGauge(ComboCounter *cc, Pair *nextPair)
{
	//la couleur
	/*for(int i=0;i<12;i++)
	{
		if(!m_fx_gauge_alpha[i]->isRunning())
		{
			m_fx_gauge_tint_left[i]->setColors(oslcolorleft, oslcolorleft);
			m_fx_gauge_tint_right[i]->setColors(oslcolorright, oslcolorright);
		}
	}
	*/
	
	
	
	int current=int((cc->getTime()*12)/cc->getTimeTotalAvailable());
	if(current>=12)
		current=11;
		
	current=11-current;
	
	if(cc->getTime()!=0 && !m_fx_gauge_alpha[current]->isRunning())
	{
		//couleur:
		OSL_COLOR oslcolorleft = AbstractPiece::getOslColor(nextPair->getPiece(0)->getColor());
		OSL_COLOR oslcolorright = AbstractPiece::getOslColor(nextPair->getPiece(1)->getColor());
		m_fx_gauge_tint_left[current]->setColors(oslcolorleft, oslcolorleft);
		m_fx_gauge_tint_right[current]->setColors(oslcolorright, oslcolorright);
		
		//m_fx_gauge_tint_left[current]->restart();
		//m_fx_gauge_tint_right[current]->restart();
		m_fx_gauge_alpha[current]->restart();
	}
	/*
	else if(cc->getTime()==0)
	{
		for(int i=0;i<12;i++)
		{
			m_fx_gauge_alpha[i]->update();
		}
	}
	*/
	
	for(int i=0;i<12;i++)
	{
		//m_fx_gauge_tint_left[i]->update();
		//m_fx_gauge_tint_right[i]->update();
		m_fx_gauge_alpha[i]->update();
	}
}

void LevelChiptunes::updateForceFxGenTransl()
{
	m_fx_gen_transl_right->stop();
	m_fx_gen_transl_left->stop();
	
	//on fait en sorte que le prochain update les fasse partir de la position d'origine
	/*delete m_fx_gen_transl_right;
	delete m_fx_gen_transl_left;
	m_fx_gen_transl_right=new TranslationEffect(0,0,0,0,1,1);
	m_fx_gen_transl_left=new TranslationEffect(0,0,0,0,1,1);
	*/
	
	updateFxGenTransl(m_fx_gen_transl_left, m_translparams_left);
	updateFxGenTransl(m_fx_gen_transl_right, m_translparams_right);
}

void LevelChiptunes::updateFxGenTransl(TranslationEffect *&fx_gen_transl, TranslationParams params)
{
	if(m_gen_moving)
	{
		fx_gen_transl->update();
		if(!fx_gen_transl->isRunning())
		{
			Pos startpos(fx_gen_transl->getCurrentPos());
			int endx=params.x_min+random(0, params.x_max-params.x_min);
			int endy=params.y_min+random(0, params.y_max-params.y_min);
			
			//calcul de la distance qu'elle va parcourir
			float dist=pow(float(endx-startpos.x),2.0f)+pow(float(endy-startpos.y),2.0f);
			dist=sqrt(dist);
			
			float speed=params.speed_slow;
			if(m_music_speed_is_fast)
				speed=params.speed_fast;
			int time=int(dist*speed);
			if(time<2)
				time=2;
				
			delete fx_gen_transl;
			fx_gen_transl=new TranslationEffect(startpos.x, startpos.y, endx, endy, time, time, true, 1);
			fx_gen_transl->start();
		}
	}
}


void LevelChiptunes::changePartgenTurnParams(TurningGenParams params)
{
	//l'effet tournant
	m_fx_partgen_turn->setRevolutionTime(params.turn_speed);
	
	//les generateurs
	m_partgen_turning_right_1->setScale(params.scale);
	m_partgen_turning_right_1->setThroughput(params.throughput_particles, params.throughput_frames);
	m_partgen_turning_right_1->setPTranslTime(params.transl_time);
	m_partgen_turning_right_1->setPDistance(params.transl_dist);
	m_partgen_turning_right_1->setPLifespan(params.lifespan);
	
	m_partgen_turning_right_2->setScale(params.scale);
	m_partgen_turning_right_2->setThroughput(params.throughput_particles, params.throughput_frames);
	m_partgen_turning_right_2->setPTranslTime(params.transl_time);
	m_partgen_turning_right_2->setPDistance(params.transl_dist);
	m_partgen_turning_right_2->setPLifespan(params.lifespan);
	
	m_partgen_turning_left_1->setScale(params.scale);
	m_partgen_turning_left_1->setThroughput(params.throughput_particles, params.throughput_frames);
	m_partgen_turning_left_1->setPTranslTime(params.transl_time);
	m_partgen_turning_left_1->setPDistance(params.transl_dist);
	m_partgen_turning_left_1->setPLifespan(params.lifespan);
	
	m_partgen_turning_left_2->setScale(params.scale);
	m_partgen_turning_left_2->setThroughput(params.throughput_particles, params.throughput_frames);
	m_partgen_turning_left_2->setPTranslTime(params.transl_time);
	m_partgen_turning_left_2->setPDistance(params.transl_dist);
	m_partgen_turning_left_2->setPLifespan(params.lifespan);
}

void LevelChiptunes::update_pause_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
{
	m_repeater_txt->update();
	
	m_fx_partgen_turn->update();
	State *state=new State();
	
	m_partgen_turning_right_1->savestate(state);
	m_fx_partgen_turn->applyOnOrientable(m_partgen_turning_right_1);
	if(m_gen_moving)
		m_fx_gen_transl_right->applyOnPositionable(m_partgen_turning_right_1);
	m_partgen_turning_right_1->update();
	m_partgen_turning_right_1->restorestate(state);
	
	m_partgen_turning_right_2->savestate(state);
	m_fx_partgen_turn->applyOnOrientable(m_partgen_turning_right_2);
	if(m_gen_moving)
		m_fx_gen_transl_right->applyOnPositionable(m_partgen_turning_right_2);
	m_partgen_turning_right_2->update();
	m_partgen_turning_right_2->restorestate(state);
	
	m_partgen_turning_left_1->savestate(state);
	m_fx_partgen_turn->applyOnOrientable(m_partgen_turning_left_1);
	if(m_gen_moving)
		m_fx_gen_transl_left->applyOnPositionable(m_partgen_turning_left_1);
	m_partgen_turning_left_1->update();
	m_partgen_turning_left_1->restorestate(state);
	
	m_partgen_turning_left_2->savestate(state);
	m_fx_partgen_turn->applyOnOrientable(m_partgen_turning_left_2);
	if(m_gen_moving)
		m_fx_gen_transl_left->applyOnPositionable(m_partgen_turning_left_2);
	m_partgen_turning_left_2->update();
	m_partgen_turning_left_2->restorestate(state);
	
	delete state;
}

//appelé uniquement si le level n'est pas en pause.
void LevelChiptunes::update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
{
	if(m_part==LEVEL_CHIPTUNES_PART_WOLVES)
		m_songbanner_wolves->update();
	else if(m_part==LEVEL_CHIPTUNES_PART_MACHINE)
		m_songbanner_machine->update();
	else
		m_songbanner_sunday->update();
	
	m_repeater_txt->update();
	
	m_fx_tint_ul->update();
	m_fx_tint_dl->update();
	m_fx_tint_ur->update();
	m_fx_tint_dr->update();
	
	m_scenario_wolves->update();
	m_scenario_machine->update();
	m_scenario_sunday->update();
	
	m_fx_sizewow_images_big->update();
	
	m_fx_tint_image_big_right->update();
	m_fx_tint_image_big_left->update();
	
	updateFxGenTransl(m_fx_gen_transl_right, m_translparams_right);
	updateFxGenTransl(m_fx_gen_transl_left, m_translparams_left);
	
	m_fx_rotation_raggae->update();
	
	m_partgen_noise_right->update();
	m_partgen_noise_left->update();
	
	m_fx_wolves_alpha->update();
	
	///// generateurs
	m_fx_partgen_turn->update();
	State *state=new State();
	
	m_partgen_turning_right_1->savestate(state);
	m_fx_partgen_turn->applyOnOrientable(m_partgen_turning_right_1);
	if(m_gen_moving)
		m_fx_gen_transl_right->applyOnPositionable(m_partgen_turning_right_1);
	m_partgen_turning_right_1->update();
	m_partgen_turning_right_1->restorestate(state);
	
	m_partgen_turning_right_2->savestate(state);
	m_fx_partgen_turn->applyOnOrientable(m_partgen_turning_right_2);
	if(m_gen_moving)
		m_fx_gen_transl_right->applyOnPositionable(m_partgen_turning_right_2);
	m_partgen_turning_right_2->update();
	m_partgen_turning_right_2->restorestate(state);
	
	m_partgen_turning_left_1->savestate(state);
	m_fx_partgen_turn->applyOnOrientable(m_partgen_turning_left_1);
	if(m_gen_moving)
		m_fx_gen_transl_left->applyOnPositionable(m_partgen_turning_left_1);
	m_partgen_turning_left_1->update();
	m_partgen_turning_left_1->restorestate(state);
	
	m_partgen_turning_left_2->savestate(state);
	m_fx_partgen_turn->applyOnOrientable(m_partgen_turning_left_2);
	if(m_gen_moving)
		m_fx_gen_transl_left->applyOnPositionable(m_partgen_turning_left_2);
	m_partgen_turning_left_2->update();
	m_partgen_turning_left_2->restorestate(state);
	
	delete state;
	////// fin generateurs
	
	//jauge
	updateGauge(cc, nextPair);
	
	///////////
	// changement de zik
	/////////////////
	if(m_difficulty==GAMEDEF_DIFFICULTY_NORMAL) //machine puis sunday
	{
		if(m_part==LEVEL_CHIPTUNES_PART_MACHINE && m_scenario_machine->getTime()>=LEVEL_CHIPTUNES_DURATION_MACHINE)
		{
			m_scenario_sunday->restart();
			oslPlaySound(m_music_sunday, 2);
			m_part=LEVEL_CHIPTUNES_PART_SUNDAY;
		}
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_HARD) //wolves puis machine
	{
		if(m_part==LEVEL_CHIPTUNES_PART_WOLVES && m_scenario_wolves->getTime()>=LEVEL_CHIPTUNES_DURATION_WOLVES)
		{
			m_scenario_machine->restart();
			oslPlaySound(m_music_machine, 1);
			m_part=LEVEL_CHIPTUNES_PART_MACHINE;
		}
	}
	
	///////////
	//     events
	/////////////////
	
	if(m_part==LEVEL_CHIPTUNES_PART_WOLVES)
	{
		/*
		#1 : debut bizarre
		#2 : partie chantée on
		#3 : partie chantée off
		#4 : wooooooo
		#5 : basses 0
		#6 : basses 1
		#7 : basses 2
		#8 : pouit
		#9 : fin
		*/
		
		if(m_scenario_wolves->hasEvent(1)) //debut bizarre
		{
			m_fx_wolves_alpha->start();
		}
		if(m_scenario_wolves->hasEvent(2)) //partie chantée on
		{
			m_fx_rotation_raggae->start();
		}
		if(m_scenario_wolves->hasEvent(3)) //partie chantée off
		{
			m_fx_rotation_raggae->stop();
		}
		if(m_scenario_wolves->hasEvent(4)) //wooooooo
		{
			m_addline+=3;
		}
		if(m_scenario_wolves->hasEvent(5)) //basses 0
		{
			m_gen_moving=false;
			m_music_speed_is_fast=false;
			updateForceFxGenTransl();
			changePartgenTurnParams(m_partgenturnparams_slow);
			m_partgen_turning_left_2->generationOff();
			m_partgen_turning_left_1->generationOff();
			m_partgen_turning_right_2->generationOff();
			m_partgen_turning_right_1->generationOff();
			
		}
		if(m_scenario_wolves->hasEvent(6)) //basses 1
		{
			m_gen_moving=true;
			m_music_speed_is_fast=false;
			updateForceFxGenTransl();
			changePartgenTurnParams(m_partgenturnparams_slow);
			m_partgen_turning_left_2->generationOn();
			m_partgen_turning_left_1->generationOn();
			m_partgen_turning_right_2->generationOn();
			m_partgen_turning_right_1->generationOn();
		}
		if(m_scenario_wolves->hasEvent(7)) //basses 2
		{
			m_fx_wolves_alpha->stop();
			
			m_gen_moving=true;
			m_music_speed_is_fast=true;
			updateForceFxGenTransl();
			changePartgenTurnParams(m_partgenturnparams_fast);
			m_partgen_turning_left_2->generationOn();
			m_partgen_turning_left_1->generationOn();
			m_partgen_turning_right_2->generationOn();
			m_partgen_turning_right_1->generationOn();
		}
		if(m_scenario_wolves->hasEvent(8)) //pouit
		{
			m_fx_sizewow_images_big->restart();
		}
		if(m_scenario_wolves->hasEvent(9)) //fin
		{
			m_partgen_turning_left_2->generationOff();
			m_partgen_turning_left_1->generationOff();
			m_partgen_turning_right_2->generationOff();
			m_partgen_turning_right_1->generationOff();
			m_gen_moving=false;
			m_partgen_turning_left_2->reset();
			m_partgen_turning_left_1->reset();
			m_partgen_turning_right_2->reset();
			m_partgen_turning_right_1->reset();
		}
	}
	else if(m_part==LEVEL_CHIPTUNES_PART_MACHINE)
	{
		/*
		#1 : noise clair
		#2 : noise sourd
		#3 : bass 0
		#4 : bass 1
		#5 : bass 2
		#6 : hello
		#7 : prouit
		#8 : fin lente
		#9 : debut ligne chip
		#10: fin ligne chip
		*/
		
		if(m_scenario_machine->hasEvent(9)) //debut ligne chip
		{
			m_partgen_turning_left_2->generationOn();
			m_partgen_turning_left_1->generationOn();
			m_partgen_turning_right_2->generationOn();
			m_partgen_turning_right_1->generationOn();
		}
		if(m_scenario_machine->hasEvent(10)) //fin ligne chip
		{
			m_partgen_turning_left_2->generationOff();
			m_partgen_turning_left_1->generationOff();
			m_partgen_turning_right_2->generationOff();
			m_partgen_turning_right_1->generationOff();
		}
		
		if(m_scenario_machine->hasEvent(1)) //noise clair
		{
			m_partgen_noise_right->restart();
			m_partgen_noise_left->restart();
		}
		if(m_scenario_machine->hasEvent(2)) //noise sourd
		{
			m_partgen_noise_right->restart();
			m_partgen_noise_left->restart();
		}
		if(m_scenario_machine->hasEvent(3)) //bass 0
		{
			m_gen_moving=false;
			m_music_speed_is_fast=false;
			updateForceFxGenTransl();
			changePartgenTurnParams(m_partgenturnparams_slow);
		}
		if(m_scenario_machine->hasEvent(4)) //bass 1
		{
			m_gen_moving=true;
			m_music_speed_is_fast=false;
			updateForceFxGenTransl();
			changePartgenTurnParams(m_partgenturnparams_slow);
		}
		if(m_scenario_machine->hasEvent(5)) //bass 2
		{
			m_gen_moving=true;
			m_music_speed_is_fast=true;
			updateForceFxGenTransl();
			changePartgenTurnParams(m_partgenturnparams_fast);
		}
		if(m_scenario_machine->hasEvent(6)) //hello
		{
			m_repeater_txt->restart();
		}
		if(m_scenario_machine->hasEvent(7)) //prouit
		{
			m_fx_sizewow_images_big->restart();
		}
		if(m_scenario_machine->hasEvent(8)) //fin lente
		{
			
		}
	}
	else if(m_part==LEVEL_CHIPTUNES_PART_SUNDAY)
	{
		/*
		#1 : chiptunes on
		#2 : chiptunes off
		#3 : people
		#4 : raggae on
		#5 : raggae off
		#6 : grande partie chiptune start
		#7 : grande partie chiptunes end
		#8 : ti tu
		#9 : fin
		*/ 
		
		if(m_scenario_sunday->hasEvent(1)) //chiptunes on
		{
			m_partgen_turning_left_2->generationOn();
			m_partgen_turning_left_1->generationOn();
			m_partgen_turning_right_2->generationOn();
			m_partgen_turning_right_1->generationOn();
		}
		if(m_scenario_sunday->hasEvent(2)) //chiptunes off
		{
			m_partgen_turning_left_2->generationOff();
			m_partgen_turning_left_1->generationOff();
			m_partgen_turning_right_2->generationOff();
			m_partgen_turning_right_1->generationOff();
		}
		if(m_scenario_sunday->hasEvent(3)) //people
		{
			m_repeater_txt->restart();
		}
		if(m_scenario_sunday->hasEvent(4)) //raggae on
		{
			m_fx_rotation_raggae->start();
		}
		if(m_scenario_sunday->hasEvent(5)) //raggae off
		{
			m_fx_rotation_raggae->stop();
		}
		if(m_scenario_sunday->hasEvent(6)) //grande partie chiptune start
		{
			m_partgen_turning_left_2->generationOn();
			m_partgen_turning_left_1->generationOn();
			m_partgen_turning_right_2->generationOn();
			m_partgen_turning_right_1->generationOn();
			m_gen_moving=true;
			m_music_speed_is_fast=true;
			updateForceFxGenTransl();
			changePartgenTurnParams(m_partgenturnparams_fast);
		}
		if(m_scenario_sunday->hasEvent(7)) //grande partie chiptune end
		{
			m_partgen_turning_left_2->generationOff();
			m_partgen_turning_left_1->generationOff();
			m_partgen_turning_right_2->generationOff();
			m_partgen_turning_right_1->generationOff();
			//m_gen_moving=false;
			m_music_speed_is_fast=false;
			updateForceFxGenTransl();
			changePartgenTurnParams(m_partgenturnparams_slow);
		}
		if(m_scenario_sunday->hasEvent(8)) //ti tu
		{
			m_fx_sizewow_images_big->restart();
		}
		if(m_scenario_sunday->hasEvent(9)) //fin
		{
			m_partgen_turning_left_2->generationOff();
			m_partgen_turning_left_1->generationOff();
			m_partgen_turning_right_2->generationOff();
			m_partgen_turning_right_1->generationOff();
			m_gen_moving=false;
			m_partgen_turning_left_2->reset();
			m_partgen_turning_left_1->reset();
			m_partgen_turning_right_2->reset();
			m_partgen_turning_right_1->reset();
		}
	}
	//fin events
	
	
}

Anim *LevelChiptunes::getAnimParticleFromShape(int piece_shape)
{
	switch(piece_shape)
	{
		case PIECE_SHAPE_SQUARE:
		{
			return m_anim_particle_square;
			break;
		}
		case PIECE_SHAPE_STAR:
		{
			return m_anim_particle_star;
			break;
		}
		case PIECE_SHAPE_TRIANGLE:
		{
			return m_anim_particle_triangle;
			break;
		}
		case PIECE_SHAPE_CIRCLE:
		{
			return m_anim_particle_circle;
			break;
		}
		case PIECE_SHAPE_DIAMOND:
		{
			return m_anim_particle_diamond;
			break;
		}
		case PIECE_SHAPE_CROSS:
		{
			return m_anim_particle_cross;
			break;
		}
		default:
		{
			return NULL;	
		}
	}
	
	return NULL;
}

OSL_IMAGE *LevelChiptunes::getPieceImageBig(int piece_shape)
{
	switch(piece_shape)
	{
		case PIECE_SHAPE_SQUARE:
		{
			return m_img_pieces_big[0];
			break;
		}
		case PIECE_SHAPE_STAR:
		{
			return m_img_pieces_big[1];
			break;
		}
		case PIECE_SHAPE_TRIANGLE:
		{
			return m_img_pieces_big[2];
			break;
		}
		case PIECE_SHAPE_CIRCLE:
		{
			return m_img_pieces_big[3];
			break;
		}
		case PIECE_SHAPE_DIAMOND:
		{
			return m_img_pieces_big[4];
			break;
		}
		case PIECE_SHAPE_CROSS:
		{
			return m_img_pieces_big[5];
			break;
		}
		default:
		{
			return NULL;	
		}
	}
	
	return NULL;
}

void LevelChiptunes::pause_specific() //pause
{
	if(m_part==LEVEL_CHIPTUNES_PART_WOLVES)
	{
		oslPauseSound(m_music_wolves, 1);
		m_scenario_wolves->pause();
	}
	else if(m_part==LEVEL_CHIPTUNES_PART_MACHINE)
	{
		oslPauseSound(m_music_machine, 1);
		m_scenario_machine->pause();
	}
	else if(m_part==LEVEL_CHIPTUNES_PART_SUNDAY)
	{
		oslPauseSound(m_music_sunday, 1);
		m_scenario_sunday->pause();
	}
}

void LevelChiptunes::resume_specific()
{
	if(m_part==LEVEL_CHIPTUNES_PART_WOLVES)
	{
		oslPauseSound(m_music_wolves, 0);
		m_scenario_wolves->start();
	}
	else if(m_part==LEVEL_CHIPTUNES_PART_MACHINE)
	{
		oslPauseSound(m_music_machine, 0);
		m_scenario_machine->start();
	}
	else if(m_part==LEVEL_CHIPTUNES_PART_SUNDAY)
	{
		oslPauseSound(m_music_sunday, 0);
		m_scenario_sunday->start();
	}
}

list<AbstractPiece *> *LevelChiptunes::getNextLine()
{
	return m_linepattern->getNextLine();
}

bool LevelChiptunes::getAddLineEvent() //dit quand une ligne doit monter, dû au scénario
{
	if(m_addline>0)
	{
		m_addline--;
		return true;
	}
	return false;
}

int LevelChiptunes::getDropSpeed() //la vitesse a un instant t (elle peut etre modifiée par le scénario)
{
	return -1;
}

int LevelChiptunes::getUpSpeed() //la vitesse ou les lignes montent a un instant t
{
	return m_upspeed;
}

void LevelChiptunes::buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid)
{
	if(key==OSL_KEY_SQUARE)
	{
		//oslPlaySound(sndtest, 5);
		
	}
	if(key==OSL_KEY_TRIANGLE)
	{
		grid->explodeAll(PIECE_MODE_COLOR);
	}
}

bool LevelChiptunes::finished()
{
	if(
		   m_difficulty==GAMEDEF_DIFFICULTY_EASY 
		&& m_scenario_machine->getTime()>=LEVEL_CHIPTUNES_DURATION_MACHINE)
	{
		return true;
	}
	if(
		   m_difficulty==GAMEDEF_DIFFICULTY_NORMAL 
		&& m_part==LEVEL_CHIPTUNES_PART_SUNDAY 
		&& m_scenario_sunday->getTime()>=LEVEL_CHIPTUNES_DURATION_SUNDAY)
	{
		return true;
	}
	if(
		   m_difficulty==GAMEDEF_DIFFICULTY_HARD 
		&& m_part==LEVEL_CHIPTUNES_PART_MACHINE 
		&& m_scenario_machine->getTime()>=LEVEL_CHIPTUNES_DURATION_MACHINE)
	{
		return true;
	}
	
	return false;
}

float LevelChiptunes::getDuration()
{
	if(m_difficulty==GAMEDEF_DIFFICULTY_EASY)
		return LEVEL_CHIPTUNES_DURATION_MACHINE;
	if(m_difficulty==GAMEDEF_DIFFICULTY_NORMAL)
		return LEVEL_CHIPTUNES_DURATION_MACHINE+LEVEL_CHIPTUNES_DURATION_SUNDAY;
	if(m_difficulty==GAMEDEF_DIFFICULTY_HARD)
		return LEVEL_CHIPTUNES_DURATION_WOLVES+LEVEL_CHIPTUNES_DURATION_MACHINE;
		
	return -1;
}



Pair *LevelChiptunes::getNewPair()
{
	Pair *nextPair=this->AbstractLevel::getNewPair();
	
	int colorleft=nextPair->getPiece(0)->getColor();
	int colorright=nextPair->getPiece(1)->getColor();
	int shapeleft=nextPair->getPiece(0)->getShape();
	int shaperight=nextPair->getPiece(1)->getShape();
	OSL_COLOR oslcolorleft = AbstractPiece::getOslColor(colorleft);
	OSL_COLOR oslcolorright = AbstractPiece::getOslColor(colorright);
		
	//on met la bonne couleur dans les effets:
	m_fx_tint_ul->setColorFrom(oslcolorleft);
	m_fx_tint_dl->setColorTo(oslcolorleft);
	m_fx_tint_ur->setColorTo(oslcolorright);
	m_fx_tint_dr->setColorFrom(oslcolorright);
	
	//on met la bonne image des pieces
	m_img_piece_left_current=getPieceImageBig(shapeleft);
	m_img_piece_right_current=getPieceImageBig(shaperight);
	//et la bonne couleur
	m_fx_tint_image_big_right->setColorFrom(oslcolorright);
	m_fx_tint_image_big_left->setColorTo(oslcolorleft);
	
	//on met les bonnes particules aux generateurs qui tournent
	Anim *particleright=getAnimParticleFromShape(shaperight);
	Anim *particleleft=getAnimParticleFromShape(shapeleft);
	m_animf_turning_right1->setAnim(particleright);
	m_animf_turning_right2->setAnim(particleright);
	m_animf_turning_left1->setAnim(particleleft);
	m_animf_turning_left2->setAnim(particleleft);
	//ainsi que la bonne couleur
	m_pfxf_partgen_turn_right1_color->replaceColorNumber(1, oslcolorright);
	m_pfxf_partgen_turn_right2_color->replaceColorNumber(1, oslcolorright);
	m_pfxf_partgen_turn_left1_color->replaceColorNumber(1, oslcolorleft);
	m_pfxf_partgen_turn_left2_color->replaceColorNumber(1, oslcolorleft);
	
	//no prob ca delete l'effet deja présent tt seul.
	m_partgen_noise_right->setGlobalEffect2(new ProgressiveTintEffect(1, 1, oslcolorright, oslcolorright, 1));
	m_partgen_noise_left->setGlobalEffect2(new ProgressiveTintEffect(1, 1, oslcolorleft, oslcolorleft, 1));
	
	//la jauge: on s'en occupe dans l'update, c + compliqué pour elle
	
	return nextPair;
}
























