#ifndef LEVELCHIPTUNES_H_
#define LEVELCHIPTUNES_H_

#include <oslib/oslib.h>
#include "code/game/levels/abstract/AbstractLevel.h"
#include "code/game/grid/GameGrid.h"
#include "code/game/Pair.h"
#include "code/game/levels/LinePattern.h"
#include "code/game/pieces/abstract/AbstractPiece.h"
#include <list>
#include "code/game/SongBanner.h"
#include "code/game/combos/ComboCounter.h"
#include "code/game/combos/ComboNumberDrawer.h"
#include "code/framework/fx/EffectsLib.h"
#include "code/game/EventsScenario.h"
#include "code/framework/particles/complete/impl/ParticleGeneratorPoint.h"
#include "code/framework/particles/complete/impl/ParticleGeneratorSegment.h"
#include "code/framework/particles/complete/animFactories/ControlledAnimationFactory.h"
#include "code/framework/Properties.h"
#include "code/framework/particles/complete/fxFactories/PFXFRandomColor.h"
#include "code/framework/time/Repeater.h"

using namespace std;

class LevelChiptunes : public AbstractLevel
{
	private:
		//inner struct
		typedef struct
		{
			int x_min;
			int x_max;
			int y_min;
			int y_max;
			float speed_slow;
			float speed_fast;
		}TranslationParams;
		
		//inner struct
		typedef struct
		{
			int scale;
			int throughput_particles;
			int throughput_frames;
			int transl_time;
			int transl_dist;
			int lifespan;
			unsigned int turn_speed;
		}TurningGenParams;
	
		SongBanner *m_songbanner_wolves;
		SongBanner *m_songbanner_machine;
		SongBanner *m_songbanner_sunday;
		
		ProgressiveTintEffect *m_fx_tint_ul;
		ProgressiveTintEffect *m_fx_tint_dl;
		ProgressiveTintEffect *m_fx_tint_ur;
		ProgressiveTintEffect *m_fx_tint_dr;
		
		OSL_SOUND *m_music_wolves;
		OSL_SOUND *m_music_machine;
		OSL_SOUND *m_music_sunday;
		
		EventsScenario *m_scenario_wolves;
		EventsScenario *m_scenario_machine;
		EventsScenario *m_scenario_sunday;
		
		LinePattern *m_linepattern;
		
		int m_part;
		int m_addline;
		
		//dessin des grosses formes de pieces
		OSL_IMAGE *m_img_piece_left_current; //pointeur uniquement
		OSL_IMAGE *m_img_piece_right_current; //pointeur uniquement
		
		OSL_IMAGE *m_img_pieces_big[6];
		OSL_IMAGE *getPieceImageBig(int piece_shape);
		Pos m_pos_images_big_right;
		Pos m_pos_images_big_left;
		
		SizeWowEffect *m_fx_sizewow_images_big;
		
		ProgressiveTintEffect *m_fx_tint_image_big_right;
		ProgressiveTintEffect *m_fx_tint_image_big_left;
		
		int m_upspeed;
	
		//les générateurs de particules qui tournent: 2 de chaque coté
		void initParticleGeneratorsTurning(Properties *props);
		ParticleGeneratorPoint *m_partgen_turning_right_1;
		ParticleGeneratorPoint *m_partgen_turning_right_2;
		ParticleGeneratorPoint *m_partgen_turning_left_1;
		ParticleGeneratorPoint *m_partgen_turning_left_2;
		ControlledAnimationFactory *m_animf_turning_right1; //pointeur
		ControlledAnimationFactory *m_animf_turning_right2; //pointeur
		ControlledAnimationFactory *m_animf_turning_left1; //pointeur
		ControlledAnimationFactory *m_animf_turning_left2; //pointeur
		Anim *m_anim_particle_square;
		Anim *m_anim_particle_circle;
		Anim *m_anim_particle_triangle;
		Anim *m_anim_particle_diamond;
		Anim *m_anim_particle_star;
		Anim *m_anim_particle_cross;
		RotationEffect *m_fx_partgen_turn;
		void changePartgenTurnParams(TurningGenParams params);
		TurningGenParams m_partgenturnparams_slow;
		TurningGenParams m_partgenturnparams_fast;
		PFXFRandomColor *m_pfxf_partgen_turn_right1_color; //pointeur
		PFXFRandomColor *m_pfxf_partgen_turn_right2_color; //pointeur
		PFXFRandomColor *m_pfxf_partgen_turn_left1_color; //pointeur
		PFXFRandomColor *m_pfxf_partgen_turn_left2_color; //pointeur
		Anim *getAnimParticleFromShape(int piece_shape);
		SizeWowEffect *m_fx_partgen_size;
		TranslationEffect *m_fx_gen_transl_right;
		TranslationEffect *m_fx_gen_transl_left;
		void updateFxGenTransl(TranslationEffect *&fx_gen_transl, TranslationParams params);
		void updateForceFxGenTransl();
		TranslationParams m_translparams_left;
		TranslationParams m_translparams_right;
		bool m_gen_moving;
		bool m_music_speed_is_fast;
		RotationEffect *m_fx_rotation_raggae;
		
		
		//les generateurs de noise
		ParticleGeneratorSegment *m_partgen_noise_right;
		ParticleGeneratorSegment *m_partgen_noise_left;
		void initParticleGeneratorsNoise(Properties *props);
		
		
		//les effets de la jauge
		ProgressiveTintEffect *m_fx_gauge_tint_left[12];
		ProgressiveTintEffect *m_fx_gauge_tint_right[12];
		ProgressiveAlphaEffect *m_fx_gauge_alpha[12];
		void drawGauge();
		void updateGauge(ComboCounter *cc, Pair *nextPair);
		
		//l'effet de depart de wolves
		SerialGlobalEffect *m_fx_wolves_alpha;
		
		//hello et people
		OSL_IMAGE *m_img_hello;
		OSL_IMAGE *m_img_people;
		Repeater *m_repeater_txt;
	
	
		void start_specific(); //démarre la musique, le timer etc
		void update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans);
		void update_pause_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans);
		void pause_specific(); 
		void resume_specific();
		
		void drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc);
		void drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc);
		
		
			
	public:
		LevelChiptunes(int difficulty=GAMEDEF_DIFFICULTY_NORMAL, int nbPlayers=1);
		~LevelChiptunes();
		
		//hérité:
		int getDropSpeed(); //la vitesse a un instant t (elle peut etre modifiée par le scénario)
		int getUpSpeed(); //la vitesse ou les lignes montent a un instant t
		void draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer);
		string getCode();
		
		void buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid);
		
		//LinePattern *getLevelPieces(); 
		bool getAddLineEvent(); //dit quand une ligne doit monter, dû au scénario
		
		bool finished();
		
		list<AbstractPiece *> *getNextLine();
		
		float getDuration();
		
		void reset();
		Pair *getNewPair();
};

#endif /*LEVELCHIPTUNES_H_*/
