#ifndef PDR4CLEVELDJBEAT_H_
#define PDR4CLEVELDJBEAT_H_

#include <oslib/oslib.h>
#include "code/game/levels/abstract/AbstractLevel.h"
#include "code/game/grid/GameGrid.h"
#include "code/game/Pair.h"
#include "code/game/levels/LinePattern.h"
#include "code/game/pieces/abstract/AbstractPiece.h"
#include <list>
#include "code/game/SongBanner.h"
#include "code/game/combos/ComboCounter.h"
#include "code/game/combos/pdr4c/PDR4CComboNumberDrawer.h"
#include "code/game/EventsScenario.h"
#include "code/framework/fx/EffectsLib.h"
#include "code/framework/anim/Anim.h"
#include "code/game/levels/utils/MCOContentSimpleAnimated.h"
#include "code/framework/Properties.h"


using namespace std;

class PDR4CLevelDjbeat : public AbstractLevel
{
	private:
		
		Anim *m_anim_grid;
			
		
		Anim **m_anims_next; //dessin de "next"
		Anim **m_anims_gauge; //dessin de la jauge "combo combo combo"
		int m_currentAnimNum;
		Pos *m_pos_next;
		Pos *m_pos_gauge;
		Delayer *m_delayer; //change les anims next et combo
		Delayer *m_delayer2; //arrete la vitesse des cercles
		int m_delayer_time, m_delayer_loops;
		
		
		Anim * m_anim_pause;
		
		//la gauge se vide
		VanishUpEffect *m_fx_gauge_vanish;
		TranslationEffect *m_fx_gauge_move;
		
		LinePattern *m_linepattern;
				
		SongBanner *m_songbanner;
		
		PDR4CComboNumberDrawer * m_comboDrawer;
		
		EventsScenario *m_scenario;
		OSL_SOUND* m_music;
		
		int m_dropspeed; //current
		int m_dropspeed_slow;
		int m_dropspeed_fast;
		int m_upspeed; //current
		int m_upspeed_slow;
		int m_upspeed_fast;
		
		int m_addline; //ajoute 1 qd on veut qu'une ligne monte
								
		void drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc);
		void drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc);
				
		int m_difficulty;
		int m_nbColors; //nombre de couleurs pour les pieces
		
		//animations qui remplacent les pieces normales
		Anim ** m_anim_overlays;//les pieces quand elles n'explosent pas, overlay
		Anim ** m_anim_bgs;//les pieces quand elles n'explosent pas, bgs
		Anim ** m_anim_explosions;//les pieces quand elles explosent
		
		//les coins
		Anim ** m_anims_corner_upleft;
		Anim ** m_anims_corner_upright;
		Anim ** m_anims_corner_downleft;
		Anim ** m_anims_corner_downright;
		
		
		void start_specific(); //démarre la musique, le timer etc
		void update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans);
		void update_pause_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans);
		void pause_specific(); 
		void resume_specific();
		
		bool m_beginning; //nous dit qd la musique commence réellement
		
		Anim *m_anim_djbeat;
		//int m_color_songbanner;
		//int m_color_songbannerbg;
		
		//SerialLocalEffect *m_fx_songbannerbg; //disparition du fond de la baniere
		//OSL_IMAGE *m_img_songbannerbg;
		//Anim *m_anim_songbannerbg;
		
		//sert a faire clignoter la songbanner
		int m_frame_counter;
		bool m_frame_even;
		
		
		//les cercles
		OSL_IMAGE *m_img_circle_blue_small;
		OSL_IMAGE *m_img_circle_blue_big;
		OSL_IMAGE *m_img_circle_red_small;
		OSL_IMAGE *m_img_circle_red_big;
		OSL_IMAGE *m_img_circle_green_small;
		OSL_IMAGE *m_img_circle_green_big;
		TranslationEffect *m_fx_circle_blue;
		TranslationEffect *m_fx_circle_red;
		TranslationEffect *m_fx_circle_green;
		
		bool m_translation_fast;
		float m_speedFast;
		float m_speedSlow;
		bool m_circles_big;
			
		AbstractPiece *randomPiece();
		void drawPairXY(Pair *pair, int x, int y, GameGrid *grid, SerialLocalEffect *fx_transl=NULL, AbstractLocalEffect *fx1=NULL);
		void drawPieceXY(AbstractPiece *piece, int x, int y, AbstractLocalEffect *fx_transl=NULL);
		void drawCircles();
		int calculateTranslationTime(int x0, int y0, int x1, int y1);
		void changeCirclesSpeed();
		//Anim *animPieceFromColor(int color);
		//Anim *animExplosionFromColor(int color);
		int rankFromColor(int color);
		MCOContentSimpleAnimated *mcoContentFromColor(int color);
		void initAnimsPieces(Properties *props);
		void initAnimsCorners();
		TranslationEffect *nextTranslationEffect(TranslationEffect *old=NULL);
		
		
	public:
		PDR4CLevelDjbeat(int difficulty=GAMEDEF_DIFFICULTY_NORMAL, int nbPlayers=1);
		~PDR4CLevelDjbeat();
		
		//hérité:
		int getDropSpeed(); //la vitesse a un instant t (elle peut etre modifiée par le scénario)
		int getUpSpeed(); //la vitesse ou les lignes montent a un instant t
		int getFastDropSpeed();
		void draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer);
		string getCode();
		
		void buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid);
		
		bool getAddLineEvent(); //dit quand une ligne doit monter, dû au scénario
		
		bool finished();
		
		list<AbstractPiece *> *getNextLine();
		
		//surchargé pour avoir les bonnes pieces
		Pair *getNewPair();
		
		float getDuration();
		
		void reset();
};

#endif /*PDR4CLEVELDJBEAT_H_*/
