#include "code/game/levels/impl/mechanicalgroundwork/PDR4CLevelMecha.h"
#include "code/framework/Pos.h"
#include "code/game/gamedef.h"
#include "code/framework/utils.h"
#include <oslib/oslib.h>
#include "code/game/pieces/abstract/AbstractPiece.h"
#include "code/framework/Properties.h"
#include "code/game/pieces/impl/PieceSimple.h"
#include "code/framework/anim/Anim.h"
#include "code/game/levels/utils/MCOContentSimple.h"
#include "code/game/levels/impl/mechanicalgroundwork/PDR4CMechaMCOContentCombo.h"
#include "code/framework/check/loaders.h"

using namespace utils;

#define PDR4C_LEVEL_MECHA_COLOR_BROWN RGB(80,40,0)
#define PDR4C_LEVEL_MECHA_COLOR_RED RGB(210,32,0)
#define PDR4C_LEVEL_MECHA_COLOR_DARKGREY RGB(20,0,0)
#define PDR4C_LEVEL_MECHA_COLOR_LIGHTGREY RGB(180,180,170)
#define PDR4C_LEVEL_MECHA_DURATION 344000.0f

string PDR4CLevelMecha::getCode()
{
	return "pdr4cmecha";
}

PDR4CLevelMecha::PDR4CLevelMecha(int difficulty, int nbPlayers):AbstractLevel(0, -1, difficulty, nbPlayers)
{		
	Properties *props=new Properties("res/pdr4c/levels/mechanicalgroundwork/mecha.properties");

	m_songbanner=new SongBanner(GAMEDEF_SONGBANNER_Y, "res/fonts/bmpFonts/russelsquare/15/nosmooth/", "normal", "italic", props->getStrValue("songbanner.text1"), props->getStrValue("songbanner.text2", ""), props->getStrValue("songbanner.text3", ""), props->getStrValue("songbanner.text4", ""));
	m_songbanner_color=PDR4C_LEVEL_MECHA_COLOR_RED;

	m_comboDrawer=new PDR4CComboNumberDrawer(PDR4C_LEVEL_MECHA_COLOR_LIGHTGREY, PDR4C_LEVEL_MECHA_COLOR_DARKGREY, PDR4C_LEVEL_MECHA_COLOR_RED, PDR4C_LEVEL_MECHA_COLOR_BROWN, PDR4C_LEVEL_MECHA_COLOR_LIGHTGREY, true);
	
	m_scenario=new EventsScenario("res/pdr4c/levels/mechanicalgroundwork/mecha.scenario");
	
	m_music=fwkLoadSoundFileMP3("res/pdr4c/levels/mechanicalgroundwork/mechanicalgroundwork.mp3", OSL_FMT_STREAM);
	
	m_addline=3;
	
	m_difficulty=difficulty;
	if(m_difficulty==GAMEDEF_DIFFICULTY_EASY)
	{
		debug("difficulty is easy");
		m_nbColors=props->getIntValue("easy.colors");
		m_dropspeed_slow=props->getIntValue("easy.dropspeed.slow");
		m_dropspeed_fast=props->getIntValue("easy.dropspeed.fast");
		m_linepattern=new LinePattern("res/pdr4c/levels/mechanicalgroundwork/mecha_easy.pattern");
		
		m_linegauge=fwkLoadImageFilePNG("res/pdr4c/levels/mechanicalgroundwork/linegauge20.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
		m_linegauge_total=props->getIntValue("easy.linegauge.total");
		m_linegauge_lineadd=props->getIntValue("easy.linegauge.lines.add");
		m_linegauge_itemadd=props->getIntValue("easy.linegauge.item.add");
		
		m_imgLineLeds=fwkLoadImageFilePNG("res/pdr4c/levels/mechanicalgroundwork/line1.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
		oslImageSetPosition(m_imgLineLeds, 419, 160);
	}
	if(m_difficulty==GAMEDEF_DIFFICULTY_NORMAL)
	{
		debug("difficulty is normal");
		m_nbColors=props->getIntValue("normal.colors");
		m_dropspeed_slow=props->getIntValue("normal.dropspeed.slow");
		m_dropspeed_fast=props->getIntValue("normal.dropspeed.fast");
		m_linepattern=new LinePattern("res/pdr4c/levels/mechanicalgroundwork/mecha_normal.pattern");
		
		m_linegauge=fwkLoadImageFilePNG("res/pdr4c/levels/mechanicalgroundwork/linegauge20.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
		m_linegauge_total=props->getIntValue("normal.linegauge.total");
		m_linegauge_lineadd=props->getIntValue("normal.linegauge.lines.add");
		m_linegauge_itemadd=props->getIntValue("normal.linegauge.item.add");
		
		m_imgLineLeds=fwkLoadImageFilePNG("res/pdr4c/levels/mechanicalgroundwork/line2.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
		oslImageSetPosition(m_imgLineLeds, 419, 160);
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_HARD)
	{
		debug("difficulty is hard");
		m_nbColors=props->getIntValue("hard.colors");
		m_dropspeed_slow=props->getIntValue("hard.dropspeed.slow");
		m_dropspeed_fast=props->getIntValue("hard.dropspeed.fast");
		m_linepattern=new LinePattern("res/pdr4c/levels/mechanicalgroundwork/mecha_hard.pattern");
		
		m_linegauge=fwkLoadImageFilePNG("res/pdr4c/levels/mechanicalgroundwork/linegauge18.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
		m_linegauge_total=props->getIntValue("hard.linegauge.total");
		m_linegauge_lineadd=props->getIntValue("hard.linegauge.lines.add");
		m_linegauge_itemadd=props->getIntValue("hard.linegauge.item.add");
		
		m_imgLineLeds=fwkLoadImageFilePNG("res/pdr4c/levels/mechanicalgroundwork/line2.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
		oslImageSetPosition(m_imgLineLeds, 419, 170);
	}
	m_dropspeed=m_dropspeed_slow;
	
	oslImageSetCenter(m_linegauge, 0, m_linegauge->sizeY-1);
	oslImageSetPosition(m_linegauge, 400, 262);
	
	m_linegauge_current=0;
		
	m_animBlue=new Anim(10, "helice", "res/pdr4c/levels/mechanicalgroundwork/pieces/helice/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animRed=new Anim(10, "roue", "res/pdr4c/levels/mechanicalgroundwork/pieces/roue/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animYellow=new Anim(10, "led", "res/pdr4c/levels/mechanicalgroundwork/pieces/led/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animPurple=new Anim(10, "ressort", "res/pdr4c/levels/mechanicalgroundwork/pieces/ressort/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animGreen=new Anim(10, "cle", "res/pdr4c/levels/mechanicalgroundwork/pieces/cle/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animOrange=new Anim(10, "rock", "res/pdr4c/levels/mechanicalgroundwork/pieces/rock/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animBlue->setReferencePointCenter();
	m_animRed->setReferencePointCenter();
	m_animYellow->setReferencePointCenter();
	m_animPurple->setReferencePointCenter();
	m_animGreen->setReferencePointCenter();
	m_animOrange->setReferencePointCenter();
	
	
	m_animComboExplode=new Anim(10, "combo", "res/pdr4c/levels/mechanicalgroundwork/pieces/combo/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animComboExplode->setReferencePointCenter();
	m_animComboChange=new Anim(5, "cc", "res/pdr4c/levels/mechanicalgroundwork/pieces/combo/change/", 8, OSL_PF_8888, 1);
	m_animComboChange->setReferencePointCenter();
	
	m_imgComboUnits=new OSL_IMAGE *[10]; //tableau de pointeurs
	for(int i=0;i<10;i++)
	{
		m_imgComboUnits[i]=fwkLoadImageFilePNG("res/pdr4c/levels/mechanicalgroundwork/pieces/combo/numbers/units/cu"+toString(i)+".png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
		oslImageSetCenter(m_imgComboUnits[i], 0, 20);
	}
	
	m_imgComboTens=new OSL_IMAGE *[6]; //tableau de pointeurs
	for(int i=0;i<6;i++)
	{
		m_imgComboTens[i]=fwkLoadImageFilePNG("res/pdr4c/levels/mechanicalgroundwork/pieces/combo/numbers/tens/ct"+toString(i)+".png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
		oslImageSetCenter(m_imgComboTens[i], 0, 20);
	}
	m_imgComboUndefined=fwkLoadImageFilePNG("res/pdr4c/levels/mechanicalgroundwork/pieces/combo/numbers/undefined.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslSetImageRotCenter(m_imgComboUndefined);
	
	
	m_imgGauge=fwkLoadImageFilePNG("res/pdr4c/levels/mechanicalgroundwork/gauge.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetCenter(m_imgGauge, 0, m_imgGauge->sizeY-1);
	oslImageSetPosition(m_imgGauge, 378, 262);
	
	m_imgGaugeItem=fwkLoadImageFilePNG("res/pdr4c/levels/mechanicalgroundwork/gauge_item.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetCenter(m_imgGaugeItem, 0, m_imgGaugeItem->sizeY-1);
	
	m_imgGrid=fwkLoadImageFilePNG("res/pdr4c/levels/mechanicalgroundwork/grid.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_imgGrid, 112, 8);
	
	m_imgNext=fwkLoadImageFilePNG("res/pdr4c/levels/mechanicalgroundwork/next.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_imgNext, 43, 8);
	
	m_imgWheels=fwkLoadImageFilePNG("res/pdr4c/levels/mechanicalgroundwork/wheels.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetCenter(m_imgWheels, 0, m_imgWheels->sizeY-1);
	oslImageSetPosition(m_imgWheels, 0, 271);
	
	m_anim_electricity=new Anim(7, "electricity", "res/pdr4c/levels/mechanicalgroundwork/electricity/", 11, OSL_PF_8888, 11);
	m_anim_electricity->setReferencePointCenter();
	
	m_anim_ledLineSuppl=new Anim("res/pdr4c/levels/mechanicalgroundwork/led_on.png", 15.0f, OSL_PF_8888, 1);
	m_anim_ledLineSuppl->setX(94);
	m_anim_ledLineSuppl->setY(65);
	
	m_anim_ledElectricity=new Anim("res/pdr4c/levels/mechanicalgroundwork/led_on.png", 11.0f*11.0f, OSL_PF_8888, 1);
	m_anim_ledElectricity->setX(94);
	m_anim_ledElectricity->setY(78);
	
	m_anim_ledRotation=new Anim("res/pdr4c/levels/mechanicalgroundwork/led_on.png", 60.0f, OSL_PF_8888, 1);
	m_anim_ledRotation->setX(94);
	m_anim_ledRotation->setY(91);
	
	m_imgEvents=fwkLoadImageFilePNG("res/pdr4c/levels/mechanicalgroundwork/events.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_imgEvents, 82, 65);
	m_longRotation=false;
	
	m_imgLedOn=fwkLoadImageFilePNG("res/pdr4c/levels/mechanicalgroundwork/led_on.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	//imageSetPosition(m_imgLedOn, 94, 104); //pour la rotation longue
	m_pos_ledLongRotation=new Pos(94,104);
	
	m_pos_ledBatteryCharging=new Pos(94, 117);
	m_pos_ledBatteryLow=new Pos(94, 130);
	m_delayer_blink=new Delayer(40);
	m_blink_on=false;
	
	m_imgSpeedZen=fwkLoadImageFilePNG("res/pdr4c/levels/mechanicalgroundwork/arrowdown_off.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_imgSpeedZen, 94, 145);
	
	m_imgSpeedFast=fwkLoadImageFilePNG("res/pdr4c/levels/mechanicalgroundwork/arrowdown_on.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_imgSpeedFast, 94, 145);
	
	m_imgSpeedDown=fwkLoadImageFilePNG("res/pdr4c/levels/mechanicalgroundwork/doublearrowdown.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_imgSpeedDown, 94, 144);
	
	m_anim_turnHelice=new Anim(3, "helice", "res/pdr4c/levels/mechanicalgroundwork/turnhelice/", 12, OSL_PF_8888, 5);
	m_anim_turnHelice->setReferencePointCenter();
	
	m_anim_turnRoue=new Anim(3, "roue", "res/pdr4c/levels/mechanicalgroundwork/turnroue/", 12, OSL_PF_8888, -1);
	m_anim_turnRoue->setReferencePointCenter();
	
	m_anim_pause=new Anim("res/pdr4c/levels/mechanicalgroundwork/pause/pause1.png", 100);
	for(int i=0;i<8;i++)
		m_anim_pause->addImage("res/pdr4c/levels/mechanicalgroundwork/pause/pause1.png");
	m_anim_pause->addImage("res/pdr4c/levels/mechanicalgroundwork/pause/pause2.png");
	m_anim_pause->addImage("res/pdr4c/levels/mechanicalgroundwork/pause/pause3.png");
	m_anim_pause->addImage("res/pdr4c/levels/mechanicalgroundwork/pause/pause4.png");
	m_anim_pause->addImage("res/pdr4c/levels/mechanicalgroundwork/pause/pause5.png");
	m_anim_pause->addImage("res/pdr4c/levels/mechanicalgroundwork/pause/pause6.png");
	m_anim_pause->setReferencePointCenter();
	m_anim_pause->start();
	
	m_songbanner_bg=fwkLoadImageFilePNG("res/pdr4c/levels/mechanicalgroundwork/songbannerbg.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetCenter(m_songbanner_bg, 0, m_songbanner_bg->sizeY-1);
	m_fx_songbannerbg=new SerialLocalEffect(1);
	int time=m_songbanner->getTotalTimePresent();
	m_fx_songbannerbg->addEffect(new AppearDownEffect(7));
	m_fx_songbannerbg->addEffect(new WaitLocalEffect(time-7));
	m_fx_songbannerbg->addEffect(new VanishDownEffect(30));
	
	delete props;
}

void PDR4CLevelMecha::reset()
{
	m_songbanner->reset();
	m_comboDrawer->reset();
	m_scenario->reset();
	oslStopSound(m_music);
	m_addline=3;
	m_linepattern->reset();
	m_dropspeed=m_dropspeed_slow;
	m_linegauge_current=0;
	m_longRotation=false;
	m_fx_songbannerbg->stop();
	m_fx_songbannerbg->reset();
}

PDR4CLevelMecha::~PDR4CLevelMecha()
{
	delete m_songbanner;
	delete m_comboDrawer;
	delete m_scenario;
	oslDeleteSound(m_music);
	delete m_linepattern;
	
	delete m_animBlue;
	delete m_animRed;
	delete m_animYellow;
	delete m_animPurple;
	delete m_animGreen;
	delete m_animOrange;
	
	delete m_animComboExplode;
	delete m_animComboChange;
	
	for(int i=0;i<10;i++)
	{
		oslDeleteImage(m_imgComboUnits[i]);
	}
	delete[] m_imgComboUnits;
	for(int i=0;i<6;i++)
	{
		oslDeleteImage(m_imgComboTens[i]);
	}
	delete[] m_imgComboTens;
	oslDeleteImage(m_imgComboUndefined);
	
	oslDeleteImage(m_imgGauge);
	oslDeleteImage(m_imgGaugeItem);
	oslDeleteImage(m_imgGrid);
	oslDeleteImage(m_imgNext);
	oslDeleteImage(m_imgWheels);
	oslDeleteImage(m_linegauge);
	oslDeleteImage(m_imgLineLeds);
	
	delete m_anim_electricity;
	
	delete m_anim_ledLineSuppl;
	delete m_anim_ledElectricity;
	delete m_anim_ledRotation;
	oslDeleteImage(m_imgEvents);
	oslDeleteImage(m_imgLedOn);
	
	delete m_pos_ledLongRotation;
	delete m_pos_ledBatteryCharging;
	delete m_pos_ledBatteryLow;
	delete m_delayer_blink;
	
	oslDeleteImage(m_imgSpeedZen);
	oslDeleteImage(m_imgSpeedFast);
	oslDeleteImage(m_imgSpeedDown);
	
	delete m_anim_turnHelice;
	delete m_anim_turnRoue;
	
	delete m_anim_pause;
	
	delete m_fx_songbannerbg;
	oslDeleteImage(m_songbanner_bg);
}


//dessin avec une paire
void PDR4CLevelMecha::draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	if(!isPaused())
	{
		//currentPair->draw(grid, fx_trans, APPLY_FX_ON_NONE);
		Pos gridPos=currentPair->getGridPos(0);
		Pos pixPos=grid->getContentPosition(gridPos.x, gridPos.y);
		drawPairXY(currentPair, pixPos.x, pixPos.y, grid, fx_trans);
	}
	drawAfter(grid, nextPair, cc);
}

//dessin avec 2 pieces
void PDR4CLevelMecha::draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	
	Pos pixPosConnected=grid->getContentPosition(pieceConnectedPos.x, pieceConnectedPos.y);
	Pos pixPosUnconnected=grid->getContentPosition(pieceUnconnectedPos.x, pieceUnconnectedPos.y);
	
	drawPieceXY(pieceConnected, pixPosConnected.x, pixPosConnected.y);
	drawPieceXY(pieceUnconnected, pixPosUnconnected.x, pixPosUnconnected.y, fx_trans);
	
	drawAfter(grid, nextPair, cc);
}

//dessin avec rien
void PDR4CLevelMecha::draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	drawAfter(grid, nextPair, cc);
}

void PDR4CLevelMecha::drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	oslClearScreen(PDR4C_LEVEL_MECHA_COLOR_LIGHTGREY);
	
	oslDrawImage(m_imgWheels);
	oslDrawImage(m_imgGauge);
	oslDrawImage(m_imgGrid);
	oslDrawImage(m_imgNext);
	oslDrawImage(m_imgEvents);
	
	if(m_anim_ledElectricity->isRunning())
	{
		m_anim_ledElectricity->draw();
	}
	if(m_anim_ledLineSuppl->isRunning())
	{
		m_anim_ledLineSuppl->draw();
	}
	if(m_anim_ledRotation->isRunning())
	{
		m_anim_ledRotation->draw();
	}
	if(m_longRotation)
	{
		oslDrawImageXY(m_imgLedOn, m_pos_ledLongRotation->x, m_pos_ledLongRotation->y);
	}
	if(scePowerIsBatteryCharging())
	{
		oslDrawImageXY(m_imgLedOn, m_pos_ledBatteryCharging->x, m_pos_ledBatteryCharging->y);
	}
	if(scePowerIsLowBattery() && m_blink_on)
	{
		oslDrawImageXY(m_imgLedOn, m_pos_ledBatteryLow->x, m_pos_ledBatteryLow->y);
	}
	if(m_dropspeed==m_dropspeed_slow)
	{
		oslDrawImage(m_imgSpeedZen);
	}
	else if(m_dropspeed==m_dropspeed_fast)
	{
		oslDrawImage(m_imgSpeedFast);
	}
	
	//oslDrawFillRect(384, 262-int((cc->getTime()*252)/cc->getTimeTotalAvailable()), 398, 262, PDR4C_LEVEL_MECHA_COLOR_RED);
	
			
	if(isPaused())
	{
		m_anim_pause->drawXY(240,136);
	}
	else
	{
		//dessin du contenu de la grille
		for(int i=0;i<grid->getNbCols();i++)
		{
			for(int j=0;j<grid->getNbLines();j++)
			{
				MatrixContentObject *mco=grid->getAt(i,j);
				if(mco->getPiece()!=NULL)
				{
					Pos pos=grid->getContentPosition(i, j);
					AbstractPiece *p=mco->getPiece();
					
					if(p->getType()==PIECE_TYPE_NORMAL || p->getType()==PIECE_TYPE_COMBO)
					{
						if(mco->getMCOContent()!=NULL)
						{
							if(!p->destructing())
							{
								if(p->getColor()==PIECE_COLOR_BLUE) //helice
								{
									if(m_anim_turnHelice->isRunning())
									{
										m_anim_turnHelice->drawXY(pos.x, pos.y);
									}
									else
									{
										mco->getMCOContent()->drawXY(pos.x,pos.y);
									}
								}
								else if(p->getColor()==PIECE_COLOR_RED) //roue dentée
								{
									if(m_longRotation)
									{
										m_anim_turnRoue->drawXY(pos.x, pos.y);
									}
									else
									{
										mco->getMCOContent()->drawXY(pos.x,pos.y);
									}
								}
								else
								{
									mco->getMCOContent()->drawXY(pos.x,pos.y);
									//ressort
									if(m_anim_electricity->isRunning() && p->getColor()==PIECE_COLOR_PURPLE)
									{
										m_anim_electricity->drawXY(pos.x,pos.y);
									}
								}
							}
							else //destructing
							{
								mco->getMCOContent()->drawXY(pos.x,pos.y);
							}
						}
					}
					else
					{
						p->drawXY(pos.x,pos.y);
					}
				}
			}	
		}
	}
	
}

void PDR4CLevelMecha::drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	if(!isPaused())
		drawPairXY(nextPair, 68, 31, grid);	
		
		
	//la jauge de combo
	int nbItems=int( (49*cc->getTime()) / cc->getTimeTotalAvailable() )+1;
	int x=381;
	int y=259;
	
	if(cc->getTime()>0)
	{
		for(int i=0;i<nbItems;i++)
		{
			oslDrawImageXY(m_imgGaugeItem, x, y);
			y-=5;
		}
	}
	
	
	//la jauge de line+
	x+=22;
	y=259;
	oslDrawImage(m_linegauge);
	for(int i=0;i<m_linegauge_current;i++)
	{
		oslDrawImageXY(m_imgGaugeItem, x, y);
		y-=5;
	}
	oslDrawImage(m_imgLineLeds);
	
	m_comboDrawer->drawXY(240, 180); 
	//oslDrawString(10, 190, (toString(m_scenario->getTime())).c_str());
	
	if(m_fx_songbannerbg->isRunning())
	{
		fx::drawXY(m_songbanner_bg, 0, 271, m_fx_songbannerbg);
		
		oslSetAlpha(OSL_FX_TINT, m_songbanner_color);
		m_songbanner->draw();
		oslSetAlpha(OSL_FX_DEFAULT,0);
	}
	
}

void PDR4CLevelMecha::start_specific()
{
	oslPlaySound(m_music, 0);
	m_scenario->restart();
	m_fx_songbannerbg->start();
}

void PDR4CLevelMecha::update_pause_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
{
	m_anim_pause->update();
}

//appelé uniquement si le level n'est pas en pause.
void PDR4CLevelMecha::update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
{
	//cas special pour la pdrom: on utilise des anims séparées
	for(int i=0;i<grid->getNbCols();i++)
	{
		for(int j=0;j<grid->getNbLines();j++)
		{
			MatrixContentObject *mco=grid->getAt(i,j);
			AbstractPiece *p=mco->getPiece();
			if(p!=NULL)
			{
				if(p->getType()==PIECE_TYPE_NORMAL)
				{
					//on cree si une piece n'en a pas
					if(mco->getMCOContent()==NULL)
					{
						Anim *anim=animFromColor(p->getColor());
						mco->setMCOContent(new MCOContentSimple(new Anim(*anim)));
					}				
				}
				else if(p->getType()==PIECE_TYPE_COMBO)
				{
					//remplir si c'est une piece combo
					if(mco->getMCOContent()==NULL)
					{
						int nbCombos=p->specialEvent(PIECE_EVENT_COMBO_GET);
						Anim *shine=new Anim(*m_animComboChange);
						Anim *explode=new Anim(*m_animComboExplode);
						mco->setMCOContent(new PDR4CMechaMCOContentCombo(explode, shine, m_imgComboUnits, m_imgComboTens, m_imgComboUndefined, nbCombos));
					}
					
					//on met a jour le nombre de combos
					int nbCombosPiece=p->specialEvent(PIECE_EVENT_COMBO_GET);
					int nbCombosMCOContent=mco->getMCOContent()->specialEvent(PDR4C_LEVEL_MECHA_EVENT_COMBO_GETNB);
					if(nbCombosPiece!=nbCombosMCOContent)
					{
						mco->getMCOContent()->specialEvent(PDR4C_LEVEL_MECHA_EVENT_COMBO_DECREMENT);
					}
				}
				//on démarre si la piece explose
				if(mco->isColorExploding() || mco->isShapeExploding() || mco->isSpecialExploding())
					mco->getMCOContent()->start();
			}
			else//mco->getPiece()==NULL
			{
				if(mco->getMCOContent()!=NULL)
				{
					mco->deleteMCOContent();
				}
			}
			
			if(mco->getMCOContent()!=NULL) //peu importe le cas, on update
			{
				mco->getMCOContent()->update();
			}
		}
	}
	
	m_fx_songbannerbg->update();
	m_anim_turnHelice->update();
	m_anim_turnRoue->update();
	m_songbanner->update();
	m_comboDrawer->sync(cc);
	m_comboDrawer->update();
	m_scenario->update();
	m_anim_electricity->update();
	m_anim_ledLineSuppl->update();
	m_anim_ledElectricity->update();
	m_anim_ledRotation->update();
	m_delayer_blink->update();
	if(m_delayer_blink->get())
		m_blink_on=!m_blink_on;
	/*if(m_songbanner_delayer->get())
	{
		if(m_songbanner_color==PDR4C_LEVEL_MECHA_COLOR_DARKGREY)
			m_songbanner_color=PDR4C_LEVEL_MECHA_COLOR_RED;
		//else if(m_songbanner_color==PDR4C_LEVEL_MECHA_COLOR_RED)
		//	m_songbanner_color=PDR4C_LEVEL_MECHA_COLOR_BROWN;
		else
			m_songbanner_color=PDR4C_LEVEL_MECHA_COLOR_DARKGREY;
	}*/
	
	//evenements du scenario
	if(m_scenario->hasEvent(1)) //tu uhhh uhh (rotation)
	{
		m_anim_ledRotation->restart();
		m_anim_turnHelice->restart();
	}
	if(m_scenario->hasEvent(2)) //tac tac (+1 element pour lignes)
	{
		m_linegauge_current+=m_linegauge_itemadd;
		if(m_linegauge_current>=m_linegauge_total)
		{
			m_addline+=m_linegauge_lineadd;
			m_linegauge_current-=m_linegauge_total;
		}
		m_anim_ledLineSuppl->restart();
	}
	if(m_scenario->hasEvent(3)) //wzzzzz (electricité)
	{
		m_anim_electricity->restart();
		m_anim_ledElectricity->restart();
	}
	if(m_scenario->hasEvent(4)) //tuuuuuuuuuuuuu (rotation longue) debut
	{
		m_longRotation=true;
		m_anim_turnRoue->restart();
	}
	if(m_scenario->hasEvent(5)) //tuuuuuuuuuuuuu (rotation longue) fin
	{
		m_longRotation=false;
		m_anim_turnRoue->stop();
	}
	if(m_scenario->hasEvent(6)) //zen
	{
		m_dropspeed=m_dropspeed_slow;
	}
	if(m_scenario->hasEvent(7)) //rapide
	{
		m_dropspeed=m_dropspeed_fast;
	}
}

void PDR4CLevelMecha::pause_specific() 
{
	oslPauseSound(m_music, 1);
	m_scenario->pause();
}

void PDR4CLevelMecha::resume_specific()
{
	oslPauseSound(m_music, 0);
	m_scenario->start();
}
		

list<AbstractPiece *> *PDR4CLevelMecha::getNextLine()
{
	return m_linepattern->getNextLine();
}

bool PDR4CLevelMecha::getAddLineEvent() //dit quand une ligne doit monter, dû au scénario
{
	if(m_addline>0)
	{
		m_addline--;
		return true;
	}
	return false;
}

int PDR4CLevelMecha::getDropSpeed() //la vitesse a un instant t (elle peut etre modifiée par le scénario)
{
	return m_dropspeed;
}

int PDR4CLevelMecha::getUpSpeed() //la vitesse ou les lignes montent a un instant t
{
	return -1;
}

void PDR4CLevelMecha::buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid)
{
	if(key==OSL_KEY_TRIANGLE)
	{
				
	}
	else if(key==OSL_KEY_SQUARE)
	{
		
	}
}

bool PDR4CLevelMecha::finished()
{
	if(m_scenario->getTime()>=PDR4C_LEVEL_MECHA_DURATION)
	{
		return true;
	}
	return false;
}

float PDR4CLevelMecha::getDuration()
{
	return PDR4C_LEVEL_MECHA_DURATION;
}


Pair *PDR4CLevelMecha::getNewPair()
{	
	Pair *p=new Pair(randomPiece(), randomPiece());
	int posX=int(m_nbCols/2)-1;
	int posY=-1; 
	p->goTo(posX, posY);
	
	return p;
}



AbstractPiece *PDR4CLevelMecha::randomPiece()
{
	//suivant la difficulté, il y a + ou - de couleurs à l'écran
	int color=random(0,m_nbColors-1);
	PieceSimple *p=NULL;
	switch(color)
	{
		case 0:
		{
			p=new PieceSimple(PIECE_SHAPE_CIRCLE, PIECE_COLOR_BLUE);
			break;
		}
		case 1:
		{
			p=new PieceSimple(PIECE_SHAPE_SQUARE, PIECE_COLOR_RED);
			break;
		}
		case 2:
		{
			p=new PieceSimple(PIECE_SHAPE_DIAMOND, PIECE_COLOR_YELLOW);	
			break;
		}
		case 3:
		{
			p=new PieceSimple(PIECE_SHAPE_STAR, PIECE_COLOR_PURPLE);	
			break;
		}
		case 4:
		{
			p=new PieceSimple(PIECE_SHAPE_CROSS, PIECE_COLOR_GREEN);
			break;
		}
		case 5:
		{
			p=new PieceSimple(PIECE_SHAPE_TRIANGLE, PIECE_COLOR_ORANGE);
			break;
		}
	}
	
	
	
	return p;
}

void PDR4CLevelMecha::drawPairXY(Pair *pair, int x, int y, GameGrid *grid, AbstractLocalEffect *fx_transl, AbstractLocalEffect *fx1)
{
	//suivant l'orientation, la 2eme piece est a un autre endroit
	int x2=x, y2=y;
	switch(pair->getOrientation())
	{
		case ORIENTATION_EAST:
			x2=x+grid->getCellWidth();
			break;	
		
		case ORIENTATION_NORTH:
			y2=y-grid->getCellHeight();
			break;
			
		case ORIENTATION_SOUTH:
			y2=y+grid->getCellHeight();
			break;
			
		case ORIENTATION_WEST:
			x2=x-grid->getCellWidth();
			break;
	}
		
	//fx::drawXY(animFromColor(pair->getPiece(1)->getColor())->getImage(), x2, y2, fx_transl, fx1);
	//fx::drawXY(animFromColor(pair->getPiece(0)->getColor())->getImage(), x, y, fx_transl, fx1);
	drawPieceXY(pair->getPiece(1), x2, y2, fx_transl);
	drawPieceXY(pair->getPiece(0), x, y, fx_transl);
}

void PDR4CLevelMecha::drawPieceXY(AbstractPiece *piece, int x, int y, AbstractLocalEffect *fx_transl)
{
	if(piece->getColor()==PIECE_COLOR_BLUE && m_anim_turnHelice->isRunning())
	{
		fx::drawXY(m_anim_turnHelice->getImage(), x, y, fx_transl);
		//m_anim_turnHelice->drawXY(x, y, fx_transl);
	}
	else if(piece->getColor()==PIECE_COLOR_RED && m_longRotation) //roue dentée
	{
		fx::drawXY(m_anim_turnRoue->getImage(), x, y, fx_transl);
	}
	else
	{
		fx::drawXY(animFromColor(piece->getColor())->getImage(), x, y, fx_transl);
		if(m_anim_electricity->isRunning() && piece->getColor()==PIECE_COLOR_PURPLE)
		{
			fx::drawXY(m_anim_electricity->getImage(), x, y, fx_transl);
			//m_anim_electricity->drawXY(x,y,fx_transl);
		}
	}
}

Anim *PDR4CLevelMecha::animFromColor(int color)
{
	switch(color)
	{
		case PIECE_COLOR_BLUE:
		{
			return m_animBlue;
			break;
		}
		case PIECE_COLOR_RED:
		{
			return m_animRed;
			break;
		}
		case PIECE_COLOR_YELLOW:
		{
			return m_animYellow;
			break;
		}
		case PIECE_COLOR_PURPLE:
		{
			return m_animPurple;
			break;
		}
		case PIECE_COLOR_GREEN:
		{
			return m_animGreen;
			break;
		}
		case PIECE_COLOR_ORANGE:
		{
			return m_animOrange;
			break;
		}
	}
	
	return NULL;
}








