#ifndef PDR4CLEVELMECHA_H_
#define PDR4CLEVELMECHA_H_

#include <oslib/oslib.h>
#include "code/game/levels/abstract/AbstractLevel.h"
#include "code/game/grid/GameGrid.h"
#include "code/game/Pair.h"
#include "code/game/levels/LinePattern.h"
#include "code/game/pieces/abstract/AbstractPiece.h"
#include <list>
#include "code/game/SongBanner.h"
#include "code/game/combos/ComboCounter.h"
#include "code/game/combos/pdr4c/PDR4CComboNumberDrawer.h"
#include "code/game/EventsScenario.h"
#include "code/framework/anim/Anim.h"

using namespace std;

class PDR4CLevelMecha : public AbstractLevel
{
	private:
		LinePattern *m_linepattern;
			
		void start_specific(); //démarre la musique, le timer etc
		void update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans);
		void update_pause_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans);
		void pause_specific(); 
		void resume_specific();
				
		SongBanner *m_songbanner;
		int m_songbanner_color;
		
		PDR4CComboNumberDrawer * m_comboDrawer;
		
		EventsScenario *m_scenario;
		OSL_SOUND* m_music;
		
		
		int m_dropspeed; //current
		int m_dropspeed_slow;
		int m_dropspeed_fast;
		
		
		int m_addline; //ajoute 1 qd on veut qu'une ligne monte
		
								
		void drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc);
		void drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc);
		
				
		int m_difficulty;
		int m_nbColors; //nombre de couleurs pour les pieces
		
		//animations qui remplacent les pieces normales
		Anim *m_animBlue,*m_animRed,*m_animYellow,*m_animPurple,*m_animGreen,*m_animOrange;
		//animations et images qui remplacent les pieces combo
		Anim *m_animComboExplode, *m_animComboChange;
		OSL_IMAGE **m_imgComboUnits; //tableau de pointeurs
		OSL_IMAGE **m_imgComboTens; //tableau de pointeurs
		OSL_IMAGE *m_imgComboUndefined;
		
		OSL_IMAGE *m_imgGauge;
		OSL_IMAGE *m_imgGaugeItem;
		OSL_IMAGE *m_imgGrid;
		OSL_IMAGE *m_imgNext;
		OSL_IMAGE *m_imgWheels;
		OSL_IMAGE *m_imgLineLeds;
		
		Anim *m_anim_electricity;
		Anim *m_anim_ledLineSuppl;
		Anim *m_anim_ledElectricity;
		Anim *m_anim_ledRotation;
		OSL_IMAGE *m_imgEvents;
		OSL_IMAGE *m_imgLedOn;
		bool m_longRotation;
		Pos *m_pos_ledLongRotation;
		Pos *m_pos_ledBatteryCharging;
		Pos *m_pos_ledBatteryLow;
		Delayer *m_delayer_blink;
		bool m_blink_on;
		
		OSL_IMAGE *m_imgSpeedZen;
		OSL_IMAGE *m_imgSpeedFast;
		OSL_IMAGE *m_imgSpeedDown;
		
		
		OSL_IMAGE *m_linegauge;
		int m_linegauge_total;
		int m_linegauge_lineadd;
		int m_linegauge_itemadd;
		int m_linegauge_current;
		
		Anim *m_anim_turnHelice;
		Anim *m_anim_turnRoue;
		Anim *m_anim_pause;
		
		OSL_IMAGE *m_songbanner_bg;
		SerialLocalEffect *m_fx_songbannerbg;
		
		AbstractPiece *randomPiece();
		void drawPairXY(Pair *pair, int x, int y, GameGrid *grid, AbstractLocalEffect *fx_transl=NULL, AbstractLocalEffect *fx1=NULL);
		void drawPieceXY(AbstractPiece *piece, int x, int y, AbstractLocalEffect *fx_transl=NULL);
		Anim *animFromColor(int color);
		
		
				
	public:
		PDR4CLevelMecha(int difficulty=GAMEDEF_DIFFICULTY_NORMAL, int nbPlayers=1);
		~PDR4CLevelMecha();
		
		//hérité:
		int getDropSpeed(); //la vitesse a un instant t (elle peut etre modifiée par le scénario)
		int getUpSpeed(); //la vitesse ou les lignes montent a un instant t
		void draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer);
		string getCode();
		
		void buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid);
		
		bool getAddLineEvent(); //dit quand une ligne doit monter, dû au scénario
		
		bool finished();
		
		list<AbstractPiece *> *getNextLine();
		
		//surchargé pour avoir les bonnes pieces
		Pair *getNewPair();
		
		float getDuration();
		
		void reset();
};

#endif /*PDR4CLEVELMECHA_H_*/
