#include "LevelProjectSexy.h"
#include <string>
#include "code/framework/Properties.h"
#include "code/game/combos/ComboNumberDrawer.h"
#include "code/game/levels/utils/MCOContentSimple.h"
#include "code/game/levels/utils/MCOContentCombo.h"
#include "code/game/pieces/impl/PieceSimple.h"
#include "code/game/pieces/impl/PieceJoker.h"
#include "code/framework/utils.h"
#include "code/game/levels/impl/projectsexy/SexyMCOContentParticles.h"
#include "code/framework/check/loaders.h"

using namespace utils;
using namespace std;

#define LEVEL_SEXY_NEXTPAIR_X 62
#define LEVEL_SEXY_NEXTPAIR_Y 34
#define LEVEL_SEXY_DURATION 478000.0f

#define LEVEL_SEXY_COMBO_COLOR_NORMAL_IN RGB(255,255,255)
#define LEVEL_SEXY_COMBO_COLOR_NORMAL_OUT RGB(0,0,0)
#define LEVEL_SEXY_COMBO_COLOR_HOT1 RGB(255,93,0)
#define LEVEL_SEXY_COMBO_COLOR_HOT2 RGB(233,130,233)
#define LEVEL_SEXY_COMBO_COLOR_HOT3 RGB(168,0,203)

string LevelProjectSexy::getCode()
{
	return "sexy";
}

LevelProjectSexy::LevelProjectSexy(int difficulty, int nbPlayers):AbstractLevel(0, -1, difficulty, nbPlayers)
{
	Properties *props=new Properties("res/levels/projectsexy/sexy.properties");
	
	m_songbanner=new SongBanner(GAMEDEF_SONGBANNER_Y, "res/fonts/bmpFonts/russelsquare/14/", "normal", "italic", props->getStrValue("songbanner.text1"), props->getStrValue("songbanner.text2"), props->getStrValue("songbanner.text3"), props->getStrValue("songbanner.text4"));
	m_scenario=new EventsScenario("res/levels/projectsexy/sexy.scenario");
	m_music=fwkLoadSoundFileMP3("res/levels/projectsexy/sexy.mp3", OSL_FMT_STREAM);
	m_img_kti=fwkLoadImageFilePNG("res/levels/projectsexy/kti.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888);
	oslImageCenterScreen(m_img_kti);
	m_img_grid=fwkLoadImageFilePNG("res/levels/projectsexy/overlay.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_img_grid, 45, 9); //113, 9
	m_difficulty=difficulty;
	m_img_pause=fwkLoadImageFilePNG("res/levels/projectsexy/pause.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_img_pause, 0, 0);
	
	
	m_comboDrawer=new ComboNumberDrawer(	LEVEL_SEXY_COMBO_COLOR_NORMAL_IN, LEVEL_SEXY_COMBO_COLOR_NORMAL_OUT, 
											LEVEL_SEXY_COMBO_COLOR_HOT1, LEVEL_SEXY_COMBO_COLOR_HOT2, LEVEL_SEXY_COMBO_COLOR_HOT3,
											"res/fonts/bmpFonts/kinkee/24/", "kinkee", "res/fonts/bmpFonts/null/", "null"
										);
	
	
	if(m_difficulty==GAMEDEF_DIFFICULTY_EASY)
	{
		m_nbColors=props->getIntValue("easy.colors");
		m_dropspeed_slow=props->getIntValue("easy.dropspeed.slow");
		m_dropspeed_medium=props->getIntValue("easy.dropspeed.medium");
		m_dropspeed_fast=props->getIntValue("easy.dropspeed.fast");
		m_upspeed_slow=props->getIntValue("easy.upspeed.slow");
		m_upspeed_medium=props->getIntValue("easy.upspeed.medium");
		m_upspeed_fast=props->getIntValue("easy.upspeed.fast");
		m_linepattern=new LinePattern("res/levels/projectsexy/sexy_easy.pattern");
		
		m_jgauge_size=props->getIntValue("easy.jokergauge.size");
		m_jgauge_add=props->getIntValue("easy.jokergauge.comboadd");
		m_jgauge_frem=props->getIntValue("easy.jokergauge.removeperframe");
		m_addline=props->getIntValue("easy.addline");
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_NORMAL)
	{
		m_nbColors=props->getIntValue("normal.colors");
		m_dropspeed_slow=props->getIntValue("normal.dropspeed.slow");
		m_dropspeed_medium=props->getIntValue("normal.dropspeed.medium");
		m_dropspeed_fast=props->getIntValue("normal.dropspeed.fast");
		m_upspeed_slow=props->getIntValue("normal.upspeed.slow");
		m_upspeed_medium=props->getIntValue("normal.upspeed.medium");
		m_upspeed_fast=props->getIntValue("normal.upspeed.fast");
		m_linepattern=new LinePattern("res/levels/projectsexy/sexy_normal.pattern");
		
		m_jgauge_size=props->getIntValue("normal.jokergauge.size");
		m_jgauge_add=props->getIntValue("normal.jokergauge.comboadd");
		m_jgauge_frem=props->getIntValue("normal.jokergauge.removeperframe");
		m_addline=props->getIntValue("normal.addline");
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_HARD)
	{
		m_nbColors=props->getIntValue("hard.colors");
		m_dropspeed_slow=props->getIntValue("hard.dropspeed.slow");
		m_dropspeed_medium=props->getIntValue("hard.dropspeed.medium");
		m_dropspeed_fast=props->getIntValue("hard.dropspeed.fast");
		m_upspeed_slow=props->getIntValue("hard.upspeed.slow");
		m_upspeed_medium=props->getIntValue("hard.upspeed.medium");
		m_upspeed_fast=props->getIntValue("hard.upspeed.fast");
		m_linepattern=new LinePattern("res/levels/projectsexy/sexy_hard.pattern");
		
		m_jgauge_size=props->getIntValue("hard.jokergauge.size");
		m_jgauge_add=props->getIntValue("hard.jokergauge.comboadd");
		m_jgauge_frem=props->getIntValue("hard.jokergauge.removeperframe");
		m_addline=props->getIntValue("hard.addline");
	}
	else
	{
		oslFatalError("LevelProjectSexy: this difficulty does not exist");
	}
	
	
	m_dropspeed=m_dropspeed_slow;
	m_upspeed=m_upspeed_slow;
	m_jgauge_current=0;
	
	initAnims();
	initFlash(props);
	initClouds(props);
	initShake(props);
	initDancer(props);
	initComboTurn(props);
	
	//piece particules
	m_blocParticles_colorFrom=RGBA(props->getIntValue("bloc.particles.color.from.r"), props->getIntValue("bloc.particles.color.from.g"), props->getIntValue("bloc.particles.color.from.b"), props->getIntValue("bloc.particles.color.from.a"));
	m_blocParticles_colorTo=RGBA(props->getIntValue("bloc.particles.color.to.r"), props->getIntValue("bloc.particles.color.to.g"), props->getIntValue("bloc.particles.color.to.b"), props->getIntValue("bloc.particles.color.to.a"));
	m_blocParticles_distance=props->getIntValue("bloc.particles.distance");
	m_blocParticles_time=props->getIntValue("bloc.particles.time");
	m_blocParticles_frames=props->getIntValue("bloc.particles.frames");
	m_blocParticles_particles=props->getIntValue("bloc.particles.particles");
	
	m_lastdraw=-1;
	m_smcocp_next1=NULL;
	m_smcocp_next2=NULL;
	m_smcocp_current1=NULL;
	m_smcocp_current2=NULL;
	m_smcocp_connected=NULL;
	m_smcocp_unconnected=NULL;
	
	m_jokerDrawer=new JokerDrawer();
	
	m_delayer=new Delayer(int(GAMEDEF_EXPLOSION_TIME));
	m_endExplosion=false;
		
	delete props;
}

void LevelProjectSexy::reset()
{
	Properties *props=new Properties("res/levels/projectsexy/sexy.properties");
	
	m_songbanner->reset();
	m_scenario->reset();
	oslStopSound(m_music);
	m_comboDrawer->reset();
	m_linepattern->reset();
	
	if(m_difficulty==GAMEDEF_DIFFICULTY_EASY)
		m_addline=props->getIntValue("easy.addline");
	else if(m_difficulty==GAMEDEF_DIFFICULTY_NORMAL)
		m_addline=props->getIntValue("normal.addline");
	else if(m_difficulty==GAMEDEF_DIFFICULTY_HARD)
		m_addline=props->getIntValue("hard.addline");
	
	m_dropspeed=m_dropspeed_slow;
	m_upspeed=m_upspeed_slow;
	m_jgauge_current=0;
	
	m_endExplosion=false;
	
	m_fx_shake->stop();
	m_anim_comboturn->setDelayTotal(m_ctspeed_slow);
	m_fx_combo->setTimeForward(m_ctintspeed_slow);
	m_fx_combo->setTimeBackward(m_ctintspeed_slow);
	
	restartFlash();
	
	delete props;
}

LevelProjectSexy::~LevelProjectSexy()
{
	delete m_songbanner;
	delete m_comboDrawer;
	delete m_scenario;
	fwkDeleteSound(m_music);
	oslDeleteImage(m_img_kti);
	oslDeleteImage(m_img_grid);
	oslDeleteImage(m_img_pause);
	delete m_linepattern;
	
	delete m_animBlue;
	delete m_animRed;
	delete m_animYellow;
	delete m_animPurple;
	delete m_animGreen;
	delete m_animOrange;
	
	delete m_animComboExplode;
	delete m_animComboChange;
	
	for(int i=0;i<=21;i++)
	{
		oslDeleteImage(m_imgCombo[i]);
	}
	delete m_fx_combo;
	
	delete[] m_imgCombo;
	
	delete[] m_flash_clouds_colorsfrom;
	delete[] m_flash_clouds_colorsto;
	delete m_fx_flash_clouds;
	delete[] m_flash_kti_colorsfrom;
	delete[] m_flash_kti_colorsto;
	delete m_fx_flash_kti;
	
	if(m_smcocp_next1!=NULL)
		delete m_smcocp_next1;
	if(m_smcocp_next2!=NULL)
		delete m_smcocp_next2;
	if(m_smcocp_current1!=NULL)
		delete m_smcocp_current1;
	if(m_smcocp_current2!=NULL)
		delete m_smcocp_current2;
	if(m_smcocp_connected!=NULL)
		delete m_smcocp_connected;
	if(m_smcocp_unconnected!=NULL)
		delete m_smcocp_unconnected;
	
	for(int i=0;i<m_nbClouds;i++)
	{
		oslDeleteImage(m_img_clouds[i]);
		delete m_fx_clouds[i];
	}
	delete [] m_img_clouds;
	delete [] m_fx_clouds;
	
	delete m_fx_shake;
	
	oslDeleteImage(m_img_dancer);
	delete m_fx_dancer;
	
	delete m_jokerDrawer;
	delete m_partgen_dancer;
	
	delete m_anim_comboturn;
	
	delete m_delayer;
	
	//enableDebug();
	//debug("LevelProjectSexy : delete ok");
	//enableDebug(false);
	
}

void LevelProjectSexy::initSmcocp(Pair *currentPair, Pair *nextPair)
{
	//debug("initsmocp");
	if(nextPair->getPiece(0)->getType()==PIECE_TYPE_NORMAL && nextPair->getPiece(0)->getColor()==PIECE_COLOR_BLUE)
	{
		if(m_smcocp_next1!=NULL)
			delete m_smcocp_next1;
		m_smcocp_next1=newParticlesMCO(animFromColor(PIECE_COLOR_BLUE));
	}
	if(nextPair->getPiece(1)->getType()==PIECE_TYPE_NORMAL && nextPair->getPiece(1)->getColor()==PIECE_COLOR_BLUE)
	{
		if(m_smcocp_next2!=NULL)
			delete m_smcocp_next2;
		m_smcocp_next2=newParticlesMCO(animFromColor(PIECE_COLOR_BLUE));
	}
	
	if(currentPair->getPiece(0)->getType()==PIECE_TYPE_NORMAL && currentPair->getPiece(0)->getColor()==PIECE_COLOR_BLUE)
	{
		if(m_smcocp_current1!=NULL)
			delete m_smcocp_current1;
		m_smcocp_current1=newParticlesMCO(animFromColor(PIECE_COLOR_BLUE));
	}
	if(currentPair->getPiece(1)->getType()==PIECE_TYPE_NORMAL && currentPair->getPiece(1)->getColor()==PIECE_COLOR_BLUE)
	{
		if(m_smcocp_current2!=NULL)
			delete m_smcocp_current2;
		m_smcocp_current2=newParticlesMCO(animFromColor(PIECE_COLOR_BLUE));
	}
	//debug("initsmocp ok");
}

//dessin avec une paire
void LevelProjectSexy::draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	//m_debugstr="paire";
	if(m_lastdraw==-1)
	{
		initSmcocp(currentPair, nextPair);
		m_lastdraw=LEVEL_SEXY_DRAWTYPE_PAIR;
	}
	else if(m_lastdraw!=LEVEL_SEXY_DRAWTYPE_PAIR)
	{
		if(m_smcocp_current1!=NULL)
			delete m_smcocp_current1;
		if(m_smcocp_current2!=NULL)
			delete m_smcocp_current2;
		m_smcocp_current1=m_smcocp_next1;
		m_smcocp_current2=m_smcocp_next2;
		m_smcocp_next1=NULL;
		m_smcocp_next2=NULL;
		
		if(nextPair->getPiece(0)->getType()==PIECE_TYPE_NORMAL && nextPair->getPiece(0)->getColor()==PIECE_COLOR_BLUE)
		{
			m_smcocp_next1=newParticlesMCO(animFromColor(PIECE_COLOR_BLUE));
		}
		if(nextPair->getPiece(1)->getType()==PIECE_TYPE_NORMAL && nextPair->getPiece(1)->getColor()==PIECE_COLOR_BLUE)
		{
			m_smcocp_next2=newParticlesMCO(animFromColor(PIECE_COLOR_BLUE));
		}
		
		m_lastdraw=LEVEL_SEXY_DRAWTYPE_PAIR;
	}
	
	
	
	
	drawBefore(grid, nextPair, cc);
	if(!isPaused() && m_dropspeed!=1)
	{
		//currentPair->draw(grid, fx_trans, APPLY_FX_ON_NONE);
		Pos gridPos=currentPair->getGridPos(0);
		Pos pixPos=grid->getContentPosition(gridPos.x, gridPos.y);
		drawCurrentPair(currentPair, pixPos.x, pixPos.y, grid, fx_trans);
	}
	drawAfter(grid, nextPair, cc);
	
}

//dessin avec 2 pieces
void LevelProjectSexy::draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	//m_debugstr="2 pieces";
	/*if(pieceConnected==NULL)
		debug("dessin avec 2 pieces : pieceConnected est NULL");
	if(pieceUnconnected==NULL)
		debug("dessin avec 2 pieces : pieceUnconnected est NULL");
		*/
		
	if(m_lastdraw!=LEVEL_SEXY_DRAWTYPE_PIECES)
	{
		if(pieceConnected->getColor()==PIECE_COLOR_BLUE)
		{
			if(m_smcocp_connected!=NULL)
			{	
				delete m_smcocp_connected;
				m_smcocp_connected=NULL;
			}
			
			if(m_smcocp_current1!=NULL)
			{
				m_smcocp_connected=m_smcocp_current1;
				m_smcocp_current1=NULL;
			}
			else if(m_smcocp_current2!=NULL)
			{
				m_smcocp_connected=m_smcocp_current2;
				m_smcocp_current2=NULL;
			}
			else
			{
				//debug("dessin avec 2 pieces : pieceConnected : aucun current n'a de smocp");
				m_smcocp_connected=newParticlesMCO(animFromColor(pieceConnected->getColor()));
			}
		}
		if(pieceUnconnected->getColor()==PIECE_COLOR_BLUE)
		{
			if(m_smcocp_unconnected!=NULL)	
			{
				delete m_smcocp_unconnected;
				m_smcocp_unconnected=NULL;
			}
			
			if(m_smcocp_current2!=NULL)
			{
				m_smcocp_unconnected=m_smcocp_current2;
				m_smcocp_current2=NULL;
			}
			else if(m_smcocp_current1!=NULL)
			{
				m_smcocp_unconnected=m_smcocp_current1;
				m_smcocp_current1=NULL;
			}
			else
			{
				//debug("dessin avec 2 pieces : pieceUnconnected : aucun current n'a de smocp");
				m_smcocp_unconnected=newParticlesMCO(animFromColor(pieceUnconnected->getColor()));
			}
		}
		
		m_lastdraw=LEVEL_SEXY_DRAWTYPE_PIECES;
	}
	
	
	
	
	drawBefore(grid, nextPair, cc);
	
	Pos pixPosConnected=grid->getContentPosition(pieceConnectedPos.x, pieceConnectedPos.y);
	Pos pixPosUnconnected=grid->getContentPosition(pieceUnconnectedPos.x, pieceUnconnectedPos.y);
	
	drawPieceConnected(pieceConnected, pixPosConnected.x, pixPosConnected.y);
	drawPieceUnconnected(pieceUnconnected, pixPosUnconnected.x, pixPosUnconnected.y, fx_trans);
	
	drawAfter(grid, nextPair, cc);
}

//dessin avec rien
void LevelProjectSexy::draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer)
{
	//debug("dessin avec rien");
	//m_debugstr="//";
	drawBefore(grid, nextPair, cc);
	drawAfter(grid, nextPair, cc);
	
	m_lastdraw=LEVEL_SEXY_DRAWTYPE_NOTHING;
}

void LevelProjectSexy::drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	//oslClearScreen(RGB(0,0,0));
	
	//nuages
	if(m_fx_flash_clouds->isRunning())
		m_fx_flash_clouds->activate();
	else
		oslSetAlpha(OSL_FX_TINT, m_flash_clouds_colorsto[m_flash_currentcolor]);
	
			for(int i=0;i<m_nbClouds;i++)
			{
				fx::draw(m_img_clouds[i], m_fx_clouds[i]);
			}
	
	oslSetAlpha(OSL_FX_DEFAULT, 0);
	
	//kti
	if(m_fx_flash_kti->isRunning())
	{
		m_fx_flash_kti->activate();
		oslDrawImage(m_img_kti);
		drawOverlay(cc);
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
	else
	{
		oslSetAlpha(OSL_FX_TINT, m_flash_kti_colorsto[m_flash_currentcolor]);
		//oslDrawImage(m_img_kti);
		drawOverlay(cc);
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
	
	if(isPaused())
	{
		//fx::drawXY(m_img_pause, 243,136,m_fx_pause_shake);
		oslDrawImage(m_img_pause);
	}
	else
	{
		//dessin du contenu de la grille
		//pieces qui explosent pas
		for(int i=0;i<grid->getNbCols();i++)
		{
			for(int j=0;j<grid->getNbLines();j++)
			{
				MatrixContentObject *mco=grid->getAt(i,j);
				/*if(mco==NULL)
					debug("drawBefore : mco est null");
					*/
					
				AbstractPiece *p=mco->getPiece();
				if(p!=NULL && !p->destructing() && !p->destructed())//pieces qui explosent pas
				{
					//Pos *pos=grid->getContentPosition(i, j);
					if(p->getType()==PIECE_TYPE_NORMAL || p->getType()==PIECE_TYPE_JOKER)
					{
						if(mco->getMCOContent()!=NULL)
						{
							if(p->getType()==PIECE_TYPE_JOKER)
								m_jokerDrawer->drawXY(mco->getMCOContent()->getX(), mco->getMCOContent()->getY());
							
							mco->getMCOContent()->draw();
						}
					}
					/*else //blindées?
					{
						p->drawXY(pos->x,pos->y);
					}*/
				}
			}	
		}
		
		//pieces qui explosent + combos
		for(int i=0;i<grid->getNbCols();i++)
		{
			for(int j=0;j<grid->getNbLines();j++)
			{
				MatrixContentObject *mco=grid->getAt(i,j);
				/*if(mco==NULL)
					debug("drawBefore : mco est null");*/
					
				AbstractPiece *p=mco->getPiece();
				if(p!=NULL)
				{
					Pos pos=grid->getContentPosition(i, j);
					if(p->getType()==PIECE_TYPE_COMBO && !p->destructing() && !p->destructed())
					{
						//animation combo turn
						oslSetAlpha(OSL_FX_ALPHA, m_ctalpha);
						m_anim_comboturn->drawXY(pos.x, pos.y);
						oslSetAlpha(OSL_FX_DEFAULT, 0);
						
						//piece combo avec progressive tint
						if(mco->getMCOContent()!=NULL)
						{
							m_fx_combo->activate();
							mco->getMCOContent()->draw();
							oslSetAlpha(OSL_FX_DEFAULT, 0);
						}
					}
					if(p->destructing() || p->destructed())//pieces qui explosent
					{
						//if(p->getType()==PIECE_TYPE_NORMAL || p->getType()==PIECE_TYPE_COMBO || p->getType()==PIECE_TYPE_JOKER)
						//{
							if(mco->getMCOContent()!=NULL)
							{
								if(p->getType()==PIECE_TYPE_JOKER)
								{
									m_jokerDrawer->drawXY(mco->getMCOContent()->getX(), mco->getMCOContent()->getY());
								}
								mco->getMCOContent()->draw();
							}
						//}
					}
				}
			}	
		}
	}
}

void LevelProjectSexy::drawOverlay(ComboCounter *cc)
{
	//jauge joker: hauteur: 193, position: (94, 65)
	if(m_jgauge_current>0)
		oslDrawFillRect(85, 262-int((m_jgauge_current*193)/m_jgauge_size), 99, 262, RGB(255,255,255));
	
	//jauge combo:
	if(cc->getTime()>0)
		oslDrawFillRect(384, 262-int((cc->getTime()*251)/cc->getTimeTotalAvailable()), 398, 262, RGB(255,255,255));
	

	if(m_fx_shake->isRunning())
		fx::draw(m_img_grid, m_fx_shake);
	else
		oslDrawImage(m_img_grid);
}

void LevelProjectSexy::drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	if(!isPaused() && m_dropspeed!=1) //m_dropspeed==1 a la fin
		drawNextPair(nextPair);	
		
	if(m_fx_dancer->isRunning())
	{
		m_fx_dancer->activate();
		oslDrawImage(m_img_dancer);
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
	m_partgen_dancer->draw();
	
	//if(m_fx_songbannerbg->isRunning())
	//{
		//fx::drawXY(m_songbanner_bg, 0, 271, m_fx_songbannerbg);
		
		//oslSetAlpha(OSL_FX_TINT, m_songbanner_color);
		m_songbanner->draw();
		//oslSetAlpha(OSL_FX_DEFAULT,0);
	//}
	
	if(m_endExplosion)
	{
		int r, g, b, a;
		oslRgbaGet8888(m_flash_kti_colorsfrom[m_flash_currentcolor], r, g, b, a);
		oslSetAlpha(OSL_FX_TINT, RGBA(r, g, b, 180));
		oslDrawImage(m_img_kti);
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
	
	//m_comboDrawer->drawXY(170, 180);
	
	if(cc->isHot())
	{
		m_comboDrawer->drawXY(170, 180);	
	}
	else
	{
		if(m_fx_flash_kti->isRunning())
		{
			m_fx_flash_kti->activate();
			m_comboDrawer->drawXY(170, 180);
			oslSetAlpha(OSL_FX_DEFAULT, 0);
		}
		else
		{
			oslSetAlpha(OSL_FX_TINT, m_flash_clouds_colorsto[m_flash_currentcolor]);
			m_comboDrawer->drawXY(170, 180);
			oslSetAlpha(OSL_FX_DEFAULT, 0);
		}
	}
	
	
	
	 
	
	//oslDrawString(150, 100, m_debugstr.c_str());
}

void LevelProjectSexy::start_specific() //démarre la musique, le timer etc
{
	oslPlaySound(m_music, 0);
	m_scenario->restart();
}

void LevelProjectSexy::update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
{
	int fxtransy=0;
	if(fx_trans!=NULL)
		fxtransy=fx_trans->getPosY();
	
	if(m_smcocp_next1!=NULL)
	{
		m_smcocp_next1->setX(LEVEL_SEXY_NEXTPAIR_X);
		m_smcocp_next1->setY(LEVEL_SEXY_NEXTPAIR_Y);
		
		m_smcocp_next1->update();
	}
	if(m_smcocp_next2!=NULL)
	{
		m_smcocp_next2->setX(LEVEL_SEXY_NEXTPAIR_X+GAMEDEF_PIECE_WIDTH);
		m_smcocp_next2->setY(LEVEL_SEXY_NEXTPAIR_Y);
		
		m_smcocp_next2->update();
	}
	if(m_smcocp_current1!=NULL)
	{
		if(currentPair==NULL)
		{
			//debug("update_specific : currentPair=NULL");
		}
		else
		{
			Pos p=getCurrentPairPixPos(0, grid, currentPair);
			m_smcocp_current1->setX(p.x);
			m_smcocp_current1->setY(p.y+fxtransy);
		}	

		m_smcocp_current1->update();
	}
	if(m_smcocp_current2!=NULL)
	{
		if(currentPair==NULL)
		{
			//debug("update_specific : currentPair=NULL");
		}
		else
		{
			Pos p=getCurrentPairPixPos(1, grid, currentPair);
			m_smcocp_current2->setX(p.x);
			m_smcocp_current2->setY(p.y+fxtransy);
		}
		
		m_smcocp_current2->update();
	}
	if(m_smcocp_connected!=NULL)
	{
		m_smcocp_connected->update();
	}
	if(m_smcocp_unconnected!=NULL)
	{
		m_smcocp_unconnected->update();
	}
	
	//cas special comme pour la pdrom: on utilise des anims séparées
	for(int i=0;i<grid->getNbCols();i++)
	{
		for(int j=0;j<grid->getNbLines();j++)
		{
			MatrixContentObject *mco=grid->getAt(i,j);
			AbstractPiece *p=mco->getPiece();
			if(p!=NULL)
			{
				if(p->getType()==PIECE_TYPE_NORMAL || p->getType()==PIECE_TYPE_JOKER)
				{
					//on cree si une piece n'en a pas
					if(mco->getMCOContent()==NULL)
					{
						Anim *anim=animFromColor(p->getColor());
						//Pos *pos=grid->getContentPosition(i, j);
						if(p->getColor()==PIECE_COLOR_BLUE)
						{
							if(m_smcocp_connected!=NULL)
							{
								mco->setMCOContent(m_smcocp_connected);
								m_smcocp_connected=NULL;
							}
							else if(m_smcocp_unconnected!=NULL)
							{
								mco->setMCOContent(m_smcocp_unconnected);
								m_smcocp_unconnected=NULL;
							}
							else
							{
								mco->setMCOContent(newParticlesMCO(anim));
							}
							
						}
						else
						{
							Anim *newAnim=new Anim(*anim);
							if(p->getType()==PIECE_TYPE_JOKER)
							{
								newAnim->setDelayTotal(newAnim->getDelayTotal()/2.0f);
							}
							mco->setMCOContent(new MCOContentSimple(newAnim));
						}
						
					}
					else //mco->getMCOContent()!=NULL
					{
						
					}	
						
				}
				else if(p->getType()==PIECE_TYPE_COMBO)
				{
					//remplir si c'est une piece combo
					if(mco->getMCOContent()==NULL)
					{
						int nbCombos=p->specialEvent(PIECE_EVENT_COMBO_GET);
						Anim *shine=new Anim(*m_animComboChange);
						Anim *explode=new Anim(*m_animComboExplode);
						mco->setMCOContent(new MCOContentCombo(explode, shine, m_imgCombo, nbCombos));
					}
					
					//on met a jour le nombre de combos
					int nbCombosPiece=p->specialEvent(PIECE_EVENT_COMBO_GET);
					int nbCombosMCOContent=mco->getMCOContent()->specialEvent(MCOCONTENT_EVENT_COMBO_GETNB);
					if(nbCombosPiece!=nbCombosMCOContent)
					{
						mco->getMCOContent()->specialEvent(MCOCONTENT_EVENT_COMBO_DECREMENT);
					}
				}
				
				//on démarre si la piece explose
				if(mco->isColorExploding() || mco->isShapeExploding() || mco->isSpecialExploding())
					mco->getMCOContent()->start();
			}
			else//mco->getPiece()==NULL
			{
				if(mco->getMCOContent()!=NULL)
				{
					mco->deleteMCOContent();
				}
			}
			
			if(mco->getMCOContent()!=NULL) //peu importe le cas, on update
			{
				Pos pos=grid->getContentPosition(i, j);
				mco->getMCOContent()->setX(pos.x);
				mco->getMCOContent()->setY(pos.y);		
				mco->getMCOContent()->update();
			}
		}
	}
	
	//joker gauge
	if(m_comboDrawer->getComboNum()<cc->getCombos() && m_comboDrawer->getComboNum()>=1)
	{
		//debug("cd="+toString(m_comboDrawer->getComboNum())+" cc="+toString(cc->getCombos()));
		m_jgauge_current+=m_jgauge_add;
	}
	else if(!cc->isEvent() && m_jgauge_current>0 )
	{
		m_jgauge_current-=m_jgauge_frem;
	}
	if(m_jgauge_current<0)
	{
		m_jgauge_current=0;
	}
		
	if(m_jgauge_current>=m_jgauge_size /*|| m_debugbool*/)
	{
		//debug("appel change");
		//if(m_debugbool)
		//	debug("raison: m_debugbool");
		//else
		//	debug("raison jauge: "+toString(m_jgauge_current)+">="+toString(m_jgauge_size));
		changeToJoker(nextPair);
		//m_debugbool=false;
	}
	
	m_comboDrawer->sync(cc);
	
	
	m_songbanner->update();
	m_comboDrawer->update();
	m_scenario->update();
	m_fx_flash_clouds->update();
	m_fx_flash_kti->update();
	m_fx_shake->update();
	m_fx_dancer->update();
	m_jokerDrawer->update();
	m_partgen_dancer->update();
	m_anim_comboturn->update();
	m_fx_combo->update();
	//m_fx_songbannerbg->update();
	for(int i=0;i<m_nbClouds;i++)
	{
		m_fx_clouds[i]->update();
	}
	
	//evenements du scenario
	if(m_scenario->hasEvent(1)) //fiou (debut ++)
	{
		/*if(m_difficulty==GAMEDEF_DIFFICULTY_EASY)
			m_addline++;
		else*/
			m_addline+=2;
		
		m_fx_dancer->restart();
		m_partgen_dancer->restart();
	}
	if(m_scenario->hasEvent(2)) //tun tu tun ti tun tu, ti ti tu tu tou, ti ti tu tu tou....
	{
		restartFlash();
	}
	if(m_scenario->hasEvent(3)) //batterie rapide debut
	{
		m_fx_shake->restart();
	}
	if(m_scenario->hasEvent(4)) //batterie rapide fin
	{
		m_fx_shake->stop();
	}
	if(m_scenario->hasEvent(5)) //pas de son debut
	{
		
	}
	if(m_scenario->hasEvent(6)) //pas de son fin
	{
		
	}
	if(m_scenario->hasEvent(20)) //basses niveau 1 (leger)
	{
		m_upspeed=m_upspeed_slow;
		m_dropspeed=m_dropspeed_slow;
		m_anim_comboturn->setDelayTotal(m_ctspeed_slow);
		m_fx_combo->setTimeForward(m_ctintspeed_slow);
		m_fx_combo->setTimeBackward(m_ctintspeed_slow);
	}
	if(m_scenario->hasEvent(21)) //basses niveau 2 (gros boum boum)
	{
		m_upspeed=m_upspeed_medium;
		m_dropspeed=m_dropspeed_medium;
		m_anim_comboturn->setDelayTotal(m_ctspeed_med);
		m_fx_combo->setTimeForward(m_ctintspeed_medium);
		m_fx_combo->setTimeBackward(m_ctintspeed_medium);
	}
	if(m_scenario->hasEvent(22)) //basses niveau 3 (gros boum boum + cymballes)
	{
		m_upspeed=m_upspeed_fast;
		m_dropspeed=m_dropspeed_fast;
		m_anim_comboturn->setDelayTotal(m_ctspeed_fast);
		m_fx_combo->setTimeForward(m_ctintspeed_fast);
		m_fx_combo->setTimeBackward(m_ctintspeed_fast);
	}
	if(m_scenario->hasEvent(30)) //dernier pfiou
	{
		startLastExplosion();
	}
	
	if(m_endExplosion)
	{
		if(m_dropspeed!=1)
		{
			m_dropspeed=-1;
			m_upspeed=-1;
			m_delayer->update();
			if(m_delayer->get())
			{
				m_dropspeed=1;
			}
			grid->explodeAll(PIECE_MODE_COLOR);
		}
		
		m_animBlue->update();
		m_animGreen->update();
		m_animOrange->update();
		m_animPurple->update();
		m_animRed->update();
		m_animYellow->update();
	}
}

void LevelProjectSexy::startLastExplosion()
{
	//grid->explodeAll(PIECE_MODE_COLOR);
	//Pair *nextPair, Pair *currentPair,
	if(m_smcocp_next1!=NULL)
		m_smcocp_next1->start();
	if(m_smcocp_next2!=NULL)
		m_smcocp_next2->start();
	if(m_smcocp_current1!=NULL)
		m_smcocp_current1->start();
	if(m_smcocp_current2!=NULL)
		m_smcocp_current2->start();
	if(m_smcocp_connected!=NULL)
		m_smcocp_connected->start();
	if(m_smcocp_unconnected!=NULL)
		m_smcocp_unconnected->start();
	m_animBlue->start();
	m_animGreen->start();
	m_animOrange->start();
	m_animPurple->start();
	m_animRed->start();
	m_animYellow->start();
	
	restartFlash();
	
	m_endExplosion=true;
}

void LevelProjectSexy::pause_specific()
{
	oslPauseSound(m_music, 1);
	m_scenario->pause();
}

void LevelProjectSexy::resume_specific()
{
	oslPauseSound(m_music, 0);
	m_scenario->start();
}


list<AbstractPiece *> *LevelProjectSexy::getNextLine()
{
	return m_linepattern->getNextLine();
}

bool LevelProjectSexy::getAddLineEvent() //dit quand une ligne doit monter, dû au scénario
{
	if(m_addline>0)
	{
		m_addline--;
		return true;
	}
	return false;
}

int LevelProjectSexy::getDropSpeed() //la vitesse a un instant t (elle peut etre modifiée par le scénario)
{
	return m_dropspeed;
}

int LevelProjectSexy::getUpSpeed() //la vitesse ou les lignes montent a un instant t
{
	return m_upspeed;
}

void LevelProjectSexy::buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid)
{
	if(key==OSL_KEY_TRIANGLE)
	{
		//m_fx_shake->restart();
	}
	else if(key==OSL_KEY_SQUARE)
	{
		/*
		restartFlash();
		m_debugbool=true;
		m_fx_dancer->restart();
		m_partgen_dancer->restart();
		*/
		//startLastExplosion();
	}
	else if(key==OSL_KEY_R)
	{
		//changeToJoker();
		//m_debugbool=true;
	}
}

bool LevelProjectSexy::finished()
{
	if(m_scenario->getTime()>=LEVEL_SEXY_DURATION)
	{
		return true;
	}
	return false;
}

float LevelProjectSexy::getDuration()
{
	return LEVEL_SEXY_DURATION;
}


Pair *LevelProjectSexy::getNewPair()
{	
	Pair *p=new Pair(randomPiece(), randomPiece());
	int posX=int(m_nbCols/2)-1;
	int posY=-1; 
	p->goTo(posX, posY);
	
	return p;
}

void LevelProjectSexy::initAnims()
{
	m_animBlue=new Anim(10, "particle_", "res/levels/projectsexy/pieces/particles/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animRed=new Anim(10, "star_", "res/levels/projectsexy/pieces/star/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animYellow=new Anim(10, "diese_", "res/levels/projectsexy/pieces/diese/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animPurple=new Anim(10, "i_", "res/levels/projectsexy/pieces/i/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animGreen=new Anim(10, "t_", "res/levels/projectsexy/pieces/t/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animOrange=new Anim(19, "k_", "res/levels/projectsexy/pieces/k/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animBlue->setReferencePointCenter();
	m_animRed->setReferencePointCenter();
	m_animYellow->setReferencePointCenter();
	m_animPurple->setReferencePointCenter();
	m_animGreen->setReferencePointCenter();
	m_animOrange->setReferencePointCenter();
		
	m_animComboExplode=new Anim(11, "comboexplode", "res/levels/projectsexy/pieces/combos/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animComboExplode->setReferencePointCenter();
	m_animComboChange=new Anim(11, "combochange", "res/levels/projectsexy/pieces/combos/", 27, OSL_PF_8888, 1);
	m_animComboChange->setReferencePointCenter();
	
	m_imgCombo=new OSL_IMAGE *[22]; //tableau de pointeurs
	for(int i=0;i<=20;i++)
	{
		m_imgCombo[i]=fwkLoadImageFilePNG("res/levels/projectsexy/pieces/combos/combo"+toString(i)+".png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
		oslImageSetRotCenter(m_imgCombo[i]);
	}
	m_imgCombo[21]=fwkLoadImageFilePNG("res/levels/projectsexy/pieces/combos/comboplus.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetRotCenter(m_imgCombo[21]);
}

AbstractPiece *LevelProjectSexy::randomPiece()
{
	int color=random(0,m_nbColors-1);
	PieceSimple *p=NULL;
	switch(color)
	{
		case 0:
		{
			p=new PieceSimple(PIECE_SHAPE_CIRCLE, PIECE_COLOR_BLUE);
			break;
		}
		case 1:
		{
			p=new PieceSimple(PIECE_SHAPE_SQUARE, PIECE_COLOR_RED);
			break;
		}
		case 2:
		{
			p=new PieceSimple(PIECE_SHAPE_DIAMOND, PIECE_COLOR_YELLOW);	
			break;
		}
		case 3:
		{
			p=new PieceSimple(PIECE_SHAPE_STAR, PIECE_COLOR_PURPLE);	
			break;
		}
		case 4:
		{
			p=new PieceSimple(PIECE_SHAPE_CROSS, PIECE_COLOR_GREEN);
			break;
		}
		case 5:
		{
			p=new PieceSimple(PIECE_SHAPE_TRIANGLE, PIECE_COLOR_ORANGE);
			break;
		}
	}
	
	if(p==NULL)
		fatalError("randomPiece: la piece est NULL");
	
	return p;
}

Anim *LevelProjectSexy::animFromColor(int color)
{
	switch(color)
	{
		case PIECE_COLOR_BLUE:
		{
			return m_animBlue;
			break;
		}
		case PIECE_COLOR_RED:
		{
			return m_animRed;
			break;
		}
		case PIECE_COLOR_YELLOW:
		{
			return m_animYellow;
			break;
		}
		case PIECE_COLOR_PURPLE:
		{
			return m_animPurple;
			break;
		}
		case PIECE_COLOR_GREEN:
		{
			return m_animGreen;
			break;
		}
		case PIECE_COLOR_ORANGE:
		{
			return m_animOrange;
			break;
		}
	}
	
	fatalError("animFromColor: l'anim est NULL");
	
	return NULL;
}




void LevelProjectSexy::initFlash(Properties *props)
{
	m_flash_nbcolors=props->getIntValue("flash.colors.nb");
	m_flash_clouds_colorsfrom=new int[m_flash_nbcolors];
	m_flash_clouds_colorsto=new int[m_flash_nbcolors];
	m_flash_kti_colorsfrom=new int[m_flash_nbcolors];
	m_flash_kti_colorsto=new int[m_flash_nbcolors];
	m_flash_currentcolor=-1;
	int kti_alpha=props->getIntValue("flash.kti.colors.alpha");
	
	for(int i=1;i<=m_flash_nbcolors;i++)
	{
		m_flash_clouds_colorsfrom[i-1]=RGB(props->getIntValue("flash.clouds.color."+toString(i)+".from.r"), props->getIntValue("flash.clouds.color."+toString(i)+".from.g"), props->getIntValue("flash.clouds.color."+toString(i)+".from.b"));
		m_flash_clouds_colorsto[i-1]=RGB(props->getIntValue("flash.clouds.color."+toString(i)+".to.r"), props->getIntValue("flash.clouds.color."+toString(i)+".to.g"), props->getIntValue("flash.clouds.color."+toString(i)+".to.b"));
		m_flash_kti_colorsfrom[i-1]=RGBA(props->getIntValue("flash.kti.color."+toString(i)+".from.r"), props->getIntValue("flash.kti.color."+toString(i)+".from.g"), props->getIntValue("flash.kti.color."+toString(i)+".from.b"), kti_alpha);
		m_flash_kti_colorsto[i-1]=RGBA(props->getIntValue("flash.kti.color."+toString(i)+".to.r"), props->getIntValue("flash.kti.color."+toString(i)+".to.g"), props->getIntValue("flash.kti.color."+toString(i)+".to.b"), kti_alpha);
	}
	
	int time=props->getIntValue("flash.duration");
	m_fx_flash_clouds=new ProgressiveTintEffect(time, time, 0, 0, 1);
	m_fx_flash_kti=new ProgressiveTintEffect(time, time, 0, 0, 1);
	restartFlash();
}

void LevelProjectSexy::restartFlash()
{
	int r=random(0,m_flash_nbcolors-1);
	if(r==m_flash_currentcolor)
		r=(r+1)%m_flash_nbcolors;
	m_flash_currentcolor=r;
	m_fx_flash_clouds->setColors(m_flash_clouds_colorsfrom[r], m_flash_clouds_colorsto[r]);
	m_fx_flash_kti->setColors(m_flash_kti_colorsfrom[r], m_flash_kti_colorsto[r]);
	m_fx_flash_clouds->restart();
	m_fx_flash_kti->restart();
}

SexyMCOContentParticles *LevelProjectSexy::newParticlesMCO(Anim *anim)
{
	MCOContentSimple *innermco=new MCOContentSimple(new Anim(*anim));
	return new SexyMCOContentParticles(m_blocParticles_particles, m_blocParticles_frames, m_blocParticles_time, m_blocParticles_distance, m_blocParticles_colorFrom, m_blocParticles_colorTo, innermco);
}

Pos LevelProjectSexy::getCurrentPairPixPos(int piecenum, GameGrid *grid, Pair *currentPair)
{
	/*if(currentPair==NULL)
		debug("getCurrentPairPixPos: currentPair est null");*/
	Pos pcurrent_grid=currentPair->getGridPos(0);
	Pos pcurrent_pix=grid->getContentPosition(pcurrent_grid.x, pcurrent_grid.y);
	short x=pcurrent_pix.x;
	short y=pcurrent_pix.y;
	
	if(piecenum==0)
		return pcurrent_pix;
	
	//else:
	switch(currentPair->getOrientation())
	{
		case ORIENTATION_EAST:
			x=x+grid->getCellWidth();
			break;	
		
		case ORIENTATION_NORTH:
			y=y-grid->getCellHeight();
			break;
			
		case ORIENTATION_SOUTH:
			y=y+grid->getCellHeight();
			break;
			
		case ORIENTATION_WEST:
			x=x-grid->getCellWidth();
			break;
	}
	
	return Pos(x, y);
}


void LevelProjectSexy::drawPieceConnected(AbstractPiece *piece, int x, int y)
{
	if(piece==NULL)
	{
		//oslFatalError("drawPieceConnected : piece est null");
		return;
	}
	
	if(piece->getType()==PIECE_TYPE_JOKER)
	{
		m_jokerDrawer->drawXY(x, y);
	}
	
	if(piece->getType()==PIECE_TYPE_NORMAL && piece->getColor()==PIECE_COLOR_BLUE)
	{
		if(m_smcocp_connected==NULL)
		{
			//debug("drawPieceConnected : m_smcocp_connected est null");
			m_smcocp_connected=newParticlesMCO(animFromColor(piece->getColor()));
		}
		m_smcocp_connected->setX(x);
		m_smcocp_connected->setY(y);
		m_smcocp_connected->draw();
	}
	else
		fx::drawXY(animFromColor(piece->getColor())->getImage(), x, y);
}

void LevelProjectSexy::drawPieceUnconnected(AbstractPiece *piece, int x, int y, SerialLocalEffect *fx_transl)
{
	if(piece==NULL)
	{
		//oslFatalError("drawPieceUnconnected : piece est null");
		return;
	}
	
	if(piece->getType()==PIECE_TYPE_JOKER)
	{
		m_jokerDrawer->drawXY(x, y, fx_transl);
	}
	
	if(piece->getType()==PIECE_TYPE_NORMAL && piece->getColor()==PIECE_COLOR_BLUE)
	{
		if(m_smcocp_unconnected==NULL)
		{
			//debug("drawPieceUnconnected : m_smcocp_unconnected est null");
			m_smcocp_unconnected=newParticlesMCO(animFromColor(piece->getColor()));
		}
		m_smcocp_unconnected->setX(x);
		m_smcocp_unconnected->setY(y);
		m_smcocp_unconnected->draw();
	}
	else
		fx::drawXY(animFromColor(piece->getColor())->getImage(), x, y, fx_transl);
}

void LevelProjectSexy::drawCurrentPair(Pair *pair, int x, int y, GameGrid *grid, SerialLocalEffect *fx_transl, AbstractLocalEffect *fx1)
{
	if(pair==NULL)
	{
		//oslFatalError("drawCurrentPair : pair est null");
		return;
	}
	
	//suivant l'orientation, la 2eme piece est a un autre endroit
	Pos piece2pos=getCurrentPairPixPos(1, grid, pair);
	
	if(pair->getPiece(0)->getType()==PIECE_TYPE_JOKER)
	{
		m_jokerDrawer->drawXY(x, y, fx_transl);
	}
	
	if(pair->getPiece(0)->getType()==PIECE_TYPE_NORMAL && pair->getPiece(0)->getColor()==PIECE_COLOR_BLUE)
	{
		if(m_smcocp_current1==NULL)
		{
			//debug("drawCurrentPair : m_smcocp_current1 est null");
			m_smcocp_current1=newParticlesMCO(animFromColor(pair->getPiece(0)->getColor()));	
			m_smcocp_current1->setX(x);
			m_smcocp_current1->setY(y);
		}
		//m_smcocp_current1->setX(x);
		//m_smcocp_current1->setY(y);
		m_smcocp_current1->draw(fx1);
	}
	else
		fx::drawXY(animFromColor(pair->getPiece(0)->getColor())->getImage(), x, y, fx_transl, fx1);
	
	
	if(pair->getPiece(1)->getType()==PIECE_TYPE_JOKER)
	{
		m_jokerDrawer->drawXY(x, y);
	}
	
	if(pair->getPiece(1)->getType()==PIECE_TYPE_NORMAL && pair->getPiece(1)->getColor()==PIECE_COLOR_BLUE)
	{
		if(m_smcocp_current2==NULL)
		{
			//debug("drawCurrentPair : m_smcocp_current2 est null");
			m_smcocp_current2=newParticlesMCO(animFromColor(pair->getPiece(1)->getColor()));	
			m_smcocp_current2->setX(piece2pos.x);
			m_smcocp_current2->setY(piece2pos.y);
		}
		//m_smcocp_current2->setX(piece2pos.x);
		//m_smcocp_current2->setY(piece2pos.y);
		m_smcocp_current2->draw(fx1);
	}
	else
		fx::drawXY(animFromColor(pair->getPiece(1)->getColor())->getImage(), piece2pos.x, piece2pos.y, fx_transl, fx1);
}

void LevelProjectSexy::drawNextPair(Pair *pair)
{
	if(pair==NULL)
	{
		//oslFatalError("drawNextPair : pair est null");
		return;
	}
	
	if(pair->getPiece(0)->getType()==PIECE_TYPE_JOKER)
	{
		m_jokerDrawer->drawXY(LEVEL_SEXY_NEXTPAIR_X, LEVEL_SEXY_NEXTPAIR_Y);
	}
	
	if(pair->getPiece(0)->getType()==PIECE_TYPE_NORMAL && pair->getPiece(0)->getColor()==PIECE_COLOR_BLUE)
	{
		if(m_smcocp_next1==NULL)
		{
			//debug("drawNextPair : m_smcocp_next1 est null");
			m_smcocp_next1=newParticlesMCO(animFromColor(pair->getPiece(0)->getColor()));	
		}
		m_smcocp_next1->setX(LEVEL_SEXY_NEXTPAIR_X);
		m_smcocp_next1->setY(LEVEL_SEXY_NEXTPAIR_Y);
		m_smcocp_next1->draw();
	}
	else
		fx::drawXY(animFromColor(pair->getPiece(0)->getColor())->getImage(), LEVEL_SEXY_NEXTPAIR_X, LEVEL_SEXY_NEXTPAIR_Y);
	
	
	if(pair->getPiece(1)->getType()==PIECE_TYPE_JOKER)
	{
		m_jokerDrawer->drawXY(LEVEL_SEXY_NEXTPAIR_X+GAMEDEF_PIECE_WIDTH, LEVEL_SEXY_NEXTPAIR_Y);
	}
	
	if(pair->getPiece(1)->getType()==PIECE_TYPE_NORMAL && pair->getPiece(1)->getColor()==PIECE_COLOR_BLUE)
	{
		if(m_smcocp_next2==NULL)
		{
			//debug("drawNextPair : m_smcocp_next2 est null");
			m_smcocp_next2=newParticlesMCO(animFromColor(pair->getPiece(0)->getColor()));	
		}
		m_smcocp_next2->setX(LEVEL_SEXY_NEXTPAIR_X+GAMEDEF_PIECE_WIDTH);
		m_smcocp_next2->setY(LEVEL_SEXY_NEXTPAIR_Y);
		m_smcocp_next2->draw();
	}
	else
		fx::drawXY(animFromColor(pair->getPiece(1)->getColor())->getImage(), LEVEL_SEXY_NEXTPAIR_X+GAMEDEF_PIECE_WIDTH, LEVEL_SEXY_NEXTPAIR_Y);
}

void LevelProjectSexy::initClouds(Properties *props)
{
	m_nbClouds=props->getIntValue("clouds.nb");
	
	m_img_clouds=new OSL_IMAGE *[m_nbClouds];
	m_fx_clouds=new SerialLocalEffect *[m_nbClouds];
	
	for(int c=0;c<m_nbClouds;c++)
	{
		m_img_clouds[c]=oslLoadImageFilePNG((char *)string("res/levels/projectsexy/clouds/cloud"+toString(c+1)+".png").c_str(), OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
		m_fx_clouds[c]=new SerialLocalEffect();
		int nbmoves=props->getIntValue("cloud"+toString(c+1)+".moves.nb");
		
		for(int m=0;m<nbmoves;m++)
		{
			int xfrom=props->getIntValue("cloud"+toString(c+1)+".move"+toString(m+1)+".from.x");
			int yfrom=props->getIntValue("cloud"+toString(c+1)+".move"+toString(m+1)+".from.y");
			int xto=props->getIntValue("cloud"+toString(c+1)+".move"+toString(m+1)+".to.x");
			int yto=props->getIntValue("cloud"+toString(c+1)+".move"+toString(m+1)+".to.y");
			int time=props->getIntValue("cloud"+toString(c+1)+".move"+toString(m+1)+".time");
			
			m_fx_clouds[c]->addEffect(new TranslationEffect(xfrom, yfrom, xto, yto, time, time, true, 1));
			m_fx_clouds[c]->start();
		}
	}
}

void LevelProjectSexy::initShake(Properties *props)
{
	//enableDebug();
	m_fx_shake=new SerialLocalEffect(1);
	
	int nb=props->getIntValue("shakes.nb");
	
	for(int i=0;i<nb;i++)
	{
		int width=props->getIntValue("shake"+toString(i+1)+".width");
		int time=props->getIntValue("shake"+toString(i+1)+".time");
		int loops=props->getIntValue("shake"+toString(i+1)+".loops");
		m_fx_shake->addEffect(new ShakeEffect(width, 0, time, true, loops));
	}
}

void LevelProjectSexy::initDancer(Properties *props)
{
	m_img_dancer=fwkLoadImageFilePNG("res/levels/projectsexy/dancer.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888);
	oslImageCenterScreen(m_img_dancer);
	int from=props->getIntValue("dancer.alpha.from");
	int to=props->getIntValue("dancer.alpha.to");
	int time=props->getIntValue("dancer.alpha.time");
	m_fx_dancer=new ProgressiveAlphaEffect(time, time, from, to, 1);
	
	//partgen
	int nbParticles=props->getIntValue("dancer.particle.throughput.nb");
	int generationTime=props->getIntValue("dancer.particle.throughput.frames");
	int movingTime=props->getIntValue("dancer.particle.translation.time");
	int dist=props->getIntValue("dancer.particle.translation.length");
	int particleLifespan=props->getIntValue("dancer.particle.translation.time");
	string filename="res/levels/projectsexy/dancer_particle.png";
	m_partgen_dancer=new ParticleGeneratorPoint(240, 136, 0, 360, nbParticles, generationTime, movingTime, dist, particleLifespan, filename, OSL_PF_8888, 1);
	m_partgen_dancer->generationOn();
	m_partgen_dancer->setLocalEffect1SimpleTranslation();
	
	int rotation=props->getIntValue("dancer.particle.rotation.time");
	int alphafrom=props->getIntValue("dancer.particle.alpha.from");
	int alphato=props->getIntValue("dancer.particle.alpha.to");
	ProgressiveAlphaEffect *fxalpha=new ProgressiveAlphaEffect(movingTime, movingTime, alphafrom, alphato, 1);
	RotationEffect *fxrot=new RotationEffect(rotation);
	m_partgen_dancer->setLocalEffect2(fxrot);
	m_partgen_dancer->setGlobalEffect1(fxalpha);
	
	int growfrom=props->getIntValue("dancer.particle.grow.from");
	int growto=props->getIntValue("dancer.particle.grow.to");
	SizeWowEffect *fxgrow=new SizeWowEffect(growfrom, growto, growfrom, growto, movingTime, movingTime, true, 1);
	m_partgen_dancer->setLocalEffect3(fxgrow);
}

void LevelProjectSexy::changeToJoker(Pair *pair)
{
	//debug("change to joker!");
	/*if(pair==NULL)
		debug("changeToJoker : pair est null");*/
	AbstractPiece * p=pair->getPiece(0);
	if(p->getColor()==PIECE_COLOR_BLUE)
	{
		if(pair->getPiece(1)->getColor()!=PIECE_COLOR_BLUE)
			p=pair->getPiece(1);
		else
			return; //il attendra d'avoir autre chose que des double générateurs
	}
	PieceJoker *jok=new PieceJoker(p->getShape(), p->getColor());
	pair->setPiece0(jok);
	m_jgauge_current=0;
}

void LevelProjectSexy::initComboTurn(Properties *props)
{
	if(m_difficulty==GAMEDEF_DIFFICULTY_EASY)
	{
		m_ctspeed_slow=props->getIntValue("easy.combo.anim.turn.speed.slow");
		m_ctspeed_med=props->getIntValue("easy.combo.anim.turn.speed.medium");
		m_ctspeed_fast=props->getIntValue("easy.combo.anim.turn.speed.fast");
		
		m_ctintspeed_slow=props->getIntValue("easy.combo.fx.tint.speed.slow");
		m_ctintspeed_medium=props->getIntValue("easy.combo.fx.tint.speed.medium");
		m_ctintspeed_fast=props->getIntValue("easy.combo.fx.tint.speed.fast");
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_NORMAL)
	{
		m_ctspeed_slow=props->getIntValue("normal.combo.anim.turn.speed.slow");
		m_ctspeed_med=props->getIntValue("normal.combo.anim.turn.speed.medium");
		m_ctspeed_fast=props->getIntValue("normal.combo.anim.turn.speed.fast");
		
		m_ctintspeed_slow=props->getIntValue("normal.combo.fx.tint.speed.slow");
		m_ctintspeed_medium=props->getIntValue("normal.combo.fx.tint.speed.medium");
		m_ctintspeed_fast=props->getIntValue("normal.combo.fx.tint.speed.fast");
	}
	else //if(m_difficulty==GAMEDEF_DIFFICULTY_HARD)
	{
		m_ctspeed_slow=props->getIntValue("hard.combo.anim.turn.speed.slow");
		m_ctspeed_med=props->getIntValue("hard.combo.anim.turn.speed.medium");
		m_ctspeed_fast=props->getIntValue("hard.combo.anim.turn.speed.fast");
		
		m_ctintspeed_slow=props->getIntValue("hard.combo.fx.tint.speed.slow");
		m_ctintspeed_medium=props->getIntValue("hard.combo.fx.tint.speed.medium");
		m_ctintspeed_fast=props->getIntValue("hard.combo.fx.tint.speed.fast");
	}
	m_anim_comboturn=new Anim(24, "comboturn_", "res/levels/projectsexy/comboturn/", m_ctspeed_slow, OSL_PF_8888);
	m_anim_comboturn->setReferencePointCenter();
	m_anim_comboturn->start();
	m_ctalpha=props->getIntValue("combo.anim.turn.alpha");
	
	//tinte progressive
	int colorfrom=RGB(props->getIntValue("combo.fx.tint.from.r"), props->getIntValue("combo.fx.tint.from.g"), props->getIntValue("combo.fx.tint.from.b"));
	int colorto=RGB(props->getIntValue("combo.fx.tint.to.r"), props->getIntValue("combo.fx.tint.to.g"), props->getIntValue("combo.fx.tint.to.b"));
	m_fx_combo=new ProgressiveTintEffect(m_ctintspeed_slow, m_ctintspeed_slow, colorfrom, colorto);
	m_fx_combo->start();
}












