#ifndef LEVELTETRIS_H_
#define LEVELTETRIS_H_

#include <oslib/oslib.h>
#include "code/game/levels/abstract/AbstractLevel.h"
#include "code/game/grid/GameGrid.h"
#include "code/game/Pair.h"
#include "code/game/levels/LinePattern.h"
#include "code/game/pieces/abstract/AbstractPiece.h"
#include <list>
#include "code/game/SongBanner.h"
#include "code/game/combos/ComboCounter.h"
#include "code/game/combos/ComboNumberDrawer.h"
#include "code/game/EventsScenario.h"
#include "code/framework/fx/EffectsLib.h"
#include "code/framework/anim/Anim.h"
#include "code/framework/time/Delayer.h"


using namespace std;

class LevelTetris : public AbstractLevel
{
	private:
			
		OSL_IMAGE *m_img_bg;
		LinePattern *m_linepattern;
		OSL_IMAGE *m_img_pause;
		ProgressiveAlphaEffect *m_fx_gauge;
			
		void start_specific(); //démarre la musique, le timer etc
		void update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans);
		void update_pause_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans);
		void pause_specific(); 
		void resume_specific();
				
		SongBanner *m_songbannerA;
		SongBanner *m_songbannerB;
		SongBanner *m_songbannerPause;
		
		ComboNumberDrawer * m_comboDrawer;
		
		EventsScenario *m_scenario;
		OSL_SOUND* m_musicA;
		OSL_SOUND* m_musicB;
		OSL_SOUND* m_musicPause;
		
		int m_dropspeed;
		int m_dropspeed_start;
		int m_dropspeed_themeB;
		int m_upspeed;
		int m_upspeed_themeB;
		
		int m_addline; //ajoute 1 qd on veut qu'une ligne monte
								
		void drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer);
		void drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer);
		
		//on va compter le nombre de pieces ajoutées dans la grilles (via lignes et next piece)
		//afin de calculer la vitesse de tombée, qui va augmenter, il faudra afficher le level
		//comme dans tetris.
		int m_nbPiecesTotal; 
		int m_currentLevel;
		int m_part; //theme A ou theme B
		int m_levelup_nbBlocks;
		int m_levelup_acceleration;
		
		int m_difficulty;
		int m_nbColors; //nombre de couleurs pour les pieces
		
		Anim *m_animBlue,*m_animRed,*m_animYellow,*m_animPurple,*m_animGreen,*m_animOrange;
		
		SerialLocalEffect *m_fx_up3;
		
		//va nous permettre de dessiner une ligne sous la grille (pointeur uniquement, ne pas deleter)
		list<AbstractPiece *> * m_nextline;
		
		BlinkEffect *m_fx_blink;
		
		bool m_test;
		
		OSL_SOUND *m_snd_tetris;
		OSL_SOUND *m_snd_levelup;
		OSL_SOUND *m_snd_pause;
		Delayer *m_delayer_levelup;
		int m_levelup_delay;
		
		OSL_IMAGE *m_img_songbannerbg;
		SerialLocalEffect *m_fx_songbanner;
		SerialLocalEffect *m_fx_songbanner_pause;
		
		//recalcule les dropspeed, upspeed, level
		void recalculateSpeeds();
		AbstractPiece *randomPiece();
		void drawPairXY(Pair *pair, int x, int y, GameGrid *grid, AbstractLocalEffect *fx_transl=NULL, AbstractLocalEffect *fx1=NULL);
		void drawPieceXY(AbstractPiece *piece, int x, int y, AbstractLocalEffect *fx_transl=NULL);
		Anim *animFromColor(int color);
		
		//affichage du score des combos et du level
		int m_score;
		int m_combos;
		BmpText *m_text_score;
		BmpText *m_text_combos;
		BmpText *m_text_level;
		BmpFont *m_font;
		
		
	public:
		LevelTetris(int difficulty=GAMEDEF_DIFFICULTY_NORMAL, int nbPlayers=1);
		~LevelTetris();
		
		//hérité:
		int getDropSpeed(); //la vitesse a un instant t (elle peut etre modifiée par le scénario)
		int getUpSpeed(); //la vitesse ou les lignes montent a un instant t
		void draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer);
		string getCode();
		
		void buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid);
		
		//LinePattern *getLevelPieces(); 
		bool getAddLineEvent(); //dit quand une ligne doit monter, dû au scénario
		
		bool finished();
		
		list<AbstractPiece *> *getNextLine();
		
		//surchargé pour avoir les bonnes pieces et compter
		Pair *getNewPair();
		
		float getDuration();
		
		void reset();
};


#endif /*LEVELTETRIS_H_*/
