#include "code/game/levels/impl/underwater/fauna/AnimalManager.h"
#include "code/framework/utils.h"
#include <string>
#include "code/game/levels/impl/underwater/fauna/AnimalProperties.h"

using namespace utils;
using namespace std;

AnimalManager::AnimalManager(Properties *props, int watery)
{
	m_watery=watery;
	
	m_animal_proportion_total=0;
	
	int nbkinds=props->getIntValue("animals.nbkinds");
	
	m_list_animalprops.reserve(nbkinds);
	
	for(int i=1; i<=nbkinds; i++)
	{
		string keyprefix="animal."+toString(i)+".";
		
		Anim *anim=new Anim(
			props->getIntValue(keyprefix+"anim.nbimages"), 
			props->getStrValue(keyprefix+"anim.filename"), 
			"res/levels/underwater/"+props->getStrValue(keyprefix+"anim.folder"), 
			props->getFloatValue(keyprefix+"anim.time")
		);
		//anim->setReferencePointCenter();
		
		int spawnproportion=props->getIntValue(keyprefix+"spawn.proportion");
		m_animal_proportion_total+=spawnproportion;
		
		AnimalProperties *fp=new AnimalProperties(
			props->getIntValue(keyprefix+"x.left"),
			props->getIntValue(keyprefix+"x.right"),
			props->getIntValue(keyprefix+"y.min"),
			props->getIntValue(keyprefix+"y.max"),
			props->getIntValue(keyprefix+"translation.time.min"),
			props->getIntValue(keyprefix+"translation.time.max"),
			anim,
			spawnproportion,
			props->getIntValue(keyprefix+"size.maxreduction.percent")
		);		
		
		m_list_animalprops.push_back(fp);
	}
	
	m_probability=props->getIntValue("animals.generation.probability");
}

AnimalManager::~AnimalManager()
{
	for(list<Animal *>::iterator it=m_list_animals.begin(); it != m_list_animals.end(); it++)
	{
		delete *it;
	}
	
	for(vector<AnimalProperties *>::iterator it=m_list_animalprops.begin(); it != m_list_animalprops.end(); it++)
	{
		delete *it;
	}
}

void AnimalManager::setWaterPosition(int y)
{
	m_watery=y;
}

void AnimalManager::update()
{
	for(list<Animal *>::iterator it=m_list_animals.begin(); it != m_list_animals.end(); /*it++*/)
	{
		Animal *animal = *it;
		animal->update();
		//on enleve les poissons qui sont arrivés au bout
		if(animal->finished())
		{
			delete animal;
			it = m_list_animals.erase(it);
		}
		else
		{
			it++;
		}
	}
	
	//creation d'un nouvel animal
	int probaresult=random(0,m_probability);
	if(probaresult==0)
	{
		int randomnumber=random(1, m_animal_proportion_total);
		int currentnumber=0;
		//on recherche sur quel animal on vient de tomber
		AnimalProperties *fp_chosen=NULL;
		for(vector<AnimalProperties *>::iterator it=m_list_animalprops.begin(); it != m_list_animalprops.end() && fp_chosen==NULL; it++)
		{
			AnimalProperties *fp = *it;
			currentnumber+=fp->getSpawnProportion();
			if(currentnumber>=randomnumber)
			{
				fp_chosen=fp;
			}
		}
		if(fp_chosen==NULL)
			fatalError("AnimalManager::update : animal creation failed because random was too high");
		
		int ymin=fp_chosen->getYMin();
		int ymax=fp_chosen->getYMax();
		
		if(m_watery+ymin<=262 && m_watery+ymax>=0) //on est visible!
		{				
			
			int y=random(ymin, ymax);
			
			//s'il est trop dehors on le crée même pas
			if(m_watery+y<-10 || m_watery+y>262)
				return;
			//else:
			
			bool direction=random(0,1)==0;
			
			int time=random(fp_chosen->getTranslationTimeMin(), fp_chosen->getTranslationTimeMax());
		
			TranslationEffect *translfx=new TranslationEffect(fp_chosen->getXLeft(), y, fp_chosen->getXRight(), y, time, time, direction, 1);
			
			int reduction=random(0, fp_chosen->getMaxReductionPercent());
			
			Animal *animal=new Animal(fp_chosen->getAnimClone(), translfx, reduction, direction);
			m_list_animals.push_back(animal);
		}
	}
}

void AnimalManager::draw()
{
	for(list<Animal *>::iterator it=m_list_animals.begin(); it != m_list_animals.end(); it++)
	{
		Animal *animal = *it;
		animal->drawRelative(m_watery);
	}
}











