#include "code/game/menus/Explosion.h"
#include "code/framework/utils.h"
#include "code/framework/check/loaders.h"

using namespace utils;

Explosion::Explosion(int duration, int x, int y, u32 colorFrom, u32 colorTo)
{
	m_time=duration; //le temps que ca dure en tout
	m_current=0; //ou on en est
	m_vanish=new ProgressiveAlphaEffect(int(duration/3), 0, 255, 0, 1); //le dernier tiers du temps, ca disparait
	m_particle=fwkLoadImageFilePNG("res/menus/main/images/explosion.png", OSL_IN_RAM, OSL_PF_4444); //la "particule"
	oslImageSetRotCenter(m_particle);
	m_particle->x=x;
	m_particle->y=y;
	
	for(int i=0;i<5;i++)//on va dessiner 5 fois l'image a la fin
	{
		m_growEffects[i]=new SizeWowEffect(100, 1300, 100, 600, duration, 0, true, 1);
		m_tint[i] = new ProgressiveTintEffect(duration, 0, colorFrom, colorTo, 1);
	}
	m_running=false;
	
	m_growEffects[0]->restart(); //il ne va pas subir les updates tant que l'explosion n'est pas en route
	m_tint[0]->restart();
}

Explosion::~Explosion()
{
	for(int i=0;i<5;i++)
	{
		delete m_growEffects[i];
		delete m_tint[i];
	}
	oslDeleteImage(m_particle);
	delete m_vanish;
	//delete m_tint;
}

bool Explosion::finished()
{
	return m_current>=m_time;
}

void Explosion::draw()
{
	if(m_current<m_time)
	{
		oslSetBilinearFilter(true);
		
		OSL_ALPHA_PARAMS *alpha = new OSL_ALPHA_PARAMS;
		oslGetAlphaEx(alpha);
		
		m_vanish->activate();
		m_tint[0]->activate();	
		fx::draw(m_particle, m_growEffects[0]);
		oslSetAlphaEx(alpha);
		if(m_current>=int(m_time/5))
		{
			m_vanish->activate();
			m_tint[1]->activate();
			fx::draw(m_particle, m_growEffects[1]);
			oslSetAlphaEx(alpha);
		}
		if(m_current>=int(2*m_time/5))
		{
			m_vanish->activate();
			m_tint[2]->activate();
			fx::draw(m_particle, m_growEffects[2]);
			oslSetAlphaEx(alpha);
		}
		if(m_current>=int(3*m_time/5))
		{
			m_vanish->activate();
			m_tint[3]->activate();
			fx::draw(m_particle, m_growEffects[3]);
			oslSetAlphaEx(alpha);
		}
		if(m_current>=int(4*m_time/5))
		{
			m_vanish->activate();
			m_tint[4]->activate();
			fx::draw(m_particle, m_growEffects[4]);
			oslSetAlphaEx(alpha);
		}
			
		
		oslSetBilinearFilter(false);
		
		delete alpha;
	}
}

void Explosion::update()
{
	if(m_running)
	{
		m_current++;
		if(m_current>=m_time)
		{
			m_running=false;
			return;	
		}		
		if(m_current==int(m_time/5))
		{
			m_growEffects[1]->restart();
			m_tint[1]->restart();
		}
		else if(m_current==int(2*m_time/5))
		{
			m_growEffects[2]->restart();
			m_tint[2]->restart();
		}
		else if(m_current==int(3*m_time/5))
		{
			m_growEffects[3]->restart();
			m_tint[3]->restart();
		}
		else if(m_current==int(4*m_time/5))
		{
			m_growEffects[4]->restart();
			m_tint[4]->restart();
		}
		
		if(m_current==int(2*m_time/3))
			m_vanish->restart();
		
			
		for(int i=0;i<5;i++)
		{
			m_growEffects[i]->update();
			m_tint[i]->update();
		}
		m_vanish->update();
		
	}
}

void Explosion::start()
{
	m_running=true;
}

void Explosion::stop()
{
	m_running=false;
}

void Explosion::reset()
{
	m_current=0;
	for(int i=0;i<5;i++)
	{
		m_growEffects[i]->reset();
		m_tint[i]->reset();
	}
	m_growEffects[0]->restart();
	m_tint[0]->restart();
	m_vanish->reset();
}

void Explosion::restart()
{
	reset();
	start();
}

bool Explosion::isRunning()
{
	return m_running;
}




