#include "code/game/menus/MainMenu.h"
#include <oslib/oslib.h>
#include "code/framework/bmpFont/BmpFont.h"
#include "code/framework/bmpFont/BmpText.h"
#include "code/framework/fx/EffectsLib.h"
#include "code/framework/utils.h"
#include "code/framework/anim/Anim.h"
#include "code/framework/I18nResourceBundle.h"
#include "code/game/menus/MenuItemManager.h"
#include "code/game/menus/Explosion.h"
#include "code/framework/fx/utils/AlphaParamsState.h"
#include "code/framework/particles/complete/fxFactories/PFXFSimpleTranslation.h"
#include "code/framework/particles/complete/fx/ParticlesProgressiveTintEffect.h"
#include "code/framework/particles/complete/animFactories/RandomAnimationFactory.h"
#include "code/framework/particles/complete/fxFactories/PFXFRandomTranslation.h"
#include "code/framework/particles/complete/fxFactories/PFXFSimpleOrientation.h"
#include "code/framework/check/loaders.h"
#include "code/framework/check/PrettyFileLogger.h"

using namespace std;
using namespace utils;

#define VINYL_X 328
#define VINYL_Y 91

//a mettre pour que le fichier les connaisse:
OSL_SOUND* MainMenu::m_music;
EventsScenario *MainMenu::m_scenario;


//construction
MainMenu::MainMenu(string locale)
{
	PFL::logConstructionStart("MainMenu");
	//oslInitAudio();
	
	m_locale=locale;
	m_action=MENU_NOACTION;
	m_skip=false;
	
	m_anim_projecteur=new Anim(3, "proj", "res/menus/main/proj/", 21.0f, OSL_PF_8888);
	m_anim_projecteur->setReferencePoint(240, 480);
	
	m_img_bg = fwkLoadImageFilePNG("res/menus/main/images/fondmenu.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_img_bg, 0, 0);
	m_img_djdrops=fwkLoadImageFilePNG("res/menus/main/images/djdrops.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888);
	oslSetImageRotCenter(m_img_djdrops);
	m_img_ray=fwkLoadImageFilePNG("res/menus/main/images/rayon.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetCenter(m_img_ray, 25, 512);
	m_img_vinyl=fwkLoadImageFilePNG("res/menus/main/images/minivinyl.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetCenter(m_img_vinyl, 26, 25);
	
	
	//effets
	m_fxmgr_effects=new SimpleEffectManager();
	
	m_fx_swow = m_fxmgr_effects->addEffect<SizeWowEffect>(new SizeWowEffect(9, 0, 9, 0, 20, 20));
	m_fx_swowprojhor = m_fxmgr_effects->addEffect<SizeWowEffect>(new SizeWowEffect(20, 33, 0, 0, 115, 215));
	m_fx_swowprojvert =m_fxmgr_effects->addEffect<SizeWowEffect>(new SizeWowEffect(0, 0, 30, 0, 215, 115, false));
	m_fx_ptint = m_fxmgr_effects->addEffect<ProgressiveTintEffect>(new ProgressiveTintEffect(31, 23, RGBA(255,100,255,255), RGBA(100,255,200,220)));
	m_fx_ptint3 = m_fxmgr_effects->addEffect<ProgressiveTintEffect>(new ProgressiveTintEffect(19, 73, RGBA(255,255,150,255), RGBA(150,200,255,230)));
	m_fx_vinyl = m_fxmgr_effects->addEffect<RotationEffect>(new RotationEffect(200));
	
	m_fx_ray_gh = m_fxmgr_effects->addEffect<SerialLocalEffect>(new SerialLocalEffect() /*new RotateBackAndForthEffect(400, 0, 110)*/);
	m_fx_ray_gb = m_fxmgr_effects->addEffect<SerialLocalEffect>(new SerialLocalEffect() /*new RotateBackAndForthEffect(157, -5, 80)*/);
	m_fx_ray_dh = m_fxmgr_effects->addEffect<SerialLocalEffect>(new SerialLocalEffect() /*new RotateBackAndForthEffect(233, -100, 10)*/);
	m_fx_ray_db = m_fxmgr_effects->addEffect<SerialLocalEffect>(new SerialLocalEffect() /*new RotateBackAndForthEffect(381, -95, 20)*/);
	
	//rayon gauche haut:
	m_fx_ray_gh->addEffect(new RotateBackAndForthEffect(200, 0, 92, 1));
	m_fx_ray_gh->addEffect(new RotateBackAndForthEffect(114, 92, 30, 1));
	m_fx_ray_gh->addEffect(new RotateBackAndForthEffect(112, 30, 85, 1));
	m_fx_ray_gh->addEffect(new RotateBackAndForthEffect(123, 85, 0, 1));
	
	//rayon gauche bas:
	m_fx_ray_gb->addEffect(new RotateBackAndForthEffect(157, 90, -5, 1));
	m_fx_ray_gb->addEffect(new RotateBackAndForthEffect(201, -5, 50, 1));
	m_fx_ray_gb->addEffect(new RotateBackAndForthEffect(83, 50, 5, 1));
	m_fx_ray_gb->addEffect(new RotateBackAndForthEffect(95, 5, 68, 1));
	m_fx_ray_gb->addEffect(new RotateBackAndForthEffect(61, 68, 90, 1));
	
	//rayon droite haut:
	m_fx_ray_dh->addEffect(new RotateBackAndForthEffect(212, 0, -88, 1));
	m_fx_ray_dh->addEffect(new RotateBackAndForthEffect(101, -88, -30, 3));
	m_fx_ray_dh->addEffect(new RotateBackAndForthEffect(159, -30, -95, 1));
	m_fx_ray_dh->addEffect(new RotateBackAndForthEffect(208, -95, -20, 3));
	m_fx_ray_dh->addEffect(new RotateBackAndForthEffect(81, -20, 0, 1));
	
	//rayon droite bas:
	m_fx_ray_db->addEffect(new RotateBackAndForthEffect(206, -75, 5, 3));
	m_fx_ray_db->addEffect(new RotateBackAndForthEffect(96, 5, -30, 1));
	m_fx_ray_db->addEffect(new RotateBackAndForthEffect(98, -30, -80, 3));
	m_fx_ray_db->addEffect(new RotateBackAndForthEffect(187, -80, 5, 1));
	m_fx_ray_db->addEffect(new RotateBackAndForthEffect(199, 5, -75, 1));
	
	m_fx_bg = m_fxmgr_effects->addEffect<SerialGlobalEffect>(new SerialGlobalEffect() /*new ProgressiveTintEffect(221, 150, RGBA(128,255,128,255), RGBA(255,128,255,255))*/);
	
	m_fx_bg->addEffect(new ProgressiveTintEffect(150, 0, RGB(255,135,0), RGB(255,135,0), 1));//wait
	m_fx_bg->addEffect(new ProgressiveTintEffect(450, 0, RGB(255,135,0), RGB(202,30,80), 1));
	m_fx_bg->addEffect(new ProgressiveTintEffect(150, 0, RGB(202,30,80), RGB(202,30,80), 1));//wait
	m_fx_bg->addEffect(new ProgressiveTintEffect(450, 0, RGB(202,30,80), RGB(255,244,9), 1));
	m_fx_bg->addEffect(new ProgressiveTintEffect(150, 0, RGB(255,244,9), RGB(255,244,9), 1));//wait
	m_fx_bg->addEffect(new ProgressiveTintEffect(450, 0, RGB(255,244,9), RGB(220,60,30), 1));
	m_fx_bg->addEffect(new ProgressiveTintEffect(150, 0, RGB(220,60,30), RGB(220,60,30), 1));//wait
	m_fx_bg->addEffect(new ProgressiveTintEffect(450, 0, RGB(220,60,30), RGB(255,135,0), 1));
		
	//effets induis par le scenario
	m_fxmgr_scenario=new SimpleEffectManager();
	m_fx_smallshake = m_fxmgr_scenario->addEffect<ShakeEffect>(new ShakeEffect(4,0,3));
	m_fx_shake = m_fxmgr_scenario->addEffect<ShakeEffect>(new ShakeEffect(6,0,3));
	m_fx_bigshake = m_fxmgr_scenario->addEffect<ShakeEffect>(new ShakeEffect(16,0,2));
	m_fx_bgtext_attention_appear = m_fxmgr_scenario->addEffect<ProgressiveAlphaEffect>(new ProgressiveAlphaEffect(1,60,0,195,2));
	m_fx_bgtext_interdit_appear = m_fxmgr_scenario->addEffect<ProgressiveAlphaEffect>(new ProgressiveAlphaEffect(1,60,0,195,2));
	m_fx_bgtexts_grow = m_fxmgr_scenario->addEffect<SizeWowEffect>(new SizeWowEffect(50, 1200, 50, 1200, 60, 60, true, 1));
	m_fx_vinyl_grow = m_fxmgr_scenario->addEffect<SizeWowEffect>(new SizeWowEffect(0, 100, 0, 100, 20, 3, true, 2));
	
	
	
	//BmpFont::initBmpFont(); //carbonblock24, carbon_block
	BmpFont * carbonblock=new BmpFont("res/fonts/bmpFonts/carbonblock24/", "carbon_block", OSL_PF_8888);
	
	//memory lost: 0
	//{
	m_menu=new MenuItemManager();
	m_menu->initAsTestMenu(locale, carbonblock);
	//}
	
	//memory lost: 0
	initBgTextImages(carbonblock);
	
	
	delete carbonblock;
	
	
	//musique sons
	//memory lost: 0
	m_music=fwkLoadSoundFileMP3("res/menus/main/interdit_cut.mp3", OSL_FMT_STREAM);
	oslSetSoundEndCallback(m_music, musicCallBackFunction);
	
	
	/*
	m_snd_move=fwkLoadSoundFileBGM("res/menus/main/sfx/whoosh2.bgm", OSL_FMT_NONE);
	m_snd_cancel=fwkLoadSoundFileBGM("res/menus/main/sfx/main_menu_cancel.bgm", OSL_FMT_NONE);
	m_snd_select=fwkLoadSoundFileBGM("res/menus/main/sfx/shawee.bgm", OSL_FMT_NONE);
	*/
	m_snd_move=fwkLoadSoundFileWAV("res/menus/main/sfx/whoosh2.wav", OSL_FMT_NONE);
	m_snd_cancel=fwkLoadSoundFileWAV("res/menus/main/sfx/main_menu_cancel.wav", OSL_FMT_NONE);
	m_snd_select=fwkLoadSoundFileWAV("res/menus/main/sfx/shawee.wav", OSL_FMT_NONE);
	
	m_snd_select->volumeLeft=28000;
	m_snd_select->volumeRight=m_snd_select->volumeLeft;
	m_snd_move->volumeLeft=0x1500;
	m_snd_move->volumeRight=m_snd_move->volumeLeft;
	m_snd_cancel->volumeLeft=48000;
	m_snd_cancel->volumeRight=m_snd_cancel->volumeLeft;
	
	//m_channel=new ChannelDistributor(1,7);
	m_chan_mgr=new ChannelManager();
	
	//memory lost: 0
	m_explosion=new Explosion(100, 240, 136, RGB(90, 90, 120), RGB(30, 30, 20));
	
	
	m_projectors_on=true;
	
	
	//effets du vinyl
	int transl_time=28;
	int lifespan=44;
	//va generer des etoiles qui tournent
	m_gen_vinyl1 = new ParticleGeneratorPoint(VINYL_X, VINYL_Y, 0, 360, 14, 5, transl_time, 25+20+10, lifespan, "res/menus/main/particles/star.png", OSL_PF_4444, 1);
	m_gen_vinyl1->setLocalEffect1(new PFXFRandomTranslation(25, 25+16+10));
	m_gen_vinyl1->setLocalEffect2(new RotationEffect(60));
	SerialGlobalEffect *transp=new SerialGlobalEffect();
	transp->addEffect(new WaitGlobalEffect(transl_time+1-5));
	transp->addEffect(new ProgressiveAlphaEffect(lifespan-transl_time+5,0,255,0));
	m_gen_vinyl1->setGlobalEffect1(transp);
	
	//va générer des traits et des mini etoiles orientés
	Anim *ani=new Anim("res/menus/main/particles/traits.png");
	ani->setReferencePointCenter();
	RandomAnimationFactory *raf=new RandomAnimationFactory(ani);
	ani=new Anim("res/menus/main/particles/small.png");
	ani->setReferencePointCenter();
	raf->addAnimation(ani);
	m_gen_vinyl2 = new ParticleGeneratorPoint(VINYL_X, VINYL_Y, 0, 360, 25, 5, transl_time+2, 25+16+10, lifespan, raf, 1);
	m_gen_vinyl2->setGlobalEffect1(transp->clone());
	m_gen_vinyl2->setLocalEffect1(new PFXFRandomTranslation(25, 25+16+10));
	m_gen_vinyl2->setLocalEffect2(new PFXFSimpleOrientation());
	
	m_scenario=new EventsScenario("res/menus/main/interdit.scenario");
	
	oslSetTextColor(RGB(255,255,255));  
	
	PFL::logConstructionEnd("MainMenu");
}

//destruction
MainMenu::~MainMenu()
{
	PFL::logDestructionStart("MainMenu");
	//enableDebug();
	//debug("destruction du main");
	
	oslDeleteImage(m_img_bg);
	oslDeleteImage(m_img_ray);
	oslDeleteImage(m_img_djdrops);
	oslDeleteImage(m_img_vinyl);
	delete m_anim_projecteur;
	m_fxmgr_effects->deleteAll();
	delete m_fxmgr_effects;
	m_fxmgr_scenario->deleteAll();
	delete m_fxmgr_scenario;
	delete m_menu;
	
	oslDeleteImage(m_img_attention);
	oslDeleteImage(m_img_interdit);
	delete m_scenario;
	
	fwkDeleteSound(m_music);
	fwkDeleteSound(m_snd_move);
	fwkDeleteSound(m_snd_cancel);
	fwkDeleteSound(m_snd_select);
	
	//delete m_channel;
	delete m_chan_mgr;
	
	delete m_explosion;
	
	delete m_gen_vinyl1;
	delete m_gen_vinyl2;	
	
	//oslDeinitAudio();	
	
	PFL::logDestructionEnd("MainMenu");
}

//demarrage du menu
int MainMenu::go(bool debugmode)
{
	//demarrage musique
	//oslPlaySound(m_music, 0);
	m_chan_mgr->playSoundAndReserve(m_music); //reserve la channel infiniment
	m_scenario->restart();
	
	//démarrage des effets et des animations
	m_fxmgr_effects->start();
	m_anim_projecteur->start();
	
	//m_test_fx_shakePartGen->start();
	
	while(m_action==MENU_NOACTION)
	{
		oslBenchmarkTest(OSL_BENCH_START);
		
		//touches
		m_action=keys();
		if(m_action==MENU_QUIT)
			return m_action;
		
		//phase d'update
		m_fxmgr_effects->update();
		m_fxmgr_scenario->update();
		m_anim_projecteur->update();
		m_action=m_menu->update(); 
		m_scenario->update();
		m_explosion->update();
		m_gen_vinyl1->update();
		m_gen_vinyl2->update();
		
		/*m_test_fx_shakePartGen->update();
		
		State * state=new State();
		m_test_partGen->savestate(state);
		m_test_fx_shakePartGen->applyOn(m_test_partGen);
		m_test_partGen->update();
		m_test_partGen->restorestate(state);
		delete state;
		*/
		
		//mise a jour des effets du scenario
		
		if(m_scenario->hasEvent(1)) //1er boum
		{
			m_explosion->restart();
			m_projectors_on=false;
		}
		if(m_scenario->hasEvent(2)) //2eme boum
		{
			m_projectors_on=true;
		}
		if(m_scenario->hasEvent(11)) //attention
		{
			m_fx_bgtext_attention_appear->restart();
			m_fx_bgtexts_grow->restart();
		}
		if(m_scenario->hasEvent(12)) //interdit
		{
			m_fx_bgtext_interdit_appear->restart();
			m_fx_bgtexts_grow->restart();
		}
		if(m_scenario->hasEvent(21)) //ahhhhhhh ouh
		{
			m_fx_vinyl_grow->restart();
			m_gen_vinyl1->restart();
			m_gen_vinyl2->restart();
		}
		if(m_scenario->hasEvent(31)) //petite foule on/off
		{
			//enableDebug();
			if(m_fx_smallshake->isRunning())
			{
				m_fx_smallshake->stop();
				//debug("petite foule on"
			}
			else
			{
				m_fx_smallshake->restart();
				//debug("petite foule off");
			}
			//enableDebug(false);
		}
		if(m_scenario->hasEvent(32)) //foule moyenne on/off
		{
			if(m_fx_shake->isRunning())
				m_fx_shake->stop();
			else
				m_fx_shake->restart();
		}
		if(m_scenario->hasEvent(33)) //foule grande on/off
		{
			if(m_fx_bigshake->isRunning())
				m_fx_bigshake->stop();
			else
				m_fx_bigshake->restart();
		}
		
		
		if (!m_skip && !debugmode)
		{
			//phase de dessin
			oslStartDrawing();
			drawScene();
			oslBenchmarkTest(OSL_BENCH_END); //TODO: peut planter si frameskip??
			oslSysBenchmarkDisplay();
			oslEndDrawing();
		}
		
		if(!debugmode)
		{
			//End of a frame
	        oslEndFrame();
			//Synchronize to ensure the game runs at 60 fps
	        m_skip = oslSyncFrame();
			//oslAudioVSync(); //ne sert plus, oslEndFrame le fait
		}
		else
		{
			m_action=MENU_CANCEL; //juste pour le faire sortir de la boucle
		}
	}
	//il a choisi une option
	//enableDebug();
	//debug("fin boucle main, return "+toString(m_action));
	//enableDebug(false);
	return m_action;	
}

//dessins
void MainMenu::drawScene()
{
	AlphaParamsState * alpha=new AlphaParamsState();
	
	alpha->saveState();
	m_fx_bg->activate();			
	oslDrawImage(m_img_bg);
	//dessins du scenario, en arriere plan
	if(m_fx_bgtext_attention_appear->isRunning())
	{
		m_fx_bgtext_attention_appear->activate();
		fx::drawXY(m_img_attention, 240, 136, m_fx_bgtexts_grow);
	}
	if(m_fx_bgtext_interdit_appear->isRunning())
	{
		m_fx_bgtext_interdit_appear->activate();
		fx::drawXY(m_img_interdit, 240, 136, m_fx_bgtexts_grow);
	}
	if(m_explosion->isRunning())
	{
		//m_fx_bg->activate();
		m_explosion->draw();	
	}
	
	//oslSetAlpha(OSL_FX_DEFAULT, 0);
	
	alpha->restoreState();
	alpha->saveState();
	
	if(!m_projectors_on)
		oslSetAlpha(OSL_FX_ALPHA, 64);
	m_fx_ptint3->activate();		
	fx::drawXY(m_img_ray, -10, 252, m_fx_ray_gh);
	fx::drawXY(m_img_ray, 460, 282, m_fx_ray_db);
	//oslSetAlpha(OSL_FX_DEFAULT, 0);
	alpha->restoreState();
	alpha->saveState();
	
	if(!m_projectors_on)
		oslSetAlpha(OSL_FX_ALPHA, 64);
	m_fx_ptint->activate();
	fx::drawXY(m_img_ray,  20, 282, m_fx_ray_gb);
	fx::drawXY(m_img_ray, 490, 252, m_fx_ray_dh);
	//slSetAlpha(OSL_FX_DEFAULT, 0);
	alpha->restoreState();
	alpha->saveState();
	
	if(!m_projectors_on)
		oslSetAlpha(OSL_FX_ALPHA, 64);
	m_fx_ptint3->activate();
	m_fx_ptint->activate();
	fx::drawXY(m_anim_projecteur->getImage(), 240, 290, m_fx_swowprojhor, m_fx_swowprojvert);
	//oslSetAlpha(OSL_FX_DEFAULT, 0);
	alpha->restoreState();
	
	
	
	m_fx_ptint3->activate();
	m_fx_ptint->activate();
	oslDrawImageXY(m_img_djdrops, 240, 70);
	
	m_gen_vinyl1->draw();
	m_gen_vinyl2->draw();
	
	if(m_fx_vinyl_grow->isRunning())
	{
		fx::drawXY(m_img_vinyl, VINYL_X, VINYL_Y, m_fx_vinyl, m_fx_vinyl_grow);
	}
	else
	{
		fx::drawXY(m_img_vinyl, 328, 91, m_fx_vinyl);
	}
	//oslSetAlpha(OSL_FX_DEFAULT, 0);
	
	alpha->restoreState();
					
//	oslSetTextColor(RGBA(255,255,255,255));
//	oslSetBkColor(RGBA(0,0,0,0));
	
	//TODO: bug peut etre ici!
	//dessin du menu avec les bons effets
	AbstractLocalEffect * eff[3];
	eff[0]=NULL;eff[1]=NULL;eff[2]=NULL;
	int i=0;
	if(m_fx_smallshake->isRunning())
	{
		eff[i]=m_fx_smallshake;
		i++;
	}
	if(m_fx_shake->isRunning())
	{
		eff[i]=m_fx_shake;
		i++;
	}
	if(m_fx_bigshake->isRunning())
	{
		eff[i]=m_fx_bigshake;
		i++;
	}
	m_menu->drawXY(240, 185, eff[0], eff[1], eff[2]);
	
	
	//tests
	//m_test_partGen->draw();
	
	//oslDrawString(10, 260, (toString(m_test_partGen->getCurrentNbParticles()) + " particles").c_str());
	
	delete alpha;
	
	oslDrawString(10, 210, toString(mallinfo().arena).c_str());
	oslDrawString(10, 220, toString(mallinfo().uordblks).c_str());
	
	oslDrawString(10, 230, toString(sceKernelMaxFreeMemSize()).c_str());
	oslDrawString(10, 240, toString(sceKernelTotalFreeMemSize()).c_str());
}

//gestion des touches
int MainMenu::keys()
{
	oslReadKeys();
	if (osl_pad.pressed.start) //Exit 
	{
		//enableDebug();
		//debug("start!");
		//enableDebug(false);
		return MENU_QUIT;
	}
	
	if (osl_pad.pressed.cross)
	{
		int action=m_menu->selectAction();
		if(action==MENU_CANCEL) //cliqué sur cancel
		{
			if(m_menu->cancelAction())
			{
				//oslPlaySound(m_snd_cancel, m_channel->next());	
				m_chan_mgr->restartSound(m_snd_cancel);
			}
		}
		else if(action!=MENU_OCCUPIED)
		{
			//oslPlaySound(m_snd_select,1);
			m_chan_mgr->restartSound(m_snd_select);
		}
	}
	else if (osl_pad.pressed.square)
	{
		if(m_menu->cancelAction())
		{
			//oslPlaySound(m_snd_cancel,m_channel->next());	
			m_chan_mgr->restartSound(m_snd_cancel);
		}
	}
	
	//no actions
	if (osl_pad.pressed.up)
	{
		if(m_menu->previousMenuAction())
		{
			//oslPlaySound(m_snd_move,m_channel->next());	
			m_chan_mgr->restartSound(m_snd_move);
		}
	}
	else if (osl_pad.pressed.down)
	{
		if(m_menu->nextMenuAction())
		{
			//oslPlaySound(m_snd_move,m_channel->next());	
			m_chan_mgr->restartSound(m_snd_move);
		}
	}
	if (osl_pad.pressed.right)
	{
		m_menu->previousValueAction();
	}
	else if (osl_pad.pressed.left)
	{
		m_menu->nextValueAction();
	}
	else if(osl_pad.pressed.R)
	{
		m_fx_vinyl_grow->restart();
		m_gen_vinyl1->restart();
		m_gen_vinyl2->restart();
	}
	else if(osl_pad.released.R)
	{
		m_fx_vinyl_grow->stop();
		m_fx_vinyl_grow->reset();
	}	
	else if(osl_pad.pressed.L)
	{
		//m_test_partGen->generationSwitchOnOff();
	}
	
	/*
	if(osl_pad.analogX>=100)
		m_test_partGen->setX(m_test_partGen->getX()+8);
	else if(osl_pad.analogX>=70)
		m_test_partGen->setX(m_test_partGen->getX()+4);
	else if(osl_pad.analogX>=20)
		m_test_partGen->setX(m_test_partGen->getX()+1);
	
	if(osl_pad.analogX<=-100)
		m_test_partGen->setX(m_test_partGen->getX()-8);
	else if(osl_pad.analogX<=-70)
		m_test_partGen->setX(m_test_partGen->getX()-4);
	else if(osl_pad.analogX<=-20)
		m_test_partGen->setX(m_test_partGen->getX()-1);
	
	if(osl_pad.analogY>=100)
		m_test_partGen->setY(m_test_partGen->getY()+8);
	else if(osl_pad.analogY>=70)
		m_test_partGen->setY(m_test_partGen->getY()+4);
	else if(osl_pad.analogY>=20)
		m_test_partGen->setY(m_test_partGen->getY()+1);
	
	if(osl_pad.analogY<=-100)
		m_test_partGen->setY(m_test_partGen->getY()-8);
	if(osl_pad.analogY<=-70)
		m_test_partGen->setY(m_test_partGen->getY()-4);
	if(osl_pad.analogY<=-20)
		m_test_partGen->setY(m_test_partGen->getY()-1);
	*/
	
	if(osl_pad.pressed.select)
	{
		///screenshot
		//oslWriteImageFilePNG(OSL_SECONDARY_BUFFER, "oslscreenshot.png", 0);
		screenshot();
	}
	
	
	return MENU_NOACTION;
}

void MainMenu::initBgTextImages(BmpFont *f)
{
	BmpText * bmptattention=new BmpText("Attention", f);
	bmptattention->compile();
	m_img_attention=bmptattention->createSwizzledImageCopy();
	oslSetImageRotCenter(m_img_attention);
	delete bmptattention;
	BmpText * bmptinterdit=new BmpText("Interdit", f);
	bmptinterdit->compile();
	m_img_interdit=bmptinterdit->createSwizzledImageCopy();
	oslSetImageRotCenter(m_img_interdit);
	delete bmptinterdit;
}


int MainMenu::musicCallBackFunction(OSL_SOUND *s, int voice)
{
	//We replay the same sound on the same voice
    oslPlaySound(s, voice);
    //restart scenario to stay in sync
    MainMenu::m_scenario->restart();
    //Return 1 because we continue sound playback
    return 1;
}





