#include "code/game/menus/MenuItemManager.h"
#include "code/framework/menus/MenuItem.h"
#include "code/framework/utils.h"
#include "code/framework/I18nResourceBundle.h"
#include <string>
#include "code/framework/fx/EffectsLib.h"
using namespace utils;
using namespace std;

#define translationtime 8


MenuItemManager::MenuItemManager()
{
	m_root=NULL;//new MenuItem("root", 0, NULL, 5);
	m_current=NULL;
	m_state=MENUSTATE_WAIT;
	
	m_fx_highlight=new SizeWowEffect(20, 10, 20, 10, 14, 14);
	m_fx_rotation=new RotationEffect(30, true, 1); //lors de la selection, l'item tourne
	m_fx_grow=new SizeWowEffect(100,1500,100,1500,30,30,true,1); //lors de la selection, l'item grossi
	m_fx_selection_disappear=new SerialGlobalEffect(1);//lors de la selection, l'item disparait au bout d'un moment
	m_fx_selection_disappear->addEffect(new WaitGlobalEffect(15));
	m_fx_selection_disappear->addEffect(new ProgressiveAlphaEffect(17, 17, 255, 30, 1));
	m_fx_selection_otherDisappear=new ProgressiveAlphaEffect(30,30,255,0,1);
	
	m_fx_cancel = new SizeWowEffect(90, 0, 90, 0, 21, 21, false, 1); //qd on annule l'item retrecit
	
	m_fx_down_appear = new AppearDownEffect(translationtime);
	m_fx_down_translation = new TranslationEffect(0,0,0,-20,translationtime,translationtime,true,1); //voir dans initAsMainMenu
	m_fx_down_vanish = new VanishUpEffect(translationtime);
	m_fx_up_translation = new TranslationEffect(0,0,0,20,translationtime,translationtime,true,1); //lorsqu'on appuie sur haut, les items descendent
	m_fx_up_appear = new AppearUpEffect(translationtime); //lorsqu'on appuie sur haut, l'item du dessus apparait
	m_fx_up_vanish = new VanishDownEffect(translationtime);
	
	m_distance=0;
}

MenuItemManager::~MenuItemManager()
{
	delete m_root;
	
	delete m_fx_highlight;
	delete m_fx_rotation;
	delete m_fx_grow;
	delete m_fx_selection_disappear;
	delete m_fx_cancel;
	delete m_fx_selection_otherDisappear;
	delete m_fx_down_appear;
	delete m_fx_down_translation;
	delete m_fx_down_vanish;
	delete m_fx_up_appear;
	delete m_fx_up_translation;
	delete m_fx_up_vanish;
}

void MenuItemManager::drawXY(int x, int y, AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3)
{
		oslSetAlpha(OSL_FX_ALPHA, 100);
		
		if(m_state==MENUSTATE_GOINGDOWN)
		{
			fx::drawXY(m_current->getBeforePreviousChild()->getImage(), x, y, m_fx_up_appear, fx1, fx2, fx3);
		}
		
		
		if(m_state==MENUSTATE_CANCELING)
			fx::drawXY(m_current->getPreviousChild()->getImage(), x, y, m_fx_cancel, fx1, fx2, fx3);
		else if(m_state==MENUSTATE_SELECTIONNING)
		{
			m_fx_selection_otherDisappear->activate();
			//oslDrawImageXY(m_current->getPreviousChild()->getImage(), x, y);
			fx::drawXY(m_current->getPreviousChild()->getImage(), x, y, fx1, fx2, fx3);
		}
		else if(m_state==MENUSTATE_GOINGUP)
		{
			fx::drawXY(m_current->getPreviousChild()->getImage(), x, y, m_fx_down_vanish, fx1, fx2, fx3);
		}
		else if(m_state==MENUSTATE_GOINGDOWN)
		{
			fx::drawXY(m_current->getPreviousChild()->getImage(), x, y, m_fx_up_translation, fx1, fx2, fx3);
		}
		else
			//oslDrawImageXY(m_current->getPreviousChild()->getImage(), x, y);
			fx::drawXY(m_current->getPreviousChild()->getImage(), x, y, fx1, fx2, fx3);
		oslSetAlpha(OSL_FX_ALPHA, 255);
		
		
		
		if(m_state==MENUSTATE_WAIT)
			fx::drawXY(m_current->getSelectedChild()->getImage(), x, y+m_distance, m_fx_highlight);
		else if(m_state==MENUSTATE_SELECTIONNING)
		{
			m_fx_selection_disappear->activate();
			fx::drawXY(m_current->getSelectedChild()->getImage(), x, y+m_distance, m_fx_grow, m_fx_rotation);
			oslSetAlpha(OSL_FX_DEFAULT, 0);
			//oslDrawImageXY(m_current->getSelectedChild()->getImage(), x, y+dist);
		}
		else if(m_state==MENUSTATE_CANCELING)
		{
			fx::drawXY(m_current->getSelectedChild()->getImage(), x, y+m_distance, m_fx_cancel);
		}
		else if(m_state==MENUSTATE_GOINGUP)
		{
			fx::drawXY(m_current->getSelectedChild()->getImage(), x, y+m_distance, m_fx_down_translation);
		}
		else if(m_state==MENUSTATE_GOINGDOWN)
		{
			fx::drawXY(m_current->getSelectedChild()->getImage(), x, y+m_distance, m_fx_up_translation);
		}
		
		
		
		oslSetAlpha(OSL_FX_ALPHA, 100);
		if(m_state==MENUSTATE_CANCELING)
			fx::drawXY(m_current->getNextChild()->getImage(), x, y+(2*m_distance), m_fx_cancel, fx1, fx2, fx3);
		else if(m_state==MENUSTATE_SELECTIONNING)
		{
			m_fx_selection_otherDisappear->activate();
			//oslDrawImageXY(m_current->getNextChild()->getImage(), x, y+(2*m_distance));
			fx::drawXY(m_current->getNextChild()->getImage(), x, y+(2*m_distance), fx1, fx2, fx3);
		}
		else if(m_state==MENUSTATE_GOINGUP)
		{
			fx::drawXY(m_current->getNextChild()->getImage(), x, y+(2*m_distance), m_fx_down_translation, fx1, fx2, fx3);
		}
		else if(m_state==MENUSTATE_GOINGDOWN)
		{
			fx::drawXY(m_current->getNextChild()->getImage(), x, y+(2*m_distance), m_fx_up_vanish, fx1, fx2, fx3);
		}
		else
			//oslDrawImageXY(m_current->getNextChild()->getImage(), x, y+(2*m_distance));
			fx::drawXY(m_current->getNextChild()->getImage(), x, y+(2*m_distance), fx1, fx2, fx3);
		
	
	
		
		if(m_state==MENUSTATE_GOINGUP)
		{
			fx::drawXY(m_current->getAfterNextChild()->getImage(), x, y+(2*m_distance), m_fx_down_appear, fx1, fx2, fx3);
		}
		oslSetAlpha(OSL_FX_ALPHA, 255);
}

int MenuItemManager::selectAction()
{
	if(m_state==MENUSTATE_WAIT)
	{
		if(m_current->hasValues())
		{
			return MENU_NOACTION;	
		}
		//else
		if(m_current->getSelectedChild()->getId()==MENU_CANCEL)
		{
			if(cancelAction())
				return MENU_CANCEL;
			return MENU_NOACTION;
		}
	
		m_state=MENUSTATE_SELECTIONNING;
		m_fx_rotation->restart();
		m_fx_grow->restart();
		m_fx_selection_disappear->restart();
		m_fx_selection_otherDisappear->restart();
	
	
		if(m_current->getSelectedChild()->hasChildren())
		{	
			return MENU_NOACTION;
		}
		return m_current->getSelectedChild()->getId();
	}
	return MENU_OCCUPIED;
}

bool MenuItemManager::cancelAction()
{
	if(m_current->getParent()!=NULL)
	{
		m_state=MENUSTATE_CANCELING;
		m_fx_cancel->restart();
		//m_current=m_current->getParent();
		return true;	
	}
	return false;
}

bool MenuItemManager::nextMenuAction()
{
	if(m_state==MENUSTATE_WAIT)
	{
		m_state=MENUSTATE_GOINGUP;
		m_fx_down_appear->restart();
		m_fx_down_vanish->restart();
		m_fx_down_translation->restart();
		return true;
	}
	
	//m_current->gotoNextChild();
	return false;
}

bool MenuItemManager::previousMenuAction()
{
	if(m_state==MENUSTATE_WAIT)
	{
		m_state=MENUSTATE_GOINGDOWN;
		m_fx_up_appear->restart();
		m_fx_up_vanish->restart();
		m_fx_up_translation->restart();
		return true;
	}
	return false;
}

bool MenuItemManager::nextValueAction()
{
	m_current->gotoNextValue();
	return false;
}

bool MenuItemManager::previousValueAction()
{
	//TODO: ajouter changements de valeurs
	m_current->gotoPreviousValue();
	return false;
}

int MenuItemManager::update()
{
	//TODO: utiliser un manager
	//fx update.
	m_fx_highlight->update();
	m_fx_grow->update();
	m_fx_rotation->update();
	m_fx_selection_disappear->update();
	m_fx_cancel->update();
	m_fx_selection_otherDisappear->update();
	m_fx_down_appear->update();
	m_fx_down_translation->update();
	m_fx_down_vanish->update();
	m_fx_up_appear->update();
	m_fx_up_translation->update();
	m_fx_up_vanish->update();
	
	
	if(m_state==MENUSTATE_SELECTIONNING)
	{
		if(m_fx_grow->finished())
		{
			if(m_current->getSelectedChild()->hasChildren())
			{
				m_state=MENUSTATE_WAIT;
				m_current=m_current->getSelectedChild();	
				return MENU_NOACTION;
			}
			return m_current->getSelectedChild()->getId();
		}	
	}
	else if(m_state==MENUSTATE_CANCELING)
	{
		if(m_fx_cancel->finished())
		{
			m_state=MENUSTATE_WAIT;
			m_current=m_current->getParent();
		}
		return MENU_NOACTION;
	}
	else if(m_state==MENUSTATE_GOINGUP) //les items montent, on veut le nextChild
	{
		if(m_fx_down_translation->finished())
		{
			m_state=MENUSTATE_WAIT;
			m_current->gotoNextChild();
		}
		return MENU_NOACTION;
	}
	else if(m_state==MENUSTATE_GOINGDOWN) //les items descendent, on veut le previousChild
	{
		if(m_fx_up_translation->finished())
		{
			m_state=MENUSTATE_WAIT;
			m_current->gotoPreviousChild();
		}
		return MENU_NOACTION;
	}
	
	return MENU_NOACTION;
}


void MenuItemManager::initAsMainMenu(string locale, BmpFont * f)
{
	//enableDebug();
	//debug("initAsMainMenu");
	
	I18nResourceBundle * bundle=new I18nResourceBundle(locale, "main_menu.properties");
	
	m_root=new MenuItem("root", f, 0, NULL, 5);

	//debug("test bundle->getMessage: "+bundle->getMessage("challenge"));
	
	MenuItem *solo=m_root->addSubMenu(new MenuItem(bundle->getMessage("solo"), f, 1, m_root, 5));
		MenuItem *challenge=solo->addSubMenu(new MenuItem(bundle->getMessage("challenge"), f, 11, solo, 4));
			challenge->addSubMenu(new MenuItem(bundle->getMessage("easy"), f, 111, challenge));
			challenge->addSubMenu(new MenuItem(bundle->getMessage("normal"), f, 112, challenge));
			challenge->addSubMenu(new MenuItem(bundle->getMessage("hard"), f, 113, challenge));
			challenge->addSubMenu(new MenuItem(bundle->getMessage("cancel"), f, MENU_CANCEL, challenge));
		
		MenuItem *custom=solo->addSubMenu(new MenuItem(bundle->getMessage("custom"), f, 12, solo, 4));
			challenge->addSubMenu(new MenuItem(bundle->getMessage("easy"), f, 121, custom));
			challenge->addSubMenu(new MenuItem(bundle->getMessage("normal"), f, 122, custom));
			challenge->addSubMenu(new MenuItem(bundle->getMessage("hard"), f, 123, custom));
			challenge->addSubMenu(new MenuItem(bundle->getMessage("cancel"), f, MENU_CANCEL, custom));
			
		MenuItem *vscpu=solo->addSubMenu(new MenuItem(bundle->getMessage("vscpu"), f, 13, solo, 4));
			challenge->addSubMenu(new MenuItem(bundle->getMessage("easy"), f, 131, vscpu));
			challenge->addSubMenu(new MenuItem(bundle->getMessage("normal"), f, 132, vscpu));
			challenge->addSubMenu(new MenuItem(bundle->getMessage("hard"), f, 133, vscpu));
			challenge->addSubMenu(new MenuItem(bundle->getMessage("cancel"), f, MENU_CANCEL, vscpu));
			
		solo->addSubMenu(new MenuItem(bundle->getMessage("cancel"), f, MENU_CANCEL, solo));
		
	
	MenuItem *multi=m_root->addSubMenu(new MenuItem(bundle->getMessage("multi"), f, 2, m_root, 3));
		multi->addSubMenu(new MenuItem(bundle->getMessage("hostgame"), f, 21, multi));
		multi->addSubMenu(new MenuItem(bundle->getMessage("searchgame"), f, 22, multi));
		multi->addSubMenu(new MenuItem(bundle->getMessage("cancel"), f, MENU_CANCEL, multi));
	
	MenuItem *options=m_root->addSubMenu(new MenuItem(bundle->getMessage("options"), f, 3, m_root, 3));
		options->addSubMenu(new MenuItem(bundle->getMessage("yourname"), f, 31, options));
		options->addSubMenu(new MenuItem(bundle->getMessage("character"), f, 32, options));
		options->addSubMenu(new MenuItem(bundle->getMessage("cancel"), f, MENU_CANCEL, options));
		
	m_root->addSubMenu(new MenuItem(bundle->getMessage("highscore"), f, 4, m_root));
	
	m_root->addSubMenu(new MenuItem(bundle->getMessage("quit"), f, MENU_QUIT, m_root));
	
	//m_root->addSubMenu(new MenuItem(bundle->getMessage("test"), f, TEST_GAMEPLAY, m_root));
	
	m_current=m_root;
		
	delete bundle;
	
	//debug("done");
	//enableDebug();
	
	m_distance=(m_current->getSelectedChild()->getImage()->sizeY * 9)/10;
	
	//debug("distance="+toString(m_distance));
	//enableDebug(false);
	
	//devrait etre dans le constructeur mais c seulement là qu'on connait m_distance
	delete m_fx_down_translation;
	delete m_fx_up_translation;
	m_fx_up_translation = new TranslationEffect(0,0,0,m_distance,translationtime,translationtime,true,1);
	m_fx_down_translation = new TranslationEffect(0,0,0,-m_distance,translationtime,translationtime,true,1);
	
	m_fx_highlight->start();
}

void MenuItemManager::initAsTestMenu(string locale, BmpFont * f)
{	
	I18nResourceBundle * bundle=new I18nResourceBundle(locale, "main_menu.properties");
	
	m_root=new MenuItem("root", f, 0, NULL, 3);
	
//	MenuItem *tests=m_root->addSubMenu(new MenuItem(bundle->getMessage("tests"), f, 0, m_root, 20));
//		tests->addSubMenu(new MenuItem(bundle->getMessage("animations"), f, TEST_ANIMATIONS, m_root));
//		tests->addSubMenu(new MenuItem(bundle->getMessage("animationsimple"), f, TEST_ANIMATION_SIMPLE, m_root));
//		tests->addSubMenu(new MenuItem(bundle->getMessage("particulesanimees"), f, TEST_PARTICLES_ANIMATED, m_root));
//		tests->addSubMenu(new MenuItem(bundle->getMessage("pleasewaitsound"), f, TEST_PLEASEWAITSOUND, m_root));
//		tests->addSubMenu(new MenuItem(bundle->getMessage("translationfx"), f, TEST_TRANSLATIONFX, m_root));
//		tests->addSubMenu(new MenuItem(bundle->getMessage("particulesorientables"), f, TEST_PARTICLES_ORIENTABLE, m_root));
//		tests->addSubMenu(new MenuItem(bundle->getMessage("particulessegment"), f, TEST_PARTICLES_SEGMENT, m_root));
//		tests->addSubMenu(new MenuItem(bundle->getMessage("particulesgravity"), f, TEST_PARTICLES_GRAVITY, m_root));
//		tests->addSubMenu(new MenuItem(bundle->getMessage("particulesartifice"), f, TEST_ARTIFICE, m_root));
//		tests->addSubMenu(new MenuItem(bundle->getMessage("alphaprogressif"), f, TEST_PROGRESSIVE_ALPHA, m_root));
//		tests->addSubMenu(new MenuItem(bundle->getMessage("megaman"), f, TEST_MEGAMAN, m_root));
//		tests->addSubMenu(new MenuItem(bundle->getMessage("linepattern"), f, TEST_LINEPATTERN, m_root));
//		tests->addSubMenu(new MenuItem(bundle->getMessage("gamepref"), f, TEST_GAMEPREF, m_root));
//		tests->addSubMenu(new MenuItem(bundle->getMessage("songbanner"), f, TEST_SONGBANNER, m_root));
//		tests->addSubMenu(new MenuItem(bundle->getMessage("appeardown"), f, TEST_APPEARDOWN, m_root));
//		tests->addSubMenu(new MenuItem(bundle->getMessage("loading"), f, TEST_LOADING, m_root));
//		tests->addSubMenu(new MenuItem(bundle->getMessage("nowloading"), f, TEST_NOW_LOADING, m_root));
//		tests->addSubMenu(new MenuItem(bundle->getMessage("drawimageonimage"), f, TEST_DRAWIMAGEONIMAGE, m_root));
//		tests->addSubMenu(new MenuItem(bundle->getMessage("RMX"), f, TEST_RMX, m_root));
//		tests->addSubMenu(new MenuItem(bundle->getMessage("cancel"), f, MENU_CANCEL, tests));
	
	
	
	MenuItem *levels=m_root->addSubMenu(new MenuItem(bundle->getMessage("levels"), f, 0, m_root, 41));
		levels->addSubMenu(new MenuItem(bundle->getMessage("frozen_easy"), f, LEVELFROZEN_EASY, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("mecha_easy"), f, LEVELMECHA_EASY, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("djbeat_easy"), f, LEVELDJBEAT_EASY, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("frozen_normal"), f, LEVELFROZEN_NORMAL, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("mecha_normal"), f, LEVELMECHA_NORMAL, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("frozen_hard"), f, LEVELFROZEN_HARD, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("djbeat_normal"), f, LEVELDJBEAT_NORMAL, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("sexy_easy"), f, LEVELSEXY_EASY, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("frozen_vhard"), f, LEVELFROZEN_VHARD, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("mecha_hard"), f, LEVELMECHA_HARD, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("sexy_normal"), f, LEVELSEXY_NORMAL, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("djbeat_hard"), f, LEVELDJBEAT_HARD, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("sexy_hard"), f, LEVELSEXY_HARD, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("bioteck_easy"), f, LEVELBIOTECK_EASY, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("bioteck_normal"), f, LEVELBIOTECK_NORMAL, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("bioteck_hard"), f, LEVELBIOTECK_HARD, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("tetris_easy"), f, LEVELTETRIS_EASY, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("tetris_normal"), f, LEVELTETRIS_NORMAL, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("tetris_hard"), f, LEVELTETRIS_HARD, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("tetris_vhard"), f, LEVELTETRIS_VHARD, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("voltigeur_easy"), f, LEVELVOLTIGEUR_EASY, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("voltigeur_normal"), f, LEVELVOLTIGEUR_NORMAL, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("voltigeur_hard"), f, LEVELVOLTIGEUR_HARD, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("turnandburn_normal"), f, LEVELTURNANDBURN_NORMAL, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("turnandburn_hard"), f, LEVELTURNANDBURN_HARD, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("ssoup_normal"), f, LEVELSSOUP_NORMAL, levels));	
		levels->addSubMenu(new MenuItem(bundle->getMessage("ssoup_hard"), f, LEVELSSOUP_HARD, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("musong_normal"), f, LEVELMUSONG_NORMAL, levels));	
		
		levels->addSubMenu(new MenuItem(bundle->getMessage("8bits_easy"), f, LEVEL8BITS_EASY, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("8bits_normal"), f, LEVEL8BITS_NORMAL, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("8bits_hard"), f, LEVEL8BITS_HARD, levels));
		
		levels->addSubMenu(new MenuItem(bundle->getMessage("cathedral_easy"), f, LEVELCATHEDRAL_EASY, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("cathedral_normal"), f, LEVELCATHEDRAL_NORMAL, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("cathedral_hard"), f, LEVELCATHEDRAL_HARD, levels));
		
		levels->addSubMenu(new MenuItem(bundle->getMessage("underwater_easy"), f, LEVELUNDERWATER_EASY, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("underwater_normal"), f, LEVELUNDERWATER_NORMAL, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("underwater_hard"), f, LEVELUNDERWATER_HARD, levels));
		
		levels->addSubMenu(new MenuItem(bundle->getMessage("chiptunes_easy"), f, LEVELCHIPTUNES_EASY, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("chiptunes_normal"), f, LEVELCHIPTUNES_NORMAL, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("chiptunes_hard"), f, LEVELCHIPTUNES_HARD, levels));
		
		levels->addSubMenu(new MenuItem(bundle->getMessage("test_easy"), f, LEVELTEST_EASY, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("test_normal"), f, LEVELTEST_NORMAL, levels));
		levels->addSubMenu(new MenuItem(bundle->getMessage("test_hard"), f, LEVELTEST_HARD, levels));
		
		levels->addSubMenu(new MenuItem(bundle->getMessage("cancel"), f, MENU_CANCEL, levels));
		
	
	MenuItem *utils=m_root->addSubMenu(new MenuItem(bundle->getMessage("utils"), f, 0, m_root, 3));
		utils->addSubMenu(new MenuItem(bundle->getMessage("leveleditor"), f, TEST_LEVELEDITOR, utils));
		utils->addSubMenu(new MenuItem(bundle->getMessage("scenario"), f, TEST_SCENARIO, utils));
		utils->addSubMenu(new MenuItem(bundle->getMessage("cancel"), f, MENU_CANCEL, utils));
	
	m_root->addSubMenu(new MenuItem(bundle->getMessage("quit"), f, MENU_QUIT, m_root));
	
	m_current=m_root;
		
	delete bundle;
	m_distance=(m_current->getSelectedChild()->getImage()->sizeY * 9)/10;
		
	//devrait etre dans le constructeur mais c seulement là qu'on connait m_distance
	delete m_fx_down_translation;
	delete m_fx_up_translation;
	m_fx_up_translation = new TranslationEffect(0,0,0,m_distance,translationtime,translationtime,true,1);
	m_fx_down_translation = new TranslationEffect(0,0,0,-m_distance,translationtime,translationtime,true,1);
	
	m_fx_highlight->start();
}

void MenuItemManager::initAsDemoMenu(string locale, BmpFont * f)
{
	I18nResourceBundle * bundle=new I18nResourceBundle(locale, "main_menu.properties");
	
	m_root=new MenuItem("root", f, 0, NULL, 2);
	m_root->addSubMenu(new MenuItem(bundle->getMessage("ssoup_normal"), f, LEVELSSOUP_NORMAL, m_root));
	m_root->addSubMenu(new MenuItem(bundle->getMessage("quit"), f, MENU_QUIT, m_root));
	
	m_current=m_root;
		
	delete bundle;
	m_distance=(m_current->getSelectedChild()->getImage()->sizeY * 9)/10;
		
	//devrait etre dans le constructeur mais c seulement là qu'on connait m_distance
	delete m_fx_down_translation;
	delete m_fx_up_translation;
	m_fx_up_translation = new TranslationEffect(0,0,0,m_distance,translationtime,translationtime,true,1);
	m_fx_down_translation = new TranslationEffect(0,0,0,-m_distance,translationtime,translationtime,true,1);
	
	m_fx_highlight->start();
}









