#include "code/game/menus/ShowImage.h"
#include <oslib/oslib.h>
#include "code/framework/Properties.h"
#include "code/framework/fx/EffectsLib.h"
#include "code/framework/utils.h"
#include "code/framework/check/loaders.h"

using namespace utils;

ShowImage::ShowImage(string png, int alphatime, int staytime)
{
	
	m_img=fwkLoadImageFilePNG((char *)png.c_str(), OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslSetImageRotCenter(m_img);
	oslImageCenterScreen(m_img);
	
	m_fx_appear=new ProgressiveAlphaEffect(alphatime, alphatime, 0, 255, 1);
	m_fx_disappear=new ProgressiveAlphaEffect(alphatime, alphatime, 255, 0, 1);
	m_delayer=new Delayer(alphatime+staytime);
	
}

ShowImage::~ShowImage()
{
	oslDeleteImage(m_img);
	
	delete m_fx_appear;
	delete m_fx_disappear;
	delete m_delayer;
}


void ShowImage::go()
{
	
	m_fx_appear->start();
	
	bool skip=false; //skip phase de dessin si trop de calculs
	while(!m_fx_disappear->finished()) //on quitte des que le fadeout a fini
	{
		//touches
		keys();
		
		//phase d'update
		m_fx_disappear->update();
		m_fx_appear->update();
		m_delayer->update();
		
		if(m_delayer->get())
		{
			m_fx_appear->stop();
			m_fx_disappear->start();
		}
				
			
		//phase de dessin
		if(!skip)
		{
			oslStartDrawing();
			drawScene();
			oslEndDrawing();
		}
		oslEndFrame();
		//Synchronize to ensure the game runs at 60 fps
	    skip = oslSyncFrame();
	}
}

void ShowImage::keys()
{
	oslReadKeys();
	if (osl_pad.pressed.start || osl_pad.pressed.cross || osl_pad.pressed.circle)  
	{
		m_fx_appear->stop();
		m_fx_disappear->start();
	}
}

void ShowImage::drawScene()
{
	oslClearScreen(RGB(0,0,0));
	
	if(m_fx_appear->isRunning())
	{
		m_fx_appear->activate();
	}
	else // if(m_fx_disappear->isRunning())
	{
		m_fx_disappear->activate();
	}
	oslDrawImage(m_img);
	
	
	oslSetAlpha(OSL_FX_DEFAULT, 0);
}



















