#include "code/game/menus/qj/QJScoreMenu.h"
#include <oslib/oslib.h>
#include "code/framework/utils.h"
//#include "code/game/profile/ProfileManager.h"
#include <string>
#include "code/game/gamedef.h"
#include "code/framework/check/loaders.h"

using namespace std;
using namespace utils;

QJScoreMenu::QJScoreMenu()
{
	m_gamesave=new GameSave();
	
	m_font=new BmpFont("res/fonts/bmpFonts/angie/14/", "angie", OSL_PF_8888);
	m_font_big=new BmpFont("res/fonts/bmpFonts/angie/22/", "angie", OSL_PF_8888);
	
	m_linesize=20;
	m_margin=5;
	m_color=RGB(0,0,185);
	m_col1width=200;
	
	m_texts_levels=new BmpText*[13];
	for(int i=0;i<13;i++)
	{
		m_texts_levels[i]=new BmpText("", m_font);
	}
	
	
	m_text_level=new BmpText("", m_font_big); //titre du level
	m_text_score_title=new BmpText("", m_font);
	m_text_score=new BmpText("", m_font_big);
	m_text_combos_title=new BmpText("", m_font);
	m_text_combos=new BmpText("", m_font_big);
	
	//initialisation des textes fixes
	m_texts_levels[0]->setText("8 bits I");
	m_texts_levels[1]->setText("8 bits II");
	m_texts_levels[2]->setText("8 bits III");
	m_texts_levels[3]->setText("Tetris I");
	m_texts_levels[4]->setText("Tetris II");
	m_texts_levels[5]->setText("Tetris III");
	m_texts_levels[6]->setText("Tetris IV");
	m_texts_levels[7]->setText("Project Sexy I");
	m_texts_levels[8]->setText("Project Sexy II");
	m_texts_levels[9]->setText("Project Sexy III");
	m_texts_levels[10]->setText("8 bits Blixt");
	m_texts_levels[11]->setText("Frozen IV");
	m_texts_levels[12]->setText("Voltigeur (alpha)");
	
	m_text_score_title->setText("Your best score:");
	m_text_combos_title->setText("Your best combos:");
	
	//initialisation des textes non fixes
	m_selected=0;
	
	initTexts();
	
	m_text_ok=new BmpText("OK", m_font);
	m_anim_buttons=new Anim("res/qj/dialogs/cross.png", 125.0f);
	m_anim_buttons->addImage("res/qj/dialogs/square.png", OSL_PF_8888);
	m_img_bg=fwkLoadImageFilePNG("res/qj/scores.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	
	m_music=fwkLoadSoundFileMP3("res/qj/dialogs/meh.mp3", OSL_FMT_STREAM);
	oslSetSoundLoop(m_music, true);
}

void QJScoreMenu::initTexts()
{
	//titre
	m_text_level->setText(m_texts_levels[m_selected]->getText());
	
	string code;
	int diff=GAMEDEF_DIFFICULTY_NORMAL;
	
	switch(m_selected)
	{
		case 0:
		{
			code="8bits";
			diff=GAMEDEF_DIFFICULTY_EASY;
			break;
		}
		case 1:
		{
			code="8bits";
			diff=GAMEDEF_DIFFICULTY_NORMAL;
			break;
		}
		case 2:
		{
			code="8bits";
			diff=GAMEDEF_DIFFICULTY_HARD;
			break;
		}
		case 3:
		{
			code="tetris";
			diff=GAMEDEF_DIFFICULTY_EASY;
			break;
		}
		case 4:
		{
			code="tetris";
			diff=GAMEDEF_DIFFICULTY_NORMAL;
			break;
		}
		case 5:
		{
			code="tetris";
			diff=GAMEDEF_DIFFICULTY_HARD;
			break;
		}
		case 6:
		{
			code="tetris";
			diff=GAMEDEF_DIFFICULTY_VERYHARD;
			break;
		}
		case 7:
		{
			code="sexy";
			diff=GAMEDEF_DIFFICULTY_EASY;
			break;
		}
		case 8:
		{
			code="sexy";
			diff=GAMEDEF_DIFFICULTY_NORMAL;
			break;
		}
		case 9:
		{
			code="sexy";
			diff=GAMEDEF_DIFFICULTY_HARD;
			break;
		}
		case 10:
		{
			code="8bits";
			diff=GAMEDEF_DIFFICULTY_BONUS;
			break;
		}
		case 11:
		{
			code="pdr4cfrozen";
			diff=GAMEDEF_DIFFICULTY_VERYHARD;
			break;
		}
		case 12:
		{
			code="voltigeur";
			diff=GAMEDEF_DIFFICULTY_HARD;
			break;
		}
	}
	
	m_text_score->setText(toString(m_gamesave->getScoreForLevel(code, diff)));
	m_text_combos->setText(toString(m_gamesave->getCombosForLevel(code, diff)));
}

QJScoreMenu::~QJScoreMenu()
{
	delete m_font;
	delete m_font_big;
	
	for(int i=0;i<13;i++)
	{
		delete m_texts_levels[i];
	}
	delete[] m_texts_levels; 
	
	delete m_text_level;
	delete m_text_score_title;
	delete m_text_score;
	delete m_text_combos_title;
	delete m_text_combos;
	
	delete m_text_ok;
	delete m_anim_buttons;
	oslDeleteImage(m_img_bg);
	
	delete m_gamesave;
}

void QJScoreMenu::go()
{
	m_anim_buttons->start();
	oslPlaySound(m_music, 0);
	bool quit=false;
	while(!quit)
	{
		quit=keys();
		
		//phase d'update
		//update();
		m_anim_buttons->update();
		
		//phase de dessin
		oslStartDrawing();
		draw();
		oslEndDrawing();
		
        oslEndFrame();
        oslSyncFrame();
	}
}


void QJScoreMenu::draw()
{	
	//oslClearScreen(RGB(0,0,0));
	oslDrawImage(m_img_bg);
	
	//levels standards
	int margin=-2;
	for(int i=0;i<13;i++)
	{
		int width=m_texts_levels[i]->getWidth();
		int x=int(18+81-width/2.0f);
		int y=margin + i*m_linesize + m_margin;
		
		if(m_selected!=i)
		{
			oslSetAlpha(OSL_FX_TINT, RGBA(0,0,255,95));
		}
		
		m_texts_levels[i]->drawXY(x, y);
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
	
	//textes de droite (scores)
	int x=201+129;
	m_text_level->drawXY(int(x-m_text_level->getWidth()/2.0f), margin + m_margin + 5 );
	m_text_score_title->drawXY(int(x-m_text_score_title->getWidth()/2.0f), 85);
	m_text_score->drawXY(int(x-m_text_score->getWidth()/2.0f), 110);
	m_text_combos_title->drawXY(int(x-m_text_combos_title->getWidth()/2.0f), 175);
	m_text_combos->drawXY(int(x-m_text_combos->getWidth()/2.0f), 200);	
	
	
	//bouton ok
	m_anim_buttons->drawXY(413, 247);
	m_text_ok->drawXY(438, 242);
}

bool QJScoreMenu::keys()
{
	oslReadKeys();
	
	if(osl_pad.pressed.cross || osl_pad.pressed.square || osl_pad.pressed.start)
	{
		return true;
	}
	if(osl_pad.pressed.select)
	{
		screenshot();
	}
	
	if(osl_pad.pressed.up)
	{
		if(m_selected>0)
		{
			m_selected--;
			initTexts();
		}
	}
	else if(osl_pad.pressed.down)
	{
		if(m_selected<12)
		{
			m_selected++;
			initTexts();
		}
	}
	
	
	return false;
}




