#include "code/game/menus/qj/dialogs/YouWinDialog.h"
#include <oslib/oslib.h>
#include "code/framework/utils.h"
#include <iostream>
#include "code/game/levels/abstract/AbstractLevel.h"
#include "code/framework/check/loaders.h"
#include "code/framework/check/PrettyFileLogger.h"

using namespace utils;

YouWinDialog::YouWinDialog(string level_code, int level_difficulty, int score, int combos, int bestscore, int bestcombos)
: AbstractDialog("res/qj/dialogs/youwin.png")
{
	PFL::logConstructionStart("YouWinDialog");
			
	
	m_font_big=new BmpFont("res/fonts/bmpFonts/angie/22/", "angie", OSL_PF_8888);
	m_font_normal=new BmpFont("res/fonts/bmpFonts/angie/20/", "angie", OSL_PF_8888);
	m_font_small=new BmpFont("res/fonts/bmpFonts/angie/16/", "angie", OSL_PF_8888);
	
	m_text_title=new BmpText("You win!", m_font_big);
	
	m_text_best_score_title=new BmpText("Best score:", m_font_small);
	//int bestscore=ProfileManager::getBestScoreForLevel(level->getCode(), level->getDifficulty()).getScore();
	
	
	m_text_best_score=new BmpText(toString(bestscore), m_font_small);
	m_text_your_score_title=new BmpText("Your score:", m_font_small);
	m_text_your_score=new BmpText(toString(score), m_font_small);
	
	m_text_best_combo_title=new BmpText("Best combos:", m_font_small);
	//int bestcombos=ProfileManager::getBestCombosForLevel(level->getCode(), level->getDifficulty()).getScore();
	//int bestcombos=gamesave->getCombosForLevel(level_code, level_difficulty);
	m_text_best_combo=new BmpText(toString(bestcombos), m_font_small);
	m_text_your_combo_title=new BmpText("Your combos:", m_font_small);
	m_text_your_combo=new BmpText(toString(combos), m_font_small);
	
	m_text_playagain=new BmpText("Play again?", m_font_normal);
	
	m_text_yes=new BmpText("Yes", m_font_normal);
	m_text_no=new BmpText("No", m_font_normal);
	
	m_music=fwkLoadSoundFileMP3("res/qj/dialogs/life_is_art.mp3", OSL_FMT_STREAM);
	//oslSetSoundLoop(m_music, true);
	
	m_isBestCombo=false;
	m_isBestScore=false;
	
	if(score>bestscore)
		m_isBestScore=true;
	if(combos>bestcombos)
		m_isBestCombo=true;
		
	m_img_cross=fwkLoadImageFilePNG("res/qj/dialogs/cross.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	m_img_square=fwkLoadImageFilePNG("res/qj/dialogs/square.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);

	PFL::logConstructionEnd("YouWinDialog");
}

YouWinDialog::~YouWinDialog()
{
	PFL::logDestructionStart("YouWinDialog");
	
	delete m_font_big;
	delete m_font_normal;
	delete m_font_small;
	
	delete m_text_title;
	delete m_text_best_score_title;
	delete m_text_best_score;
	delete m_text_your_score_title;
	delete m_text_your_score;
	delete m_text_best_combo_title;
	delete m_text_best_combo;
	delete m_text_your_combo_title;
	delete m_text_your_combo;
	delete m_text_playagain;
	
	delete m_text_yes;
	delete m_text_no;
	
	fwkDeleteSound(m_music);
	
	oslDeleteImage(m_img_cross);
	oslDeleteImage(m_img_square);
	
	PFL::logDestructionEnd("YouWinDialog");
}


void YouWinDialog::startSpecific()
{
	oslPlaySound(m_music, 0);
}

void YouWinDialog::updateSpecific()
{
	//rien
}


void YouWinDialog::drawSpecific()
{
	m_text_title->drawXY(123, 42);
	
	m_text_best_score_title->drawXY(123, 77);
	m_text_best_score->drawXY(361-m_text_best_score->getWidth(), 77);
	
	if(m_isBestScore)
	{
		oslSetAlpha(OSL_FX_TINT, RGB(255,0,0));
	}
	m_text_your_score_title->drawXY(122, 97);
	m_text_your_score->drawXY(361-m_text_your_score->getWidth(), 97);
	oslSetAlpha(OSL_FX_DEFAULT, 0);
	
	m_text_best_combo_title->drawXY(123, 127);
	m_text_best_combo->drawXY(361-m_text_best_combo->getWidth(), 127);
	
	if(m_isBestCombo)
	{
		oslSetAlpha(OSL_FX_TINT, RGB(255,0,0));
	}
	m_text_your_combo_title->drawXY(122, 147);
	m_text_your_combo->drawXY(361-m_text_your_combo->getWidth(), 147);
	oslSetAlpha(OSL_FX_DEFAULT, 0);
	
	m_text_playagain->drawXY(123, 177);
	
	
	oslDrawImageXY(m_img_cross, 247, 212);
	m_text_yes->drawXY(267, 201);
	oslDrawImageXY(m_img_square, 317, 212);
	m_text_no->drawXY(337, 201);	
}

int YouWinDialog::keys()
{
	oslReadKeys();
	
	if(osl_pad.pressed.square)
	{
		return DIALOG_NO;
	}
	else if(osl_pad.pressed.cross)
	{
		return DIALOG_YES;
	}
	else if(osl_pad.pressed.select)
	{
		screenshot();
	}
	
	
	return DIALOG_NONE;
}

		
		
		
