#include "code/game/pieces/PiecesBufferFactory.h"
#include "code/framework/anim/Anim.h"
#include <vector>
#include "code/game/gamedef.h"
#include <string>
#include "code/framework/utils.h"
#include "code/game/pieces/abstract/AbstractPiece.h"
#include "code/game/pieces/impl/PieceJoker.h"
#include "code/framework/check/loaders.h"
#include "code/framework/check/PrettyFileLogger.h"
#include "code/framework/Properties.h"

using namespace std;
using namespace utils;


//c++ fait chier...:
vector<Anim *> *PiecesBufferFactory::m_pieceDestrAnim=NULL;
OSL_IMAGE **PiecesBufferFactory::m_comboNumbers=NULL;
bool PiecesBufferFactory::m_isBuffered=false;
Anim *PiecesBufferFactory::m_comboShine=NULL;
OSL_IMAGE **PiecesBufferFactory::m_shieldedImages=NULL;
OSL_IMAGE *PiecesBufferFactory::m_joker1=NULL;
OSL_IMAGE *PiecesBufferFactory::m_joker2=NULL;
int PiecesBufferFactory::m_combo_image_number=0;

Anim *PiecesBufferFactory::newPieceDestrAnim(int shape)
{
	if(!m_isBuffered)
		fatalError("PiecesBufferFactory::newPieceDestrAnim : n'est pas en buffer");
	return new Anim(*(m_pieceDestrAnim->at(shape))); //constructeur par copie.
}

Anim *PiecesBufferFactory::newComboDestrAnim()
{
	if(!m_isBuffered)
		fatalError("PiecesBufferFactory::newComboDestrAnim : n'est pas en buffer");
	return new Anim(*(m_pieceDestrAnim->at(6))); //on a mis l'anim "combo" en place num 6 dans le vecteur
}

Anim *PiecesBufferFactory::newComboShineAnim()
{
	if(!m_isBuffered)
		fatalError("PiecesBufferFactory::newComboShineAnim : n'est pas en buffer");
	return new Anim(*m_comboShine); //constructeur par copie.
}

OSL_IMAGE *PiecesBufferFactory::getJokerImage1()
{
	if(!m_isBuffered)
		fatalError("PiecesBufferFactory::getJokerImage1 : n'est pas en buffer");
	return m_joker1;
}

OSL_IMAGE *PiecesBufferFactory::getJokerImage2()
{
	if(!m_isBuffered)
		fatalError("PiecesBufferFactory::getJokerImage2 : n'est pas en buffer");
	return m_joker2;
}

void PiecesBufferFactory::bufferResources()
{
	PFL::logFunctionStart("PiecesBufferFactory::bufferResources");
	
	if(m_isBuffered)
		fatalError("PiecesBufferFactory::bufferResources : ressources already buffered.");	
	
	Properties *props=new Properties("res/pieces/pieces.properties");
	
	m_pieceDestrAnim=new vector<Anim *>();
	
	Anim *circle =	new Anim(	props->getIntValue("piece.circle.nbframes"), 
								"circle", "res/pieces/circle/", 
								GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	Anim *cross =	new Anim(	props->getIntValue("piece.cross.nbframes"), 
								"cross", "res/pieces/cross/", 
								GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	Anim *square =	new Anim(	props->getIntValue("piece.square.nbframes"), 
								"square", "res/pieces/square/", 
								GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	Anim *triangle =new Anim(	props->getIntValue("piece.triangle.nbframes"), 
								"triangle", "res/pieces/triangle/", 
								GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	Anim *star =	new Anim(	props->getIntValue("piece.star.nbframes"), 
								"star", "res/pieces/star/", 
								GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	Anim *diamond =	new Anim(	props->getIntValue("piece.diamond.nbframes"), 
								"diamond", "res/pieces/diamond/", 
								GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	Anim *combo =	new Anim(	props->getIntValue("piece.combo.nbframes"), 
								"combo", "res/pieces/combo/", 
								GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	
	//ordre très important, car accès par numéro après.
	//TODO: utiliser insert at position en utilisant les defines?
	m_pieceDestrAnim->push_back(circle);
	m_pieceDestrAnim->push_back(cross);
	m_pieceDestrAnim->push_back(square);
	m_pieceDestrAnim->push_back(triangle);
	m_pieceDestrAnim->push_back(star);
	m_pieceDestrAnim->push_back(diamond);
	m_pieceDestrAnim->push_back(combo);
	
	//on va définir le centre de chaque image
	for(vector<Anim *>::iterator it = m_pieceDestrAnim->begin(); it!=m_pieceDestrAnim->end(); ++it) 
	{
		Anim *anim=*it;
		anim->setReferencePointCenter();
	}
	
	m_combo_image_number=props->getIntValue("piece.combo.numbers.max")+2; //de 0 a 20 et la piece "?"
	m_comboNumbers=new OSL_IMAGE *[m_combo_image_number];
	for(int i=0;i<m_combo_image_number-1;i++)
	{
		m_comboNumbers[i]=fwkLoadImageFilePNG((char *)string("res/pieces/combo/numbers/combo_num_"+toString(i)+".png").c_str(), OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
		oslImageSetRotCenter(m_comboNumbers[i]);
	}
	m_comboNumbers[m_combo_image_number-1]=fwkLoadImageFilePNG("res/pieces/combo/numbers/combo_num_more.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetRotCenter(m_comboNumbers[m_combo_image_number-1]);
	
	m_comboShine =	new Anim(	props->getIntValue("piece.combo.shine.nbframes"), 
								"combo_change_", "res/pieces/combo/change/", 
								props->getIntValue("piece.combo.shine.time"), 
								OSL_PF_8888, 1);
	m_comboShine->setReferencePointCenter();
	
	m_shieldedImages=new OSL_IMAGE *[PIECE_NB_SHAPES];
	m_shieldedImages[PIECE_SHAPE_CIRCLE]=fwkLoadImageFilePNG("res/pieces/shielded/shielded_circle.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetRotCenter(m_shieldedImages[PIECE_SHAPE_CIRCLE]);
	m_shieldedImages[PIECE_SHAPE_SQUARE]=fwkLoadImageFilePNG("res/pieces/shielded/shielded_square.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetRotCenter(m_shieldedImages[PIECE_SHAPE_SQUARE]);
	m_shieldedImages[PIECE_SHAPE_TRIANGLE]=fwkLoadImageFilePNG("res/pieces/shielded/shielded_triangle.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetRotCenter(m_shieldedImages[PIECE_SHAPE_TRIANGLE]);
	m_shieldedImages[PIECE_SHAPE_STAR]=fwkLoadImageFilePNG("res/pieces/shielded/shielded_star.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetRotCenter(m_shieldedImages[PIECE_SHAPE_STAR]);
	m_shieldedImages[PIECE_SHAPE_DIAMOND]=fwkLoadImageFilePNG("res/pieces/shielded/shielded_diamond.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetRotCenter(m_shieldedImages[PIECE_SHAPE_DIAMOND]);
	m_shieldedImages[PIECE_SHAPE_CROSS]=fwkLoadImageFilePNG("res/pieces/shielded/shielded_cross.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetRotCenter(m_shieldedImages[PIECE_SHAPE_CROSS]);
	
	m_joker1=fwkLoadImageFilePNG("res/pieces/joker/joker_ray_big.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetRotCenter(m_joker1);
	m_joker2=fwkLoadImageFilePNG("res/pieces/joker/joker_ray2_big.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetRotCenter(m_joker2);
	
	m_isBuffered=true;
	
	delete props;
	
	PFL::logFunctionEnd("PiecesBufferFactory::bufferResources");
}

void PiecesBufferFactory::unbufferResources()
{
	PFL::logFunctionStart("PiecesBufferFactory::unbufferResources");
	
	if(!m_isBuffered)
		fatalError("PiecesBufferFactory::unbufferResources : ressources were not buffered.");
	//enableDebug();
	debug("unbuffer ressources");
	for(vector<Anim *>::iterator it = m_pieceDestrAnim->begin(); it!=m_pieceDestrAnim->end(); ++it) 
	{
		delete *it;	
	}
	delete m_pieceDestrAnim;
	m_pieceDestrAnim=NULL;
	
	for(int i=0; i<m_combo_image_number; i++)
	{
		oslDeleteImage(m_comboNumbers[i]);
	}
	delete[] m_comboNumbers;
	
	for(int i=0; i<PIECE_NB_SHAPES; i++)
	{
		oslDeleteImage(m_shieldedImages[i]);
	}
	delete[] m_shieldedImages;
	
	delete m_comboShine;
	
	oslDeleteImage(m_joker1);
	oslDeleteImage(m_joker2);
	
	m_isBuffered=false;
	debug("unbuffer done");
	PFL::logFunctionEnd("PiecesBufferFactory::unbufferResources");
}

OSL_IMAGE *PiecesBufferFactory::getComboImage(int number)
{
	if(!m_isBuffered)
		fatalError("PiecesBufferFactory::getComboImage : n'est pas en buffer");
		
	if(number>=m_combo_image_number-1)
		return m_comboNumbers[m_combo_image_number-1];
		//oslFatalError("PiecesBufferFactory::getComboImage : ne peut pas fournir un nombre > COMBO_IMAGE_MAX_NUMBER");
	
	return m_comboNumbers[number];
}

int PiecesBufferFactory::getNbAnims()
{
	if(!m_isBuffered)
		fatalError("PiecesBufferFactory::getNbAnims : n'est pas en buffer");
	return m_pieceDestrAnim->size();
}

OSL_IMAGE *PiecesBufferFactory::getShieldedImage(int shape)
{
	return m_shieldedImages[shape];
}



















