#include "code/game/pieces/impl/PieceLevel.h"
#include "code/game/gamedef.h"

PieceLevel::PieceLevel(int shape, int color, int id)
{
	m_color_num=color;
	m_shape_num=shape;
	m_id=id;
	m_random=false;
	
	m_repeater=new Repeater(int(GAMEDEF_EXPLOSION_TIME), 1);
}

PieceLevel::PieceLevel(int id)
{
	m_color_num=random(0,5);
	m_shape_num=random(0,5);
	m_id=id;
	m_random=true;
	
	m_repeater=new Repeater(int(GAMEDEF_EXPLOSION_TIME), 1);
}

void PieceLevel::setExplosionTime(float time)
{
	m_repeater->setDuration(int(time));
}


PieceLevel::~PieceLevel()
{
	delete m_repeater;
}

PieceLevel::PieceLevel(const PieceLevel &piece)//copie
{
	m_color_num=piece.m_color_num;
	m_shape_num=piece.m_shape_num;
	
	m_id=piece.m_id;
	
	m_random=piece.m_random;
}

PieceLevel& PieceLevel::operator =(const PieceLevel& piece)
{
	m_color_num=piece.m_color_num;
	m_shape_num=piece.m_shape_num;

	m_random=piece.m_random;
	
	m_id=piece.m_id;
	
	return *this;
}

PieceLevel * PieceLevel::clone() const
{
	PieceLevel *p=new PieceLevel(m_shape_num, m_color_num, m_id);
	p->m_random=m_random;
	p->setExplosionTime(m_repeater->getDuration());
	return p;
}

int PieceLevel::getShape()
{
	return m_shape_num;
}

int PieceLevel::getColor()
{
	return m_color_num;
}

void PieceLevel::setColor(int color)
{
	m_color_num=color;
}

void PieceLevel::setShape(int shape)
{
	m_shape_num=shape;
}

int PieceLevel::getType()
{
	return PIECE_TYPE_LEVEL;
}

void PieceLevel::startDestruct()
{
	m_repeater->start();
}

void PieceLevel::drawXY(int x, int y, AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3, AbstractLocalEffect *fx4, AbstractLocalEffect *fx5)
{
	//le level doit s'en charger	
}

void PieceLevel::update()
{
	m_repeater->update();
}
 
bool PieceLevel::destructed()
{
	return m_repeater->finished();
}

bool PieceLevel::destructing()
{
	return m_repeater->isRunning();
}

bool PieceLevel::isRandom()
{
	return m_random;
}

int PieceLevel::specialEvent(int e) //get id est la dedans
{
	if(e==PIECE_EVENT_LEVEL_GETID)
		return m_id;
	else if(e==PIECE_EVENT_LEVEL_ID_ADD)
		m_id++;
	else if(e==PIECE_EVENT_LEVEL_ID_REDUCE && m_id>0)
		m_id--;
		
	return 0;
}

void PieceLevel::setId(int id)
{
	m_id=id;
}







