#include "code/game/saves/GameSave.h"
#include "code/framework/utils.h"
#include <string>
#include "code/game/gamedef.h"
#include "code/framework/check/PrettyFileLogger.h"

using namespace utils;
using namespace std;

//TODO: diminuer ca? 8ko pour le moment;
#define GAMESAVE_BUFFER_LEN 8192

unsigned int GameSave::m_size_icon0=0;

GameSave::GameSave()
{
	m_props=NULL;
	m_save_exists=false;
	m_size_icon0=0;
	load();
}

GameSave::~GameSave()
{
	if(m_props==NULL)
		fatalError("GameSave::~GameSave : m_props is null");
	delete m_props;
}

void GameSave::initSavedata(SceUtilitySavedataParam * savedata, PspUtilitySavedataMode mode)
{	
	char key[] = "DJDROPSDAAA57150";	// Key to encrypt or decrypt savedata
	
	memset(savedata, 0, sizeof(SceUtilitySavedataParam));
	savedata->base.size = sizeof(SceUtilitySavedataParam);

	savedata->base.graphicsThread = 0x11;
	savedata->base.accessThread = 0x13;
	savedata->base.fontThread = 0x12;
	savedata->base.soundThread = 0x10;

	savedata->mode = mode;
	savedata->overwrite = 1;

#if _PSP_FW_VERSION >= 200
	strncpy(savedata->key, key, 16);
#endif

	strcpy(savedata->gameName, "DJDROPS");	// First part of the save name, game identifier name
	strcpy(savedata->saveName, "DATA");	// Second part of the save name, save identifier name

	strcpy(savedata->fileName, "DATA.BIN");	// name of the data file

	

	// Set some data
	if (mode == PSP_UTILITY_SAVEDATA_AUTOSAVE)
	{
		//char test[] = "je suis un test";
		string str_to_save=m_props->toString();
		//char tosave[str_to_save.length()+1];
		//strcpy(&tosave, str_to_save.c_str());
		long mem_len=str_to_save.length()+1;
		
		//Allocate buffer used to store various parts of the save data
		savedata->dataBuf = malloc(mem_len);
		savedata->dataBufSize = mem_len;
		savedata->dataSize = mem_len;
		
		memset(savedata->dataBuf, 0, mem_len); 
		memcpy(savedata->dataBuf, str_to_save.c_str(), mem_len);
	
		strcpy(savedata->sfoParam.title, "DJ Drops");
		strcpy(savedata->sfoParam.savedataTitle, "DJ Drops Save Data");
		strcpy(savedata->sfoParam.detail, "");
		savedata->sfoParam.parentalLevel = 1;
		
		// No icon1
		savedata->icon1FileData.buf = NULL;
		savedata->icon1FileData.bufSize = 0;
		savedata->icon1FileData.size = 0;
	
		if(!m_save_exists)
		{
			//on lui file une image
			savedata->icon0FileData.buf = bin2c("res/icon0.png");
		    savedata->icon0FileData.bufSize = m_size_icon0;
			savedata->icon0FileData.size = m_size_icon0;
			    
			//OSL_IMAGE * icon0=oslLoadImageFilePNG("res/icon0.png", OSL_IN_RAM|OSL_UNSWIZZLED, OSL_PF_8888);
			
			/*savedata->icon0FileData.buf = malloc(icon0->totalSize);
			savedata->icon0FileData.bufSize = icon0->totalSize;
			savedata->icon0FileData.size = icon0->totalSize;
			
			memcpy(savedata->icon0FileData.buf, icon0->data, icon0->totalSize);
			*/
			//oslDeleteImage(icon0);
		}
		else
		{
			//No icon0
			savedata->icon0FileData.buf = NULL;
			savedata->icon0FileData.bufSize = 0;
			savedata->icon0FileData.size = 0;
		}
		
		// No pic1
		savedata->pic1FileData.buf = NULL;
		savedata->pic1FileData.bufSize = 0;
		savedata->pic1FileData.size = 0;
		
		
		
		// No snd0
		savedata->snd0FileData.buf = NULL;
		savedata->snd0FileData.bufSize = 0;
		savedata->snd0FileData.size = 0;
	
	}
	else //LOAD
	{
		//Allocate buffer used to store various parts of the save data
		savedata->dataBuf = malloc(GAMESAVE_BUFFER_LEN);
		savedata->dataBufSize = GAMESAVE_BUFFER_LEN;
		savedata->dataSize = GAMESAVE_BUFFER_LEN;
		
		//memset(savedata->dataBuf, 0, sizeof(test));
		
		strcpy(savedata->sfoParam.title, "NULL");
		savedata->focus = PSP_UTILITY_SAVEDATA_FOCUS_LATEST;
	}
}

void GameSave::cleanSavedata(SceUtilitySavedataParam * savedata)
{
	free(savedata->dataBuf);
	if(savedata->icon0FileData.buf!=NULL)
		free(savedata->icon0FileData.buf);
}

void GameSave::load()
{
	PFL::logFunctionStart("GameSave::load");
	
	if(m_props!=NULL)
	{
		PFL::logMessage("gamesave already loaded, exiting...");
		PFL::logFunctionEnd("GameSave::load");
		return;
	}	
	
	SceUtilitySavedataParam params;
	
	initSavedata(&params, PSP_UTILITY_SAVEDATA_AUTOLOAD);
	
	int r=sceUtilitySavedataInitStart(&params);
	if(r!=0)
	{
		fatalError("GameSave::load : sceUtilitySavedataInitStart error: "+toString(r));
	}
	
	//debug("debut while");
	while(true)
	{
		int status=sceUtilitySavedataGetStatus();
		switch(status) 
		{
			case PSP_UTILITY_DIALOG_VISIBLE :
			{
				//debug("visible "+getTimestamp());
				sceUtilitySavedataUpdate(1);
	    		break;
			}
			case PSP_UTILITY_DIALOG_QUIT :
			{
				//debug("quit "+getTimestamp());
	    		sceUtilitySavedataShutdownStart();
	    		break;
			}
			case PSP_UTILITY_DIALOG_FINISHED :
			{
				//no break, continue down
			}	
			case PSP_UTILITY_DIALOG_NONE :
			{
				if(string(params.sfoParam.title)=="NULL")
				{
					//Save doesn't exist
					//enableDebug();
					//debug("save does not exist");
					//enableDebug(false);
					m_props=new Properties();
					/*m_props->setValue("bob", "0");
					m_props->setValue("bob1", "1");
					m_props->setValue("test", "ok");
					*/
				}
				else
				{
					//Save loaded ok!!
					//enableDebug();
					//debug("save loaded ok");
					/*char *result=(char *)malloc(m_buffer_len);
					debug("malloc ok");
					memcpy(result, params.dataBuf, m_buffer_len);
					debug("memcpy ok");
					debug("result = "+string(result));
					free(result);*/
					
					m_props=new Properties();
					m_props->loadFromString(string((char *)params.dataBuf));
					//debug(m_props->toString());
					
					//enableDebug(false);
					
					m_save_exists=true;
				}
				
				cleanSavedata(&params);
				PFL::logFunctionEnd("GameSave::load");
	    		return;
			}
		}
		sceDisplayWaitVblankStart();
	}
	fatalError("GameSave::load : returning without loading");
	return;
		
	
}

void GameSave::save()
{
	SceUtilitySavedataParam params;
	
	initSavedata(&params, PSP_UTILITY_SAVEDATA_AUTOSAVE);
	
	if(sceUtilitySavedataInitStart(&params)!=0)
		fatalError("GameSave::save : sceUtilitySavedataInitStart error");
	
	while(true)
	{
		switch(sceUtilitySavedataGetStatus()) 
		{
			case PSP_UTILITY_DIALOG_VISIBLE :
			{
				sceUtilitySavedataUpdate(1);
	    		break;
			}
			case PSP_UTILITY_DIALOG_QUIT :
			{
	    		sceUtilitySavedataShutdownStart();
	    		break;
			}
			case PSP_UTILITY_DIALOG_FINISHED :
			{
				message("GameSave::save ok");
				//no break;		
			}	
			case PSP_UTILITY_DIALOG_NONE :
			{
				cleanSavedata(&params);
				m_save_exists=true;
	    		return;
			}
		}
	
		//oslEndFrame();
		sceDisplayWaitVblankStart();
	}
	
	fatalError("GameSave::save : returning without saving");
	return;
}


unsigned char * __attribute__((aligned(16))) GameSave::bin2c(string filename)
{
	PFL::logFunctionStart("GameSave::bin2c");
	FILE *source;

	if((source=fopen(filename.c_str(), "rb")) == NULL) 
	{
		fatalError("GameSave::bin2c : cannot open icon0 file");
	}

	fseek(source,0,SEEK_END);
	m_size_icon0 = (unsigned int)ftell(source);
	fseek(source,0,SEEK_SET);

	unsigned char * buffer;
	buffer=(unsigned char *)malloc(m_size_icon0);

	if(fread(buffer, 1, m_size_icon0, source) != m_size_icon0) 
	{
		fatalError("GameSave::bin2c : Failed to read file.");
	}
	fclose(source);
	
	
	PFL::logFunctionEnd("GameSave::bin2c");
	return buffer;
}





Properties* GameSave::getProps()
{
	return m_props;
}


//reprises de profile.cpp


int GameSave::getScoreForLevel(string levelcode, int difficulty)
{
	string key="level."+levelcode+"."+difficultyToString(difficulty)+".score";
	if(!m_props->containsKey(key))
		return 0;
	return m_props->getIntValue(key);
}
void GameSave::setScoreForLevel(string levelcode, int difficulty, int score)
{
	m_props->setValue("level."+levelcode+"."+difficultyToString(difficulty)+".score", score);
}

int GameSave::getCombosForLevel(string levelcode, int difficulty)
{
	string key="level."+levelcode+"."+difficultyToString(difficulty)+".maxcombos";
	if(!m_props->containsKey(key))
		return 0;
	return m_props->getIntValue(key);
}
void GameSave::setCombosForLevel(string levelcode, int difficulty, int maxcombos)
{
	m_props->setValue("level."+levelcode+"."+difficultyToString(difficulty)+".maxcombos", maxcombos);
}

int GameSave::getNbLevelsPlayedTotal()
{
	string key="profile.levels.played";
	if(!m_props->containsKey(key))
		return 0;
	return m_props->getIntValue(key);
}		
void GameSave::setNbLevelsPlayedTotal(int nb)
{
	m_props->setValue("profile.levels.played", nb);
}

int GameSave::getNbLevelsWonTotal()
{
	string key="profile.levels.won";
	if(!m_props->containsKey(key))
		return 0;
	return m_props->getIntValue(key);
}
void GameSave::setNbLevelsWonTotal(int nb)
{
	m_props->setValue("profile.levels.won", nb);
}

int GameSave::getNbLevelsLostTotal()
{
	return getNbLevelsPlayedTotal()-getNbLevelsWonTotal();
}

float GameSave::getWinRatioTotal()
{
	if(getNbLevelsPlayedTotal()==0)
	{
		return 0.0f;
	}
	
	return (getNbLevelsWonTotal()*100.0f)/getNbLevelsPlayedTotal();
}

string GameSave::getWinRatioStrTotal()
{
	float ratio=getWinRatioTotal();
	
	ostringstream ost;
	ost.precision(2);
	ost.setf(ios::fixed,ios::floatfield);
	ost << ratio;
	
	return ost.str()+"%";
}

int GameSave::getNbPlayedForLevel(string levelcode, int difficulty)
{
	string key="level."+levelcode+"."+difficultyToString(difficulty)+".played";
	if(!m_props->containsKey(key))
		return 0;
	return m_props->getIntValue(key);
}

void GameSave::setNbPlayedForLevel(string levelcode, int difficulty, int nb)
{
	m_props->setValue("level."+levelcode+"."+difficultyToString(difficulty)+".played", nb);
}


int GameSave::getNbWonForLevel(string levelcode, int difficulty)
{
	string key="level."+levelcode+"."+difficultyToString(difficulty)+".won";
	if(!m_props->containsKey(key))
		return 0;
	return m_props->getIntValue(key);
}

void GameSave::setNbWonForLevel(string levelcode, int difficulty, int nb)
{
	m_props->setValue("level."+levelcode+"."+difficultyToString(difficulty)+".won", nb);
}

void GameSave::addWonForLevel(string levelcode, int difficulty)
{
	//ajout partie gagnée
	setNbWonForLevel(levelcode, difficulty, getNbWonForLevel(levelcode, difficulty)+1);
	
	//ajout partie jouée
	setNbPlayedForLevel(levelcode, difficulty, getNbPlayedForLevel(levelcode, difficulty)+1);
	
	//idem sur total
	setNbLevelsWonTotal(getNbLevelsWonTotal()+1);
	setNbLevelsPlayedTotal(getNbLevelsPlayedTotal()+1);
}

void GameSave::addLostForLevel(string levelcode, int difficulty)
{
	//ajout partie jouée
	setNbPlayedForLevel(levelcode, difficulty, getNbPlayedForLevel(levelcode, difficulty)+1);
	
	//idem sur total
	setNbLevelsPlayedTotal(getNbLevelsPlayedTotal()+1);
}

int GameSave::getNbLostForLevel(string levelcode, int difficulty)
{
	return getNbPlayedForLevel(levelcode, difficulty)-getNbWonForLevel(levelcode, difficulty);
}

float GameSave::getWinRatioForLevel(string levelcode, int difficulty)
{
	if(getNbPlayedForLevel(levelcode, difficulty)==0)
	{
		return 0.0f;
	}
	
	return (getNbWonForLevel(levelcode, difficulty)*100.0f)/getNbPlayedForLevel(levelcode, difficulty);
}

string GameSave::getWinRatioStrForLevel(string levelcode, int difficulty)
{
	float ratio=getWinRatioForLevel(levelcode, difficulty);
	
	ostringstream ost;
	ost.precision(2);
	ost.setf(ios::fixed,ios::floatfield);
	ost << ratio;
	
	return ost.str()+"%";
}

string GameSave::difficultyToString(int difficulty)
{
	switch(difficulty)
	{
		case GAMEDEF_DIFFICULTY_EASY:
		{
			return "easy";
		}
		case GAMEDEF_DIFFICULTY_NORMAL:
		{
			return "normal";
		}
		case GAMEDEF_DIFFICULTY_HARD:
		{
			return "hard";
		}
		case GAMEDEF_DIFFICULTY_VERYHARD:
		{
			return "vhard";
		}
		case GAMEDEF_DIFFICULTY_BONUS:
		{
			return "bonus";
		}
	}
	
	fatalError("GameSave::difficultyToString : unknown difficulty!");
	
	return "";
}



















