#include "code/tests/TestAnimations.h"
#include <oslib/oslib.h>
#include <string>
#include "code/framework/utils.h"
#include "code/game/menus/MenusDefines.h"

using namespace std;
using namespace utils;

//construction
TestAnimations::TestAnimations()
{
	oslSetKeyAutorepeat(OSL_KEYMASK_R|OSL_KEYMASK_L,40,4);
	m_action=MENU_NOACTION;
	m_skip=false;
	
	m_model=new Anim("res/pieces/circle/circle1.png", 70.0f); //test creation avec ajout d'images
	m_model->addImage("res/pieces/circle/circle2.png");
	m_model->addImage("res/pieces/circle/circle3.png");
	m_model->addImage("res/pieces/circle/circle4.png");
	m_model->addImage("res/pieces/circle/circle5.png");
	m_model->addImage("res/pieces/circle/circle6.png");
	m_model->addImage("res/pieces/circle/circle7.png");
	m_model->addImage("res/pieces/circle/circle8.png");
	m_model->addImage("res/pieces/circle/circle7.png");
	m_model->addImage("res/pieces/circle/circle6.png");
	m_model->addImage("res/pieces/circle/circle5.png");
	m_model->addImage("res/pieces/circle/circle4.png");
	m_model->addImage("res/pieces/circle/circle3.png");
	m_model->addImage("res/pieces/circle/circle2.png");
	//m_model->setReferencePointCenter();
	m_model->setReferencePoint(0,0);
	//8 images différentes mises en buffer normalement
	
	
	m_animations = new list<Anim *>();
	
}

//destruction
TestAnimations::~TestAnimations()
{
	delete m_model;
	
	for(list<Anim *>::iterator it = m_animations->begin(); it!=m_animations->end(); ++it)
	{
		delete (*it);
	}
	delete m_animations;
}

//demarrage du menu
int TestAnimations::go()
{
	while(m_action==MENU_NOACTION)
	{
		oslBenchmarkTest(OSL_BENCH_START);
		
		//touches
		m_action=keys();
		if(m_action==MENU_QUIT)
			return m_action;
		
		//phase d'update
		for(list<Anim *>::iterator it = m_animations->begin(); it!=m_animations->end(); ++it)
		{
			(*it)->update();
		}
		
		if (!m_skip)
		{
			//phase de dessin
			oslStartDrawing();
			drawScene();
			oslBenchmarkTest(OSL_BENCH_END);
			oslSysBenchmarkDisplay();
			oslEndDrawing();
		}
		
		//End of a frame
        oslEndFrame();
		//Synchronize to ensure the game runs at 60 fps
        m_skip = oslSyncFrame();
		//oslAudioVSync(); //ne sert plus, oslEndFrame le fait
	}
	//il a choisi une option
	enableDebug();
	debug("fin boucle TestAnimations, return "+toString(m_action));
	enableDebug(false);
	
	return m_action;	
}

//dessins
void TestAnimations::drawScene()
{
	oslClearScreen(RGB(0,0,0));
	
	//dessin des animations a leur place
	for(list<Anim *>::iterator it = m_animations->begin(); it!=m_animations->end(); ++it)
	{
		(*it)->draw();
	}
	
	oslDrawString(10, 250, ("nb animations: " + toString(m_animations->size())).c_str());
	oslDrawString(10, 260, ("images in buffer: " + toString(Anim::getNbBufferedImages())).c_str());
}

//gestion des touches
int TestAnimations::keys()
{
	oslReadKeys();
	if (osl_pad.pressed.start) //Exit 
	{
		enableDebug();
		debug("start!");
		enableDebug(false);
		return MENU_MAIN;
	}
	
	//ajout d'une animation
	if(osl_pad.pressed.R)
	{
		if(m_animations->size()<MAX_ANIMS) //on a de la place
		{
			Anim *anim=new Anim(*m_model);
			int nbanims=m_animations->size();
			int x, y; //position de l'anim
			x=(nbanims%24)*20;
			y=int(nbanims/24)*20;
			anim->setX(x);
			anim->setY(y);
			anim->start();
			m_animations->push_back(anim);
		}
	}
	//retrait d'une animation
	else if(osl_pad.pressed.L)
	{
		if(!m_animations->empty())
		{
			delete m_animations->back();
			m_animations->pop_back();
		}
	}
	
	
		
	if (osl_pad.pressed.up)
	{
		
	}
	else if(osl_pad.pressed.down)
	{
		
	}
	else if(osl_pad.pressed.left)
	{
		
	}
	else if(osl_pad.pressed.right)
	{
		
	}
	
	if(osl_pad.pressed.select)
	{
	
	}
	
	return MENU_NOACTION;
}
































