#include "code/tests/TestArtifice.h"
#include <oslib/oslib.h>
#include "code/framework/utils.h"
#include "code/framework/fx/EffectsLib.h"
#include "code/game/menus/MenusDefines.h"
#include "code/framework/particles/complete/fxFactories/PFXFRandomTranslation.h"
#include "code/framework/particles/complete/fxFactories/PFXFSimpleOrientation.h"

#define NBIMAGES4PARTICLE 14

TestArtifice::TestArtifice()
{
	oslSetKeyAutorepeat(OSL_KEYMASK_R,40,40);
	
	m_action=MENU_NOACTION;
	m_skip=false;
	
	//tests
	int transl_time=10; 
	int lifespan=40; 
	int transl_dist=120;
		
	PFXFRandomTranslation *rt=new PFXFRandomTranslation(16, transl_dist);
	PFXFSimpleOrientation *so=new PFXFSimpleOrientation();
	
	SerialGlobalEffect *transp=new SerialGlobalEffect();
	transp->addEffect(new WaitGlobalEffect(transl_time+1));
	transp->addEffect(new ProgressiveAlphaEffect(lifespan-transl_time,0,255,0));
	
	m_test_partGen=new ParticleGeneratorPoint(240, 136, 0, 360, 150, 1, transl_time, transl_dist, lifespan, "res/tests/artifice1.png", OSL_PF_4444, 1);
	m_test_partGen->setLocalEffect1(rt);
	m_test_partGen->setLocalEffect2(so);
	m_test_partGen->setGlobalEffect1(transp);
	
	//m_colors=new OSL_COLOR[12];
	m_colors=new OSL_COLOR[12];
	m_colors[0]=RGB(255,0,0);
	m_colors[1]=RGB(0,255,0);
	m_colors[2]=RGB(0,0,255);
	m_colors[3]=RGB(255,255,0);
	m_colors[4]=RGB(255,0,255);
	m_colors[5]=RGB(0,255,255);
	m_colors[6]=RGB(255,128,0);
	m_colors[7]=RGB(255,0,128);
	m_colors[8]=RGB(128,255,0);
	m_colors[9]=RGB(0,255,128);
	m_colors[10]=RGB(128,0,255);
	m_colors[11]=RGB(0,128,255);
	m_curentColor=m_colors[0];
	
	m_imageNum=1;
}

//destruction
TestArtifice::~TestArtifice()
{
	delete m_test_partGen;
	delete[] m_colors;
}

//demarrage du menu
int TestArtifice::go()
{
	oslSetBilinearFilter(true);
	//m_test_partGen->start();
	
	
	while(m_action==MENU_NOACTION)
	{
		oslBenchmarkTest(OSL_BENCH_START);
		
		//touches
		m_action=keys();
		if(m_action==MENU_QUIT)
			return m_action;
		
		//phase d'update		
		m_test_partGen->update();
		
		
		if (!m_skip)
		{
			//phase de dessin
			oslStartDrawing();
			drawScene();
			oslBenchmarkTest(OSL_BENCH_END);
			oslSysBenchmarkDisplay();
			oslEndDrawing();
		}
		
		//End of a frame
        oslEndFrame();
		//Synchronize to ensure the game runs at 60 fps
        m_skip = oslSyncFrame();
		//oslAudioVSync(); //ne sert plus, oslEndFrame le fait
	}
	//il a choisi une option
	enableDebug();
	debug("fin boucle main, return "+toString(m_action));
	enableDebug(false);
	
	oslSetBilinearFilter(false);
	
	return m_action;	
}

//dessins
void TestArtifice::drawScene()
{
	oslClearScreen(RGB(0,0,0));
	
	oslSetAlpha(OSL_FX_TINT, m_curentColor);
	m_test_partGen->draw();
	oslSetAlpha(OSL_FX_DEFAULT, 0);
	
	oslDrawString(10, 260, (toString(m_test_partGen->getCurrentNbParticles()) + " particles").c_str());
	
	
}

//gestion des touches
int TestArtifice::keys()
{
	oslReadKeys();
	if (osl_pad.pressed.start) //Exit 
	{
		enableDebug();
		debug("start!");
		enableDebug(false);
		return MENU_MAIN;
	}
	if(osl_pad.pressed.triangle)
	{
		if(m_test_partGen->isRunning())
			m_test_partGen->stop();
		else
			m_test_partGen->start();
	}
	
	if(osl_pad.pressed.R)
	{
		//TODO vrai random ici
		int r=utils::random(0, 11);
		m_curentColor=m_colors[r];
		m_test_partGen->restart();
		
		//ne pas faire ca en prod, vaut mieux les charger à l'avance en tant qu'anim.
		r=utils::random(1, NBIMAGES4PARTICLE);
		m_test_partGen->setAnimation("res/tests/artifice"+toString(r)+".png", OSL_PF_4444);
	}
	
	return MENU_NOACTION;
}



