#include "code/tests/TestGeneratorWithGravity.h"
#include <oslib/oslib.h>
#include "code/framework/anim/Anim.h"
#include "code/framework/utils.h"
#include "code/framework/fx/EffectsLib.h"
#include "code/game/menus/MenusDefines.h"
#include "code/framework/particles/complete/fx/ParticlesProgressiveTintEffect.h"
//#include "code/framework/particles/complete/animFactories/RandomAnimationFactory.h"
#include "code/framework/particles/complete/animFactories/SuccessiveAnimationFactory.h"

//construction
TestGeneratorWithGravity::TestGeneratorWithGravity()
{
	oslSetKeyAutorepeat(OSL_KEYMASK_UP|OSL_KEYMASK_DOWN|OSL_KEYMASK_RIGHT|OSL_KEYMASK_LEFT|OSL_KEYMASK_CIRCLE|OSL_KEYMASK_SQUARE|OSL_KEYMASK_CROSS|OSL_KEYMASK_TRIANGLE|OSL_KEYMASK_L|OSL_KEYMASK_R,40,2);
	
	m_action=MENU_NOACTION;
	m_skip=false;
	
	//tests
	int transl_time=84; 
	int lifespan=transl_time; 
	int transl_dist=272;
		
	/*
	SerialGlobalEffect *transp_fx=new SerialGlobalEffect(1);
	m_fx_transp1=new ProgressiveAlphaEffect((int)((transl_time*27)/100), 0, 0, 255, 1);
	m_fx_transp2=new WaitGlobalEffect((int)((transl_time*18)/100));
	m_fx_transp3=new ProgressiveAlphaEffect((int)((transl_time*55)/100), 0, 255, 0, 1);
	transp_fx->addEffect(m_fx_transp1); 
	transp_fx->addEffect(m_fx_transp2);
	transp_fx->addEffect(m_fx_transp3);
	*/
	
	m_fx_gravity=new GravityEffect(0.0f, 0.0f, 0.0f, 0.11f);
	
	//m_fx_tint=new ParticlesProgressiveTintEffect(transl_time, 0, RGB(0,255,255), RGB(0,0,200), 1);
	SerialGlobalEffect *tint_fx=new SerialGlobalEffect(1);
	m_fx_alpha1=new ParticlesProgressiveTintEffect((int)((transl_time*27)/100), 0, RGBA(0,255,255,0), RGBA(0,186,240,255), 1);
	m_fx_alpha2=new ParticlesProgressiveTintEffect((int)((transl_time*18)/100), 0, RGBA(0,186,240,255), RGBA(0,140,230,255), 1);
	m_fx_alpha3=new ParticlesProgressiveTintEffect((int)((transl_time*55)/100), 0, RGBA(0,140,230,255), RGBA(0,0,200,0), 1);
	tint_fx->addEffect(m_fx_alpha1); 
	tint_fx->addEffect(m_fx_alpha2);
	tint_fx->addEffect(m_fx_alpha3);
	
	
	
	//Anim *anim=new Anim(12, "smoke", "res/tests/goute.png/", float(transl_time), OSL_PF_4444, 1);
	Anim *anim=new Anim("res/tests/goutes.png", OSL_PF_4444);
	anim->setReferencePointCenter();
	SuccessiveAnimationFactory *saf=new SuccessiveAnimationFactory(anim);
	anim=new Anim("res/tests/goutes2.png", OSL_PF_4444);
	anim->setReferencePointCenter();
	saf->addAnimation(anim);
	anim=new Anim("res/tests/goutes3.png", OSL_PF_4444);
	anim->setReferencePointCenter();
	saf->addAnimation(anim);
	anim=new Anim("res/tests/goutes4.png", OSL_PF_4444);
	anim->setReferencePointCenter();
	saf->addAnimation(anim);
	m_test_partGen=new ParticleGeneratorPoint(240, 136, 57, 21, 5, 3, transl_time, transl_dist, lifespan, saf);
	m_test_partGen->setLocalEffect1SimpleTranslation();
	m_test_partGen->setLocalEffect2(m_fx_gravity);
	m_test_partGen->setGlobalEffect1(tint_fx);
}

//destruction
TestGeneratorWithGravity::~TestGeneratorWithGravity()
{
	delete m_test_partGen;
}

//demarrage du menu
int TestGeneratorWithGravity::go()
{
	oslSetBilinearFilter(true);
	m_test_partGen->start();
	
	
	while(m_action==MENU_NOACTION)
	{
		oslBenchmarkTest(OSL_BENCH_START);
		
		//touches
		m_action=keys();
		if(m_action==MENU_QUIT)
			return m_action;
		
		//phase d'update		
		m_test_partGen->update();
		
		
		if (!m_skip)
		{
			//phase de dessin
			oslStartDrawing();
			drawScene();
			oslBenchmarkTest(OSL_BENCH_END);
			oslSysBenchmarkDisplay();
			oslEndDrawing();
		}
		
		//End of a frame
        oslEndFrame();
		//Synchronize to ensure the game runs at 60 fps
        m_skip = oslSyncFrame();
		//oslAudioVSync(); //ne sert plus, oslEndFrame le fait
	}
	//il a choisi une option
	enableDebug();
	debug("fin boucle main, return "+toString(m_action));
	enableDebug(false);
	
	oslSetBilinearFilter(false);
	
	return m_action;	
}

//dessins
void TestGeneratorWithGravity::drawScene()
{
	oslClearScreen(RGB(0,0,0));
	
	m_test_partGen->draw();
	
	oslDrawString(10, 200, ("gravity: " + toString(m_fx_gravity->getAccelerationY()) + "pixels").c_str());
	oslDrawString(10, 210, ("orientation: " + toString(m_test_partGen->getOrientation()) + "degr.").c_str());
	oslDrawString(10, 220, ("scale: " + toString(m_test_partGen->getScale()) + "degr.").c_str());
	oslDrawString(10, 230, ("life: " + toString(m_test_partGen->getPLifespan()) + "frames").c_str());
	oslDrawString(10, 240, ("time: " + toString(m_test_partGen->getPTranslTime()) + "frames").c_str());
	oslDrawString(10, 250, ("distance: " + toString(m_test_partGen->getPDistance()) + "pixels").c_str());
	oslDrawString(10, 260, (toString(m_test_partGen->getCurrentNbParticles()) + " particles").c_str());
	
}

//gestion des touches
int TestGeneratorWithGravity::keys()
{
	oslReadKeys();
	if (osl_pad.pressed.start) //Exit 
	{
		enableDebug();
		debug("start!");
		enableDebug(false);
		return MENU_MAIN;
	}
	
	//gravité
	if(osl_pad.pressed.triangle)
	{
		m_fx_gravity->setAccelerationY(m_fx_gravity->getAccelerationY()+0.01f);
	}
	else if(osl_pad.pressed.cross)
	{
		m_fx_gravity->setAccelerationY(m_fx_gravity->getAccelerationY()-0.01f);
	}
	
	
	
	
	else if(osl_pad.pressed.square)
	{
		int scale=m_test_partGen->getScale();
		scale--;
		if(scale<=0)
			scale=1;		
		m_test_partGen->setScale(scale);
	}
	else if(osl_pad.pressed.circle)
	{
		int scale=m_test_partGen->getScale();
		scale++;
		if(scale>359)
			scale=359;		
		m_test_partGen->setScale(scale);
	}
	
	if(osl_pad.pressed.R)
	{
		int orientation=m_test_partGen->getOrientation();
		orientation+=3;
		if(orientation>=360)
			orientation=0;
		m_test_partGen->setOrientation(orientation);
	}
	else if(osl_pad.pressed.L)
	{
		int orientation=m_test_partGen->getOrientation();
		orientation-=3;
		if(orientation<0)
			orientation=359;
		m_test_partGen->setOrientation(orientation);
	}
	
	if(osl_pad.analogX>=100)
		m_test_partGen->setX(m_test_partGen->getX()+8);
	else if(osl_pad.analogX>=70)
		m_test_partGen->setX(m_test_partGen->getX()+4);
	else if(osl_pad.analogX>=20)
		m_test_partGen->setX(m_test_partGen->getX()+1);
	
	if(osl_pad.analogX<=-100)
		m_test_partGen->setX(m_test_partGen->getX()-8);
	else if(osl_pad.analogX<=-70)
		m_test_partGen->setX(m_test_partGen->getX()-4);
	else if(osl_pad.analogX<=-20)
		m_test_partGen->setX(m_test_partGen->getX()-1);
	
	if(osl_pad.analogY>=100)
		m_test_partGen->setY(m_test_partGen->getY()+8);
	else if(osl_pad.analogY>=70)
		m_test_partGen->setY(m_test_partGen->getY()+4);
	else if(osl_pad.analogY>=20)
		m_test_partGen->setY(m_test_partGen->getY()+1);
	
	if(osl_pad.analogY<=-100)
		m_test_partGen->setY(m_test_partGen->getY()-8);
	if(osl_pad.analogY<=-70)
		m_test_partGen->setY(m_test_partGen->getY()-4);
	if(osl_pad.analogY<=-20)
		m_test_partGen->setY(m_test_partGen->getY()-1);
		
	//vitesse des particules
	if (osl_pad.pressed.up)
	{
		int t=m_test_partGen->getPTranslTime();
		t+=2;
		adjustEffects(t);
	}
	else if(osl_pad.pressed.down)
	{
		int t=m_test_partGen->getPTranslTime();
		if(t>=5)
		{
			t-=2;
			adjustEffects(t);
		}
	}
	//distance de jet des particules
	else if(osl_pad.pressed.left)
	{
		int d=m_test_partGen->getPDistance();
		if(d>=10)
		{
			d-=2;
			m_test_partGen->setPDistance(d);
		}
	}
	else if(osl_pad.pressed.right)
	{
		int d=m_test_partGen->getPDistance();
		d+=2;
		m_test_partGen->setPDistance(d);
	}
	
	if(osl_pad.pressed.select)
	{
		m_test_partGen->generationSwitchOnOff();
	}
	
	return MENU_NOACTION;
}

void TestGeneratorWithGravity::adjustEffects(int newTime)
{
	m_test_partGen->setPTranslTime(newTime);
	m_test_partGen->setPAnimTime(float(newTime));
	m_test_partGen->setPLifespan(newTime);
	m_fx_alpha1->setTimeForward((newTime*27)/100);
	m_fx_alpha2->setTimeForward((newTime*18)/100);
	m_fx_alpha3->setTimeForward((newTime*55)/100);
}





