#include "code/tests/TestLevelEditor.h"
#include <oslib/oslib.h>
#include <string>
#include "code/framework/utils.h"
#include "code/game/menus/MenusDefines.h"
#include "code/game/levels/LinePattern.h"
#include "code/game/gamedef.h"
#include "code/framework/Pos.h"
#include "code/game/pieces/PiecesBufferFactory.h"
#include "code/framework/utils.h"
#include "code/game/pieces/impl/PieceShielded.h"
#include "code/game/pieces/impl/PieceSimple.h"
#include "code/game/pieces/impl/PieceCombo.h"
#include "code/game/pieces/impl/PieceLevel.h"
#include <fstream>
#include "code/framework/check/loaders.h"

using namespace std;
using namespace utils;

#define LEVELEDITOR_MODIFIER_NONE 0
#define LEVELEDITOR_MODIFIER_PIECE 1
#define LEVELEDITOR_MODIFIER_LINE 2
#define LEVELEDITOR_MODIFIER_CELL 3
#define LEVELEDITOR_MODIFIER_GRID 4

#define LEVELEDITOR_COPYMARKER_NONE 0
#define LEVELEDITOR_COPYMARKER_CELL 1
#define LEVELEDITOR_COPYMARKER_LINE 2

#define LEVELEDITOR_FILE "res/tests/testcenter/test.pattern"

//construction
TestLevelEditor::TestLevelEditor()
{
	//enableDebug();
	
	oslSetKeyAutorepeat(OSL_KEYMASK_UP|OSL_KEYMASK_DOWN|OSL_KEYMASK_RIGHT|OSL_KEYMASK_LEFT,40,4);
	oslSetTextColor(RGB(255,255,255));   
	//PiecesBufferFactory::bufferResources();
	
	m_action=MENU_NOACTION;
	m_skip=false;
	
	m_nbCols=12;
	
	//debug("chargement line pattern");
	m_grid=new vector<vector<AbstractPiece *> *>();
	//chargement du line pattern
	LinePattern *lp=new LinePattern(LEVELEDITOR_FILE, 1);
	
	//debug("line pattern chargement ok");
	
	list<AbstractPiece *> *line=lp->getNextLine();
	unsigned int max=0; //le max de pieces sur une ligne, trouvé dans le fichier
	while(line!=NULL)
	{
		vector<AbstractPiece *> *v=new vector<AbstractPiece *>();
		
		list<AbstractPiece *>::iterator it;
    	for (it=line->begin(); it!=line->end(); it++)
    	{
    		AbstractPiece *p=*it;
    		if(p!=NULL)
    		{
    			v->push_back(p->clone());
    		}
    		else
    		{
    			v->push_back(NULL);
    		}
    	}
    	
    	m_grid->push_back(v);
    	
    	if(v->size()>max)
    	{
    		max=v->size();
    	}
    	
    	line=lp->getNextLine();
	}
	
	m_linesDrawn=11; //=12;
	if(max>0)
		m_nbCols=max;
	m_lineTopNum=0; //=0;
	m_lineDownNum=10; //=11;
	
	//Pos p=calculateGridPosition();
	//int x=p.x;
	//int y=p.y;
	m_gridPixPos=calculateGridPosition();
	
	m_selectorPixPos=Pos(m_gridPixPos.x,m_gridPixPos.y);
//	m_selectorPixPos.y=m_gridPixPos.y;
	m_selectorGridPos=Pos(0,0);
//	m_selectorGridPos.y=0;
	
	/*m_modifier_pieceSelect=false;
	m_modifier_lineSelect=false;
	m_modifier_cellSelect=false;
	m_modifier_gridSelect=false;
	*/
	
	m_modifier=LEVELEDITOR_MODIFIER_NONE;
	
	//toutes les lignes doivent avoir la meme longueur, ie m_nbCols donc compléter les autres
	vector<vector<AbstractPiece *> *>::iterator it;
	for (it=m_grid->begin(); it<m_grid->end();it++)
    {
    	vector<AbstractPiece *> *pline=*it;
    	
    	if(int(pline->size())<m_nbCols)
    	{
    		for(int i=0;i<m_nbCols-int(pline->size());i++)
    		{
    			pline->push_back(NULL);
    		}
    	}
    }
	
	if(m_grid->size()==0)
	{
		addEmptyLine();
	}
	
	delete lp;
	
	m_msg=" ";
	
	m_img_random=fwkLoadImageFilePNG("res/pieces/random/random.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_4444);
	m_img_randomShielded=fwkLoadImageFilePNG("res/pieces/random/random_shielded.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_4444);
	m_img_piecelevel=fwkLoadImageFilePNG("res/pieces/level/level.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_4444);
	
	oslSetImageRotCenter(m_img_random);
	oslSetImageRotCenter(m_img_randomShielded);
	oslSetImageRotCenter(m_img_piecelevel);
	
	m_pieceUnderCursor=findPieceUnderCursor();
	
	m_copyMarkerPixPos=Pos(0,0);
	m_copyMarkerGridPos=Pos(0,0);
	m_copyMarkerType=LEVELEDITOR_COPYMARKER_NONE;
}

void TestLevelEditor::addEmptyLine()
{
	vector<AbstractPiece *> *v=new vector<AbstractPiece *>();
	for(int i=0;i<m_nbCols;i++)
	{
		v->push_back(NULL);
	}
	m_grid->push_back(v);
}

Pos TestLevelEditor::calculateGridPosition()
{
	int middleScreenX=480/2;
	int middleScreenY=272/2;
	int gridWidth=GAMEDEF_PIECE_WIDTH*m_nbCols;
	int gridHeight=GAMEDEF_PIECE_HEIGHT*m_linesDrawn;
	int middleGridX=int(gridWidth/2);
	int middleGridY=int(gridHeight/2);
	
	int startX=middleScreenX-middleGridX;
	int startY=middleScreenY-middleGridY;
	
	return Pos(startX, startY);
}

//destruction
TestLevelEditor::~TestLevelEditor()
{
	vector<vector<AbstractPiece *> *>::iterator it;
	for (it=m_grid->begin(); it<m_grid->end();it++)
    {
    	vector<AbstractPiece *> *pline=*it;
    	
    	vector<AbstractPiece *>::iterator it2;
    	for (it2=pline->begin(); it2!=pline->end(); it2++)
    	{
    		AbstractPiece *p=*it2;
    		if(p!=NULL)
    			delete p;
    	}
    	delete pline;

    }
    delete m_grid;
    
    //PiecesBufferFactory::unbufferResources();
    
    oslDeleteImage(m_img_random);
    oslDeleteImage(m_img_randomShielded);
    oslDeleteImage(m_img_piecelevel);
}

Pos TestLevelEditor::calculateSelectorPosition()
{
	int x, y;
	x=m_gridPixPos.x + m_selectorGridPos.x*GAMEDEF_PIECE_WIDTH;
	y=m_gridPixPos.y + m_selectorGridPos.y*GAMEDEF_PIECE_HEIGHT;
	return Pos(x, y);
}

//demarrage du menu
int TestLevelEditor::go()
{
	while(m_action==MENU_NOACTION)
	{
		//oslBenchmarkTest(OSL_BENCH_START);
		
		//touches
		m_action=keys();
		if(m_action==MENU_QUIT)
			return m_action;
		
		//phase d'update
		
				
		if (!m_skip)
		{
			//phase de dessin
			oslStartDrawing();
			drawScene();
			//oslBenchmarkTest(OSL_BENCH_END);
			//oslSysBenchmarkDisplay();
			oslEndDrawing();
		}
		
		//End of a frame
        oslEndFrame();
		//Synchronize to ensure the game runs at 60 fps
        m_skip = oslSyncFrame();
		//oslAudioVSync(); //ne sert plus, oslEndFrame le fait
	}
	//il a choisi une option
	enableDebug();
	debug("fin boucle TestLevelEditor, return "+toString(m_action));
	enableDebug(false);
	
	return m_action;	
}

AbstractPiece *TestLevelEditor::findPieceUnderCursor()
{
	//enableDebug();
	//debug(toString(m_selectorGridPos.y+m_lineTopNum));
	AbstractPiece *p=m_grid->at(m_selectorGridPos.y+m_lineTopNum)->at(m_selectorGridPos.x);
	//debug("piece trouvee");
	//enableDebug(false);
	//m_msg=toString(m_selectorGridPos.y+m_lineTopNum)+" piece trouvee";
	m_msg=generateMessage(p);
	return p;
}

AbstractPiece *TestLevelEditor::findPieceMarked()
{
	return m_grid->at(m_copyMarkerGridPos.y+m_lineTopNum)->at(m_copyMarkerGridPos.x);
}

//dessins
void TestLevelEditor::drawScene()
{
	oslClearScreen(RGB(0,0,0));
	
	//infos utiles
	oslPrintf_xy(5, 10, "Col=%i", m_nbCols);
	oslPrintf_xy(5, 20, "Lin=%i", m_grid->size());
	
	oslPrintf_xy(5, 30, "selector:");
	oslPrintf_xy(5, 40, "x=%i", m_selectorGridPos.x);
	oslPrintf_xy(5, 50, "y=%i", m_selectorGridPos.y);
	
	//dessin des numeros de colonne:
	for(int i=0;i<m_nbCols;i++)
	{
		oslPrintf_xy(m_gridPixPos.x+i*GAMEDEF_PIECE_WIDTH+5, m_gridPixPos.y-10, "%i", i);
	}
	
	//dessin des numeros de ligne
	for(int i=0;i<m_linesDrawn;i++)
	{
		oslPrintf_xy(m_gridPixPos.x-15, m_gridPixPos.y+i*GAMEDEF_PIECE_HEIGHT+5, "%i", i+m_lineTopNum);
	}
	
	//dessin de la grille
	int x=m_gridPixPos.x, y=m_gridPixPos.y;
	int xend=m_gridPixPos.x+GAMEDEF_PIECE_WIDTH*m_nbCols+1;
	int yend=m_gridPixPos.y+GAMEDEF_PIECE_HEIGHT*m_linesDrawn+1;
	
	//vertical
	for(int i=0;i<m_nbCols+1;i++)
	{
		oslDrawLine(x,y,x,yend,RGB(255,255,255));
		x+=GAMEDEF_PIECE_WIDTH;
	}
	//horizontal
	x=m_gridPixPos.x;
	for(int i=0;i<m_linesDrawn+1;i++)
	{
		oslDrawLine(x,y,xend,y,RGB(255,255,255));
		y+=GAMEDEF_PIECE_HEIGHT;
	}
	
	//dessin du marqueur de copie
	if(m_copyMarkerGridPos.y>=0 && m_copyMarkerGridPos.y<m_linesDrawn)
	{
		if(m_copyMarkerType==LEVELEDITOR_COPYMARKER_CELL)
		{
			oslDrawFillRect(m_copyMarkerPixPos.x, m_copyMarkerPixPos.y,m_copyMarkerPixPos.x+GAMEDEF_PIECE_WIDTH+1, m_copyMarkerPixPos.y+GAMEDEF_PIECE_HEIGHT+1, RGB(255,255,0));
			oslDrawRect(m_copyMarkerPixPos.x, m_copyMarkerPixPos.y,m_copyMarkerPixPos.x+GAMEDEF_PIECE_WIDTH+1, m_copyMarkerPixPos.y+GAMEDEF_PIECE_HEIGHT+1, RGB(0,255,0));
		}
		else if(m_copyMarkerType==LEVELEDITOR_COPYMARKER_LINE)
		{
			oslDrawFillRect(m_gridPixPos.x, m_copyMarkerPixPos.y,m_gridPixPos.x+m_nbCols*GAMEDEF_PIECE_WIDTH+1, m_copyMarkerPixPos.y+GAMEDEF_PIECE_HEIGHT+1, RGB(255,255,0));
			oslDrawRect(m_gridPixPos.x, m_copyMarkerPixPos.y,m_gridPixPos.x+m_nbCols*GAMEDEF_PIECE_WIDTH+1, m_copyMarkerPixPos.y+GAMEDEF_PIECE_HEIGHT+1, RGB(0,255,0));
		}
	}
	
	//dessin du selecteur
	if(m_modifier==LEVELEDITOR_MODIFIER_CELL || m_modifier==LEVELEDITOR_MODIFIER_NONE || m_modifier==LEVELEDITOR_MODIFIER_PIECE)
	{
		oslDrawFillRect(m_selectorPixPos.x, m_selectorPixPos.y,m_selectorPixPos.x+GAMEDEF_PIECE_WIDTH+1, m_selectorPixPos.y+GAMEDEF_PIECE_HEIGHT+1, RGB(255,0,255));
		oslDrawRect(m_selectorPixPos.x, m_selectorPixPos.y,m_selectorPixPos.x+GAMEDEF_PIECE_WIDTH+1, m_selectorPixPos.y+GAMEDEF_PIECE_HEIGHT+1, RGB(255,128,0));
	}
	else if(m_modifier==LEVELEDITOR_MODIFIER_LINE)
	{
		oslDrawFillRect(m_gridPixPos.x, m_selectorPixPos.y,m_gridPixPos.x+m_nbCols*GAMEDEF_PIECE_WIDTH+1, m_selectorPixPos.y+GAMEDEF_PIECE_HEIGHT+1, RGB(255,0,255));
		oslDrawRect(m_gridPixPos.x, m_selectorPixPos.y,m_gridPixPos.x+m_nbCols*GAMEDEF_PIECE_WIDTH+1, m_selectorPixPos.y+GAMEDEF_PIECE_HEIGHT+1, RGB(255,128,0));
	}
	else if(m_modifier==LEVELEDITOR_MODIFIER_GRID)
	{
		oslDrawFillRect(m_gridPixPos.x, m_gridPixPos.y,m_gridPixPos.x+m_nbCols*GAMEDEF_PIECE_WIDTH+1, m_linesDrawn*m_gridPixPos.y+GAMEDEF_PIECE_HEIGHT+1, RGB(255,0,255));
		oslDrawRect(m_gridPixPos.x, m_gridPixPos.y,m_gridPixPos.x+m_nbCols*GAMEDEF_PIECE_WIDTH+1, m_linesDrawn*m_gridPixPos.y+GAMEDEF_PIECE_HEIGHT+1, RGB(255,128,0));
	}
		
	//dessin des pieces
	y=m_gridPixPos.y+int(GAMEDEF_PIECE_HEIGHT/2);
	for(int i=m_lineTopNum;i<=m_lineDownNum && i<int(m_grid->size());i++)
	{
		vector<AbstractPiece *> *pline=m_grid->at(i);
		
		x=m_gridPixPos.x+int(GAMEDEF_PIECE_WIDTH/2);
		
		vector<AbstractPiece *>::iterator it;
    	for (it=pline->begin(); it!=pline->end(); it++)
    	{
    		AbstractPiece *p=*it;
    		if(p!=NULL)
    		{
    			if(p->getType()==PIECE_TYPE_LEVEL)
    			{
    				oslDrawImageXY(m_img_piecelevel, x, y);
    			}
    			else if(p->isRandom())
    			{
    				if(p->getType()==PIECE_TYPE_SHIELDED)
    				{
    					oslDrawImageXY(m_img_randomShielded, x, y);
    				}
    				else if(p->getType()==PIECE_TYPE_NORMAL)
    				{
    					oslDrawImageXY(m_img_random, x, y);
    				}
    			}
    			else
    			{
    				p->drawXY(x, y);
    			}
    		}
    		x+=GAMEDEF_PIECE_WIDTH;
    	}
    	y+=GAMEDEF_PIECE_HEIGHT;
	}
	
	//dessin des commandes
	if(m_modifier==LEVELEDITOR_MODIFIER_NONE)
	{
		oslPrintf_xy(2,160, "SELECTION:");
		oslPrintf_xy(5,170, "[]: cell");
		oslPrintf_xy(5,180, "X: block");
		oslPrintf_xy(5,190, "O: line");
		oslPrintf_xy(5,200, "st: grid");
		
		oslPrintf_xy(5,215, "sel: marker");
	}
	else if(m_modifier==LEVELEDITOR_MODIFIER_CELL)
	{
		oslPrintf_xy(2,160, "CELL:");
		oslPrintf_xy(5,170, "<>: type");
		oslPrintf_xy(5,180, "R: copy");
		oslPrintf_xy(5,190, "L: paste");
		oslPrintf_xy(5,200, "/\\: delete");
	}
	else if(m_modifier==LEVELEDITOR_MODIFIER_PIECE)
	{
		oslPrintf_xy(2,160, "BLOCK:");
		
		if(m_pieceUnderCursor==NULL)
		{
			oslPrintf_xy(5,170, "-empty-");
		}
		else if(m_pieceUnderCursor->getType()==PIECE_TYPE_COMBO)
		{
			oslPrintf_xy(5,170, "-combo-");
			oslPrintf_xy(5,180, "^v: number");
		}
		else if(m_pieceUnderCursor->getType()==PIECE_TYPE_LEVEL)
		{
			oslPrintf_xy(5,170, "-special-");
			oslPrintf_xy(5,180, "^v: id");
		}
		else if(m_pieceUnderCursor->isRandom())
		{
			oslPrintf_xy(5,170, "-random-");
			oslPrintf_xy(5,180, "[]: shield");
		}
		else if(m_pieceUnderCursor->getType()==PIECE_TYPE_NORMAL || m_pieceUnderCursor->getType()==PIECE_TYPE_SHIELDED)
		{
			oslPrintf_xy(5,170, "-normal-");
			oslPrintf_xy(5,180, "[]: shield");
			oslPrintf_xy(5,190, "<>: color");
			oslPrintf_xy(5,200, "^v: shape");
		}
	}
	else if(m_modifier==LEVELEDITOR_MODIFIER_GRID)
	{
		oslPrintf_xy(2,160, "GRID:");
		oslPrintf_xy(5,170, "R: col+");
		oslPrintf_xy(5,180, "L: col-");
		oslPrintf_xy(5,190, "O: save");
		oslPrintf_xy(5,200, "sel: quit"); 
		oslPrintf_xy(5,210, "[]+/\\: reset");
	}
	else if(m_modifier==LEVELEDITOR_MODIFIER_LINE)
	{
		oslPrintf_xy(2,160, "LINE:");
		oslPrintf_xy(5,170, "R: copy");
		oslPrintf_xy(5,180, "L: paste");
		oslPrintf_xy(5,190, "dpad: shift");
		oslPrintf_xy(5,200, "/\\: clear");
		oslPrintf_xy(5,210, "[]: remove");
		oslPrintf_xy(5,220, "X: insert");
	}
	
	//dessin du message:
	oslPrintf_xy(10,262, "msg:");
	oslPrintf_xy(55,262, m_msg.c_str());
}

//gestion des touches
int TestLevelEditor::keys()
{
	oslReadKeys();
	
	/********************************/
	//            NORMAL            //
	/********************************/
	
	if(m_modifier==LEVELEDITOR_MODIFIER_NONE)
	{
		//X: selecteur de case
		if(osl_pad.pressed.square)
		{
			if(m_modifier==LEVELEDITOR_MODIFIER_NONE)
			{
				m_modifier=LEVELEDITOR_MODIFIER_CELL;
			}
		}
		//[]: selecteur de piece
		else if(osl_pad.pressed.cross)
		{
			if(m_modifier==LEVELEDITOR_MODIFIER_NONE)
			{
				m_modifier=LEVELEDITOR_MODIFIER_PIECE;
			}
		}
		
		//start: selecteur de grille
		else if(osl_pad.pressed.start)
		{
			if(m_modifier==LEVELEDITOR_MODIFIER_NONE)
			{
				m_modifier=LEVELEDITOR_MODIFIER_GRID;
			}
		}
	
		// /\: selecteur de ligne
		else if(osl_pad.pressed.circle)
		{
			if(m_modifier==LEVELEDITOR_MODIFIER_NONE)
			{
				m_modifier=LEVELEDITOR_MODIFIER_LINE;
			}
		}
		
		//déplacements
		if(osl_pad.pressed.right)
		{
			if(m_selectorGridPos.x<m_nbCols-1)
			{
				m_selectorGridPos.x++;
				m_selectorPixPos.x+=GAMEDEF_PIECE_WIDTH; //=calculateSelectorPosition();
				m_pieceUnderCursor=findPieceUnderCursor();
			}
		}
		else if(osl_pad.pressed.left)
		{
			if(m_selectorGridPos.x>0)
			{
				m_selectorGridPos.x--;
				m_selectorPixPos.x-=GAMEDEF_PIECE_WIDTH;
				m_pieceUnderCursor=findPieceUnderCursor();
			}
		}
		else if(osl_pad.pressed.up)
		{
			goUp();
		}
		else if(osl_pad.pressed.down)
		{
			goDown();
		}
		
		else if(osl_pad.pressed.select)
		{
			m_copyMarkerType=LEVELEDITOR_COPYMARKER_NONE;
		}
		
		//commun a tous les déplacements
		if(osl_pad.pressed.down || osl_pad.pressed.up || osl_pad.pressed.left || osl_pad.pressed.right)
		{
			m_msg=generateMessage(m_pieceUnderCursor);			
		}
		
		
	}
	
	/********************************/
	//           CELLULE            //
	/********************************/
	
	else if(m_modifier==LEVELEDITOR_MODIFIER_CELL)
	{
		if(osl_pad.released.square)
		{
			m_modifier=LEVELEDITOR_MODIFIER_NONE;
		}
		
		//changement de piece
		if(osl_pad.pressed.left || osl_pad.pressed.right)
		{
			AbstractPiece *piece;
			if(osl_pad.pressed.left)
				piece=previousType(m_pieceUnderCursor);
			else
				piece=nextType(m_pieceUnderCursor);
				
			if(m_pieceUnderCursor!=NULL)
			{
				delete m_pieceUnderCursor;
			}
			
			m_grid->at(m_selectorGridPos.y+m_lineTopNum)->at(m_selectorGridPos.x)=piece;
			m_pieceUnderCursor=piece;
			
			m_msg=generateMessage(m_pieceUnderCursor);
		}
		
		//copy
		else if(osl_pad.pressed.R)
		{
			m_copyMarkerPixPos=m_selectorPixPos;
			m_copyMarkerGridPos=m_selectorGridPos;
			m_copyMarkerType=LEVELEDITOR_COPYMARKER_CELL;
			m_msg="cell copied ok.";
		}
		
		//paste
		else if(osl_pad.pressed.L)
		{
			if(m_copyMarkerType==LEVELEDITOR_COPYMARKER_NONE)
			{
				m_msg="nothing to paste";
			}
			else if(m_copyMarkerType==LEVELEDITOR_COPYMARKER_LINE)
			{
				m_msg="cannot paste a line into a cell";
			}
			else
			{
				AbstractPiece *clone;
				AbstractPiece *p_marked=findPieceMarked();
				
				if(p_marked==NULL)
					/*m_grid->at(m_selectorGridPos.y+m_lineTopNum)->at(m_selectorGridPos.x)*/clone=NULL;
				else
					/*m_grid->at(m_selectorGridPos.y+m_lineTopNum)->at(m_selectorGridPos.x)*/clone=p_marked->clone();
				
				AbstractPiece *p_dest=m_grid->at(m_selectorGridPos.y+m_lineTopNum)->at(m_selectorGridPos.x);
				if(p_dest!=NULL)
					delete p_dest;
					
				m_grid->at(m_selectorGridPos.y+m_lineTopNum)->at(m_selectorGridPos.x)=clone;
				
				m_pieceUnderCursor=findPieceUnderCursor();
				
				m_msg="cell pasted ok";
			}
		}
		
		//delete
		else if(osl_pad.pressed.triangle)
		{
			if(m_pieceUnderCursor!=NULL)
			{
				delete m_pieceUnderCursor;
			}
			
			m_grid->at(m_selectorGridPos.y+m_lineTopNum)->at(m_selectorGridPos.x)=NULL;
			m_pieceUnderCursor=NULL;
			m_msg=generateMessage(m_pieceUnderCursor);
		}
	}
	
	
	/********************************/
	//            PIECE             //
	/********************************/
	
	else if(m_modifier==LEVELEDITOR_MODIFIER_PIECE)
	{
		if(osl_pad.released.cross)
		{
			m_modifier=LEVELEDITOR_MODIFIER_NONE;
		}
		
		if(m_pieceUnderCursor==NULL)
		{
			return MENU_NOACTION;
		}
		
		//pieces speciales
		else if(m_pieceUnderCursor->getType()==PIECE_TYPE_LEVEL)
		{
			if(osl_pad.pressed.up)
			{
				m_pieceUnderCursor->specialEvent(PIECE_EVENT_LEVEL_ID_ADD);
				m_msg="id changed to "+toString(m_pieceUnderCursor->specialEvent(PIECE_EVENT_LEVEL_GETID));
			}
			else if(osl_pad.pressed.down)
			{
				m_pieceUnderCursor->specialEvent(PIECE_EVENT_LEVEL_ID_REDUCE);
				m_msg="id changed to "+toString(m_pieceUnderCursor->specialEvent(PIECE_EVENT_LEVEL_GETID));
			}
		}
		
		//pieces random
		else if(m_pieceUnderCursor->isRandom())
		{
			if(osl_pad.pressed.square)
			{
				if(m_pieceUnderCursor->getType()==PIECE_TYPE_NORMAL)
				{
					m_grid->at(m_selectorGridPos.y+m_lineTopNum)->at(m_selectorGridPos.x)=new PieceShielded();
					delete m_pieceUnderCursor;
					m_pieceUnderCursor=findPieceUnderCursor();
					m_msg="shield added";
				}
				else //shielded
				{
					m_grid->at(m_selectorGridPos.y+m_lineTopNum)->at(m_selectorGridPos.x)=new PieceSimple();
					delete m_pieceUnderCursor;
					m_pieceUnderCursor=findPieceUnderCursor();
					m_msg="shield removed";
				}
			}
		}
		
		//pieces normales (shielded or not)
		else if(m_pieceUnderCursor->getType()==PIECE_TYPE_NORMAL || m_pieceUnderCursor->getType()==PIECE_TYPE_SHIELDED)
		{
			//couleur
			if(osl_pad.pressed.left)
			{
				m_pieceUnderCursor->setColor(previousColor(m_pieceUnderCursor->getColor()));
			}
			else if(osl_pad.pressed.right)
			{
				m_pieceUnderCursor->setColor(nextColor(m_pieceUnderCursor->getColor()));
			}
			
			
			//forme
			if(osl_pad.pressed.up)
			{
				m_pieceUnderCursor->setShape(previousColor(m_pieceUnderCursor->getShape()));
			}
			else if(osl_pad.pressed.down)
			{
				m_pieceUnderCursor->setShape(nextColor(m_pieceUnderCursor->getShape()));
			}
			
			if(osl_pad.pressed.square)
			{
				if(m_pieceUnderCursor->getType()==PIECE_TYPE_NORMAL)
				{
					m_grid->at(m_selectorGridPos.y+m_lineTopNum)->at(m_selectorGridPos.x)=new PieceShielded(m_pieceUnderCursor->getShape(), m_pieceUnderCursor->getColor());
					delete m_pieceUnderCursor;
					m_pieceUnderCursor=findPieceUnderCursor();
				}
				else //shielded
				{
					m_grid->at(m_selectorGridPos.y+m_lineTopNum)->at(m_selectorGridPos.x)=new PieceSimple(m_pieceUnderCursor->getShape(), m_pieceUnderCursor->getColor());
					delete m_pieceUnderCursor;
					m_pieceUnderCursor=findPieceUnderCursor();
				}
			}
		}
		
		//pieces combo
		else if(m_pieceUnderCursor->getType()==PIECE_TYPE_COMBO)
		{
			if(osl_pad.pressed.up)
			{
				m_pieceUnderCursor->specialEvent(PIECE_EVENT_COMBO_ADD);
			}
			else if(osl_pad.pressed.down)
			{
				if(m_pieceUnderCursor->specialEvent(PIECE_EVENT_COMBO_GET)>1)
				{
					m_pieceUnderCursor->specialEvent(PIECE_EVENT_COMBO_REDUCE);
				}
			}
			
			int nb=m_pieceUnderCursor->specialEvent(PIECE_EVENT_COMBO_GET);
			m_msg="combos changed to "+toString(nb);
		}
	}
	
	
	/********************************/
	//            GRILLE            //
	/********************************/
	
	else if(m_modifier==LEVELEDITOR_MODIFIER_GRID)
	{
		if(osl_pad.released.start)
		{
			m_modifier=LEVELEDITOR_MODIFIER_NONE;
		}
		
		//ajoute/supprime une colonne
		if(osl_pad.pressed.R)
		{
			if(m_nbCols<16)
			{
				m_nbCols++;
			
				vector<vector<AbstractPiece *> *>::iterator it;
				for (it=m_grid->begin(); it<m_grid->end();it++)
			    {
			    	vector<AbstractPiece *> *pline=*it;
			    	pline->push_back(NULL);
			    }
			}
			
			m_gridPixPos=calculateGridPosition();
			
			m_selectorPixPos=Pos(m_gridPixPos.x+m_selectorGridPos.x*GAMEDEF_PIECE_WIDTH,m_gridPixPos.y+m_selectorGridPos.y*GAMEDEF_PIECE_HEIGHT);
						
		}
		if(osl_pad.pressed.L)
		{
			if(m_nbCols>6)
			{
				m_nbCols--;
			
				vector<vector<AbstractPiece *> *>::iterator it;
				for (it=m_grid->begin(); it<m_grid->end();it++)
			    {
			    	vector<AbstractPiece *> *pline=*it;
			    	//AbstractPiece *end=*(pline->end());
			    	AbstractPiece *end=pline->at(pline->size()-1);
			    	if(end!=NULL)
			    		delete end;
			    	pline->pop_back();
			    }
			}
			
			m_gridPixPos=calculateGridPosition();
			if(m_selectorGridPos.x>=m_nbCols)
			{
				m_selectorGridPos.x--;
			}
			
			m_selectorPixPos=Pos(m_gridPixPos.x+m_selectorGridPos.x*GAMEDEF_PIECE_WIDTH,m_gridPixPos.y+m_selectorGridPos.y*GAMEDEF_PIECE_HEIGHT);
		}
		
		if(osl_pad.pressed.select)
		{
			return MENU_MAIN;
		}
		
		if(osl_pad.pressed.circle)
		{
			save();
		}
		
		if(osl_pad.pressed.square && osl_pad.held.triangle)
			resetGrid();
		else if(osl_pad.pressed.triangle && osl_pad.held.square)
			resetGrid();
			
		
		
	}
	
	
	/********************************/
	//            LIGNE             //
	/********************************/
	
	else if(m_modifier==LEVELEDITOR_MODIFIER_LINE)
	{
		if(osl_pad.released.circle)
		{
			m_modifier=LEVELEDITOR_MODIFIER_NONE;
		}
		
		//copy
		else if(osl_pad.pressed.R)
		{
			m_copyMarkerPixPos=m_selectorPixPos;
			m_copyMarkerGridPos=m_selectorGridPos;
			m_copyMarkerType=LEVELEDITOR_COPYMARKER_LINE;
			m_msg="line copied ok.";
		}
		
		//paste
		else if(osl_pad.pressed.L)
		{
			if(m_copyMarkerType==LEVELEDITOR_COPYMARKER_NONE)
			{
				m_msg="nothing to paste";
			}
			else if(m_copyMarkerType==LEVELEDITOR_COPYMARKER_CELL)
			{
				AbstractPiece *p_marked=findPieceMarked();
				
				for(unsigned int i=0;i<(unsigned int)(m_nbCols);i++)
				{
					AbstractPiece *clone;
					if(p_marked==NULL)
						/*m_grid->at(m_selectorGridPos.y+m_lineTopNum)->at(i)*/clone=NULL;
					else
						/*m_grid->at(m_selectorGridPos.y+m_lineTopNum)->at(i)*/clone=p_marked->clone();
					
					AbstractPiece *p_dest=m_grid->at(m_selectorGridPos.y+m_lineTopNum)->at(i);
					if(p_dest!=NULL)
						delete p_dest;
					
					m_grid->at(m_selectorGridPos.y+m_lineTopNum)->at(i)=clone;
				}
				
				m_pieceUnderCursor=findPieceUnderCursor();
				
				m_msg="cell pasted into whole line ok";
			}
			else
			{
				for(unsigned int i=0;i<(unsigned int)(m_nbCols);i++)
				{
					AbstractPiece *p_orig=m_grid->at(m_copyMarkerGridPos.y+m_lineTopNum)->at(i);
					AbstractPiece *p_dest=m_grid->at(m_selectorGridPos.y+m_lineTopNum)->at(i);
					
					AbstractPiece *clone;
					if(p_orig==NULL)
						/*m_grid->at(m_selectorGridPos.y+m_lineTopNum)->at(i)*/clone=NULL;
					else
						/*m_grid->at(m_selectorGridPos.y+m_lineTopNum)->at(i)*/clone=p_orig->clone();
					
					if(p_dest!=NULL)
						delete p_dest;
						
					m_grid->at(m_selectorGridPos.y+m_lineTopNum)->at(i)=clone;
					
					
				}
				
				m_pieceUnderCursor=findPieceUnderCursor();
				
				m_msg = "line pasted ok";
			}
		}
		
		//delete all
		else if(osl_pad.pressed.triangle)
		{
			vector<AbstractPiece *> *pline=m_grid->at(m_selectorGridPos.y+m_lineTopNum);
			
			for(unsigned int i=0;i<pline->size();i++)
			{
				if(pline->at(i)!=NULL)
					delete pline->at(i);
				pline->at(i)=NULL;
			}
			
			m_pieceUnderCursor=findPieceUnderCursor();
			
			m_msg = "line at position "+toString(m_selectorGridPos.y+m_lineTopNum)+" cleared";
		}
		
		//remove
		else if(osl_pad.pressed.square)
		{
			vector<AbstractPiece *> *pline=m_grid->at(m_selectorGridPos.y+m_lineTopNum);
			
			for(unsigned int i=0;i<pline->size();i++)
			{
				if(pline->at(i)!=NULL)
					delete pline->at(i);
			}
			m_grid->erase(m_grid->begin()+m_selectorGridPos.y+m_lineTopNum);
			
			m_copyMarkerType=LEVELEDITOR_COPYMARKER_NONE;
			m_pieceUnderCursor=findPieceUnderCursor();
			
			m_msg = "line at position "+toString(m_selectorGridPos.y+m_lineTopNum)+" removed";
		}
		
		//insert
		else if(osl_pad.pressed.cross)
		{
			
			vector<AbstractPiece *> *v=new vector<AbstractPiece *>();
			for(int i=0;i<m_nbCols;i++)
			{
				v->push_back(NULL);
			}
			
			m_grid->insert (m_grid->begin()+m_selectorGridPos.y+m_lineTopNum, v);
			
			m_copyMarkerType=LEVELEDITOR_COPYMARKER_NONE;
			m_pieceUnderCursor=findPieceUnderCursor();
			
			m_msg = "line inserted at position "+toString(m_selectorGridPos.y+m_lineTopNum);
		}
		
		//switch avec celle du dessus
		else if(osl_pad.pressed.up)
		{
			if(m_selectorGridPos.y+m_lineTopNum > 0)
			{
				vector<AbstractPiece *> *line1=m_grid->at(m_selectorGridPos.y+m_lineTopNum);
				vector<AbstractPiece *> *line2=m_grid->at(m_selectorGridPos.y+m_lineTopNum-1);
				
				m_grid->at(m_selectorGridPos.y+m_lineTopNum)=line2;
				m_grid->at(m_selectorGridPos.y+m_lineTopNum-1)=line1;
				
				goUp();
				
				m_msg = "switched lines "+toString(m_selectorGridPos.y+m_lineTopNum-1)+" and "+toString(m_selectorGridPos.y+m_lineTopNum);
			}
		}
		
		//switch avec celle du dessous
		else if(osl_pad.pressed.down)
		{
			if(m_selectorGridPos.y<m_linesDrawn-1 || m_lineDownNum < int(m_grid->size())-1)
			{
				vector<AbstractPiece *> *line1=m_grid->at(m_selectorGridPos.y+m_lineTopNum);
				vector<AbstractPiece *> *line2=m_grid->at(m_selectorGridPos.y+m_lineTopNum+1);
				
				m_grid->at(m_selectorGridPos.y+m_lineTopNum)=line2;
				m_grid->at(m_selectorGridPos.y+m_lineTopNum+1)=line1;
				
				goDown();
				
				m_msg = "switched lines "+toString(m_selectorGridPos.y+m_lineTopNum)+" and "+toString(m_selectorGridPos.y+m_lineTopNum+1);
			}
		}
		
	}
	
	
	return MENU_NOACTION;
}

int TestLevelEditor::nextColor(int color)
{
	if(color==PIECE_COLOR_BLUE)
	{
		return PIECE_COLOR_GREEN;
	}
	else if(color==PIECE_COLOR_GREEN)
	{
		return PIECE_COLOR_ORANGE;
	}
	else if(color==PIECE_COLOR_ORANGE)
	{
		return PIECE_COLOR_PURPLE;
	}
	else if(color==PIECE_COLOR_PURPLE)
	{
		return PIECE_COLOR_RED;
	}
	else if(color==PIECE_COLOR_RED)
	{
		return PIECE_COLOR_YELLOW;
	}
	else if(color==PIECE_COLOR_YELLOW)
	{
		return PIECE_COLOR_BLUE;
	}
	
	return PIECE_COLOR_NONE;
}

int TestLevelEditor::nextShape(int shape)
{
	if(shape==PIECE_SHAPE_CIRCLE)
	{
		return PIECE_SHAPE_CROSS;
	}
	else if(shape==PIECE_SHAPE_CROSS)
	{
		return PIECE_SHAPE_DIAMOND;
	}
	else if(shape==PIECE_SHAPE_DIAMOND)
	{
		return PIECE_SHAPE_SQUARE;
	}
	else if(shape==PIECE_SHAPE_SQUARE)
	{
		return PIECE_SHAPE_STAR;
	}
	else if(shape==PIECE_SHAPE_STAR)
	{
		return PIECE_SHAPE_TRIANGLE;
	}
	else if(shape==PIECE_SHAPE_TRIANGLE)
	{
		return PIECE_SHAPE_CIRCLE;
	}
	
	return PIECE_SHAPE_NONE;
}

int TestLevelEditor::previousColor(int color)
{
	if(color==PIECE_COLOR_BLUE)
	{
		return PIECE_COLOR_YELLOW;
	}
	else if(color==PIECE_COLOR_GREEN)
	{
		return PIECE_COLOR_BLUE;
	}
	else if(color==PIECE_COLOR_ORANGE)
	{
		return PIECE_COLOR_GREEN;
	}
	else if(color==PIECE_COLOR_PURPLE)
	{
		return PIECE_COLOR_ORANGE;
	}
	else if(color==PIECE_COLOR_RED)
	{
		return PIECE_COLOR_PURPLE;
	}
	else if(color==PIECE_COLOR_YELLOW)
	{
		return PIECE_COLOR_RED;
	}
	
	return PIECE_COLOR_NONE;
}

int TestLevelEditor::previousShape(int shape)
{
	if(shape==PIECE_SHAPE_CIRCLE)
	{
		return PIECE_SHAPE_TRIANGLE;
	}
	else if(shape==PIECE_SHAPE_CROSS)
	{
		return PIECE_SHAPE_CIRCLE;
	}
	else if(shape==PIECE_SHAPE_DIAMOND)
	{
		return PIECE_SHAPE_CROSS;
	}
	else if(shape==PIECE_SHAPE_SQUARE)
	{
		return PIECE_SHAPE_DIAMOND;
	}
	else if(shape==PIECE_SHAPE_STAR)
	{
		return PIECE_SHAPE_SQUARE;
	}
	else if(shape==PIECE_SHAPE_TRIANGLE)
	{
		return PIECE_SHAPE_STAR;
	}
	
	return PIECE_SHAPE_NONE;
}


//simple - random - combo - NULL
AbstractPiece *TestLevelEditor::nextType(AbstractPiece *p)
{
	if(p==NULL)
	{
		return new PieceSimple(PIECE_SHAPE_SQUARE, PIECE_COLOR_BLUE);
	}
	else if(p->getType()==PIECE_TYPE_NORMAL || p->getType()==PIECE_TYPE_SHIELDED)
	{
		if(p->isRandom())
		{
			return new PieceCombo(5);
		}
		else
		{
			return new PieceSimple(); //random
		}
	}
	else if(p->getType()==PIECE_TYPE_COMBO)
	{
		return new PieceLevel();
	}
	else if(p->getType()==PIECE_TYPE_LEVEL)
	{
		return NULL;
	}
	
	return NULL;
}

AbstractPiece *TestLevelEditor::previousType(AbstractPiece *p)
{
	if(p==NULL)
	{
		return new PieceLevel(); 
	}
	else if(p->getType()==PIECE_TYPE_NORMAL || p->getType()==PIECE_TYPE_SHIELDED)
	{
		if(p->isRandom())
		{
			return new PieceSimple(PIECE_SHAPE_SQUARE, PIECE_COLOR_BLUE);
		}
		else
		{
			return NULL; 
		}
	}
	else if(p->getType()==PIECE_TYPE_COMBO)
	{
		return new PieceSimple(); //random
	}
	else if(p->getType()==PIECE_TYPE_LEVEL)
	{
		return new PieceCombo(5);
	}
	
	return NULL;
}

string TestLevelEditor::generateMessage(AbstractPiece *p)
{
	string res="";
	if(p==NULL)
		res="empty";
	else if(p->getType()==PIECE_TYPE_COMBO)
	{
		int nb=p->specialEvent(PIECE_EVENT_COMBO_GET);
		res="combo: "+toString(nb);
	}
	else if(p->getType()==PIECE_TYPE_NORMAL || p->getType()==PIECE_TYPE_SHIELDED)
	{
		string shield="";
		if(p->getType()==PIECE_TYPE_SHIELDED)
		{
			shield=" + shield";
		}
		
		if(p->isRandom())
		{
			res="random"+shield;
		}
		else
		{
			res="normal"+shield;
		}				
	}	
	else if(p->getType()==PIECE_TYPE_LEVEL)
	{
		res="special: "+toString(p->specialEvent(PIECE_EVENT_LEVEL_GETID));
	}
	
	return res;	
}

void TestLevelEditor::goUp() //recalcule le selecteur et les num de ligne haut et bas
{
	if(m_selectorGridPos.y>0)
	{
		m_selectorGridPos.y--;
		m_selectorPixPos.y-=GAMEDEF_PIECE_HEIGHT;
		m_pieceUnderCursor=findPieceUnderCursor();
	}
	else if(m_lineTopNum>0)
	{
		m_lineTopNum--;
		m_lineDownNum--;
		m_pieceUnderCursor=findPieceUnderCursor();
		m_copyMarkerGridPos.y++;
		m_copyMarkerPixPos.y+=GAMEDEF_PIECE_HEIGHT;
	}
}

void TestLevelEditor::goDown()
{
	if(m_selectorGridPos.y<m_linesDrawn-1)
	{
		if(m_selectorGridPos.y>=int(m_grid->size())-1)
		{
			addEmptyLine();
		}
		
		m_selectorGridPos.y++;
		m_selectorPixPos.y+=GAMEDEF_PIECE_HEIGHT;
		
		
		m_pieceUnderCursor=findPieceUnderCursor();
	}
	else if(m_lineDownNum<int(m_grid->size())-1)
	{
		m_lineTopNum++;
		m_lineDownNum++;
		m_pieceUnderCursor=findPieceUnderCursor();
		m_copyMarkerGridPos.y--;
		m_copyMarkerPixPos.y-=GAMEDEF_PIECE_HEIGHT;
	}
	else
	{
		m_lineTopNum++;
		m_lineDownNum++;
		addEmptyLine();
		m_pieceUnderCursor=findPieceUnderCursor();
		m_copyMarkerGridPos.y--;
		m_copyMarkerPixPos.y-=GAMEDEF_PIECE_HEIGHT;
	}
}

void TestLevelEditor::resetGrid()
{
	vector<vector<AbstractPiece *> *>::iterator it;
	for (it=m_grid->begin(); it<m_grid->end();it++)
    {
    	vector<AbstractPiece *> *pline=*it;
    	
    	vector<AbstractPiece *>::iterator it2;
    	for (it2=pline->begin(); it2!=pline->end(); it2++)
    	{
    		AbstractPiece *p=*it2;
    		if(p!=NULL)
    			delete p;
    	}
    	delete pline;
    }
    delete m_grid;
    
    m_grid=new vector<vector<AbstractPiece *> *>();
    
    
    addEmptyLine();
    
    m_lineTopNum=0; //=0;
	m_lineDownNum=10; //=11;
	
	m_selectorPixPos=Pos(m_gridPixPos.x,m_gridPixPos.y);
	m_selectorGridPos=Pos(0,0);
	
	m_pieceUnderCursor=findPieceUnderCursor();
	
	m_copyMarkerType=LEVELEDITOR_COPYMARKER_NONE;
	
	m_msg = "grid cleared ok";
}

void TestLevelEditor::save()
{
	ofstream patternfile(LEVELEDITOR_FILE, ofstream::trunc);
	if(patternfile.fail())
		oslFatalError("TestLevelEditor : impossible de charger le fichier");
		
	patternfile << "//generated with the level editor" << endl;
	patternfile << "//please save this file somewhere as it will be replaced each time you save." << endl;
	patternfile << endl;
	patternfile << endl;
	
	vector<vector<AbstractPiece *> *>::iterator it;
	for (it=m_grid->begin(); it<m_grid->end();it++)
    {
    	vector<AbstractPiece *> *pline=*it;
    	
    	vector<AbstractPiece *>::iterator it2;
    	
    	//recherche si toute la ligne est null?
    	bool allnull=true;
    	for(it2=pline->begin(); it2!=pline->end() && allnull; it2++)
    	{
    		AbstractPiece *p=*it2;
    		if(p!=NULL)
    			allnull=false;
    	}
    	
    	if(!allnull)
    	{
    		patternfile << ">   ";
    		
	    	for(it2=pline->begin(); it2!=pline->end(); it2++)
	    	{
	    		AbstractPiece *p=*it2;
	    		
	    		if(p==NULL)
	    		{
	    			patternfile << "  . ";
	    		}
	    		else if(p->getType()==PIECE_TYPE_COMBO)
	    		{
	    			int combos=p->specialEvent(PIECE_EVENT_COMBO_GET);
	    			patternfile << "(";
	    			if(combos<10)
	    				patternfile << " ";
	    			patternfile << combos;
	    			patternfile << ")";
	    		}
	    		else if(p->getType()==PIECE_TYPE_LEVEL)
	    		{
	    			patternfile << " #";
	    			int id=p->specialEvent(PIECE_EVENT_LEVEL_GETID);
	    			if(id==0)
	    				patternfile << "#";
	    			else
	    				patternfile << id;
	    			 patternfile << " ";	    			
	    		}
	    		else if(p->getType()==PIECE_TYPE_NORMAL || p->getType()==PIECE_TYPE_SHIELDED)
	    		{
	    			if(p->getType()==PIECE_TYPE_SHIELDED)
	    				patternfile << "[";
	    			else
	    				patternfile << " ";
	    				
	    			if(p->isRandom())
	    				patternfile << "!!";
	    			else
	    			{
	    				int shape=p->getShape();
	    				if(shape==PIECE_SHAPE_CIRCLE)
	    					patternfile << "O";
	    				else if(shape==PIECE_SHAPE_CROSS)
	    					patternfile << "X";
	    				else if(shape==PIECE_SHAPE_DIAMOND)
	    					patternfile << "◊";
	    				else if(shape==PIECE_SHAPE_SQUARE)
	    					patternfile << "□";
	    				else if(shape==PIECE_SHAPE_STAR)
	    					patternfile << "*";
	    				else if(shape==PIECE_SHAPE_TRIANGLE)
	    					patternfile << "▲";
	    					
	    				int color=p->getColor();
	    				if(color==PIECE_COLOR_BLUE)
	    					patternfile << "B";
	    				else if(color==PIECE_COLOR_GREEN)
	    					patternfile << "G";
	    				else if(color==PIECE_COLOR_ORANGE)
	    					patternfile << "O";
	    				else if(color==PIECE_COLOR_PURPLE)
	    					patternfile << "P";
	    				else if(color==PIECE_COLOR_RED)
	    					patternfile << "R";
	    				else if(color==PIECE_COLOR_YELLOW)
	    					patternfile << "Y";
	    			}
	    			
	    			if(p->getType()==PIECE_TYPE_SHIELDED)
	    				patternfile << "]";
	    			else
	    				patternfile << " ";
	    		}
	    		
	    		if(it2<pline->end()-1)
	    			patternfile << "|";
	    		
	    	}//for pieces
	    	
	    	patternfile << endl;
    	}
    } //for lines
    
    patternfile << endl;
    patternfile << endl;
    
	patternfile.close();
	
	m_msg = string("saved in '") + string(LEVELEDITOR_FILE) + string("'");
}































