#include "code/tests/TestParticleGeneratorSegment.h"
#include <oslib/oslib.h>
#include "code/framework/anim/Anim.h"
#include "code/framework/utils.h"
#include "code/framework/fx/EffectsLib.h"
//#include "code/def.h"
#include "code/game/menus/MenusDefines.h"



//construction
TestParticleGeneratorSegment::TestParticleGeneratorSegment()
{
	//enableDebug();
	//debug("construction TestParticleGeneratorSegment");
	oslSetKeyAutorepeat(OSL_KEYMASK_UP|OSL_KEYMASK_DOWN|OSL_KEYMASK_RIGHT|OSL_KEYMASK_LEFT|OSL_KEYMASK_CIRCLE|OSL_KEYMASK_SQUARE|OSL_KEYMASK_L|OSL_KEYMASK_R,40,2);
	
	m_action=MENU_NOACTION;
	m_skip=false;
	
	//tests
	int transl_time=26; //apres 90 frames la particule est au bout du chemin
	int lifespan=transl_time; //duree de vie d'une particule: une fois arrivée au bout elle meurt, -1 pour attendre la fin de tous les effets (effets avec loop infini=problème :)
	int transl_dist=60;//ca parcours 160 pixels
	
	//debug("construction du generateur");
	m_test_partGen=new ParticleGeneratorSegment(240, 136, 0, 30, 2, 3, transl_time, transl_dist, lifespan, "res/pieces/star/star1.png", OSL_PF_4444);
	//debug("construction du generateur ok");
	m_test_partGen->setLocalEffect1SimpleTranslation();
	
	m_img=oslLoadImageFilePNG("res/pieces/cross/cross1.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_4444);
	oslSetImageRotCenter(m_img);
	
	//debug("fin construction TestParticleGeneratorSegment");
}

//destruction
TestParticleGeneratorSegment::~TestParticleGeneratorSegment()
{
	delete m_test_partGen;
	
	oslDeleteImage(m_img);
}

//demarrage du menu
int TestParticleGeneratorSegment::go()
{
	//debug("go");
	m_test_partGen->start();
	//debug("start ok");
	
	while(m_action==MENU_NOACTION)
	{
		oslBenchmarkTest(OSL_BENCH_START);
		
		//touches
		m_action=keys();
		if(m_action==MENU_QUIT)
			return m_action;
		
		//phase d'update	
		//debug("update");	
		m_test_partGen->update();
		//debug("update ok");
		
		if (!m_skip)
		{
			//phase de dessin
			oslStartDrawing();
			//debug("dessin");
			drawScene();
			//debug("dessin ok");
			oslBenchmarkTest(OSL_BENCH_END);
			oslSysBenchmarkDisplay();
			oslEndDrawing();
		}
		
		//End of a frame
        oslEndFrame();
		//Synchronize to ensure the game runs at 60 fps
        m_skip = oslSyncFrame();
		//oslAudioVSync(); //ne sert plus, oslEndFrame le fait
	}
	//il a choisi une option
	enableDebug();
	debug("fin boucle main, return "+toString(m_action));
	enableDebug(false);
		
	return m_action;	
}

//dessins
void TestParticleGeneratorSegment::drawScene()
{
	oslClearScreen(RGB(0,0,0));
	
	//debug("m_test_partGen draw..");
	m_test_partGen->draw();
	//debug("m_test_partGen draw ok");
	fx::drawXY(m_img, m_test_partGen->getX(), m_test_partGen->getY());
	oslDrawLine(m_test_partGen->getXA()+m_test_partGen->getX(), m_test_partGen->getYA()+m_test_partGen->getY(), m_test_partGen->getXB()+m_test_partGen->getX(), m_test_partGen->getYB()+m_test_partGen->getY(), RGB(255,0,0));
	oslDrawString(m_test_partGen->getXA()+m_test_partGen->getX(), m_test_partGen->getYA()+m_test_partGen->getY(), "A");
	oslDrawString(m_test_partGen->getXB()+m_test_partGen->getX(), m_test_partGen->getYB()+m_test_partGen->getY(), "B");
	
	oslDrawString(10, 210, ("orientation: " + toString(m_test_partGen->getOrientation()) + "degr.").c_str());
	oslDrawString(10, 220, ("scale: " + toString(m_test_partGen->getScale()) + "pixels").c_str());
	oslDrawString(10, 230, ("life: " + toString(m_test_partGen->getPLifespan()) + "frames").c_str());
	oslDrawString(10, 240, ("time: " + toString(m_test_partGen->getPTranslTime()) + "frames").c_str());
	oslDrawString(10, 250, ("distance: " + toString(m_test_partGen->getPDistance()) + "pixels").c_str());
	oslDrawString(10, 260, (toString(m_test_partGen->getCurrentNbParticles()) + " particles").c_str());
	
}

//gestion des touches
int TestParticleGeneratorSegment::keys()
{
	oslReadKeys();
	if (osl_pad.pressed.start) //Exit 
	{
		enableDebug();
		debug("start!");
		enableDebug(false);
		return MENU_MAIN;
	}
	
	//switch pause on/off
	if(osl_pad.pressed.triangle)
	{
		if(m_test_partGen->isRunning())
			m_test_partGen->stop();
		else
			m_test_partGen->start();
	}
	else if(osl_pad.pressed.square)
	{
		int scale=m_test_partGen->getScale();
		scale--;
		if(scale<=0)
			scale=1;		
		m_test_partGen->setScale(scale);
	}
	else if(osl_pad.pressed.circle)
	{
		int scale=m_test_partGen->getScale();
		scale++;	
		m_test_partGen->setScale(scale);
	}
	
	if(osl_pad.pressed.R)
	{
		int orientation=m_test_partGen->getOrientation();
		orientation+=3;
		if(orientation>=360)
			orientation=0;
		m_test_partGen->setOrientation(orientation);
	}
	else if(osl_pad.pressed.L)
	{
		int orientation=m_test_partGen->getOrientation();
		orientation-=3;
		if(orientation<0)
			orientation=359;
		m_test_partGen->setOrientation(orientation);
	}
	
	if(osl_pad.analogX>=100)
		m_test_partGen->setX(m_test_partGen->getX()+8);
	else if(osl_pad.analogX>=70)
		m_test_partGen->setX(m_test_partGen->getX()+4);
	else if(osl_pad.analogX>=20)
		m_test_partGen->setX(m_test_partGen->getX()+1);
	
	if(osl_pad.analogX<=-100)
		m_test_partGen->setX(m_test_partGen->getX()-8);
	else if(osl_pad.analogX<=-70)
		m_test_partGen->setX(m_test_partGen->getX()-4);
	else if(osl_pad.analogX<=-20)
		m_test_partGen->setX(m_test_partGen->getX()-1);
	
	if(osl_pad.analogY>=100)
		m_test_partGen->setY(m_test_partGen->getY()+8);
	else if(osl_pad.analogY>=70)
		m_test_partGen->setY(m_test_partGen->getY()+4);
	else if(osl_pad.analogY>=20)
		m_test_partGen->setY(m_test_partGen->getY()+1);
	
	if(osl_pad.analogY<=-100)
		m_test_partGen->setY(m_test_partGen->getY()-8);
	if(osl_pad.analogY<=-70)
		m_test_partGen->setY(m_test_partGen->getY()-4);
	if(osl_pad.analogY<=-20)
		m_test_partGen->setY(m_test_partGen->getY()-1);
		
	//vitesse des particules
	if (osl_pad.pressed.up)
	{
		int t=m_test_partGen->getPTranslTime();
		t+=2;
		m_test_partGen->setPTranslTime(t);
		m_test_partGen->setPLifespan(t);
	}
	else if(osl_pad.pressed.down)
	{
		int t=m_test_partGen->getPTranslTime();
		if(t>=5)
		{
			t-=2;
			m_test_partGen->setPTranslTime(t);
			m_test_partGen->setPLifespan(t);
		}
	}
	//distance de jet des particules
	else if(osl_pad.pressed.left)
	{
		int d=m_test_partGen->getPDistance();
		if(d>=10)
		{
			d-=2;
			m_test_partGen->setPDistance(d);
		}
	}
	else if(osl_pad.pressed.right)
	{
		int d=m_test_partGen->getPDistance();
		d+=2;
		m_test_partGen->setPDistance(d);
	}
	
	if(osl_pad.pressed.select)
	{
		m_test_partGen->generationSwitchOnOff();
	}
	
	return MENU_NOACTION;
}






























